/*
 * (C) Copyright 2008 Texas Insturments
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * (C) Copyright 2002
 * Gary Jennejohn, DENX Software Engineering, <garyj@denx.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * CPU specific code
 */

#include <common.h>
#include <command.h>
#include <asm/system.h>
#include <asm/cache.h>
#include <asm/armv7.h>
#include <asm/arch/io.h>
#include <asm/arch/system.h>

void save_boot_params_default(u32 r0, u32 r1, u32 r2, u32 r3)
{
}

void save_boot_params(u32 r0, u32 r1, u32 r2, u32 r3)
	__attribute__((weak, alias("save_boot_params_default")));

int cleanup_before_linux(void)
{
	printf("Reset ethernet/disable interrupt/flush cache...\n");
#if defined(CONFIG_SYS_SUPPORT_MULTICORE)
	extern uint g_console_protect_enable;
	g_console_protect_enable = 0; // disable console protection
#endif

#if 0 // MAC3_FIX_ME_ASAP
	//Reset ethernet
	rtd_outl(0xb8016234, 0x0);
	rtd_outl(0xb801603c, 0xffff);
	rtd_outl(0xb8016038, 0x1);
#endif

	//Disable UART0/UART1
#ifndef IS_FPGA
	rtd_clearbits(ISO_MISC_OFF_INT_EN_reg, _BIT2);
#endif

#if 0 // FIXME in Mac3 (unknown register)
	rtd_clearbits(SCPU_INT_EN_reg, _BIT1 | _BIT2 | _BIT3);
#endif

	/*
	 * this function is called just before we call linux
	 * it prepares the processor for linux
	 *
	 * we turn off caches etc ...
	 */
#ifndef CONFIG_SPL_BUILD
	disable_interrupts();
#endif

	/*
	 * Turn off I-cache and invalidate it
	 */
	icache_disable();
	invalidate_icache_all();

	/*
	 * turn off D-cache
	 * dcache_disable() in turn flushes the d-cache and disables MMU
	 */
	dcache_disable();
	v7_outer_cache_disable();

	/*
	 * After D-cache is flushed and before it is disabled there may
	 * be some new valid entries brought into the cache. We are sure
	 * that these lines are not dirty and will not affect our execution.
	 * (because unwinding the call-stack and setting a bit in CP15 SCTRL
	 * is all we did during this. We have not pushed anything on to the
	 * stack. Neither have we affected any static data)
	 * So just invalidate the entire d-cache again to avoid coherency
	 * problems for kernel
	 */
	invalidate_dcache_all();

	return 0;
}

int cleanup_before_bootcode(void) {
	return cleanup_before_linux();
}

int cleanup_before_dvrbootexe(void) {
	return cleanup_before_linux();
}


/* simulate the individual cpu power down sequence */
void cpu_reset(void *jump)  __attribute__((naked, noreturn));
void cpu_reset(void *jump) 
{
#ifndef CONFIG_JUMP_TO_64BIT_KERNEL
	register unsigned long rst asm("lr") = (unsigned long) jump;
#endif
	

	/* disable irq */
	__asm__ __volatile__ ( "cpsid	i \n\t"  );

	/* disable D, and ENTRY(v7_flush_dcache_all) */
	__asm__ __volatile__ (
		"1: dsb \n\t"
		"mrc	p15, 0, r0, c1, c0, 0		@ ctrl register \n\t"
		"bic	r0, r0, #0x0006			@ .............ca. \n\t"
		"mcr	p15, 0, r0, c1, c0, 0		@ disable caches \n\t"
		"isb \n\t"

		"dsb					@ ensure ordering with previous memory accesses \n\t"
		"mrc	p15, 1, r0, c0, c0, 1		@ read clidr \n\t"
		"ands	r3, r0, #0x7000000		@ extract loc from clidr  \n\t"
		"mov	r3, r3, lsr #23			@ left align loc bit field \n\t"
		"beq	finished			@ if loc is 0, then no need to clean \n\t"
		"mov	r10, #0				@ start clean at cache level 0 \n\t"
		"flush_levels: \n\t"
		"add	r2, r10, r10, lsr #1		@ work out 3x current cache level \n\t"
		"mov	r1, r0, lsr r2			@ extract cache type bits from clidr \n\t"
		"and	r1, r1, #7			@ mask of the bits for current cache only \n\t"
		"cmp	r1, #2				@ see what cache we have at this level \n\t"
		"blt	skip				@ skip if no cache, or just i-cache \n\t"

		"mcr	p15, 2, r10, c0, c0, 0		@ select current cache level in cssr \n\t"
		"isb					@ isb to sych the new cssr&csidr \n\t"
		"mrc	p15, 1, r1, c0, c0, 0		@ read the new csidr \n\t"

		"and	r2, r1, #7			@ extract the length of the cache lines \n\t"
		"add	r2, r2, #4			@ add 4 (line length offset) \n\t"
		"ldr	r4, =(0x3ff+1) \n\t"
		"sub	r4, r4, #1 \n\t"
		"ands	r4, r4, r1, lsr #3		@ find maximum number on the way size \n\t"
		"clz	r5, r4				@ find bit position of way size increment \n\t"
		"ldr	r7, =(0x7fff+1) \n\t"
		"sub	r7, r7, #1 \n\t"
		"ands	r7, r7, r1, lsr #13		@ extract max number of the index size \n\t"
		"loop1: \n\t"
		"mov	r9, r4				@ create working copy of max way size \n\t"
		"loop2: \n\t"
		"orr	r11, r10, r9, lsl r5		@ factor way and cache number into r11 \n\t"
		"orr	r11, r11, r7, lsl r2		@ factor index number into r11 \n\t"
		"mcr	p15, 0, r11, c7, c14, 2		@ clean & invalidate by set/way \n\t"
		"subs	r9, r9, #1			@ decrement the way \n\t"
		"bge	loop2 \n\t"
		"subs	r7, r7, #1			@ decrement the index \n\t"
		"bge	loop1 \n\t"
		"skip: \n\t"
		"add	r10, r10, #2			@ increment cache number  \n\t"
		"cmp	r3, r10 \n\t"
		"bgt	flush_levels \n\t"
		"finished: \n\t"
		"mov	r10, #0				@ swith back to cache level 0 \n\t"
		"mcr	p15, 2, r10, c0, c0, 0		@ select current cache level in cssr \n\t"
		"dsb \n\t"
		"isb \n\t"
		:
		:
		: /* "r0", "r1", "r2", "r3", "r4", "r5", "r7",
		     "r9", "r10", "r11"  */);


	/* flush i$ and disable i$, btb */
	__asm__ __volatile__ (
		"dsb \n\t"
		"mrc	p15, 0, r0, c1, c0, 0		@ ctrl register \n\t"
		"bic	r0, r0, #0x1000			@ ...i............ \n\t"
		"mcr	p15, 0, r0, c1, c0, 0		@ disable caches \n\t"
		"isb \n\t"
		"dsb \n\t"
		"mov	r0, #0 \n\t"
#if defined (CONFIG_SYS_SUPPORT_MULTICORE)
		"mcr	p15, 0, r0, c7, c1, 0 		@ ICIALLUIS, invalidate I-cache inner shareable \n\t"
		"mcr    p15, 0, r0, c7, c1, 6           @ BPIALLIS \n\t"
#endif  //#if defined (CONFIG_SYS_MULTICORE)
		"mcr	p15, 0, r0, c7, c5, 0		@ ICIALLU, I+BTB cache invalidate \n\t"
		"mcr    p15, 0, r0, c7, c5, 6           @ BPIALL \n\t"
		:::"r0") ;

	__asm__ __volatile__ (
		"isb \n\t"
		"dsb \n\t"
		"1: dsb \n\t"
		"mrc	p15, 0, r0, c1, c0, 0		@ ctrl register \n\t"
		"bic	r0, r0, #0x1			@ ...............m \n\t"
		"mcr	p15, 0, r0, c1, c0, 0		@ disable MMU \n\t"
		"isb    \n\t"
		"nop    \n\t"
		"nop    \n\t"
		"nop    \n\t"

		"mov	r0, #0 \n\t"
		"mcr p15, 0, r0, c8, c7, 0   @TLBIALL \n\t"  
		"mcr p15, 0, r0, c8, c6, 0   @DTLBIALL \n\t"
		"mcr p15, 0, r0, c8, c5, 0   @ITLBIALL \n\t"
#if defined (CONFIG_SYS_SUPPORT_MULTICORE)
		"mcr p15, 0, r0, c8, c3, 0   @TLBIALLIS \n\t"
#endif //#if !defined (CONFIG_SYS_SUPPORT_MULTICORE)

		"isb \n\t"
		"dsb \n\t"
		::: "r0");

#if 0  //optee enable this register is secure register uboot can't access
	/*ENTRY(cpu_v7_reset) ?  */
	__asm__ __volatile__ (
		"@@ SMP disable \n\t"
		"mrrc	p15, 1, r0, r1, c15 \n\t"
		"bic	r0, #(1 << 6) \n\t"
		"mcrr	p15, 1, r0, r1, c15 \n\t"
		"isb \n\t"
		"dsb \n\t"
		:::"r0", "r1");

#endif
#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
	//call_smc();
#else
/*mask sw reset for optee */
	__asm__ __volatile__ (
		"cmp    %0, #0 \n\t" 
		"bx  %0 \n\t"
		"isb    \n\t"

		::"r"(rst):"r0", "r1");
#endif	
}

