/* Copyright (C) 2011
 * Corscience GmbH & Co. KG - Simon Schwarz <schwarz@corscience.de>
 *  - Added prep subcommand support
 *  - Reorganized source - modeled after powerpc version
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <asm/bootm.h>
#include <exports.h>
#include <environment.h>
#include <wakeup.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/bootparam.h>
#include <platform_lib/board/pcb_mgr.h>
#include <watchdog.h>
#include <factorylib.h>
#include <panel.h>

#if defined(CONFIG_REALTEK_EERPOM_TV010_A)
#ifndef UINT8
#define UINT8 unsigned char
#endif

extern int getKeypadDisable(UINT8 *tmpflag);
#endif

#ifndef IS_FPGA
#include <platform_lib/board/pcb.h>
#endif
#ifdef CONFIG_SYS_SUPPORT_MULTICORE
#include <asm/arch/smp.h>
#endif

#include <gpt_parse.h>
DECLARE_GLOBAL_DATA_PTR;
#include <panel_config_parameter.h>

#include <dc_sys_reg.h>

#define ENABLE_SHOW_MOUNT_POINT
//#define ENABLE_SHOW_PART_NAME
#define arg_next(p)             (char *)((unsigned long)p+strlen(p))
#define KARGS_SIZE     4096
#define args_size(p)   ((KARGS_SIZE-strlen(p)) > 0 ? (KARGS_SIZE-strlen(p)) : 0)

#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
#define EARLYPRINTK   "earlyprintk=uart8250-32bit,0x18062300"
#else
#define EARLYPRINTK   "earlyprintk"
#endif


#if defined(CONFIG_SETUP_MEMORY_TAGS) || \
	defined(CONFIG_CMDLINE_TAG) || \
	defined(CONFIG_INITRD_TAG) || \
	defined(CONFIG_SERIAL_TAG) || \
	defined(CONFIG_REVISION_TAG)
static struct tag *params;
#endif

#if 0 // without k
extern uint *KcpuFW_entry_ptr;
#endif
extern uint *boot_av_info_ptr;
extern int boot_power_on_mode;
extern uint b_fw_desc_table_vaild;
extern unsigned int KernelImagePosition;
uint gdma_length = 0;
extern unsigned int rescue_mode_with_audio_flag;
extern uchar boot_panel_type;
#if defined(Config_Vulcan3_PCB_TRUE)
extern int EnableFHD;
#endif
extern uchar Mura_CUC_data_transfer;

#if defined(Config_Board_TV010_PANEL)
extern struct _PANEL_CONFIG_PARAMETER  default_panel_parameter;
#endif
extern void enable_console(void);
extern void disable_console(void);
extern int rtk_plat_read_fw_desc_table_from_eMMC(void);
extern int rtk_plat_read_fw_desc_table_from_NAND(void);
extern int rtk_plat_read_fw_desc_table_from_SPI_NAND(void);

extern int CusApi_get_powerdown_param(char *param, int buf_length);
extern void set_boot_reason(char *kargs);
#ifdef CONFIG_SYS_SUPPORT_MULTICORE
extern void Core_Wakeup(CoreWakeupFunc __Addr, u32 CoreID);
#endif
extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);
extern int rtk_plat_boot_prep_logbuffer_value(void);
extern void release_non_boot_core(void);

extern int rtk_plat_boot_prep_dcmt(void);
extern int rtk_plat_boot_prep_sb2(void);
extern int rtk_plat_boot_prep_hwm(void);

extern void CusApi_boot_prep_linux_for_standby(void);
extern void CusApi_boot_prep_linux_bootargs(void);
extern void CusApi_linux_param_set(char *kargs);

#ifdef CONFIG_OF_LIBFDT
extern int Fdt_prep_linux(bootm_headers_t *images);
extern int Kernel_mode_DTB_Verify(bootm_headers_t *images);
#endif

#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
void release_non_boot_core(void);
#endif

#if defined(CONFIG_SYS_RTK_SPI_FLASH) // from layout.txt
#include <parse_h.h>
#include <flashdev_s.h>

int layout_part_no = -1;
layout_part_desc layout_part_info[8];
#endif //defined(CONFIG_SYS_RTK_SPI_FLASH)


#ifdef CONFIG_LMB
static ulong get_sp(void)
{
	ulong ret;

	asm("mov %0, sp" : "=r"(ret) : );
	return ret;
}

void arch_lmb_reserve(struct lmb *lmb)
{
	ulong sp;

	/*
	 * Booting a (Linux) kernel image
	 *
	 * Allocate space for command line and board info - the
	 * address should be as high as possible within the reach of
	 * the kernel (see CONFIG_SYS_BOOTMAPSZ settings), but in unused
	 * memory, which means far enough below the current stack
	 * pointer.
	 */
	sp = get_sp();
	debug("## Current stack ends at 0x%08lx ", sp);

	/* adjust sp by 1K to be safe */
	sp -= 1024;
	lmb_reserve(lmb, sp,
		    gd->bd->bi_dram[0].start + gd->bd->bi_dram[0].size - sp);
}
#endif

#if 0//def CONFIG_OF_LIBFDT
static int fixup_memory_node(void *blob)
{
	bd_t	*bd = gd->bd;
	int bank;
	u64 start[CONFIG_NR_DRAM_BANKS];
	u64 size[CONFIG_NR_DRAM_BANKS];

	for (bank = 0; bank < CONFIG_NR_DRAM_BANKS; bank++) {
		start[bank] = bd->bi_dram[bank].start;
		size[bank] = bd->bi_dram[bank].size;
	}

	return fdt_fixup_memory_banks(blob, start, size, CONFIG_NR_DRAM_BANKS);
}
#endif

static void announce_and_cleanup(void)
{
	display_evaluate_time("bootcode end");
#if 0//defined(Config_Board_TV035) || defined(Config_Board_TV035_INT_DDR)  || defined(Config_Board_TV035_INT_DDR_DCU1_4G4G_DCU2_4G4G)
	if (konka_uart_enable){
		printf("konka_uart_enable = 1\n");
	}
	else{
		printf("konka_uart_enable = 0\n");
	}
#endif
	printf("Starting kernel ... \n");
	bootstage_mark_name(BOOTSTAGE_ID_BOOTM_HANDOFF, "start_kernel");
#ifdef CONFIG_BOOTSTAGE_REPORT
	bootstage_report();
#endif

#ifdef CONFIG_USB_DEVICE
	udc_disconnect();
#endif
	cleanup_before_linux();
}

#if defined(CONFIG_SETUP_MEMORY_TAGS) || \
	defined(CONFIG_CMDLINE_TAG) || \
	defined(CONFIG_INITRD_TAG) || \
	defined(CONFIG_SERIAL_TAG) || \
	defined(CONFIG_REVISION_TAG)
static void setup_start_tag (bd_t *bd)
{
	params = (struct tag *)bd->bi_boot_params;

	params->hdr.tag = ATAG_CORE;
	params->hdr.size = tag_size (tag_core);

	params->u.core.flags = 0;
	params->u.core.pagesize = 0;
	params->u.core.rootdev = 0;

	params = tag_next (params);
}
#endif

#ifdef CONFIG_SETUP_MEMORY_TAGS
static void setup_memory_tags(bd_t *bd)
{
	int i;

	for (i = 0; i < CONFIG_NR_DRAM_BANKS; i++) {
		params->hdr.tag = ATAG_MEM;
		params->hdr.size = tag_size (tag_mem32);

		params->u.mem.start = bd->bi_dram[i].start;
		params->u.mem.size = bd->bi_dram[i].size;

		params = tag_next (params);
	}
}
#endif

#ifdef CONFIG_CMDLINE_TAG
static void setup_commandline_tag(bd_t *bd, char *commandline)
{
	char *p;

	if (!commandline)
		return;

	/* eat leading white space */
	for (p = commandline; *p == ' '; p++);

	/* skip non-existent command lines so the kernel will still
	 * use its default command line.
	 */
	if (*p == '\0')
		return;

	params->hdr.tag = ATAG_CMDLINE;
	params->hdr.size =
		(sizeof (struct tag_header) + strlen (p) + 1 + 4) >> 2;

	strcpy (params->u.cmdline.cmdline, p);

	params = tag_next (params);
}
#endif

#ifdef CONFIG_INITRD_TAG
static void setup_initrd_tag(bd_t *bd, ulong initrd_start, ulong initrd_end)
{
	/* an ATAG_INITRD node tells the kernel where the compressed
	 * ramdisk can be found. ATAG_RDIMG is a better name, actually.
	 */
	params->hdr.tag = ATAG_INITRD2;
	params->hdr.size = tag_size (tag_initrd);

	params->u.initrd.start = initrd_start;
	params->u.initrd.size = initrd_end - initrd_start;

	params = tag_next (params);
}
#endif

#ifdef CONFIG_SERIAL_TAG
void setup_serial_tag(struct tag **tmp)
{
	struct tag *params = *tmp;
	struct tag_serialnr serialnr;
	void get_board_serial(struct tag_serialnr *serialnr);

	get_board_serial(&serialnr);
	params->hdr.tag = ATAG_SERIAL;
	params->hdr.size = tag_size (tag_serialnr);
	params->u.serialnr.low = serialnr.low;
	params->u.serialnr.high= serialnr.high;
	params = tag_next (params);
	*tmp = params;
}
#endif

#ifdef CONFIG_REVISION_TAG
void setup_revision_tag(struct tag **in_params)
{
	u32 rev = 0;
	u32 get_board_rev(void);

	rev = get_board_rev();
	params->hdr.tag = ATAG_REVISION;
	params->hdr.size = tag_size (tag_revision);
	params->u.revision.rev = rev;
	params = tag_next (params);
}
#endif

#if defined(CONFIG_SETUP_MEMORY_TAGS) || \
	defined(CONFIG_CMDLINE_TAG) || \
	defined(CONFIG_INITRD_TAG) || \
	defined(CONFIG_SERIAL_TAG) || \
	defined(CONFIG_REVISION_TAG)
static void setup_end_tag(bd_t *bd)
{
	params->hdr.tag = ATAG_NONE;
	params->hdr.size = 0;
}
#endif

#if 0//def CONFIG_OF_LIBFDT
static int create_fdt(bootm_headers_t *images)
{
	ulong of_size = images->ft_len;
	char **of_flat_tree = &images->ft_addr;
	ulong *initrd_start = &images->initrd_start;
	ulong *initrd_end = &images->initrd_end;
	struct lmb *lmb = &images->lmb;
	ulong rd_len;
	int ret;

	debug("using: FDT\n");

	boot_fdt_add_mem_rsv_regions(lmb, *of_flat_tree);

	rd_len = images->rd_end - images->rd_start;
	ret = boot_ramdisk_high(lmb, images->rd_start, rd_len,
			initrd_start, initrd_end);
	if (ret)
		return ret;

	ret = boot_relocate_fdt(lmb, of_flat_tree, &of_size);
	if (ret)
		return ret;

	fdt_chosen(*of_flat_tree, 1);
	fixup_memory_node(*of_flat_tree);
	fdt_fixup_ethernet(*of_flat_tree);
	fdt_initrd(*of_flat_tree, *initrd_start, *initrd_end, 1);

	return 0;
}
#endif

/**********************************************************
 * Append the pointer information of env_vars to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_envp(void)
{
	char *commandline = getenv("bootargs");
	char *wdt_env = getenv("wdt");
	char *tmp_cmdline = NULL;
	int len = 0;

	// Modify so that Coverity won't complain about tainted strings
	char *cmdline = NULL;
	cmdline = strdup(commandline);
	if(!cmdline) {
		return 0;
	}

	if (env_export() < 0) {
		printf("[ERR] %s: Cannot export environment\n", __func__);
	}
	else {
		len = strlen(commandline) + strlen(wdt_env) + 32;

		tmp_cmdline = (char *)malloc(len);
		if (!tmp_cmdline) {
			printf("[ERR] %s: Malloc failed\n", __func__);
		}
		else {
			memset(tmp_cmdline, 0, len);

			snprintf(tmp_cmdline, len, "%s envp=%lx wdt=%s ", cmdline, (ulong)env_get_export_all(), wdt_env);

			setenv("bootargs", tmp_cmdline);

			free(tmp_cmdline);
		}
	}

	free(cmdline);
	return 0;
}

/**********************************************************
 * Determine whether watchdog needs to be enabled
 **********************************************************/
extern int g_wdog_env;
static int rtk_plat_boot_prep_watchdog(void)
{
    char *wdt_env = getenv("wdt");
    if(wdt_env != NULL)
    {
        if(strcmp(wdt_env, "on") == 0){
            rtd_outl(ISO_MISC_TCWOV_reg, 0x08000000);	/*0.621377*8=4.97  (9.320655)s*/
            rtd_maskl(ISO_MISC_TCWCR_reg, ~0xFF, 0x00);	/*enable WDEN*/
            printf("Watchdog Enabled\n");
            g_wdog_env = 1;
        }
        else{
            g_wdog_env = 0;
        }
    }
    return 0;
}

/**********************************************************
 * Append the claim selinux mode
 **********************************************************/
static int rtk_plat_boot_prep_selinux(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	char *selinux_str = "androidboot.selinux=permissive";
	int len = 0;

	len = strlen(commandline) + strlen(selinux_str) + 4;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		snprintf(tmp_cmdline, len, "%s %s", commandline, selinux_str);

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

#ifdef LED_PWM_REG
	rtd_maskl(LED_PWM_REG, ~(LED_PWM_REG_MASK), LED_PWM_REG_MASK_VALUE);
	IO_Config(LED_PWM_SET_IO_CONFIG);
#endif

	return 0;
}

/**********************************************************
 * Append the flash type information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_flashtype(void)
{
#ifndef IS_FPGA
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;

	len = strlen(commandline) + 32;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);
#ifdef CONFIG_SYS_PCB_MANAGER
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, pcb_get_boot_flash_type_string());
#else
	#if defined(CONFIG_SYS_RTK_SPI_FLASH)
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, FLASH_TYPE_SPI_STRING);
	#elif defined(CONFIG_SYS_RTK_NAND_FLASH)
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, FLASH_TYPE_NAND_STRING);
	#elif defined(CONFIG_SYS_RTK_SPI_NAND)
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, FLASH_TYPE_SPI_NAND_STRING)
	#elif defined(CONFIG_SYS_RTK_EMMC_FLASH)
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, FLASH_TYPE_EMMC_STRING);
	#else
		snprintf(tmp_cmdline, len, "%s flashtype=%s", commandline, FLASH_TYPE_UNKNOWN_STRING);
	#endif
#endif

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}
#endif

	return 0;
}

/**********************************************************
 * Append the reclaim memory information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_reclaim(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;
	boot_av_info_t *boot_av = NULL;
	uint pov_size = 0;
	uint reclaim_size = 0;

	/* consider the video stream buffer size */
	if(gdma_length){
		pov_size = gdma_length;
	}else{
		boot_av = (boot_av_info_t *) MIPS_BOOT_AV_INFO_ADDR;
		pov_size = SWAPEND32(boot_av->dwVideoStreamLength);
	}

	if (pov_size % 0x100000 != 0) { // align MB
		pov_size += 0x100000;
	}

	/* dynamic compute the reclaim size */
	reclaim_size = (POWER_ON_MUSIC_BUFFER_SIZE +
					POWER_ON_VIDEO_BUFFER_SIZE +
					POWER_ON_MUSIC_STREAM_SIZE +
					POWER_ON_RESOURCE_SIZE +
					pov_size) >> 20;

	len = strlen(commandline) + 64;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		snprintf(tmp_cmdline, len, "%s reclaim=%dM@%dM last_image=%dM@%dM", commandline,
			reclaim_size,
			KERNEL_RESERVE_FOR_POM_POV_ADDR >> 20,
			POWER_ON_VIDEO_LAST_IMAGE_SIZE >> 20,
			POWER_ON_VIDEO_LAST_IMAGE_ADDR >> 20
		);

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}
	CusApi_boot_prep_linux_bootargs();
	return 0;
}

/**********************************************************
 * Append the quiet or earlyprintk parameter to control kernel print.
 **********************************************************/
#if 0
static int rtk_plat_boot_prep_kernel_print(char *string)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;

	if (strstr(commandline, string) != NULL) { // check the specific string is existed or not
		return 0;
	}

	len = strlen(commandline) + strlen(string) + 4;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		if (!strncmp(getenv("print"),"off",3))
		{
			sprintf(tmp_cmdline, "%s quiet loglevel=0", commandline);
		}
		else
		{
			if (strcmp(string, "loglevel") == 0)
			{
				extern int get_current_loglevel (void);
				sprintf(tmp_cmdline, "%s %s=%d", commandline, string,get_current_loglevel());
			}
			else
			{
				sprintf(tmp_cmdline, "%s %s", commandline, string);
			}
		}

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
#endif

#ifdef CONFIG_SYS_RTK_EMMC_FLASH
/**********************************************************
 * Append the mmcparts parameter to pass mmc partition info.
 **********************************************************/
static int rtk_plat_boot_prep_mmcparts(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	char tmp_str_buf[512];
	int len = 0;
	unsigned int size = 0;

	unsigned int part_num = 0, loop = 0;
	gpt_entry entry;
	char partname[256] = {0};
	unsigned long long start = 0, end = 0;
	int ret = 0;
	int root_index = -1;

	// Modify so that Coverity won't complain about tainted strings
	char *cmdline = NULL;

	ret = get_gpt_entry_num(&part_num);
	if(ret < 0){
		printf("Get gpt partition info fail: ignore %s\n", __FUNCTION__);
		return 0;
	}

	cmdline = strdup(commandline);
	if(!cmdline) {
		return 0;
	}

	len = strlen(commandline) + 768;  // [R2852TV010-207] len must be big, fix kernel cmd lost

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}else{
		memset(tmp_cmdline, 0, len);
		snprintf(tmp_cmdline, len, "%s mmcparts=rtkemmc:", cmdline);

		/*get first partition offset*/
		ret = get_gpt_entry_by_index(&entry, 0);
		if(!ret){
			start = get_partition_starting_by_gpt_entry(entry);
			size = (unsigned int)(start >> 10); // kbytes

			snprintf((char *)tmp_str_buf, sizeof(tmp_str_buf), "%uk,", size);
			strcat(tmp_cmdline, (char *)tmp_str_buf);
		}

		/*parse gpt partition info*/
		for(loop = 0; loop < part_num; loop ++){
			ret = get_gpt_entry_by_index(&entry, loop);
			if(ret < 0){
				continue;
			}

			start = get_partition_starting_by_gpt_entry(entry);
			end = get_partition_ending_by_gpt_entry(entry);
			get_partition_partname_by_gpt_entry(entry, partname);

			memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
			size = (unsigned int)((end - start) >> 10); // kbytes

			snprintf((char *)tmp_str_buf, sizeof(tmp_str_buf), "%uk(/%s)", size, partname);

			strcat(tmp_cmdline, (char *)tmp_str_buf);

			if (loop != (part_num - 1)) {
	       		strncat(tmp_cmdline, ",", 1);
	       	}

			if(strncmp("system", (char *)partname, strlen("system"))== 0)
				root_index = loop + 1;
		}

		memset(tmp_str_buf, 0, sizeof(tmp_str_buf));

		if(root_index < 0){
			if(boot_mode != BOOT_RESCUE_MODE){
				/*android 10 patch: super partition*/
				if(getenv("boot_initrd") != NULL)
					sprintf(tmp_str_buf, " %s",getenv("boot_initrd"));
			}else{
				/*this is a workaroud solution for an install_a bug when using ext4  !!*/
				snprintf(tmp_str_buf, sizeof(tmp_str_buf), " root=/dev/mmcblk0p1 rootfstype=ext4 ro rootwait");
			}
		}else{
			/*other android solution, android 9*/
			snprintf(tmp_str_buf,  sizeof(tmp_str_buf), " root=/dev/mmcblk0p%d rootfstype=ext4  rootwait", root_index);
		}

		strncat(tmp_cmdline, tmp_str_buf, strlen(tmp_str_buf));
		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	free(cmdline);

	return 0;
}
#endif

#ifdef CONFIG_SYS_RTK_NAND_FLASH

/**********************************************************
 * Append the mtdparts information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_mtdparts(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	fw_desc_table_v1_t *fw_desc_table_v1 = NULL;
	part_desc_entry_v1_t *part_entry = NULL;
	int fw_desc_table_base = rtk_mem_info->fw_desc_table_base_addr;
	unsigned char empty_mount[sizeof(part_entry->mount_point)];
	char tmp_str_buf[64];
	fw_type_code_t  root_type = FW_TYPE_SQUASH;
	int len = 0;
	int i = 0;
	int part_count = 0;
	unsigned int size = 0;
	int ret = RTK_PLAT_ERR_OK;
	char* tmp_rootstr = NULL;

	// check if already copied fw desc table from flash to ram
	if (!b_fw_desc_table_vaild) {
		ret = rtk_plat_read_fw_desc_table_from_NAND();
		if (ret != RTK_PLAT_ERR_OK)
		{
			printf("Read failed: ignore %s\n", __FUNCTION__);
			return 0;
		}
	}

	len = strlen(commandline) + 768;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);
		memset(empty_mount, 0, sizeof(empty_mount));

		sprintf(tmp_cmdline, "%s mtdparts=rtk_nand:", commandline);

		/* get partition table from fw_desc_table */
		fw_desc_table_v1 = (fw_desc_table_v1_t *)fw_desc_table_base;
		part_count = fw_desc_table_v1->part_list_len / sizeof(part_desc_entry_v1_t);
		part_entry = (part_desc_entry_v1_t *)(fw_desc_table_base + sizeof(fw_desc_table_v1_t));

		/* append each partition info to command line */
		for (i = 0; i < part_count; i++)
		{
			memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
			size = part_entry[i].length / 0x400; // kbytes

			if (memcmp(empty_mount, part_entry[i].mount_point, sizeof(empty_mount)) != 0) {
				sprintf((char *)tmp_str_buf, "%uk(%s)", size, part_entry[i].mount_point);
			}
			else {
				sprintf((char *)tmp_str_buf, "%uk", size);
			}
			if ((part_entry[i].type == PART_TYPE_FS) &&
	       		(strlen((char *)part_entry[i].mount_point) == 1) &&	// rootfs is "/"
	       		(strcmp("/", (char *)part_entry[i].mount_point) == 0)) {
	       		if (part_entry[i].fw_type == FW_TYPE_RESERVED) {
	       			root_type = FW_TYPE_YAFFS2;
					tmp_rootstr = "yaffs2";
	       		} else {
	       			root_type = part_entry[i].fw_type;
					printf("claire check @ %dth - type:%d\n\n", i, part_entry[i].fw_type);
					if (root_type == FW_TYPE_SQUASH) {
						tmp_rootstr = "squashfs";
					} else if (root_type == FW_TYPE_EXT3) {
						tmp_rootstr = "ext4";
					} else {
						tmp_rootstr = "yaffs2";
					}
	       		}
			}
			strcat(tmp_cmdline, (char *)tmp_str_buf);

			if (i != (part_count - 1)) {
	       		strncat(tmp_cmdline, ",", 1);
	       	}
		}

		memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
#ifdef CONFIG_SYS_RTK_NAND_FLASH
		if (tmp_rootstr  != NULL)
			sprintf(tmp_str_buf, " rootfstype=%s root=31:01", tmp_rootstr);
#else
		if (tmp_rootstr  != NULL)
			sprintf(tmp_str_buf, " rootfstype=%s", tmp_rootstr);
#endif
		strncat(tmp_cmdline, tmp_str_buf, strlen(tmp_str_buf));

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
#endif

#ifdef CONFIG_SYS_RTK_SPI_NAND

/**********************************************************
 * Append the mtdparts information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_sn_mtdparts(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	fw_desc_table_v1_t *fw_desc_table_v1 = NULL;
	part_desc_entry_v1_t *part_entry = NULL;
	int fw_desc_table_base = rtk_mem_info->fw_desc_table_base_addr;
	unsigned char empty_mount[sizeof(part_entry->mount_point)];
	char tmp_str_buf[64];
	fw_type_code_t  root_type = FW_TYPE_SQUASH;
	int len = 0;
	int i = 0;
	int part_count = 0;
	unsigned int size = 0;
	int ret = RTK_PLAT_ERR_OK;
	char* tmp_rootstr = NULL;

	// check if already copied fw desc table from flash to ram
	if (!b_fw_desc_table_vaild) {
		ret = rtk_plat_read_fw_desc_table_from_SPI_NAND();
		if (ret != RTK_PLAT_ERR_OK)
		{
			printf("Read failed: ignore %s\n", __FUNCTION__);
			return 0;
		}
	}

	len = strlen(commandline) + 256;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);
		memset(empty_mount, 0, sizeof(empty_mount));

		sprintf(tmp_cmdline, "%s mtdparts=rtk_sn:", commandline);

		/* get partition table from fw_desc_table */
		fw_desc_table_v1 = (fw_desc_table_v1_t *)fw_desc_table_base;
		part_count = fw_desc_table_v1->part_list_len / sizeof(part_desc_entry_v1_t);
		part_entry = (part_desc_entry_v1_t *)(fw_desc_table_base + sizeof(fw_desc_table_v1_t));

		/* append each partition info to command line */
		for (i = 0; i < part_count; i++)
		{
			memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
			size = part_entry[i].length / 0x400; // kbytes

			if (memcmp(empty_mount, part_entry[i].mount_point, sizeof(empty_mount)) != 0) {
				sprintf((char *)tmp_str_buf, "%uk(%s)", size, part_entry[i].mount_point);
			}
			else {
				sprintf((char *)tmp_str_buf, "%uk", size);
			}
			if ((part_entry[i].type == PART_TYPE_FS) &&
	       		(strlen((char *)part_entry[i].mount_point) == 1) &&	// rootfs is "/"
	       		(strcmp("/", (char *)part_entry[i].mount_point) == 0)) {
	       		if (part_entry[i].fw_type == FW_TYPE_RESERVED) {
	       			root_type = FW_TYPE_YAFFS2;
					tmp_rootstr = "yaffs2";
	       		} else {
	       			root_type = part_entry[i].fw_type;
					printf("claire check @ %dth - type:%d\n\n", i, part_entry[i].fw_type);
					if (root_type == FW_TYPE_SQUASH) {
						tmp_rootstr = "squashfs";
					} else if (root_type == FW_TYPE_EXT3) {
						tmp_rootstr = "ext4";
					} else {
						tmp_rootstr = "yaffs2";
					}
	       		}
			}
			strcat(tmp_cmdline, (char *)tmp_str_buf);

			if (i != (part_count - 1)) {
	       		strncat(tmp_cmdline, ",", 1);
	       	}
		}

		memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
#ifdef CONFIG_SYS_RTK_SPI_NAND
		sprintf(tmp_str_buf, " rootfstype=%s root=31:01", (root_type == FW_TYPE_SQUASH ? "squashfs" : "yaffs2"));
#else
		if (tmp_rootstr  != NULL)
			sprintf(tmp_str_buf, " rootfstype=%s", tmp_rootstr);
#endif
		strncat(tmp_cmdline, tmp_str_buf, strlen(tmp_str_buf));

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
#endif

/**********************************************************
 * Append the mtdparts information to bootargs.
 **********************************************************/
#ifdef CONFIG_SYS_RTK_SPI_FLASH
static int rtk_plat_partition_sort_by_offset(void)
{
	layout_part_desc tmp_part;
	int i, j;

	for(i=0; i<layout_part_no; i++){
		for(j=0; j<(layout_part_no-1-i); j++){

			if(layout_part_info[j].offset > layout_part_info[j+1].offset){
				tmp_part = layout_part_info[j];
				layout_part_info[j] = layout_part_info[j+1];
				layout_part_info[j+1] = tmp_part;
			}
		}
	}

	return 0;
}

static int rtk_plat_boot_prep_mtdparts_nor(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;
	int i = 0;

	char default_block[384] = "";
	char tmp_str_buf[64];

	int flash_size = rtk_spi_nor_get_size();

	int root_part_mtd_no=0;
	int root_part_no=0;
	int free_part_no=0;

	// Arrange memory for command
	memset(default_block, 0, sizeof(default_block));

	len = strlen(commandline) + sizeof(default_block);

	tmp_cmdline = (char *)malloc(len);
	if(!tmp_cmdline)
	{
		DBG_ERR("Malloc(%d) failed!\n", len);
	}
	else
	{
		// Initial Flags
		//unsigned int Next_offset = 0;
		memset(tmp_cmdline, 0, len);
		int factory_size = factory_Get_FACSize();

		// Make sure we got information from layout.txt
		if (layout_part_no == -1)
		{
			get_layout();
		}

		if(layout_part_no > 0)
		{
			//sort partition by partition offset
			rtk_plat_partition_sort_by_offset();

			//sanity check
			if((layout_part_info[layout_part_no-1].offset + layout_part_info[layout_part_no-1].size + factory_size) > flash_size)
			{
				DBG_ERR("Sanity check fail!\n");
			}

			for(i=0; i<layout_part_no; i++){
				//process first partition
				if(i == 0){
					if(layout_part_info[i].offset != 0){
						memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
						sprintf((char *)tmp_str_buf, "%uk", (unsigned int)(layout_part_info[i].offset/1024));
						strcat(default_block, tmp_str_buf);
						free_part_no++;

						DBG_MSG("default_block:%s!\n", default_block);
					}
				}else{
					//process free space between partition[i-1] and partition[i]
					if((layout_part_info[i-1].offset + layout_part_info[i-1].size) != layout_part_info[i].offset){
						memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
						sprintf((char *)tmp_str_buf, ",%uk", \
							(unsigned int)(layout_part_info[i].offset - (layout_part_info[i-1].offset+ layout_part_info[i-1].size))/1024);
						strcat(default_block, tmp_str_buf);
						free_part_no++;

						DBG_MSG("default_block:%s!\n", default_block);
					}
				}

				// Check mount_point is root
				if (!strcmp(layout_part_info[i].mount_point, "/"))
				{
					root_part_mtd_no = free_part_no+i;
					root_part_no = i;
					DBG_MSG("mount_point:/, root_part_mtd_no:%d, root_part_no:%d!\n", root_part_mtd_no, root_part_no);
				}

				// process current part
				memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
				sprintf((char *)tmp_str_buf, ",%uk", (unsigned int)layout_part_info[i].size/1024);
				strcat(default_block, tmp_str_buf);
				DBG_MSG("default_block:%s!\n", default_block);

		#if (defined(ENABLE_SHOW_MOUNT_POINT) || defined(ENABLE_SHOW_PART_NAME))
			// Prepare current part mount point string
			#if defined(ENABLE_SHOW_MOUNT_POINT)
				char *tmp_str = layout_part_info[i].mount_point;
			#else //#elif defined(ENABLE_SHOW_PART_NAME)
				char *tmp_str = layout_part_info[i].partname;
			#endif //defined(ENABLE_SHOW_MOUNT_POINT)
				DBG_MSG("offset:0x%x, %u:%s(%d)!\n", layout_part_info[i].offset, layout_part_info[i].size, tmp_str, strlen(tmp_str));
				if(strlen(tmp_str)!=0)
				{
					memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
					sprintf((char *)tmp_str_buf, "(%s)", tmp_str);
					strcat(default_block, tmp_str_buf);
					DBG_MSG("default_block:%s!\n", default_block);
				}
		#endif //(defined(ENABLE_SHOW_MOUNT_POINT) || defined(ENABLE_SHOW_PART_NAME))

			#if defined(Config_Secure_AES_TRUE)
				// Prepare current part type string
				if (*layout_part_info[i].type)
				{
					memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
					sprintf((char *)tmp_str_buf, "[%s]", layout_part_info[i].type);
					strcat(default_block, tmp_str_buf);
					DBG_MSG("default_block:%s!\n", default_block);
				}
			#endif //defined(Config_Secure_AES_TRUE)

			}
		}

		if (flash_size-layout_part_info[layout_part_no-1].offset - layout_part_info[layout_part_no-1].size -factory_size != 0)
		{
			// Undefine space
			memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
			sprintf((char *)tmp_str_buf, ",%dk", \
				(unsigned int)(flash_size-layout_part_info[layout_part_no-1].offset - layout_part_info[layout_part_no-1].size -factory_size)/1024);
			strcat(default_block, tmp_str_buf);
			DBG_MSG("default_block:%s!\n", default_block);
		}

		// Factory size
		memset(tmp_str_buf, 0, sizeof(tmp_str_buf));
		sprintf((char *)tmp_str_buf, ",%dk", (factory_size)/1024);
		strcat(default_block, tmp_str_buf);
		DBG_MSG("default_block:%s!\n", default_block);

		// Prepare command line
		sprintf(tmp_cmdline, "%s rootfstype=%s root=31:%02d mtdparts=rtk_nor:%s", commandline, layout_part_info[root_part_no].filesystem, root_part_mtd_no, default_block);
		DBG_MSG("rootfstype=%s root=31:%02d\n", layout_part_info[root_part_no].filesystem, root_part_mtd_no	);

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
#endif //CONFIG_SYS_RTK_SPI_FLASH

/**********************************************************
 * Append standby information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_for_standby(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	char *tmp_cmdline_emcu = NULL;
	char *cus_param = NULL;
	int len = 0;
	extern uchar boot_ac_on;
	len = strlen(commandline) + 64 + 1; // 1 for space

#if defined(CONFIG_CUSTOMER_TV010)
	unsigned int Ir_Custom_Code = 0xf1f1;
#endif

	tmp_cmdline = (char *)malloc(len);
	tmp_cmdline_emcu = (char *)malloc(64);
	cus_param = (char *)malloc(64);

	if ((!tmp_cmdline) || (!tmp_cmdline_emcu) || (!cus_param)) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);
		memset(tmp_cmdline_emcu, 0, 64);
		memset(cus_param, 0, 64);

		if (boot_power_on_mode == POWER_OFF)
		{
			if (CusApi_get_powerdown_param(cus_param, 64) == 1) // need append cus_param
			{
				snprintf(tmp_cmdline_emcu, 64, "%s=%s", "POWERDOWN", cus_param);
			}
			else // original
			{
				strcpy(tmp_cmdline_emcu, "POWERDOWN");
			}
		}

		if (boot_ac_on)
		{
			//move to boot_prep_linux()
			//strcat(tmp_cmdline_emcu, " ac_on");
#ifdef CONFIG_MISC_MGR_WAKEUP
			rtkpm_clr_share_mem();
#endif
		}

#if defined(CONFIG_CUSTOMER_TV010) && defined(CONFIG_MISC_MGR_CUSTOM_TV010) && defined(CONFIG_MISC_MGR_WAKEUP)
		if(boot_power_on_mode == POWER_OFF)
		{
			unsigned int  keyarray[15] = {  0x916ef1f1, 0x0cf3f1f1,  //pwr
                                                        0xb6db6fff,
                                                                0x12edf1f1,
                                                                0x13ecf1f1,
                                                                0xfff1f1,  //zero
                                                                0x2fdf1f1,
                                                                0x3fcf1f1,
                                                                0x4fbf1f1,
                                                                0x5faf1f1,
                                                                0x6f9f1f1,
                                                                0x7f8f1f1,
                                                                0x8f7f1f1,
                                                                0x9f6f1f1,
                                                                0x1fef1f1       };      //nine
                        unsigned long long value1=0,value2=0,value3=0;
                        unsigned long long pinarray[3] = {0x00,0x00,0x00};

			if (pcb_mgr_get_enum_info_byname("PIN_LEDG", &value1) == 0)
			{
				pinarray[0] = value1;
			}
			if (pcb_mgr_get_enum_info_byname("PIN_LEDR", &value2) == 0)
			{
				pinarray[1] = value2;
			}
			if (pcb_mgr_get_enum_info_byname("PIN_STR", &value3) == 0)
			{
				pinarray[2] = value3;
			}
			unsigned long long keypad[5]={0x00,0x00,0x00,0x00,0x00};
			unsigned long long value01=0,value02=0,value03=0;
			//value04=0,value05=0;
            UINT8 keypaddisabled = 0;
#ifdef CONFIG_REALTEK_EERPOM_TV010_A
            getKeypadDisable(&keypaddisabled);
#endif
            if (keypaddisabled == 0) {
			if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_PWR", &value01) == 0){
			keypad[0] = value01;
			}
			if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_UP", &value02) == 0){
			keypad[1] = value02;
			}
			if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_DOWN", &value03) == 0){
			keypad[2] = value03;
			}
            }

			rtkpm_clr_share_mem();
			powerMgr_setup_power_en(3, &pinarray[0]);

			char *Custom_Ir = getenv("Custom_Ir_Code");
			if(NULL != Custom_Ir)
			{
				Ir_Custom_Code = (unsigned int)simple_strtoull(Custom_Ir,NULL,16);
				printf("Ir_Custom_Code %4X to MCU51 \n", Ir_Custom_Code);
				if(Ir_Custom_Code != 0xf1f1)
				{
					for(int i = 0;i < sizeof(keyarray)/sizeof(unsigned int);i++)
					{
						if((keyarray[i] & 0xffff) == 0xf1f1)
						{
							keyarray[i] = (keyarray[i] & 0xffff0000) | (Ir_Custom_Code & 0x0000ffff);
						//	printf("keyarray[%d] is 0x%x \n",i, keyarray[i]);
						}
					}
				}
			}
			powerMgr_setup_irda( 5, 15, &keyarray[0]);
			powerMgr_setup_keypad_multi(3, &keypad[0]);

              }
#endif

#if defined(CONFIG_MISC_MGR_CUSTOM_TV035) && defined(CONFIG_MISC_MGR_WAKEUP)
		unsigned long long pinarray[2] = { 0x00 ,0x00 };
		unsigned int  keyarray[5] = { 0xa05ffb04 ,0x0bf4e7e7 ,0x20b ,0x18e7e7e7 , 0x2379ff00};  //Power/Source/CH+/CH-/Logo LED  mode
		unsigned long long value1=0,value2=0;
		if (pcb_mgr_get_enum_info_byname("POWER_EN", &value1) == 0)
		{
			pinarray[0] = value1;//0x100010c04;//value1;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_POWER_SOC_EN", &value2) == 0){
			pinarray[1] = value2;
		}
		unsigned long long keypad[5]={0x00 ,0x00,0x00 ,0x00,0x00,0x00};
		unsigned long long value=0;//[5]={0};//value0=0;
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_PWR", &value) == 0){
			keypad[0] = value;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_DOWN", &value) == 0){
			keypad[1] = value;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_UP", &value) == 0){
			keypad[2] = value;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_LEFT", &value) == 0){
			keypad[3] = value;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_RIGHT", &value) == 0){
			keypad[4] = value;
		}
		if (pcb_mgr_get_enum_info_byname("PIN_KEYPAD_SOURCE", &value) == 0){
			keypad[5] = value;
		}
		powerMgr_setup_power_en(2,&pinarray[0]);
		powerMgr_setup_irda(10,5,&keyarray[0]);
		  powerMgr_setup_keypad_multi(1,&keypad[0]);
		  #if defined(CONFIG_REALTEK_AMP_RT9107B)
		  // RT9107b need to do that if standby, need to set PIN_AMP_MUTE low for PWDN ON/OFF Sequence
		  if(boot_power_on_mode == POWER_OFF)
		  {
			  printf("\n [denney]ac on to standby,need to turn on led IO_Set(PIN_AMP_MUTE, 1)\n");
			  IO_Set(PIN_AMP_MUTE, 1);//to fix standby off ,then AC off ,AC on ,standby on,cause no audio ; beacause ac off ,ac on to standby,PIN_AMP_MUTE pin is high,so when standby ,set PIN_AMP_MUTE low
		  }
		  #endif
#endif

		snprintf(tmp_cmdline, len, "%s %s", commandline, tmp_cmdline_emcu);

		setenv("bootargs", tmp_cmdline);
		WATCHDOG_DISABLE();
	}

	free(tmp_cmdline);
	free(tmp_cmdline_emcu);
	free(cus_param);

	return 0;
}

/**********************************************************
 * Append the cpu_nr information to bootargs.
 **********************************************************/
#ifdef  CONFIG_SYS_SUPPORT_MULTICORE
int rtk_plat_boot_prep_cpus(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;
	int cpu_nr;
#if 0
	unsigned int hw_cpu_nr;

	hw_cpu_nr = ((rtd_inl(0xb8017194) & (_BIT16 | _BIT17)) >> 16);
	if ((CONFIG_SYS_NR_CPUS == 0) || (hw_cpu_nr != 0)) { /* 0 means auto detect */
		cpu_nr = 4 - hw_cpu_nr;
	}
	else {
		cpu_nr = CONFIG_SYS_NR_CPUS;
		//set to max count, if setting is larger than 4.
		if (cpu_nr > 4) cpu_nr = 4;
		//set to min count, if setting is smaller than 0.
		if (cpu_nr < 0) cpu_nr = 1;
	}
#endif
	cpu_nr = smp_active_cores();
	len = strlen(commandline) + 32;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		sprintf(tmp_cmdline, "%s nr_cpus=%d maxcpus=%d", commandline, cpu_nr, cpu_nr);

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

   return 0;
}
#endif
/**********************************************************
 * Append the kcpu memory entry and size information to bootargs.
 **********************************************************/
#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
static int rtk_plat_boot_prep_kcpu(void)
{
   char *commandline = getenv("bootargs");
   char *tmp_cmdline = NULL;
   int len = 0;

   unsigned int kentry = 0x1b000000 ;
   unsigned int ksize = 0x20000000 - 0x1b000000;

   len = strlen(commandline) + 64;

   tmp_cmdline = (char *)malloc(len);
   if (!tmp_cmdline) {
      printf("[ERR] %s: Malloc failed\n", __func__);
   }
   else {
      memset(tmp_cmdline, 0, len);

      sprintf(tmp_cmdline, "%s kentry=%dM@0x%x", commandline, ksize >> 20, kentry);
		setenv("bootargs", tmp_cmdline);

      free(tmp_cmdline);
	}

	return 0;
}
#else //#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
#if 0 // without k
static int rtk_plat_boot_prep_kcpu(void)
{
   char *commandline = getenv("bootargs");
   char *tmp_cmdline = NULL;
   int len = 0;
	unsigned int kentry = (*KcpuFW_entry_ptr & 0x1fffffff)- 0x1000;
	unsigned int ksize = 0x1dc00000 - kentry;

   len = strlen(commandline) + 64;

   tmp_cmdline = (char *)malloc(len);
   if (!tmp_cmdline) {
      printf("[ERR] %s: Malloc failed\n", __func__);
   }
   else {
      memset(tmp_cmdline, 0, len);

      sprintf(tmp_cmdline, "%s kentry=%dM@0x%x", commandline, ksize >> 20, kentry);
		setenv("bootargs", tmp_cmdline);

      free(tmp_cmdline);
	}

	return 0;
}
#endif
#endif //#ifdef Config_Booting_Linux_type_is_SECURE_LINUX

#ifdef CONFIG_IP_TO_KERNEL
/**********************************************************
 * Append the network information to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_network(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;

	len = strlen(commandline) + 64;

	tmp_cmdline = (char *)malloc(len);
	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		sprintf(tmp_cmdline, "%s ip=%s::%s:%s", commandline, getenv("ipaddr"), getenv("gatewayip"), getenv("netmask"));

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
#endif	//CONFIG_IP_TO_KERNEL

#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
/**********************************************************
 *  Set secure linux env parameter to CONFIG_SECURE_LINUX_BOOT_PARAM_BASE
 **********************************************************/
static int rtk_plat_boot_prep_secure_linux_envp(void)
{

	#define ATAT_COMMAND_LINE_BUFFER_LENGTH_MAX	256

	struct tag *tagptr;
	int length;
	char flashtypedata[64]= {'\0'};
	char commanddata[ATAT_COMMAND_LINE_BUFFER_LENGTH_MAX] = {'\0'};
	unsigned long *envdst = (unsigned long *)CONFIG_SECURE_LINUX_BOOT_PARAM_BASE;

	#if defined(CONFIG_SYS_RTK_SPI_FLASH)
		sprintf(flashtypedata,"flashtype=%s", FLASH_TYPE_SPI_STRING);
	#elif defined(CONFIG_SYS_RTK_NAND_FLASH)
		sprintf(flashtypedata,"flashtype=%s", FLASH_TYPE_NAND_STRING);
	#elif defined(CONFIG_SYS_RTK_EMMC_FLASH) || defined(CONFIG_SYS_RTK_EMMC_DW_FLASH)
		sprintf(flashtypedata,"flashtype=%s", FLASH_TYPE_EMMC_STRING);
	#else
		sprintf(flashtypedata,"flashtype=%s", FLASH_TYPE_UNKNOWN_STRING);
	#endif

	sprintf(commanddata, "%s %s console=%s normalLinuxATAG=0x%x\n",EARLYPRINTK,flashtypedata,getenv("console"), env_get_config_boot_param_base());
	printf("secure linux command line:%s\n", commanddata);

	/*ATAG format : size tag data, data length is size-2*/
	for_each_tag(tagptr, (struct tag *)env_get_config_boot_param_base())
	{
		//printf("original ATAG %x,%x\n",tagptr->hdr.tag,  tagptr->hdr.size);

		if(tagptr->hdr.tag != ATAG_CMDLINE)
		{
			*envdst++ = tagptr->hdr.size;
			*envdst++ = tagptr->hdr.tag;
			memcpy(envdst,&tagptr->u,(tagptr->hdr.size - 2) * 4);
			envdst += (tagptr->hdr.size - 2);
		}
	}

	length = strlen(commanddata);

	//size
	*envdst++ = (length + 3)/4 + 2;
	//tag
	*envdst++ = ATAG_CMDLINE;
	//data
	strcpy((char *)envdst, commanddata);
	envdst += (length + 3)/4;

	//size and tag for ending ATAG
	*envdst++ = 0x0;
	*envdst++ = 0x0;


	#if 0 //debug
	for_each_tag(tagptr, (struct tag *)CONFIG_SECURE_LINUX_BOOT_PARAM_BASE)
	{
		printf("secure ATAG %x,%x\n",tagptr->hdr.tag,  tagptr->hdr.size);//
	}
	#endif

}
#endif

/**********************************************************
 * Append the bootcode version to bootargs.
 **********************************************************/
static int rtk_plat_boot_prep_bootcode_version(void)
{
	char *commandline = getenv("bootargs");
	char *tmp_cmdline = NULL;
	int len = 0;

	len = strlen(commandline) + 256;	// for dependency case, such as dependency_LOGBUF_Muye_20170222_140905

	tmp_cmdline = (char *)malloc(len);

	if (!tmp_cmdline) {
		printf("[ERR] %s: Malloc failed\n", __func__);
	}
	else {
		memset(tmp_cmdline, 0, len);

		sprintf(tmp_cmdline, "%s %s", commandline, CONFIG_BOOTCODE_VERSION);

		setenv("bootargs", tmp_cmdline);

		free(tmp_cmdline);
	}

	return 0;
}
extern int rtk_check_ip_bisr(void);
void linux_param_set(char *kargs)
{
	char *cmdline = getenv ("bootargs");
	extern uchar boot_ac_on;
	extern int get_current_loglevel (void);

	if(cmdline != NULL) snprintf(arg_next(kargs), args_size(kargs), "%s ", cmdline);

	cmdline = getenv("bootargs_ex"); // bootargs_ex is for user test kernel bootargs
	if(cmdline != NULL) snprintf(arg_next(kargs), args_size(kargs), "%s ", cmdline);

	cmdline = getenv("avb_bootargs");
	if(cmdline != NULL) snprintf(arg_next(kargs), args_size(kargs), "%s ", cmdline);

	//if(strlen(oem_key) !=0) { the method is not use in vulcan model
	//	snprintf(arg_next(kargs), args_size(kargs), "androidboot.hardware.sku=%s ", oem_key);
	//}

	//loglevel
	if (boot_mode != BOOT_RESCUE_MODE)
	{
		if(!(strncmp(getenv("print"),"off",3))) // print is off
		{
			printf("SILENT MODE!\n");
#ifdef Config_Boot_Type_RELEASE
			snprintf(arg_next(kargs), args_size(kargs),"quiet loglevel=0 "); // silent mode
#else
			snprintf(arg_next(kargs), args_size(kargs),"loglevel=%d ",get_current_loglevel()); // silent mode, temperarily close it.
#endif
			#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
			snprintf(arg_next(kargs), args_size(kargs),"%s ", EARLYPRINTK); // always add earlyprintk for arm64
			#endif
		}
		else
		{
			snprintf(arg_next(kargs), args_size(kargs),"loglevel=%d %s ", get_current_loglevel(),EARLYPRINTK);
		}
#ifdef CONFIG_CANONICAL_BOOT
		set_boot_reason(kargs);
#endif
	}
	else
	{
		snprintf(arg_next(kargs), args_size(kargs),"loglevel=%d %s ", get_current_loglevel(),EARLYPRINTK);
	}

	if (boot_ac_on) {
		snprintf(arg_next(kargs), args_size(kargs), "ac_on ");
	}

	snprintf(arg_next(kargs), args_size(kargs), "chip=RTD2885M ");

	//sprintf(arg_next(kargs), "chip_model=%dK ", CURRENT_PLATFORM_MODEL);
    if(boot_panel_type==P_LVDS_2K1K){
        snprintf(arg_next(kargs), args_size(kargs), "chip_model=%dK ", 2) ;
        printf("\033[1;33m linux_param_set chip_model=%dK \033[m\n", 2);
    }
    else{
        snprintf(arg_next(kargs), args_size(kargs), "chip_model=%dK ", 4) ;
        printf("\033[1;33m linux_param_set chip_model=%dK \033[m\n", 4);
    }

    if(Mura_CUC_data_transfer==1){
        snprintf(arg_next(kargs), args_size(kargs), "androidboot.mura_cuc_data_transfer=1 ") ;
    }
    else{
        snprintf(arg_next(kargs), args_size(kargs), "androidboot.mura_cuc_data_transfer=0 ") ;
    }

    /* mc_dram_num = 0: (1-ddr), 1: (2-ddr), 2: (3-ddr) */
	snprintf(arg_next(kargs), args_size(kargs), "mc=%d ", DC_SYS_DC_SYS_MISC_get_mem_num(rtd_inl(DC_SYS_DC_SYS_MISC_reg)) + 1);

	CusApi_linux_param_set(kargs);
	char *timelog = getenv("timelog"); // show kernel timelog
	if(!strcmp("on", timelog))
		snprintf(arg_next(kargs), args_size(kargs), "timelog=1 ");

        char *usbotg = getenv("usbotg");
        if(!strcmp("on", usbotg))
		snprintf(arg_next(kargs), args_size(kargs), "usbotg=1 ");

	char *pwminvert = getenv("pwminvert");
	if(!strcmp("on", pwminvert))
		snprintf(arg_next(kargs), args_size(kargs), "pwm_invert=true ");

	char *qhb_boot = getenv("qhb_boot");
	if((qhb_boot) && (!strcmp("true", qhb_boot))){
		snprintf(arg_next(kargs), args_size(kargs), "qhb_boot=true ");
	}

	char *builtin_mic = getenv("builtin_mic");
	if((builtin_mic) && (!strcmp("true", builtin_mic))){
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.product.vendor.sku=mic ");
	}

	char *PanelID_Out_list = getenv("PANELID_out_list");
	if((PanelID_Out_list) && (!strcmp("true", PanelID_Out_list))){
		snprintf(arg_next(kargs), args_size(kargs), "PANELID_out_list=true ");
	}

	char *enable_white_led_tvon = getenv("enable_white_led_tvon");
	if(enable_white_led_tvon){
		if(!strcmp("false", enable_white_led_tvon))
			snprintf(arg_next(kargs), args_size(kargs), "enable_white_led_tvon=false ");
		else
			snprintf(arg_next(kargs), args_size(kargs), "enable_white_led_tvon=true ");
	}

	//for irda
	{
		//printf("bootm irda_default_value=%s\n",getenv("irda_default_value"));
		//printf("bootm irda_select_value=%s\n",getenv("irda_select_value"));
		//printf("bootm power up=%s\n",getenv("irda_powerup"));
		if(getenv("irda_select_value") != NULL && strcmp(getenv("irda_select_value"),"DefaultSetting(none)") != 0) {
                        snprintf(arg_next(kargs), args_size(kargs), "irda=%s ", getenv("irda_select_value"));
                } else if(getenv("irda_default_value") != NULL && strcmp(getenv("irda_default_value"),"DefaultSetting(none)") != 0) {
                        snprintf(arg_next(kargs), args_size(kargs), "irda=%s ", getenv("irda_default_value"));
                } else {
#if defined(CUSTOMER_IRDA_DEFAULT_VALUE)
			snprintf(arg_next(kargs), args_size(kargs), "irda=%s ", CUSTOMER_IRDA_DEFAULT_VALUE);
#else
			snprintf(arg_next(kargs), args_size(kargs), "irda=%s ", IR_DEFAULT_PROTOCOL_STRING);
#endif
                }

		if(NULL!=getenv("irda_powerup"))
		{
			snprintf(arg_next(kargs), args_size(kargs), "irda_powerup=%s ", getenv("irda_powerup"));
		}
                if(NULL!=getenv("irda_filter"))
		{
			snprintf(arg_next(kargs), args_size(kargs), "irda_filter=%s ", getenv("irda_filter"));
		}
                if(getenv("irda_select_value1") != NULL && strcmp(getenv("irda_select_value1"),"DefaultSetting(none)") != 0) {
                        snprintf(arg_next(kargs), args_size(kargs), "irda1=%s ", getenv("irda_select_value1"));
                } else if(getenv("irda_default_value1") != NULL && strcmp(getenv("irda_default_value1"),"DefaultSetting(none)") != 0){
                        snprintf(arg_next(kargs), args_size(kargs), "irda1=%s ", getenv("irda_default_value1"));
                }

		if(NULL!=getenv("irda_powerup1")) {
                        snprintf(arg_next(kargs), args_size(kargs), "irda_powerup1=%s ", getenv("irda_powerup1"));
                }
	}

	if(getenv("amp_select") != NULL && getenv("amp_select_value") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "amp=%s-%s ", getenv("amp_select"),getenv("amp_select_value"));
	}

        cmdline = getenv("dvfs_low"); // set dvfs number
        if(cmdline != NULL) snprintf(arg_next(kargs), args_size(kargs), "dvfs_low=%s ", cmdline);

        cmdline = getenv("dvfs_high"); // set dvfs number
        if(cmdline != NULL) snprintf(arg_next(kargs), args_size(kargs), "dvfs_high=%s ", cmdline);

	if(getenv("TP0_FE") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "TP0_FE=%s ", getenv("TP0_FE")) ;
	}
	if(getenv("TP1_FE") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "TP1_FE=%s ", getenv("TP1_FE")) ;
	}
#ifdef POWER_ON_RESOURCE_ADDR
	if (*boot_av_info_ptr != 0 || (gdma_length != 0))
		snprintf(arg_next(kargs), args_size(kargs), "rtk_rcs=0x%x ", POWER_ON_RESOURCE_ADDR);
	else
		snprintf(arg_next(kargs), args_size(kargs), "rtk_rcs=0x%x ", POWER_ON_RESOURCE_LOGO_OFF_ADDR);
#endif
#ifdef POWER_ON_IR_TABLE_ADDR
	if(1) {
		unsigned int *p_tmp = (unsigned int *)POWER_ON_IR_TABLE_ADDR;
		if(p_tmp[0] == 0x49525442 && p_tmp[1] > 0)
       			snprintf(arg_next(kargs), args_size(kargs), "ir_table=0x%x ", POWER_ON_IR_TABLE_ADDR);
	}
#endif

	cmdline = getenv("irda_filter_en");
	if(cmdline != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "irda_filter_en=%s ", cmdline);
	}

#if (CONFIG_SYS_NS16550_CLK == V_NS16550_CLK_108M)
	snprintf(arg_next(kargs), args_size(kargs), "uartclk=108M");
#endif
#ifdef CONFIG_CUSTOM_SET_SERIAL_NUMBER
	if(getenv("FINAL_SERIAL_NUMBER") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.serialno=%s ", getenv("FINAL_SERIAL_NUMBER")) ;
	}
#endif
#ifdef CONFIG_FSW
	if(getenv("OEM_KEY1") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.oem.key1=%s ", getenv("OEM_KEY1")) ;
	}
	if(getenv("OEM_KEY1") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.hardware.sku=%s ", getenv("OEM_KEY1")) ;
	}
	//CR-951
	if(getenv("PRODUCT_NAME") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.product.name=%s ", getenv("PRODUCT_NAME")) ;
	}
	if(getenv("PRODUCT_DEVICE") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.product.device=%s ", getenv("PRODUCT_DEVICE")) ;
	}
	if(getenv("PRODUCT_MODEL") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.product.model=%s ", getenv("PRODUCT_MODEL")) ;
	}
	if(getenv("PRODUCT_RESOLUTION") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.resolution=%s ", getenv("PRODUCT_RESOLUTION")) ;
	} else {
		if(boot_panel_type==P_LVDS_2K1K) {
#if defined(Config_Vulcan3_PCB_TRUE)
			if(EnableFHD){
				snprintf(arg_next(kargs), args_size(kargs), "androidboot.resolution=FHD ");
				snprintf(arg_next(kargs), args_size(kargs), "panel_type=FHD ");
			}
			else
#endif
			 {
				snprintf(arg_next(kargs), args_size(kargs), "androidboot.resolution=WXGA ") ;
				snprintf(arg_next(kargs), args_size(kargs), "panel_type=WXGA ");
			 }

		} else {
			snprintf(arg_next(kargs), args_size(kargs), "androidboot.resolution=QFHD ");
		}
	}
	if(getenv("PHY_SWING") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "androidboot.swing=%s ", getenv("PHY_SWING")) ;
	}
	if(getenv("EV") != NULL) {
		snprintf(arg_next(kargs), args_size(kargs), "EV=%s ", getenv("EV")) ;
	}
#endif
}

/* Subcommand: PREP */
static void boot_prep_linux(bootm_headers_t *images)
{
#if 1//def CONFIG_CMDLINE_TAG
	char *commandline = NULL;
	char kargs[4096] = {0};


	rtk_plat_boot_prep_logbuffer_value();
	rtk_plat_boot_prep_dcmt();
	rtk_plat_boot_prep_sb2();
	rtk_plat_boot_prep_hwm();

	rtk_plat_boot_prep_envp();

	if (boot_mode == BOOT_RESCUE_MODE) {
		rtk_plat_boot_prep_selinux();
	}

	rtk_plat_boot_prep_flashtype();

#ifdef CONFIG_SYS_SUPPORT_MULTICORE
	// 294x/296x is single core
	rtk_plat_boot_prep_cpus();
#endif

#ifdef CONFIG_SYS_RTK_EMMC_FLASH
	rtk_plat_boot_prep_mmcparts();
#endif

#ifdef CONFIG_SYS_RTK_NAND_FLASH
	rtk_plat_boot_prep_mtdparts();
#endif

#ifdef CONFIG_SYS_RTK_SPI_NAND
	rtk_plat_boot_prep_sn_mtdparts();
#endif

#ifdef CONFIG_SYS_RTK_SPI_FLASH
    rtk_plat_boot_prep_mtdparts_nor();
#endif

#ifdef CONFIG_IP_TO_KERNEL
	rtk_plat_boot_prep_network();
#endif

	#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
	//if (*KcpuFW_entry_ptr != 0) //FIXME, need to get kentry and ksize from KcpuFW_entry_ptr later
		rtk_plat_boot_prep_kcpu();
	#else //#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
#if 0 // without k
	if (*KcpuFW_entry_ptr != 0) {
		rtk_plat_boot_prep_kcpu();
	}
#endif
	#endif // #ifdef Config_Booting_Linux_type_is_SECURE_LINUX

#ifndef IS_FPGA
	// boot animation existed
	if (*boot_av_info_ptr != 0
		||(gdma_length != 0))
	{
		rtk_plat_boot_prep_reclaim();
	}
#endif

	if (boot_mode != BOOT_RESCUE_MODE) // Add for fixing Rescue Kernel stop at 8051 standby
	{
		if (boot_power_on_mode == POWER_OFF) {
			rtd_outl(MAGIC_TAG_CORE4_reg, 0x1234abcd);
			flush_dcache_range(MAGIC_TAG_CORE4_reg, MAGIC_TAG_CORE4_reg + 8);
			rtk_plat_boot_prep_for_standby();
			CusApi_boot_prep_linux_for_standby(); // call customized function
		}
		//no need enable watchdog when run rescue.
		rtk_plat_boot_prep_watchdog();
	}
	else
	{
		if(rescue_mode_with_audio_flag == 0){
			rtd_outl(MAGIC_TAG_CORE4_reg, 0x1234abcd); // if enter rescue, let core4 loop in uboot
			flush_dcache_range(MAGIC_TAG_CORE4_reg, MAGIC_TAG_CORE4_reg + 8);
		}
	}

	// add bootcode version to commandline
	rtk_plat_boot_prep_bootcode_version();

	linux_param_set(kargs);

	setenv("bootargs", kargs);
	commandline = getenv("bootargs");
	printf("Kernel command line: %s\n", commandline);
#endif

	if(rtk_check_ip_bisr() != 0)
	{
		if(!(strncmp(getenv("print"),"off",3))) // log mode is off
		{
			enable_console();
		}
		printf("\n\033[31m[ERR] BISR CHECK FAIL, PLEASE CHECK ABOVE LOG!!\033[0m\n");
		while(1);
	}

#ifdef CONFIG_OF_LIBFDT
        if ((boot_mode == BOOT_RESCUE_MODE) ||(KernelImagePosition))
        {
            Kernel_mode_DTB_Verify(images);
        }

	if (images->ft_len) {
		debug("using: FDT\n");
                images->cmdline_start = (ulong)commandline;
                images->kbd= gd->bd;
                if(Fdt_prep_linux(images)!= 0)
                {
                    printf("fdt prepare fail\n");
                    hang();
                }
	} else
#endif
	{
#if defined(CONFIG_SETUP_MEMORY_TAGS) || \
	defined(CONFIG_CMDLINE_TAG) || \
	defined(CONFIG_INITRD_TAG) || \
	defined(CONFIG_SERIAL_TAG) || \
	defined(CONFIG_REVISION_TAG)
		debug("using: ATAGS\n");
		setup_start_tag(gd->bd);
#ifdef CONFIG_SERIAL_TAG
		setup_serial_tag(&params);
#endif
#ifdef CONFIG_CMDLINE_TAG
		setup_commandline_tag(gd->bd, commandline);
#endif
#ifdef CONFIG_REVISION_TAG
		setup_revision_tag(&params);
#endif
#ifdef CONFIG_SETUP_MEMORY_TAGS
		setup_memory_tags(gd->bd);
#endif
#ifdef CONFIG_INITRD_TAG
		if (images->rd_start && images->rd_end) {
			setup_initrd_tag(gd->bd, images->rd_start,
			images->rd_end);
		}
#endif
		setup_end_tag(gd->bd);

#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
		rtk_plat_boot_prep_secure_linux_envp();
#endif

#else /* all tags */
		printf("FDT and ATAGS support not compiled in - hanging\n");
		hang();
#endif /* all tags */
	}
}

//#define DEBUG_DUMP_ENV_RESULT
#ifdef DEBUG_DUMP_ENV_RESULT
int *_prom_envp;
#define GET_ENV(index) ((char *)(int)_prom_envp[(index)])
#endif

/* Subcommand: GO */
static void boot_jump_linux(bootm_headers_t *images)
{
	unsigned long machid = gd->bd->bi_arch_number;
	char *s;
	void (*kernel_entry)(int zero, int arch, uint params);
	unsigned long r2;

#ifdef CONFIG_CUSTOMER_TV010
	if((default_panel_parameter.iCONFIG_LD_Enable == 1)&&(default_panel_parameter.iCONFIG_LD_Partition_Amount==12)
		&&(default_panel_parameter.iCONFIG_LD_SPIDecoderMcuInPower_Enable == 0))
	{
		rtd_part_outl(0xb802c268,9,9,0x1);  // ld algo on
	}
#endif

	kernel_entry = (void (*)(int, int, uint))images->ep;

	s = getenv("machid");
	if (s) {
		strict_strtoul(s, 16, &machid);
		printf("Using machid 0x%lx from environment\n", machid);
	}

	debug("## Transferring control to Linux (at address %08lx)" \
		"...\n", (ulong) kernel_entry);
	bootstage_mark(BOOTSTAGE_ID_RUN_OS);

#ifdef CONFIG_SYS_SUPPORT_MULTICORE
#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
      release_non_boot_core();
#else
      Core_Wakeup(smp_release_coproc, 0x1);
      Core_Wakeup(smp_release_coproc, 0x2);
      Core_Wakeup(smp_release_coproc, 0x3);
#endif
#endif
#ifndef CONFIG_NO_TIMER_LOG
#ifdef CONFIG_CMD_MISC
       display_boottime_log("jump to kernel");
#endif
#endif
	announce_and_cleanup();

	r2 = gd->bd->bi_boot_params;

#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
	if (boot_mode != BOOT_RESCUE_MODE ) {
#ifdef Config_Boot_ATF_TRUE
                extern void call_smc(unsigned int, unsigned int, unsigned int, unsigned int);
		if(!KernelImagePosition)
		{
			rtd_outl(MAGIC_TAG_UIMAGE, 0x23792378);
			flush_dcache_range(MAGIC_TAG_UIMAGE, MAGIC_TAG_UIMAGE+8);
		}
                printf("bootargs=%s\n", getenv("bootargs"));
                printf("%s (0x%lx,0x%lx,0x%lx, entry:%x)\n", __func__, r2, machid, r2,LINUX64_KERNEL_ENTRY);
                rtd_outl(MAGIC_TAG_UBOOT_EXIT, 0x1234abcd); //sync world for secondary issue smc
                flush_dcache_range(MAGIC_TAG_UBOOT_EXIT, MAGIC_TAG_UBOOT_EXIT+8);
                call_smc(r2,machid,r2,LINUX64_KERNEL_ENTRY);
#else
		unsigned long kernel_entry;
		extern void call_smc(unsigned int, unsigned int, unsigned int);
		printf("bootargs=%s\n", getenv("bootargs"));
		kernel_entry=(unsigned long)simple_strtol(getenv("kernel_entry2"), NULL, 16);

		//work around patch
		if (!kernel_entry)
		{
		    kernel_entry=0x1b100000;
		}

		printf("jump to redirect64 (0x%lx,0x%lx,0x%lx) %lx\n", r2, machid, r2, kernel_entry);
		printf("kernel_entry2=%lx\n", kernel_entry);
		rtd_outl(0x520,kernel_entry);
		rtd_outl(0x524,0);
                call_smc(r2,machid,r2);
#endif
	} else
#endif
	{
#ifdef Config_Boot_ATF_TRUE
		extern void call_smc(unsigned int, unsigned int, unsigned int, unsigned int);
		if(!KernelImagePosition)
		{
			if(boot_mode != BOOT_RESCUE_MODE)
			{
				rtd_outl(MAGIC_TAG_UIMAGE, 0x23792378); // sync world for secondary issue smc
				flush_dcache_range(MAGIC_TAG_UIMAGE, MAGIC_TAG_UIMAGE+8);
			}
		}
		else
		{
			rtd_outl(MAGIC_TAG_UIMAGE,0); //sync world for secondary issue smc
			flush_dcache_range(MAGIC_TAG_UIMAGE, MAGIC_TAG_UIMAGE+8);
		}
		rtd_outl(MAGIC_TAG_UBOOT_EXIT,0x1234abcd); //sync world for secondary issue smc
		flush_dcache_range(MAGIC_TAG_UBOOT_EXIT, MAGIC_TAG_UBOOT_EXIT+8);
		printf("bootargs=%s\n", getenv("bootargs"));
		printf("%s (0x%x,0x%lx,0x%lx, entry:%x)\n", __func__, 0, machid, r2,LINUX_KERNEL_ENTRY);
		call_smc(0,machid,r2,LINUX_KERNEL_ENTRY);
#else
		#ifdef Config_Booting_Linux_type_is_SECURE_LINUX
		if(images->ep == SECURE_LINUX_KERNEL_ENTRY)
			r2 = CONFIG_SECURE_LINUX_BOOT_PARAM_BASE;
		#endif

		printf("bootargs=%s\n", getenv("bootargs"));
		printf("%s 0x%08x(0x%x,0x%lx,0x%lx)\n", __func__, (unsigned int)kernel_entry, 0, machid, r2);
		kernel_entry(0, machid, r2);
#endif
	}
}

/* Main Entry point for arm bootm implementation
 *
 * Modeled after the powerpc implementation
 * DIFFERENCE: Instead of calling prep and go at the end
 * they are called if subcommand is equal 0.
 */
int do_bootm_linux(int flag, int argc, char *argv[], bootm_headers_t *images)
{
		/* No need for those on ARM */
	if (flag & BOOTM_STATE_OS_BD_T || flag & BOOTM_STATE_OS_CMDLINE) {
		return -1;
	}

	if (flag & BOOTM_STATE_OS_PREP) {
		boot_prep_linux(images);
		return 0;
	}

	if (flag & BOOTM_STATE_OS_GO) {
		boot_jump_linux(images);
		return 0;
	}

	boot_prep_linux(images);
	boot_jump_linux(images);

	return 0;
}

#ifdef CONFIG_CMD_BOOTZ

struct zimage_header {
	uint32_t	code[9];
	uint32_t	zi_magic;
	uint32_t	zi_start;
	uint32_t	zi_end;
};

#define	LINUX_ARM_ZIMAGE_MAGIC	0x016f2818

int bootz_setup(void *image, void **start, void **end)
{
	struct zimage_header *zi = (struct zimage_header *)image;

	if (zi->zi_magic != LINUX_ARM_ZIMAGE_MAGIC) {
		puts("Bad Linux ARM zImage magic!\n");
		return 1;
	}

	*start = (void *)zi->zi_start;
	*end = (void *)zi->zi_end;

	debug("Kernel image @ 0x%08x [ 0x%08x - 0x%08x ]\n",
		(uint32_t)image, (uint32_t)*start, (uint32_t)*end);

	return 0;
}
#endif	/* CONFIG_CMD_BOOTZ */
