/* Copyright (C) 2011
 *  * Corscience GmbH & Co. KG - Simon Schwarz <schwarz@corscience.de>
 *   *  - Added prep subcommand support
 *    *  - Reorganized source - modeled after powerpc version
 *     *
 *      * (C) Copyright 2002
 *       * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 *        * Marius Groeger <mgroeger@sysgo.de>
 *         *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <asm/bootm.h>
#include <exports.h>
#include <environment.h>
#include <wakeup.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/bootparam.h>
#include <platform_lib/board/pcb_mgr.h>
#include <watchdog.h>
#include <factorylib.h>
#include <gpt_parse.h>
#include <part.h>
#include <ext4fs.h>

/* Android reboot reason stored in this property */
#define METADATA_PART_NAME  "metadata"
#define REBOOT_REASON_FILE  "/bootstat/persist.sys.boot.reason"
#define FILE_NOT_FOUND -2

#define QUIESCENT    "quiescent"
#define REBOOT_COLD "reboot,cold"
#define REBOOT_UNKNOWN "reboot,unknown"
#define POWERKEY "powerkey"

#define arg_next(p)             (char *)((unsigned long)p+strlen(p))

int read_android_reboot_reason(char *buffer, int length)
{
	int ret = 0;
	unsigned int dev_index = 0;
	char dev_string[32] = {0};
	int part = -1;
	disk_partition_t metadata_ext4_info;
	block_dev_desc_t *metadata_ext4_dev_desc = NULL;
	int file_len = 0;
	int read_len = 0;


	// get metadata partition index
	ret = get_dev_index_by_name(METADATA_PART_NAME, &dev_index);
	if (ret < 0)
	{
		printf("Get %s partition block device index fail!\n", METADATA_PART_NAME);
		return -1;
	}

	// get metadata partition info
	snprintf(dev_string, sizeof(dev_string), "0:0x%x", dev_index);
	part = get_device_and_partition("mmc", dev_string,
			&metadata_ext4_dev_desc, &metadata_ext4_info, 1);
	if (part < 0)
	{
		printf("Get %s partition desc info fail!\n", METADATA_PART_NAME);
		return -1;
	}

	// set device as block device
	ext4fs_set_blk_dev(metadata_ext4_dev_desc, &metadata_ext4_info);

	// mount filesystem
	if (!ext4fs_mount(metadata_ext4_info.size))
	{
		ret = -1;
		printf("mount ext4 partition %s fail!\n", METADATA_PART_NAME);
		goto fail;
	}

	// open file and get file size
	ret = ext4fs_open(REBOOT_REASON_FILE, (loff_t *)&file_len);
	if (ret < 0)
	{
		printf("file %s not found in %s partition!\n", REBOOT_REASON_FILE, METADATA_PART_NAME);
		ext4fs_close();
		return FILE_NOT_FOUND;
	}

	ret = ext4fs_read(buffer, 0, (loff_t)length, (loff_t *)&read_len);
	if (ret < 0)
	{
		printf("read file %s to ddr fail!\n", REBOOT_REASON_FILE);
	}
	else
	{
		ret = read_len;
	}

fail:
	ext4fs_close();
	return ret;
}

void set_boot_reason(char *kargs)
{
	int ret = 0;
	extern uchar boot_ac_on;
	extern int boot_power_on_mode;
	char boot_reason[128] = {0};

	// Print ac on
	printf("AC ON: %d\n", boot_ac_on);

	// Read boot reason from persist.sys.boot.reason
	ret = read_android_reboot_reason(boot_reason, sizeof(boot_reason));

	// Determine boot reason
	if (boot_ac_on && ret == FILE_NOT_FOUND) {
		printf("Boot Reason: %s\n", REBOOT_COLD);
		sprintf(arg_next(kargs),"androidboot.bootreason=%s ", REBOOT_COLD);
	}
	else if ((!boot_ac_on) && ret > 0) // boot reason is found in persist.sys.boot.reason
	{
		printf("Boot Reason: %s\n", boot_reason);
		sprintf(arg_next(kargs),"androidboot.bootreason=%s ", boot_reason);

		// for reboot quiescent
		if (strstr(boot_reason, QUIESCENT) != NULL)
		{
			sprintf(arg_next(kargs),"androidboot.quiescent=1 ");
		}
	}
	/* [TODO] power key detected
	else if ((!boot_ac_on) && ret == FILE_NOT_FOUND && powerkey_detected)
	{
		printf("Boot Reason: %s\n", POWERKEY);
		sprintf(arg_next(kargs),"androidboot.bootreason=%s ", POWERKEY);
	}
	*/
	else // unknown cases
	{
		printf("Boot Reason: %s\n", REBOOT_UNKNOWN);
		sprintf(arg_next(kargs),"androidboot.bootreason=%s ", REBOOT_UNKNOWN);
	}

	// for RTC wake-up
	if ((!boot_ac_on) && boot_power_on_mode == BACKLIGHT_OFF)
	{
		sprintf(arg_next(kargs),"androidboot.quiescent=1 ");
	}
}
