#include <config.h>
#include <common.h>
#include <command.h>
#include <asm/io.h>
#include <lzma_reg.h>

unsigned int g_status = 0;

#ifdef CONFIG_SYS_SUPPORT_MULTICORE
extern int smp_processor_id(unsigned int mpidr);
#else
static unsigned int cpu_id_list[4] = {0x81000000, 0x81000100, 0x81000200, 0x81000300};
static int smp_processor_id(unsigned int mpidr)
{
    int i = 0;
    while(cpu_id_list[i] != 0xdeaddead) {
        if (cpu_id_list[i] == mpidr)
            return i;
        i++;
    }
    return -1;
}
#endif

#define get_cpu_id()   smp_processor_id(get_MPIDR())

static void my_mdelay(unsigned int ms)
{
#if 0
    unsigned int i = 0, j = 0;
    for (i = 0; i < 0x100; i++) {
    	for (j = 0; j < 0x100; j++) {
    		nop();
    	}
    }
#else
    mdelay(ms);
#endif
}

static void my_memset(void *ptr, int value, size_t num )
{
#if 0
    unsigned int i = 0;
    for (i = 0; i < num; i++) {
    	*(((char *)ptr + i)) = value;
    }
#else
    memset(ptr, value, num);
#endif
}

static void decode_cmd_parser(const unsigned int *desc_addr)
{
    unsigned int crc = (((desc_addr[2] >> 24) & 0xff) << 8) | ((desc_addr[3] >> 24) & 0xff);

    printf("/*************************************/\n");
    printf("E: %01x\n", desc_addr[0] & 0x01);
    printf("D: %01x\n", (desc_addr[0] >> 1)& 0x01);
    printf("B: %01x\n", (desc_addr[0] >> 2)& 0x01);
    printf("Cmd Error: %01x\n", (desc_addr[0] >> 3)& 0x01);
    printf("Data source address: %08x\n", (desc_addr[0] >> 4)& 0x0fffffff);
    printf("HW done: %01x\n", desc_addr[1] & 0x01);
    printf("Size Error: %01x\n", (desc_addr[1] >> 1)& 0x01);
    printf("Decoder Error: %01x\n", (desc_addr[1] >> 2)& 0x01);
    printf("Distance Error: %01x\n", (desc_addr[1] >> 3)& 0x01);
    printf("Data distance address: %08x\n", (desc_addr[1] >> 4)& 0x0fffffff);
    printf("Compressed size: %06x\n", desc_addr[2] & 0x007fffff);
    printf("CRC Error: %01x\n", (desc_addr[2] >> 23) & 0x1);
    printf("Golden CRC msb: %02x(%04x)\n", (desc_addr[2] >> 24) & 0xff, crc);
    printf("Decompressed size: %06x\n", desc_addr[3] & 0x007fffff);
    printf("Unused: %01x\n", (desc_addr[3] >> 23) & 0x1);
    printf("Golden CRC lsb: %02x(%04x)\n", (desc_addr[3] >> 24) & 0xff, crc);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[0], desc_addr[0]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[1], desc_addr[1]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[2], desc_addr[2]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[3], desc_addr[3]);
    printf("/*************************************/\n");
}

static void encode_cmd_parser(const unsigned int *desc_addr)
{
    //unsigned int crc = (((desc_addr[2] >> 24) & 0xff) << 8) | ((desc_addr[3] >> 24) & 0xff);

    printf("/*************************************/\n");
    printf("E: %01x\n", desc_addr[0] & 0x01);
    printf("D: %01x\n", (desc_addr[0] >> 1)& 0x01);
    printf("B: %01x\n", (desc_addr[0] >> 2)& 0x01);
    printf("Cmd Error: %01x\n", (desc_addr[0] >> 3)& 0x01);
    printf("Data source address: %08x\n", (desc_addr[0] >> 4)& 0x0fffffff);
    printf("HW done: %01x\n", desc_addr[1] & 0x01);
    printf("Size over 4KB: %01x\n", (desc_addr[1] >> 1)& 0x01);
    printf("Unused: %01x\n", (((desc_addr[1] >> 2) & 0x01) << 1) | ((desc_addr[1] >> 3)& 0x01));
    printf("Data distance address: %08x\n", (desc_addr[1] >> 4)& 0x0fffffff);
    printf("Compressed size: %04x\n", desc_addr[2] & 0x00001fff);
    printf("Unused: %01x\n", (desc_addr[2] >> 13) & 0x7);
    printf("CRC result: %02x\n", (desc_addr[2] >> 16) & 0xffff);
    printf("Unused: %08x\n", desc_addr[3]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[0], desc_addr[0]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[1], desc_addr[1]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[2], desc_addr[2]);
    printf("[%08x] = %08x\n", (unsigned int)&desc_addr[3], desc_addr[3]);
    printf("/*************************************/\n");
}

int do_lzma_mcmq_test(unsigned int start_addr, unsigned int end_addr)
{
    int ret = 0;
    unsigned int cpu_base = (get_cpu_id() == 0) ? (0x0000000):(0x8000000);//(0x00000000):(0x10000000);
    unsigned int out_addr = cpu_base + 0x6000000;//cpu_base + 0xC000000;
    unsigned int tmp_addr = cpu_base + 0x5000000;//cpu_base + 0x8000000;
    unsigned int i = 0, j = 0, k = 0, l = 0, m = 0, n = 0;
    unsigned char *desc_addr = 0, *cmd_addr = 0, *wptr = 0;
    unsigned int align_addr = 0;
    unsigned int is_decode = 0;
    volatile unsigned char *desc0_addr = (unsigned char *)(cpu_base + 0x100000);
    volatile unsigned char *desc1_addr = (unsigned char *)(cpu_base + 0x200000);
    volatile unsigned char *desc2_addr = (unsigned char *)(cpu_base + 0x300000);
    volatile unsigned char *desc3_addr = (unsigned char *)(cpu_base + 0x400000);
    volatile unsigned int *comp_size_addr = (unsigned int *)(cpu_base + 0x1000000);
    volatile unsigned int *decomp_size_addr = (unsigned int *)(cpu_base + 0x2000000);
    volatile unsigned int *crc_addr = (unsigned int *)(cpu_base + 0x3000000);
    bool is_found = 0;
    unsigned int timeout = 1000;
    unsigned int count = 0;
    const unsigned int desc_count_per_core = 2;//1;//2;
    const unsigned int cmd_count_per_queue = 10;
    unsigned int compressed_size = 0;

#define CHK_CRC_FAIL 0
#define RUN_SINGLE_CORE 0

#if (RUN_SINGLE_CORE == 1)
    if (get_cpu_id() != 0)
    	return 0;
#endif

    m = (get_cpu_id() == 0) ? 0:desc_count_per_core;

    printf("@@@ %s, cpu_id = %d, desc_count_per_core = %d\n", __FUNCTION__, get_cpu_id(), desc_count_per_core);

    rtd_maskl(LZMA_LZMA_DMA_RD_Ctrl_reg, 0xfff0fcff, (0x3 << 8) | (0xf << 16));
    rtd_maskl(LZMA_LZMA_DMA_WR_Ctrl_reg, 0xfffcfcff, (0x3 << 8) | (0x3 << 16));
    printf("[0x%08x] = 0x%08x\n", LZMA_LZMA_DMA_RD_Ctrl_reg, rtd_inl(LZMA_LZMA_DMA_RD_Ctrl_reg));
    printf("[0x%08x] = 0x%08x\n", LZMA_LZMA_DMA_WR_Ctrl_reg, rtd_inl(LZMA_LZMA_DMA_WR_Ctrl_reg));

//retry:
    invalidate_dcache_all();
    is_decode = 0; //encode
    my_memset((void *)out_addr, 0, 0x1000000);
    my_memset((void *)tmp_addr, 0, 0x1000000);
    my_memset((void *)comp_size_addr, 0, 0x1000000);
    my_memset((void *)decomp_size_addr, 0, 0x1000000);
    my_memset((void *)crc_addr, 0, 0x1000000);
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
    flush_dcache_range((unsigned long)out_addr, (unsigned long)out_addr + 0x1000000);
    flush_dcache_range((unsigned long)tmp_addr, (unsigned long)tmp_addr + 0x1000000);
    flush_dcache_range((unsigned long)comp_size_addr, (unsigned long)comp_size_addr + 0x1000000);
    flush_dcache_range((unsigned long)decomp_size_addr, (unsigned long)decomp_size_addr + 0x1000000);
    flush_dcache_range((unsigned long)crc_addr, (unsigned long)crc_addr + 0x1000000);
//    flush_dcache_range((unsigned long)desc0_addr, (unsigned long)desc0_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc1_addr, (unsigned long)desc1_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc2_addr, (unsigned long)desc2_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc3_addr, (unsigned long)desc3_addr + 0x100000);
    for (n = 0; n < cmd_count_per_queue; n++) {
        (desc0_addr + n * 0x10)[4] = 0x1; //done bit = 0x1
        (desc1_addr + n * 0x10)[4] = 0x1;
        (desc2_addr + n * 0x10)[4] = 0x1;
        (desc3_addr + n * 0x10)[4] = 0x1;
    }
    for (k = 0; k < desc_count_per_core; k++) {
        rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), 0); //desc addr
        rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), 0); //desc limit
        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), 0); //read pointer
    }
    k = 0;
    n = 0;
    compressed_size = 0;
    for (i = start_addr, j = 0; i < end_addr; i += 4096, j++) {
        timeout = 1000;
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
              desc_addr = (unsigned char *)(((k + 1) * 0x100000) + cpu_base);
              cmd_addr = desc_addr + n * 0x10;
//                invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
                if ((cmd_addr[4] & 0x1) == 0x01) {
                //if (((cmd_addr[4] & 0x1) == 0x01) && !(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m))))) {
                //if (((cmd_addr[4] & 0x1) == 0x01) && (((unsigned int)(cmd_addr + 0x10)) != rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))))) {
                    align_addr = (cmd_addr[3] << 24) | (cmd_addr[2] << 16) | (cmd_addr[1] << 8) | (cmd_addr[0] & 0xf0);
                    if (align_addr != 0) {
                        if ((cmd_addr[0] & 0x08) || (cmd_addr[4] & 0x02)) {
                            printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                            encode_cmd_parser((unsigned int *)cmd_addr);
                            g_status = 1;
                            return 1;
                        }
                        //align_addr = (cmd_addr[3] << 24) | (cmd_addr[2] << 16) | (cmd_addr[1] << 8) | (cmd_addr[0] & 0xf0);
                        l = (align_addr - start_addr) >> 12;
                        *(comp_size_addr + l) = ((cmd_addr[9] & 0x1f) << 8) | cmd_addr[8];
                        compressed_size += *(comp_size_addr + l);
                        *(decomp_size_addr + l) = 0x1000;
                        *(crc_addr + l) = (cmd_addr[11] << 8) | cmd_addr[10];
                        //printf("[Info][%d] offset = %x, comp_size = %04x, decomp_size = %04x, crc = %04x\n", k, l, *(comp_size_addr + l), *(decomp_size_addr + l), *(crc_addr + l));
                        //encode_cmd_parser((unsigned int *)cmd_addr);
                    }
                    my_memset((void *)cmd_addr, 0, 0x10);

                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    cmd_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    cmd_addr[1] = (align_addr >> 8) & 0xff;
                    cmd_addr[2] = (align_addr >> 16) & 0xff;
                    cmd_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((tmp_addr + (j * 4096)) >> 4) << 4;
                    cmd_addr[4] = (align_addr) & 0xf0;
                    cmd_addr[5] = (align_addr >> 8) & 0xff;
                    cmd_addr[6] = (align_addr >> 16) & 0xff;
                    cmd_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        cmd_addr[8] = *(comp_size_addr + j) & 0xff;
                        cmd_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        cmd_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        cmd_addr[12] = *(decomp_size_addr + j) & 0xff;
                        cmd_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        cmd_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        ;//cmd_addr[10] = *(crc_addr + j) & 0xff;
                        ;//cmd_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        cmd_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        cmd_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[0], cmd_addr[3], cmd_addr[2], cmd_addr[1], cmd_addr[0]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[4], cmd_addr[7], cmd_addr[6], cmd_addr[5], cmd_addr[4]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[8], cmd_addr[11], cmd_addr[10], cmd_addr[9], cmd_addr[8]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[12], cmd_addr[15], cmd_addr[14], cmd_addr[13], cmd_addr[12]);

//                    flush_dcache_range((unsigned long)cmd_addr, (unsigned long)desc_addr + 0x10);
//                    invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)desc_addr + 0x10);

                    if (rtd_inl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //desc addr
                    if (rtd_inl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), (unsigned int)desc_addr + (cmd_count_per_queue * 0x10) + 0x10); //desc limit
                    if (rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer

                    wptr = cmd_addr + 0x10;
                    if (cmd_addr == desc_addr) {
                        if ((rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) &&
                            ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0)) {
                            rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
                        }
                        while(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) {
                            my_mdelay(1);
                        }
                        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer
                    }

                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), (unsigned int)wptr); //write pointer

                    //printf("@@@[%d][%d][%d] rptr[0x%x] = 0x%x, wptr[0x%x] = 0x%x\n", __LINE__, k, n, LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))), LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m))));

                    if ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0) {
                        rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * (k + m))));
                    }

                    //printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                    if (k + 1 == desc_count_per_core) {
                        n = (n + 1) % cmd_count_per_queue;
                    }
                    k  = (k + 1) % desc_count_per_core;
                    //printf("@@@ line = %d, k = %d, n = %d\n", __LINE__, k, n);
                }
            }

            if (is_found) {
                break;
            }
            else {
                if ((rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) &&
                     ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0)) {
                        rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
                }
            }
            if (!(--timeout)) {
                printf("[ERR][%d][%d] wait for available command timeout!\n", __LINE__, k);
                encode_cmd_parser((unsigned int *)cmd_addr);
                g_status = 1;
                return 1;
            }
            my_mdelay(1);
        }
    }

    timeout = 1000;
    while(rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) {
        if ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0) {
            rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
        }
        if (!(--timeout)) {
            printf("[ERR][%d][%d] wait for rptr = wptr timeout!\n", __LINE__, k);
            encode_cmd_parser((unsigned int *)cmd_addr);
            g_status = 1;
            return 1;
        }
        my_mdelay(1);
    }

    for (k = 0; k < desc_count_per_core; k++) {
        for (n = 0; n < cmd_count_per_queue; n++) {
            desc_addr = (unsigned char *)((k + 1) * 0x100000) + cpu_base;
            cmd_addr = desc_addr + n * 0x10;
//            invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
            timeout = 1000;
            while(!(cmd_addr[4] & 0x01)) {
                if ((cmd_addr[0] & 0x08) || (cmd_addr[4] & 0x02)) {
                    printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                    encode_cmd_parser((unsigned int *)cmd_addr);
                    g_status = 1;
                    return 1;
                }
                if (!(--timeout)) {
                    printf("[ERR][%d][%d] wait for command timeout!\n", __LINE__, k);
                    encode_cmd_parser((unsigned int *)cmd_addr);
                    g_status = 1;
                    return 1;
                }
                my_mdelay(1);
//                invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
            }
            align_addr = (cmd_addr[3] << 24) | (cmd_addr[2] << 16) | (cmd_addr[1] << 8) | (cmd_addr[0] & 0xf0);
            l = (align_addr - start_addr) >> 12;
            *(comp_size_addr + l) = ((cmd_addr[9] & 0x1f) << 8) | cmd_addr[8];
            compressed_size += *(comp_size_addr + l);
            *(decomp_size_addr + l) = 0x1000;
            *(crc_addr + l) = (cmd_addr[11] << 8) | cmd_addr[10];
            //encode_cmd_parser((unsigned int *)cmd_addr);
        }
    }

    is_decode = 1; //decode
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
//    flush_dcache_range((unsigned long)desc0_addr, (unsigned long)desc0_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc1_addr, (unsigned long)desc1_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc2_addr, (unsigned long)desc2_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc3_addr, desc3_addr + 0x100000);
    for (n = 0; n < cmd_count_per_queue; n++) {
        (desc0_addr + n * 0x10)[4] = 0x1; //done bit = 0x1
        (desc1_addr + n * 0x10)[4] = 0x1;
        (desc2_addr + n * 0x10)[4] = 0x1;
        (desc3_addr + n * 0x10)[4] = 0x1;
    }
    for (k = 0; k < desc_count_per_core; k++) {
        rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), 0); //desc addr
        rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), 0); //desc limit
        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), 0); //read pointer
    }
    k = 0;
    n = 0;
    for (i = tmp_addr, j = 0; i < (tmp_addr + (end_addr - start_addr)); i += 4096, j++) {
        timeout = 1000;
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
                desc_addr = (unsigned char *)((k + 1) * 0x100000) + cpu_base;
                cmd_addr = desc_addr + n * 0x10;
//                invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
#if (CHK_CRC_FAIL == 0)
                if ((cmd_addr[4] & 0x1) == 0x01) {
                //if (((cmd_addr[4] & 0x1) == 0x01) && !(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m))))) {
                //if (((cmd_addr[4] & 0x1) == 0x01) && (((unsigned int)(cmd_addr + 0x10)) != rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))))) {
#else
                if (((cmd_addr[4] & 0x1) == 0x01) && ((cmd_addr[10] & 0x80) == 0x80)) {
#endif
                    align_addr = (cmd_addr[3] << 24) | (cmd_addr[2] << 16) | (cmd_addr[1] << 8) | (cmd_addr[0] & 0xf0);
                    if (align_addr != 0) {
#if (CHK_CRC_FAIL == 0)
                        if ((cmd_addr[10] & 0x80) || (cmd_addr[4] & 0x0E)) {
#else
                        if ((cmd_addr[4] & 0x0E)) {
#endif
                            printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                            decode_cmd_parser((unsigned int *)cmd_addr);
                            g_status = 1;
                            return 1;
                        }
                        //printf("----k = %d----\n", k);
                        //decode_cmd_parser((unsigned int *)cmd_addr);
                    }
                    my_memset((void *)cmd_addr, 0, 0x10);
                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    cmd_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    cmd_addr[1] = (align_addr >> 8) & 0xff;
                    cmd_addr[2] = (align_addr >> 16) & 0xff;
                    cmd_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((out_addr + (j * 4096)) >> 4) << 4;
                    cmd_addr[4] = (align_addr) & 0xf0;
                    cmd_addr[5] = (align_addr >> 8) & 0xff;
                    cmd_addr[6] = (align_addr >> 16) & 0xff;
                    cmd_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        cmd_addr[8] = *(comp_size_addr + j) & 0xff;
                        cmd_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        cmd_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        cmd_addr[12] = *(decomp_size_addr + j) & 0xff;
                        cmd_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        cmd_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        ;//cmd_addr[10] = *(crc_addr + j) & 0xff;
                        ;//cmd_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        cmd_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        cmd_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[0], cmd_addr[3], cmd_addr[2], cmd_addr[1], cmd_addr[0]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[4], cmd_addr[7], cmd_addr[6], cmd_addr[5], cmd_addr[4]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[8], cmd_addr[11], cmd_addr[10], cmd_addr[9], cmd_addr[8]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, &cmd_addr[12], cmd_addr[15], cmd_addr[14], cmd_addr[13], cmd_addr[12]);

//                    flush_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
//                    invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);

                    //if (get_cpu_id() != 0)
                    //    m = desc_count_per_core;//2;

                    if (rtd_inl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //desc addr
                    if (rtd_inl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), (unsigned int)desc_addr + (cmd_count_per_queue * 0x10) + 0x10); //desc limit
                    if (rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) == 0x0)
                        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer

                    //wptr = ((cmd_addr + 0x10) == desc_addr + (cmd_count_per_queue * 0x10)) ? (desc_addr):(cmd_addr + 0x10);
                    wptr = cmd_addr + 0x10;
                    //if (wptr == desc_addr + (cmd_count_per_queue * 0x10)) {
                    if (cmd_addr == desc_addr) {
                        if ((rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) &&
                            ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0)) {
                            rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
                        }
                        while(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) {
                            my_mdelay(1);
                        }
                        rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer
                    }

                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), (unsigned int)wptr); //write pointer
                    //printf("@@@[%d][%d][%d] rptr[0x%x] = 0x%x, wptr[0x%x] = 0x%x\n", __LINE__, k, n, LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))), LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m))));

                    if ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0) {
                        rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * (k + m))));
                    }
                    //printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                    if (k + 1 == desc_count_per_core) {
                        n = (n + 1) % cmd_count_per_queue;
                    }
                    k  = (k + 1) % desc_count_per_core;
                    //printf("@@@ k = %d, n = %d\n", k, n);
                }
            }

            if (is_found) {
            	break;
            }
            else {
                if ((rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) &&
                     ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0)) {
                    rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
                }
            }
            if (!(--timeout)) {
                printf("[ERR][%d][%d] wait for available command timeout!\n", __LINE__, k);
                encode_cmd_parser((unsigned int *)cmd_addr);
                g_status = 1;
                return 1;
            }
            my_mdelay(1);
        }
    }

    timeout = 1000;
    while(rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) < rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)))) {
        if ((rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m)))) == 0X0) {
            rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3 << (0x6 * (k + m))));
        }
        if (!(--timeout)) {
            printf("[ERR][%d][%d] wait for rptr = wptr timeout!\n", __LINE__, k);
            encode_cmd_parser((unsigned int *)cmd_addr);
            g_status = 1;
            return 1;
        }
        my_mdelay(1);
    }

    for (k = 0; k < desc_count_per_core; k++) {
        for (n = 0; n < cmd_count_per_queue; n++) {
            desc_addr = (unsigned char *)((k + 1) * 0x100000) + cpu_base;
            cmd_addr = desc_addr + n * 0x10;
    //        invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
            timeout = 1000;
#if (CHK_CRC_FAIL == 0)
            while(!(cmd_addr[4] & 0x01)) {
#else
            while(!((cmd_addr[4] & 0x01) && (cmd_addr[10] & 0x80) == 0x80))) {
#endif

#if (CHK_CRC_FAIL == 0)
                if ((cmd_addr[10] & 0x80) || (cmd_addr[4] & 0x0E)) {
#else
                if ((cmd_addr[4] & 0x0E))
#endif
                    printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                    decode_cmd_parser((unsigned int *)cmd_addr);
                    g_status = 1;
                    return 1;
                }
                if (!(--timeout)) {
                    printf("[ERR][%d][%d] wait for command timeout!\n", __LINE__, k);
                    decode_cmd_parser((unsigned int *)cmd_addr);
                    g_status = 1;
                    return 1;
                }
                my_mdelay(1);
    //            invalidate_dcache_range((unsigned long)cmd_addr, (unsigned long)cmd_addr + 0x10);
            }
            //decode_cmd_parser((unsigned int *)cmd_addr);
        }
    }
    invalidate_dcache_range((unsigned long)out_addr, (unsigned long)out_addr + (end_addr - start_addr));
    for (i = start_addr, j = 0; i < end_addr; i++, j++) {
        if (*((char *)i) != *((char *)(out_addr + j))) {
            printf("{ERR] compressed data is not match, addr:%x, data:%x/%x\n", i, *((char *)start_addr), *((char *)out_addr));
            break;
        }
    }
    printf("[%d][%d] %s finish !! compressed_size = %d\n", get_cpu_id(), count++, __FUNCTION__, compressed_size);

    //if ((get_cpu_id() == 0 && count == 100))
    if (g_status == 1)
    	return 0;

    //goto retry;
    return ret;
}

int do_lzma_mc_test(unsigned int start_addr, unsigned int end_addr)
{
    int ret = 0;
    unsigned int cpu_base = (get_cpu_id() == 0) ? (0x0000000):(0x8000000);//(0x00000000):(0x10000000);
    unsigned int out_addr = cpu_base + 0x6000000;//cpu_base + 0xC000000;
    unsigned int tmp_addr = cpu_base + 0x5000000;//cpu_base + 0x8000000;
    unsigned int i = 0, j = 0, k = 0, l = 0, m = 0;
    unsigned char *desc_addr = 0;
    unsigned int align_addr = 0;
    unsigned int is_decode = 0;
    volatile unsigned char *desc0_addr = (unsigned char *)(cpu_base + 0x100000);
    volatile unsigned char *desc1_addr = (unsigned char *)(cpu_base + 0x200000);
    volatile unsigned char *desc2_addr = (unsigned char *)(cpu_base + 0x300000);
    volatile unsigned char *desc3_addr = (unsigned char *)(cpu_base + 0x400000);
    volatile unsigned int *comp_size_addr = (unsigned int *)(cpu_base + 0x1000000);
    volatile unsigned int *decomp_size_addr = (unsigned int *)(cpu_base + 0x2000000);
    volatile unsigned int *crc_addr = (unsigned int *)(cpu_base + 0x3000000);
    bool is_found = 0;
    unsigned int timeout = 1000;
    unsigned int count = 0;
    const unsigned int desc_count_per_core = 2;//1;//2;
    unsigned int compressed_size = 0;

#define CHK_CRC_FAIL 0
#define RUN_SINGLE_CORE 0

#if (RUN_SINGLE_CORE == 1)
    if (get_cpu_id() != 0)
    	return 0;
#endif

    m = (get_cpu_id() == 0) ? 0:desc_count_per_core;

    printf("@@@ %s, cpu_id = %d, desc_count_per_core = %d\n", __FUNCTION__, get_cpu_id(), desc_count_per_core);

    rtd_maskl(LZMA_LZMA_DMA_RD_Ctrl_reg, 0xfff0fcff, (0x3 << 8) | (0xf << 16));
    rtd_maskl(LZMA_LZMA_DMA_WR_Ctrl_reg, 0xfffcfcff, (0x3 << 8) | (0x3 << 16));
    printf("[0x%08x] = 0x%08x\n", LZMA_LZMA_DMA_RD_Ctrl_reg, rtd_inl(LZMA_LZMA_DMA_RD_Ctrl_reg));
    printf("[0x%08x] = 0x%08x\n", LZMA_LZMA_DMA_WR_Ctrl_reg, rtd_inl(LZMA_LZMA_DMA_WR_Ctrl_reg));

retry:
    invalidate_dcache_all();
    is_decode = 0; //encode
    my_memset((void *)out_addr, 0, 0x1000000);
    my_memset((void *)tmp_addr, 0, 0x1000000);
    //my_memset((void *)comp_size_addr, 0, 0x1000000);
    //my_memset((void *)decomp_size_addr, 0, 0x1000000);
    //my_memset((void *)crc_addr, 0, 0x1000000);
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
    flush_dcache_range((unsigned long)out_addr, (unsigned long)out_addr + 0x1000000);
    flush_dcache_range((unsigned long)tmp_addr, (unsigned long)tmp_addr + 0x1000000);
    //flush_dcache_range((unsigned long)comp_size_addr, (unsigned long)comp_size_addr + 0x1000000);
    //flush_dcache_range((unsigned long)decomp_size_addr, (unsigned long)decomp_size_addr + 0x1000000);
    //flush_dcache_range((unsigned long)crc_addr, (unsigned long)crc_addr + 0x1000000);
//    flush_dcache_range((unsigned long)desc0_addr, (unsigned long)desc0_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc1_addr, (unsigned long)desc1_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc2_addr, (unsigned long)desc2_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc3_addr, desc3_addr + 0x100000);
    (desc0_addr + 0x10)[4] = 0x1; //done bit = 0x1
    (desc1_addr + 0x10)[4] = 0x1;
    (desc2_addr + 0x10)[4] = 0x1;
    (desc3_addr + 0x10)[4] = 0x1;
    k = 0;
    compressed_size = 0;
    for (i = start_addr, j = 0; i < end_addr; i += 4096, j++) {
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
              desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//                invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
                //if ((desc_addr[4] & 0x1) == 0x01) {
                if (((desc_addr[4] & 0x1) == 0x01) && !(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m))))) {
                    align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                    if (align_addr != 0) {
                        if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                            printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                            encode_cmd_parser((unsigned int *)desc_addr);
                            g_status = 1;
                            return 1;
                        }
                        //align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                        l = (align_addr - start_addr) >> 12;
                        *(comp_size_addr + l) = ((desc_addr[9] & 0x1f) << 8) | desc_addr[8];
                        compressed_size += *(comp_size_addr + l);
                        *(decomp_size_addr + l) = 0x1000;
                        *(crc_addr + l) = (desc_addr[11] << 8) | desc_addr[10];
                        //printf("[Info][%d] offset = %x, comp_size = %04x, decomp_size = %04x, crc = %04x\n", k, l, *(comp_size_addr + l), *(decomp_size_addr + l), *(crc_addr + l));
                        //encode_cmd_parser((unsigned int *)desc_addr);
                    }
                    my_memset((void *)desc_addr, 0, 0x10);

                    //if (((desc_addr + 0x10)[4] == 0x1) || ((desc_addr + 0x20)[4] == 0x1) || ((desc_addr + 0x30)[4] == 0x1) || ((desc_addr + 0x40)[4] == 0x1)) {
                    //     printf("[ERR][%d][%d] wrong descriptor !!\n", __LINE__, k);
                    //     return 1;
                    //}

                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    desc_addr[1] = (align_addr >> 8) & 0xff;
                    desc_addr[2] = (align_addr >> 16) & 0xff;
                    desc_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((tmp_addr + (j * 4096)) >> 4) << 4;
                    desc_addr[4] = (align_addr) & 0xf0;
                    desc_addr[5] = (align_addr >> 8) & 0xff;
                    desc_addr[6] = (align_addr >> 16) & 0xff;
                    desc_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        desc_addr[8] = *(comp_size_addr + j) & 0xff;
                        desc_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        desc_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        desc_addr[12] = *(decomp_size_addr + j) & 0xff;
                        desc_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        desc_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        ;//desc_addr[10] = *(crc_addr + j) & 0xff;
                        ;//desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        desc_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

//                    flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
//                    invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

                    //if (get_cpu_id() != 0)
                    //    m = desc_count_per_core;//2;

                    rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), (unsigned int)desc_addr - 0x10); //desc addr
                    rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), (unsigned int)desc_addr + 0x40); //desc limit
                    rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer
                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr + 0x10); //write pointer
                    if ((rtd_inl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m))) != (unsigned int)desc_addr - 0x10) ||
                    	  (rtd_inl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m))) != (unsigned int)desc_addr + 0x40) ||
                    	  (rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) != (unsigned int)desc_addr) ||
                    	  (rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m))) != (unsigned int)desc_addr + 0x10)) {
                        printf("[ERR] wrong register setting !!\n");
                        return 1;
                    }

                    //dmb();
                    rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * (k + m))));

                    //printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                }
                k  = (k + 1) % desc_count_per_core;
            }

            if (is_found) {
                break;
            }
            else {
                k = 0;
                while(1) {
                    {//for (k = 0; k < 4 ;k++) {
                        desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//                        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

                        if ((desc_addr[4] & 0x01) == 0x01) {
                            if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                                printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                                encode_cmd_parser((unsigned int *)desc_addr);
                                g_status = 1;
                                return 1;
                            }
                            is_found = 1;
                            break;
                        }
                        k = (k + 1) % desc_count_per_core;
                    }
                    my_mdelay(1);
                }
            }
        }
    }

    for (k = 0; k < desc_count_per_core ;k++) {
        desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        timeout = 1000;
        while(!(desc_addr[4] & 0x01)) {
            if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                encode_cmd_parser((unsigned int *)desc_addr);
                g_status = 1;
                return 1;
            }
            if (!(--timeout)) {
                printf("[ERR][%d][%d] wait for command timeout!\n", __LINE__, k);
                encode_cmd_parser((unsigned int *)desc_addr);
                g_status = 1;
                return 1;
            }
            my_mdelay(1);
//            invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        }
        align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
        l = (align_addr - start_addr) >> 12;
        *(comp_size_addr + l) = ((desc_addr[9] & 0x1f) << 8) | desc_addr[8];
        compressed_size += *(comp_size_addr + l);
        *(decomp_size_addr + l) = 0x1000;
        *(crc_addr + l) = (desc_addr[11] << 8) | desc_addr[10];
        //encode_cmd_parser((unsigned int *)desc_addr);
    }

    is_decode = 1; //decode
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
//    flush_dcache_range((unsigned long)desc0_addr, (unsigned long)desc0_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc1_addr, (unsigned long)desc1_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc2_addr, (unsigned long)desc2_addr + 0x100000);
//    flush_dcache_range((unsigned long)desc3_addr, (unsigned long)desc3_addr + 0x100000);
    (desc0_addr + 0x10)[4] = 0x1; //done bit = 0x1
    (desc1_addr + 0x10)[4] = 0x1;
    (desc2_addr + 0x10)[4] = 0x1;
    (desc3_addr + 0x10)[4] = 0x1;
    k = 0;
    for (i = tmp_addr, j = 0; i < (tmp_addr + (end_addr - start_addr)); i += 4096, j++) {
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
                desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//                invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
#if (CHK_CRC_FAIL == 0)
                //if ((desc_addr[4] & 0x1) == 0x01) {
                if (((desc_addr[4] & 0x1) == 0x01) && !(rtd_inl(LZMA_CMD_GO_SWAP_reg) & (0x1 << (0x6 * (k + m))))) {
#else
                if (((desc_addr[4] & 0x1) == 0x01) && ((desc_addr[10] & 0x80) == 0x80)) {
#endif
                    align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                    if (align_addr != 0) {
#if (CHK_CRC_FAIL == 0)
                        if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
#else
                        if ((desc_addr[4] & 0x0E)) {
#endif
                            printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                            decode_cmd_parser((unsigned int *)desc_addr);
                            g_status = 1;
                            return 1;
                        }
                        //printf("----k = %d----\n", k);
                        //decode_cmd_parser((unsigned int *)desc_addr);
                    }
                    my_memset((void *)desc_addr, 0, 0x10);
                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    desc_addr[1] = (align_addr >> 8) & 0xff;
                    desc_addr[2] = (align_addr >> 16) & 0xff;
                    desc_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((out_addr + (j * 4096)) >> 4) << 4;
                    desc_addr[4] = (align_addr) & 0xf0;
                    desc_addr[5] = (align_addr >> 8) & 0xff;
                    desc_addr[6] = (align_addr >> 16) & 0xff;
                    desc_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        desc_addr[8] = *(comp_size_addr + j) & 0xff;
                        desc_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        desc_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        desc_addr[12] = *(decomp_size_addr + j) & 0xff;
                        desc_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        desc_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        ;//desc_addr[10] = *(crc_addr + j) & 0xff;
                        ;//desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        desc_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
                    //printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

//                    flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
//                    invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

                    //if (get_cpu_id() != 0)
                    //    m = desc_count_per_core;//2;

                    rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m)), (unsigned int)desc_addr - 0x10); //desc addr
                    rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m)), (unsigned int)desc_addr + 0x40); //desc limit
                    rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr); //read pointer
                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m)), (unsigned int)desc_addr + 0x10); //write pointer
                    if ((rtd_inl(LZMA_CMD_0_BASE_reg + (0x10 * (k + m))) != (unsigned int)desc_addr - 0x10) ||
                    	  (rtd_inl(LZMA_CMD_0_LIMIT_reg + (0x10 * (k + m))) != (unsigned int)desc_addr + 0x40) ||
                    	  (rtd_inl(LZMA_CMD_0_RPTR_reg + (0x10 * (k + m))) != (unsigned int)desc_addr) ||
                    	  (rtd_inl(LZMA_CMD_0_WPTR_reg + (0x10 * (k + m))) != (unsigned int)desc_addr + 0x10)) {
                        printf("[ERR] wrong register setting !!\n");
                        return 1;
                    }

                    //dmb();
                    rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * (k + m))));

                    //printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                }
                k = (k + 1) % desc_count_per_core;
            }

            if (is_found) {
            	break;
            }
            else {
                k = 0;
                while(1) {
                    {//for (k = 0; k < 4 ;k++) {
                        desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//                        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

#if (CHK_CRC_FAIL == 0)
                        if ((desc_addr[4] & 0x01) == 0x01) {
#else
                        if (((desc_addr[4] & 0x01) == 0x01) && ((desc_addr[10] & 0x80) == 0x80)) {
#endif
#if (CHK_CRC_FAIL == 0)
                            if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
#else
                            if ((desc_addr[4] & 0x0E)) {
#endif
                                printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                                decode_cmd_parser((unsigned int *)desc_addr);
                                g_status = 1;
                                return 1;
                            }
                            is_found = 1;
                            break;
                        }
        	              k = (k + 1) % desc_count_per_core;
                    }
                    my_mdelay(1);
                }
            }
        }
    }

    for (k = 0; k < desc_count_per_core ;k++) {
        desc_addr = (unsigned char *)((k + 1) * 0x100000) + 0x10 + cpu_base;
//        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        timeout = 1000;
#if (CHK_CRC_FAIL == 0)
        while(!(desc_addr[4] & 0x01)) {
#else
        while(!((desc_addr[4] & 0x01) && (desc_addr[10] & 0x80) == 0x80))) {
#endif

#if (CHK_CRC_FAIL == 0)
            if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
#else
            if ((desc_addr[4] & 0x0E))
#endif
                printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                decode_cmd_parser((unsigned int *)desc_addr);
                g_status = 1;
                return 1;
            }
            if (!(--timeout)) {
                printf("[ERR][%d][%d] wait for command timeout!\n", __LINE__, k);
                decode_cmd_parser((unsigned int *)desc_addr);
                g_status = 1;
                return 1;
            }
            my_mdelay(1);
//            invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        }
        //decode_cmd_parser((unsigned int *)desc_addr);
    }

    invalidate_dcache_range((unsigned long)out_addr, (unsigned long)out_addr + (end_addr - start_addr));
    for (i = start_addr, j = 0; i < end_addr; i++, j++) {
        if (*((char *)i) != *((char *)(out_addr + j))) {
            printf("{ERR] compressed data is not match, addr:%x, data:%x/%x\n", i, *((char *)start_addr), *((char *)out_addr));
            break;
        }
    }
    printf("[%d][%d] %s finish !! compressed_size = %d\n", get_cpu_id(), count++, __FUNCTION__, compressed_size);

    //if ((get_cpu_id() == 0 && count == 100))
    if (g_status == 1)
    	return 0;

    goto retry;
    return ret;
}

int do_lzma_mq_test(unsigned int start_addr, unsigned int end_addr)
{
    int ret = 0;

    unsigned int out_addr = 0x20000000;
    unsigned int tmp_addr = 0x10000000;
    unsigned int i = 0, j = 0, k = 0, l = 0;
    unsigned char *desc_addr = 0;
    unsigned int align_addr = 0;
    unsigned int is_decode = 0;
    volatile unsigned char *desc0_addr = (unsigned char *)0x100000;
    volatile unsigned char *desc1_addr = (unsigned char *)0x200000;
    volatile unsigned char *desc2_addr = (unsigned char *)0x300000;
    volatile unsigned char *desc3_addr = (unsigned char *)0x400000;
    volatile unsigned int *comp_size_addr = (unsigned int *)0x1000000;
    volatile unsigned int *decomp_size_addr = (unsigned int *)0x2000000;
    volatile unsigned int *crc_addr = (unsigned int *)0x3000000;
    bool is_found = 0;
    unsigned int timeout = 1000;

    is_decode = 0; //encode
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
    desc0_addr[4] = 0x1; //done bit = 0x1
    desc1_addr[4] = 0x1;
    desc2_addr[4] = 0x1;
    desc3_addr[4] = 0x1;
    for (i = start_addr, j = 0; i < end_addr; i += 4096, j++) {
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
                desc_addr = (unsigned char *)((k + 1) * 0x100000);
                invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
                if ((desc_addr[4] & 0x1) == 0x01) {
                    align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                    if (align_addr != 0) {
                        if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                	          printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                	          encode_cmd_parser((unsigned int *)desc_addr);
                	          return 1;
                        }
                        //align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                        l = (align_addr - start_addr) >> 12;
                        *(comp_size_addr + l) = ((desc_addr[9] & 0x1f) << 8) | desc_addr[8];
                        *(decomp_size_addr + l) = 0x1000;
                        *(crc_addr + l) = (desc_addr[11] << 8) | desc_addr[10];
                        printf("[Info][%d] offset = %x, comp_size = %04x, decomp_size = %04x, crc = %04x\n", k, l, *(comp_size_addr + l), *(decomp_size_addr + l), *(crc_addr + l));
                        encode_cmd_parser((unsigned int *)desc_addr);
                    }
                    my_memset(desc_addr, 0, 0x10);
                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    desc_addr[1] = (align_addr >> 8) & 0xff;
                    desc_addr[2] = (align_addr >> 16) & 0xff;
                    desc_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((tmp_addr + (j * 4096)) >> 4) << 4;
                    desc_addr[4] = (align_addr) & 0xf0;
                    desc_addr[5] = (align_addr >> 8) & 0xff;
                    desc_addr[6] = (align_addr >> 16) & 0xff;
                    desc_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        desc_addr[8] = *(comp_size_addr + j) & 0xff;
                        desc_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        desc_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        desc_addr[12] = *(decomp_size_addr + j) & 0xff;
                        desc_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        desc_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        desc_addr[10] = *(crc_addr + j) & 0xff;
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        desc_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

                    flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);      

                    rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * k), (unsigned int)desc_addr); //desc addr
                    rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * k), (unsigned int)desc_addr + 0x40); //desc limit
                    rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * k), (unsigned int)desc_addr); //read pointer
                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * k), (unsigned int)desc_addr + 0x10); //write pointer

                    rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * k)));

										printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                }
                k  = (k + 1) % 4;
            }

            if (is_found) {
                break;
            }
            else {
                while(1) {
                    {//for (k = 0; k < 4 ;k++) {
                        desc_addr = (unsigned char *)((k + 1) * 0x100000);
                        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

                        if ((desc_addr[4] & 0x01) == 0x01) {
                            if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                	              printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                	              encode_cmd_parser((unsigned int *)desc_addr);
                	              return 1;
                            }
                            is_found = 1;
                            //k = 4;
                            k = (k + 1) % 4;
        	              }
        	          }
        	          if (is_found)
        	              break;
        	          else
        	              my_mdelay(1);
        	      }
            }
        }
    }

    for (k = 0; k < 4 ;k++) {
        desc_addr = (unsigned char *)((k + 1) * 0x100000);
        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        timeout = 1000;
        while(!(desc_addr[4] & 0x01)) {
            if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                encode_cmd_parser((unsigned int *)desc_addr);
                return 1;
            }
            if (!(--timeout)) {
        	  	printf("[ERR][%d] wait for command timeout!\n", k);
        	  	break;
        	  }
            my_mdelay(1);
            invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        }
        align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
        l = (align_addr - start_addr) >> 12;
        *(comp_size_addr + l) = ((desc_addr[9] & 0x1f) << 8) | desc_addr[8];
        *(decomp_size_addr + l) = 0x1000;
        *(crc_addr + l) = (desc_addr[11] << 8) | desc_addr[10];
        encode_cmd_parser((unsigned int *)desc_addr);
    }

    is_decode = 1; //encode
    my_memset((void *)desc0_addr, 0, 0x100000);
    my_memset((void *)desc1_addr, 0, 0x100000);
    my_memset((void *)desc2_addr, 0, 0x100000);
    my_memset((void *)desc3_addr, 0, 0x100000);
    desc0_addr[4] = 0x1; //done bit = 0x1
    desc1_addr[4] = 0x1;
    desc2_addr[4] = 0x1;
    desc3_addr[4] = 0x1;
    for (i = tmp_addr, j = 0; i < (tmp_addr + (end_addr - start_addr)); i += 4096, j++) {
        while(1) {
            is_found = 0;
            {//for (k = 0; k < 4; k++) {
                desc_addr = (unsigned char *)((k + 1) * 0x100000);
                invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
                if ((desc_addr[4] & 0x1) == 0x01) {
                    align_addr = (desc_addr[3] << 24) | (desc_addr[2] << 16) | (desc_addr[1] << 8) | (desc_addr[0] & 0xf0);
                    if (align_addr != 0) {
                        if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
                	          printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                	          decode_cmd_parser((unsigned int *)desc_addr);
                	          return 1;
                        }
                        printf("----k = %d----\n", k);
                        decode_cmd_parser((unsigned int *)desc_addr);
                    }
                    my_memset((void *)desc_addr, 0, 0x10);
                    // aligned start address - [31:4]
                    align_addr = (i >> 4) << 4;
                    desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
                    desc_addr[1] = (align_addr >> 8) & 0xff;
                    desc_addr[2] = (align_addr >> 16) & 0xff;
                    desc_addr[3] = (align_addr >> 24) & 0xff;

                    // aligned output address - [63:36]
                    align_addr = ((out_addr + (j * 4096)) >> 4) << 4;
                    desc_addr[4] = (align_addr) & 0xf0;
                    desc_addr[5] = (align_addr >> 8) & 0xff;
                    desc_addr[6] = (align_addr >> 16) & 0xff;
                    desc_addr[7] = (align_addr >> 24) & 0xff;

                    // compressed size - [86:64]
                    if (is_decode) {
                        desc_addr[8] = *(comp_size_addr + j) & 0xff;
                        desc_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                        desc_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
                    }

                    // decompressd size - [118:96]
                    if (is_decode) {
                        desc_addr[12] = *(decomp_size_addr + j) & 0xff;
                        desc_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                        desc_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
                    }

                    // crc - [95:88], [127:120]
                    if (!is_decode) {
                        desc_addr[10] = *(crc_addr + j) & 0xff;
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                    } else {
                        desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                        desc_addr[15] = *(crc_addr + j) & 0xff;
                    }

                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
                    printf("[%d][%08x] = %02x%02x%02x%02x\n", k, (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

                    flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);      

                    rtd_outl(LZMA_CMD_0_BASE_reg + (0x10 * k), (unsigned int)desc_addr); //desc addr
                    rtd_outl(LZMA_CMD_0_LIMIT_reg + (0x10 * k), (unsigned int)desc_addr + 0x40); //desc limit
                    rtd_outl(LZMA_CMD_0_RPTR_reg + (0x10 * k), (unsigned int)desc_addr); //read pointer
                    rtd_outl(LZMA_CMD_0_WPTR_reg + (0x10 * k), (unsigned int)desc_addr + 0x10); //write pointer

                    rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x40000000 + (0x3f << (0x6 * k)));

                    printf("[Info] found empty descritpor: %d\n", k);
                    is_found = 1;
                    //k = 4;
                }
                k = (k + 1) % 4;
            }

            if (is_found) {
            	break;
            }
            else {
                while(1) {
                    {//for (k = 0; k < 4 ;k++) {
                        desc_addr = (unsigned char *)((k + 1) * 0x100000);
                        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

                        if ((desc_addr[4] & 0x01) == 0x01) {
                            if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
                	              printf("[ERR][%d][%d] error bit is assert!\n", __LINE__, k);
                	              decode_cmd_parser((unsigned int *)desc_addr);
                	              return 1;
                            }
                            is_found = 1;
                            k = 4;
        	              }
        	              k = (k + 1) % 4;
        	          }
        	          if (is_found)
        	              break;
        	          else
        	              my_mdelay(1);
        	      }
            }
        }
    }

    for (k = 0; k < 4 ;k++) {
        desc_addr = (unsigned char *)((k + 1) * 0x100000);
        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        timeout = 1000;
        while(!(desc_addr[4] & 0x01)) {
            if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
                printf("[ERR] error bit is assert!\n");
                decode_cmd_parser((unsigned int *)desc_addr);
                return 1;
            }
            if (!(--timeout)) {
        	  	printf("[ERR][%d] wait for command timeout!\n", k);
        	  	break;
        	  }
            my_mdelay(1);
            invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        }
        decode_cmd_parser((unsigned int *)desc_addr);
    }
    invalidate_dcache_range((unsigned long)out_addr, (unsigned long)out_addr + (end_addr - start_addr));
    for (i = start_addr, j = 0; i < end_addr; i++, j++) {
        	if (*((char *)i) != *((char *)(out_addr + j))) {
        	    printf("{ERR] compressed data is not match, addr:%x, data:0x%x/0x%x\n", i, *((char *)start_addr), *((char *)out_addr));
        	    break;
        	}
    }
    printf("%s finish !!\n", __FUNCTION__);
    return ret;
}

int do_lzma(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int ret = 0;
    unsigned int start_addr = 0, end_addr = 0, out_addr = 0;
    unsigned int align_addr = 0;
    unsigned int is_batch = 0, is_decode = 0;
    unsigned int i, j;
    unsigned int crc = 0, comp_size = 0, decomp_size = 0;
    unsigned int timeout = 1000;
    volatile unsigned char *desc_addr = (unsigned char *)0x100000;
    volatile unsigned int *comp_size_addr = (unsigned int *)0x400000;
    volatile unsigned int *decomp_size_addr = (unsigned int *)0x800000;
    volatile unsigned int *crc_addr = (unsigned int *)0xc00000;

    if (argc < 5) {
        if ((argc == 4) && (strcmp(argv[1], "mq") == 0)) {
            start_addr = (unsigned int)simple_strtoul(argv[2], NULL, 16);
            end_addr = (unsigned int)simple_strtoul(argv[3], NULL, 16);
            return do_lzma_mq_test(start_addr, end_addr);
        }
        else if ((argc == 4) && (strcmp(argv[1], "mcmq") == 0)) {
            start_addr = (unsigned int)simple_strtoul(argv[2], NULL, 16);
            end_addr = (unsigned int)simple_strtoul(argv[3], NULL, 16);
            return do_lzma_mcmq_test(start_addr, end_addr);
        }
        else if ((argc == 4) && (strcmp(argv[1], "mc") == 0)) {
            start_addr = (unsigned int)simple_strtoul(argv[2], NULL, 16);
            end_addr = (unsigned int)simple_strtoul(argv[3], NULL, 16);
            return do_lzma_mc_test(start_addr, end_addr);
        }
        printf("[ERR] the number of parameters should be larger than 5\n");
        return CMD_RET_USAGE;
    }

    if (((argv[1][0] != 's') && (argv[1][0] != 'b')) ||
    	  ((argv[1][1] != 'e') && (argv[1][1] != 'd'))) {
        printf("[ERR] wrong argv[1]=%s\n", argv[1]);
		    return CMD_RET_USAGE;
    }
    else {
        is_batch = (argv[1][0] == 'b') ? 1 : 0;
        is_decode = (argv[1][1] == 'd') ? 1 : 0;

        if (!is_batch) {
            if (argc < 7) {
        		    printf("[ERR] the number of parameters should be larger than 7\n");
        		    return CMD_RET_USAGE;
            }
        }
    }

    if (is_batch) {
        start_addr = (unsigned int)simple_strtoul(argv[2], NULL, 16);
        end_addr = (unsigned int)simple_strtoul(argv[3], NULL, 16);
        out_addr = (unsigned int)simple_strtoul(argv[4], NULL, 16);
        decomp_size = (unsigned int)simple_strtoul(argv[5], NULL, 16);

        if (((start_addr < 0x100000) || (start_addr >= 0x40000000)) ||
    	      ((end_addr < 0x100000) || (end_addr >= 0x40000000)) ||
    	      ((out_addr < 0x100000) || (out_addr > 0x40000000))) {
    	  	    printf("[ERR] wrong address, start=0x%x, end=0x%x, out=0x%x\n", start_addr, end_addr, out_addr);
    	  	    return 1;
        }

        if (start_addr >= end_addr) {
    	      printf("[ERR] end aadr is greater than start_addr, start=0x%x, end=0x%x\n", start_addr, end_addr);
    	      return 1;
        }

        for (i = start_addr, j = 0; i < end_addr; i += 4096, j++) {
        	  my_memset((void *)desc_addr, 0x0, 16);

            // aligned start address - [31:4]
            align_addr = (i >> 4) << 4;
            desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
            desc_addr[1] = (align_addr >> 8) & 0xff;
            desc_addr[2] = (align_addr >> 16) & 0xff;
            desc_addr[3] = (align_addr >> 24) & 0xff;

            // aligned output address - [63:36]
            align_addr = ((out_addr + (j * 4096)) >> 4) << 4;
            desc_addr[4] = (align_addr) & 0xf0;
            desc_addr[5] = (align_addr >> 8) & 0xff;
            desc_addr[6] = (align_addr >> 16) & 0xff;
            desc_addr[7] = (align_addr >> 24) & 0xff;

            // compressed size - [86:64]
            if (is_decode) {
                desc_addr[8] = *(comp_size_addr + j) & 0xff;
                desc_addr[9] = (*(comp_size_addr + j) >> 8) & 0xff;
                desc_addr[10] = (*(comp_size_addr + j) >> 16) & 0x7f;
            }

            // decompressd size - [118:96]
            if (is_decode) {
                desc_addr[12] = *(decomp_size_addr + j) & 0xff;
                desc_addr[13] = (*(decomp_size_addr + j) >> 8) & 0xff;
                desc_addr[14] = (*(decomp_size_addr + j) >> 16) & 0x7f;
            }

            // crc - [95:88], [127:120]
            if (!is_decode) {
                desc_addr[10] = *(crc_addr + j) & 0xff;
                desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
            } else {
                desc_addr[11] = (*(crc_addr + j) >> 8) & 0xff;
                desc_addr[15] = *(crc_addr + j) & 0xff;
            }

            printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
            printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
            printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
            printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

            flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

            rtd_outl(LZMA_CMD_0_BASE_reg, (unsigned int)desc_addr); //desc addr
            rtd_outl(LZMA_CMD_0_LIMIT_reg, (unsigned int)desc_addr + 0x40); //desc limit
            rtd_outl(LZMA_CMD_0_RPTR_reg, (unsigned int)desc_addr); //read pointer
            rtd_outl(LZMA_CMD_0_WPTR_reg, (unsigned int)desc_addr + 0x10); //write pointer

            rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x4000003f);

            invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
            timeout = 1000;
            while(!(desc_addr[4] & 0x01)) {
        	      my_mdelay(1);
        	      printf("waiting for command finish!\n");
        	      if (!(--timeout)) {
        	  	      printf("[ERR] wait for command timeout!\n");
        	  	      break;
        	      }
        	      invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
            }
            if (!is_decode) {
                if ((desc_addr[0] & 0x08) || (desc_addr[4] & 0x02)) {
                	printf("[ERR] error bit is assert!\n");
                	encode_cmd_parser((unsigned int *)desc_addr);
                	break;
                }

                *(comp_size_addr + j) = ((desc_addr[9] & 0x1f) << 8) | desc_addr[8];
                *(decomp_size_addr + j) = 0x1000;
                *(crc_addr + j) = (desc_addr[11] << 8) | desc_addr[10];
                printf("[Info] comp_size = %04x, decomp_size = %04x, crc = %04x\n", *(comp_size_addr + j), *(decomp_size_addr + j), *(crc_addr + j));
                encode_cmd_parser((unsigned int *)desc_addr);
            }
            else {
                if ((desc_addr[10] & 0x80) || (desc_addr[4] & 0x0E)) {
                	printf("[ERR] error bit is assert!\n");
                	decode_cmd_parser((unsigned int *)desc_addr);
                	break;
                }
                decode_cmd_parser((unsigned int *)desc_addr);
            }

            printf("command finish!\n\n\n\n");
        }
    }
    else {
        my_memset((void *)desc_addr, 0x0, 16);

        start_addr = (unsigned int)simple_strtoul(argv[2], NULL, 16);
        out_addr = (unsigned int)simple_strtoul(argv[3], NULL, 16);
        crc = (unsigned int)simple_strtoul(argv[4], NULL, 16);
        comp_size = (unsigned int)simple_strtoul(argv[5], NULL, 16);
        decomp_size = (unsigned int)simple_strtoul(argv[6], NULL, 16);

        // aligned start address - [31:4]
        align_addr = (start_addr >> 4) << 4;
        desc_addr[0] = (align_addr & 0xf0) | (1 << is_decode);
        desc_addr[1] = (align_addr >> 8) & 0xff;
        desc_addr[2] = (align_addr >> 16) & 0xff;
        desc_addr[3] = (align_addr >> 24) & 0xff;

        // aligned output address - [63:36]
        align_addr = (out_addr >> 4) << 4;
        desc_addr[4] = (align_addr) & 0xf0;
        desc_addr[5] = (align_addr >> 8) & 0xff;
        desc_addr[6] = (align_addr >> 16) & 0xff;
        desc_addr[7] = (align_addr >> 24) & 0xff;

        // compressed size - [86:64]
        if (is_decode) {
            desc_addr[8] = comp_size & 0xff;
            desc_addr[9] = (comp_size >> 8) & 0xff;
            desc_addr[10] = (comp_size >> 16) & 0x7f;
        }

        // decompressd size - [118:96]
        if (is_decode) {
            desc_addr[12] = decomp_size & 0xff;
            desc_addr[13] = (decomp_size >> 8) & 0xff;
            desc_addr[14] = (decomp_size >> 16) & 0x7f;
        }

        // crc - [95:88], [127:120]
        desc_addr[11] = (crc >> 8) & 0xff;
        desc_addr[15] = crc & 0xff;

        printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[0], desc_addr[3], desc_addr[2], desc_addr[1], desc_addr[0]);
        printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[4], desc_addr[7], desc_addr[6], desc_addr[5], desc_addr[4]);
        printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[8], desc_addr[11], desc_addr[10], desc_addr[9], desc_addr[8]);
        printf("[%08x] = %02x%02x%02x%02x\n", (unsigned int)&desc_addr[12], desc_addr[15], desc_addr[14], desc_addr[13], desc_addr[12]);

        flush_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);

        rtd_outl(LZMA_CMD_0_BASE_reg, (unsigned int)desc_addr); //desc addr
        rtd_outl(LZMA_CMD_0_LIMIT_reg, (unsigned int)desc_addr + 0x40); //desc limit
        rtd_outl(LZMA_CMD_0_RPTR_reg, (unsigned int)desc_addr); //read pointer
        rtd_outl(LZMA_CMD_0_WPTR_reg, (unsigned int)desc_addr + 0x10); //write pointer

        rtd_outl(LZMA_CMD_GO_SWAP_reg, 0x4000003f);

        invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        timeout = 1000;
        while(!(desc_addr[4] & 0x01)) {
        	  my_mdelay(1);
        	  printf("waiting for command finish!\n");
        	  if (!(--timeout)) {
        	  	printf("[ERR] wait for command timeout!\n");
        	  	break;
        	  }
        	  invalidate_dcache_range((unsigned long)desc_addr, (unsigned long)desc_addr + 0x10);
        }
        printf("command finish!\n\n\n\n");

        if (!is_decode) {
            encode_cmd_parser((unsigned int *)desc_addr);
        }
        else {
            decode_cmd_parser((unsigned int *)desc_addr);
        }
    }

    return ret;
}

U_BOOT_CMD(
    lzma, 7, 0, do_lzma,
    "HW LZMA Accelerator",
    "lzma [b[d/e]] start_addr end_addr out_addr\n"
    "lzma [s[d/e]] start_addr out_addr crc comp_size decomp_size\n"
    "lzma mq start_addr end_addr\n"
    "lzma mc start_addr end_addr\n"
    "lzma mcmq start_addr end_addr\n"
);
