/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#define GET_MPIDR()  (get_MPIDR() & 0x0fff)
#define ULONG_MAX	4294967295UL //#include <limits.h>

/*
 * Memory Functions
 *
 * Copied from FADS ROM, Dan Malek (dmalek@jlc.net)
 */

#include <common.h>
#include <command.h>
#ifdef CONFIG_HAS_DATAFLASH
#include <dataflash.h>
#endif
#include <watchdog.h>
#include <io.h>

#if defined(CONFIG_CMD_MM) || defined(CONFIG_CMD_NM)
static int mod_mem(cmd_tbl_t *, int, int, int, char * const []);
#endif

#define HWSEMA (0xb801A620)
inline int hwsema_trylock(void)
{
	return (rtd_inl(HWSEMA) == 1);
}

inline void hwsema_lock(unsigned long ra)
{
	int tmp = 0;
	while(1) {
		if (hwsema_trylock() == 1) {
			break;
		}
		
		asm volatile (
			"\n\t"
			"mov %0, %1 \n\t"
			"1: subs %0, #1\n\t"
			"bne 1b \n\t"
			::"r"(tmp), "I"(100));
	}
}

inline void hwsema_unlock(unsigned long ra)
{
	rtd_outl(HWSEMA, 0);
}


#if 1
#define smp_printf(fmt...)					\
	({							\
		int ret = 0;					\
		hwsema_lock(get_MPIDR());			\
		ret = printf(fmt);				\
		hwsema_unlock(get_MPIDR());			\
		ret;						\
	})
#else
#define smp_printf(fmt...)    printf(fmt)
#endif

/* Display values from last command.
 * Memory modify remembered values are different from display memory.
 */
static uint	dp_last_addr, dp_last_size;
static uint	dp_last_length = 0x40;
#if defined(CONFIG_CMD_MM) || defined(CONFIG_CMD_NM)
static uint	mm_last_addr, mm_last_size;
#endif
extern unsigned int cop_mtest_log_en;
static	ulong	base_address = 0;

/* Memory Display
 *
 * Syntax:
 *	md{.b, .w, .l} {addr} {len}
 */
#define DISP_LINE_LEN	16
int do_mem_md ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr, length;
#if defined(CONFIG_HAS_DATAFLASH)
	ulong	nbytes, linebytes;
#endif
	int	size;
	int rc = 0;

	/* We use the last specified parameters, unless new ones are
	 * entered.
	 */
	addr = dp_last_addr;
	size = dp_last_size;
	length = dp_last_length;

	if (argc < 2)
		return CMD_RET_USAGE;

	if ((flag & CMD_FLAG_REPEAT) == 0) {
		/* New command specified.  Check for a size specification.
		 * Defaults to long if no or incorrect specification.
		 */
		if ((size = cmd_get_data_size(argv[0], 4)) < 0)
			return 1;

		/* Address is specified since argc > 1
		*/
		addr = simple_strtoul(argv[1], NULL, 16);
		addr += base_address;

		/* If another parameter, it is the length to display.
		 * Length is the number of objects, not number of bytes.
		 */
		if (argc > 2)
			length = simple_strtoul(argv[2], NULL, 16);
	}

#if defined(CONFIG_HAS_DATAFLASH)
	/* Print the lines.
	 *
	 * We buffer all read data, so we can make sure data is read only
	 * once, and all accesses are with the specified bus width.
	 */
	nbytes = length * size;
	do {
		char	linebuf[DISP_LINE_LEN];
		void* p;
		linebytes = (nbytes>DISP_LINE_LEN)?DISP_LINE_LEN:nbytes;

		rc = read_dataflash(addr, (linebytes/size)*size, linebuf);
		p = (rc == DATAFLASH_OK) ? linebuf : (void*)addr;
		print_buffer(addr, p, size, linebytes/size, DISP_LINE_LEN/size);

		nbytes -= linebytes;
		addr += linebytes;
		if (ctrlc()) {
			rc = 1;
			break;
		}
	} while (nbytes > 0);
#else

# if defined(CONFIG_BLACKFIN)
	/* See if we're trying to display L1 inst */
	if (addr_bfin_on_chip_mem(addr)) {
		char linebuf[DISP_LINE_LEN];
		ulong linebytes, nbytes = length * size;
		do {
			linebytes = (nbytes > DISP_LINE_LEN) ? DISP_LINE_LEN : nbytes;
			memcpy(linebuf, (void *)addr, linebytes);
			print_buffer(addr, linebuf, size, linebytes/size, DISP_LINE_LEN/size);

			nbytes -= linebytes;
			addr += linebytes;
			if (ctrlc()) {
				rc = 1;
				break;
			}
		} while (nbytes > 0);
	} else
# endif
	{

#ifdef CONFIG_BSP_REALTEK
		/* Patch for rbus or spi address read */
		if (addr < (RBUS_BASE_VIRT_OLD + RBUS_BASE_SIZE) && addr >= RBUS_BASE_VIRT_OLD)
		{
			addr = addr - RBUS_BASE_VIRT_OLD + RBUS_BASE_VIRT;
		}
		else if (addr < (SPI_FLASH_BASE_VIRT_OLD + SPI_FLASH_BASE_SIZE) && addr >= SPI_FLASH_BASE_VIRT_OLD)
		{
			addr = addr - SPI_FLASH_BASE_VIRT_OLD + SPI_FLASH_BASE_VIRT;
		}
#endif

		/* Print the lines. */
		print_buffer(addr, (void*)addr, size, length, DISP_LINE_LEN/size);
		addr += size*length;
	}
#endif

	dp_last_addr = addr;
	dp_last_length = length;
	dp_last_size = size;
	return (rc);
}

#ifdef CONFIG_CMD_MM
int do_mem_mm ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	return mod_mem (cmdtp, 1, flag, argc, argv);
}
#endif

#ifdef CONFIG_CMD_NM
int do_mem_nm ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	return mod_mem (cmdtp, 0, flag, argc, argv);
}
#endif

int do_mem_mw ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr, writeval, count;
	int	size;

	if ((argc < 3) || (argc > 4))
		return CMD_RET_USAGE;

	/* Check for size specification.
	*/
	if ((size = cmd_get_data_size(argv[0], 4)) < 1)
		return 1;

	/* Address is specified since argc > 1
	*/
	addr = simple_strtoul(argv[1], NULL, 16);
	addr += base_address;

#ifdef CONFIG_BSP_REALTEK
	/* Patch for rbus or spi address write */
	if (addr < (RBUS_BASE_VIRT_OLD + RBUS_BASE_SIZE) && addr >= RBUS_BASE_VIRT_OLD)
	{
		addr = addr - RBUS_BASE_VIRT_OLD + RBUS_BASE_VIRT;
	}
	else if (addr < (SPI_FLASH_BASE_VIRT_OLD + SPI_FLASH_BASE_SIZE) && addr >= SPI_FLASH_BASE_VIRT_OLD)
	{
		addr = addr - SPI_FLASH_BASE_VIRT_OLD + SPI_FLASH_BASE_VIRT;
	}
#endif

	/* Get the value to write.
	*/
	writeval = simple_strtoul(argv[2], NULL, 16);

	/* Count ? */
	if (argc == 4) {
		count = simple_strtoul(argv[3], NULL, 16);
	} else {
		count = 1;
	}

	while (count-- > 0) {
		if (size == 4)
			*((ulong  *)addr) = (ulong )writeval;
		else if (size == 2)
			*((ushort *)addr) = (ushort)writeval;
		else
			*((u_char *)addr) = (u_char)writeval;
		addr += size;
	}
	return 0;
}

#ifdef CONFIG_MX_CYCLIC
int do_mem_mdc ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int i;
	ulong count;

	if (argc < 4)
		return CMD_RET_USAGE;

	count = simple_strtoul(argv[3], NULL, 10);

	for (;;) {
		do_mem_md (NULL, 0, 3, argv);

		/* delay for <count> ms... */
		for (i=0; i<count; i++)
			udelay (1000);

		/* check for ctrl-c to abort... */
		if (ctrlc()) {
			puts("Abort\n");
			return 0;
		}
	}

	return 0;
}

int do_mem_mwc ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int i;
	ulong count;

	if (argc < 4)
		return CMD_RET_USAGE;

	count = simple_strtoul(argv[3], NULL, 10);

	for (;;) {
		do_mem_mw (NULL, 0, 3, argv);

		/* delay for <count> ms... */
		for (i=0; i<count; i++)
			udelay (1000);

		/* check for ctrl-c to abort... */
		if (ctrlc()) {
			puts("Abort\n");
			return 0;
		}
	}

	return 0;
}
#endif /* CONFIG_MX_CYCLIC */

#ifdef CONFIG_CMD_CMP
int do_mem_cmp (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr1, addr2, count, ngood;
	int	size;
	int     rcode = 0;
	const char *type;

	if (argc != 4)
		return CMD_RET_USAGE;

	/* Check for size specification.
	*/
	if ((size = cmd_get_data_size(argv[0], 4)) < 0)
		return 1;
	type = size == 4 ? "word" : size == 2 ? "halfword" : "byte";

	addr1 = simple_strtoul(argv[1], NULL, 16);
	addr1 += base_address;

	addr2 = simple_strtoul(argv[2], NULL, 16);
	addr2 += base_address;

	count = simple_strtoul(argv[3], NULL, 16);

#ifdef CONFIG_HAS_DATAFLASH
	if (addr_dataflash(addr1) | addr_dataflash(addr2)){
		puts ("Comparison with DataFlash space not supported.\n\r");
		return 0;
	}
#endif

#ifdef CONFIG_BLACKFIN
	if (addr_bfin_on_chip_mem(addr1) || addr_bfin_on_chip_mem(addr2)) {
		puts ("Comparison with L1 instruction memory not supported.\n\r");
		return 0;
	}
#endif

	for (ngood = 0; ngood < count; ++ngood) {
		ulong word1, word2;
		if (size == 4) {
			word1 = *(ulong *)addr1;
			word2 = *(ulong *)addr2;
		} else if (size == 2) {
			word1 = *(ushort *)addr1;
			word2 = *(ushort *)addr2;
		} else {
			word1 = *(u_char *)addr1;
			word2 = *(u_char *)addr2;
		}
		if (word1 != word2) {
			printf("%s at 0x%08lx (%#0*lx) != %s at 0x%08lx (%#0*lx)\n",
				type, addr1, size, word1,
				type, addr2, size, word2);
			rcode = 1;
			break;
		}

		addr1 += size;
		addr2 += size;

		/* reset watchdog from time to time */
		if ((ngood % (64 << 10)) == 0)
			WATCHDOG_RESET();
	}

	printf("Total of %ld %s(s) were the same\n", ngood, type);
	return rcode;
}
#endif

#ifdef CONFIG_CMD_CP
int do_mem_cp ( cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr, dest, count;
	int	size;

	if (argc != 4)
		return CMD_RET_USAGE;

	/* Check for size specification.
	*/
	if ((size = cmd_get_data_size(argv[0], 4)) < 0)
		return 1;

	addr = simple_strtoul(argv[1], NULL, 16);
	addr += base_address;

	dest = simple_strtoul(argv[2], NULL, 16);
	dest += base_address;

	count = simple_strtoul(argv[3], NULL, 16);

	if (count == 0) {
		puts ("Zero length ???\n");
		return 1;
	}

#ifndef CONFIG_SYS_NO_FLASH
	/* check if we are copying to Flash */
	if ( (addr2info(dest) != NULL)
#ifdef CONFIG_HAS_DATAFLASH
	   && (!addr_dataflash(dest))
#endif
	   ) {
		int rc;

		puts ("Copy to Flash... ");

		rc = flash_write ((char *)addr, dest, count*size);
		if (rc != 0) {
			flash_perror (rc);
			return (1);
		}
		puts ("done\n");
		return 0;
	}
#endif

#ifdef CONFIG_HAS_DATAFLASH
	/* Check if we are copying from RAM or Flash to DataFlash */
	if (addr_dataflash(dest) && !addr_dataflash(addr)){
		int rc;

		puts ("Copy to DataFlash... ");

		rc = write_dataflash (dest, addr, count*size);

		if (rc != 1) {
			dataflash_perror (rc);
			return (1);
		}
		puts ("done\n");
		return 0;
	}

	/* Check if we are copying from DataFlash to RAM */
	if (addr_dataflash(addr) && !addr_dataflash(dest)
#ifndef CONFIG_SYS_NO_FLASH
				 && (addr2info(dest) == NULL)
#endif
	   ){
		int rc;
		rc = read_dataflash(addr, count * size, (char *) dest);
		if (rc != 1) {
			dataflash_perror (rc);
			return (1);
		}
		return 0;
	}

	if (addr_dataflash(addr) && addr_dataflash(dest)){
		puts ("Unsupported combination of source/destination.\n\r");
		return 1;
	}
#endif

#ifdef CONFIG_BLACKFIN
	/* See if we're copying to/from L1 inst */
	if (addr_bfin_on_chip_mem(dest) || addr_bfin_on_chip_mem(addr)) {
		memcpy((void *)dest, (void *)addr, count * size);
		return 0;
	}
#endif

	while (count-- > 0) {
		if (size == 4)
			*((ulong  *)dest) = *((ulong  *)addr);
		else if (size == 2)
			*((ushort *)dest) = *((ushort *)addr);
		else
			*((u_char *)dest) = *((u_char *)addr);
		addr += size;
		dest += size;

		/* reset watchdog from time to time */
		if ((count % (64 << 10)) == 0)
			WATCHDOG_RESET();
	}
	return 0;
}
#endif

#ifdef CONFIG_CMD_BASE
int do_mem_base (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc > 1) {
		/* Set new base address.
		*/
		base_address = simple_strtoul(argv[1], NULL, 16);
	}
	/* Print the current base address.
	*/
	printf("Base Address: 0x%08lx\n", base_address);
	return 0;
}
#endif /* CONFIG_CMD_BASE */

#ifdef CONFIG_CMD_LOOP
int do_mem_loop (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr, length, i;
	int	size;
	volatile uint	*longp;
	volatile ushort *shortp;
	volatile u_char	*cp;

	if (argc < 3)
		return CMD_RET_USAGE;

	/* Check for a size spefication.
	 * Defaults to long if no or incorrect specification.
	 */
	if ((size = cmd_get_data_size(argv[0], 4)) < 0)
		return 1;

	/* Address is always specified.
	*/
	addr = simple_strtoul(argv[1], NULL, 16);

	/* Length is the number of objects, not number of bytes.
	*/
	length = simple_strtoul(argv[2], NULL, 16);

	/* We want to optimize the loops to run as fast as possible.
	 * If we have only one object, just run infinite loops.
	 */
	if (length == 1) {
		if (size == 4) {
			longp = (uint *)addr;
			for (;;)
				i = *longp;
		}
		if (size == 2) {
			shortp = (ushort *)addr;
			for (;;)
				i = *shortp;
		}
		cp = (u_char *)addr;
		for (;;)
			i = *cp;
	}

	if (size == 4) {
		for (;;) {
			longp = (uint *)addr;
			i = length;
			while (i-- > 0)
				*longp++;
		}
	}
	if (size == 2) {
		for (;;) {
			shortp = (ushort *)addr;
			i = length;
			while (i-- > 0)
				*shortp++;
		}
	}
	for (;;) {
		cp = (u_char *)addr;
		i = length;
		while (i-- > 0)
			*cp++;
	}
}
#endif /* CONFIG_CMD_LOOP */

#ifdef CONFIG_LOOPW
int do_mem_loopw (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr, length, i, data;
	int	size;
	volatile uint	*longp;
	volatile ushort *shortp;
	volatile u_char	*cp;

	if (argc < 4)
		return CMD_RET_USAGE;

	/* Check for a size spefication.
	 * Defaults to long if no or incorrect specification.
	 */
	if ((size = cmd_get_data_size(argv[0], 4)) < 0)
		return 1;

	/* Address is always specified.
	*/
	addr = simple_strtoul(argv[1], NULL, 16);

	/* Length is the number of objects, not number of bytes.
	*/
	length = simple_strtoul(argv[2], NULL, 16);

	/* data to write */
	data = simple_strtoul(argv[3], NULL, 16);

	/* We want to optimize the loops to run as fast as possible.
	 * If we have only one object, just run infinite loops.
	 */
	if (length == 1) {
		if (size == 4) {
			longp = (uint *)addr;
			for (;;)
				*longp = data;
					}
		if (size == 2) {
			shortp = (ushort *)addr;
			for (;;)
				*shortp = data;
		}
		cp = (u_char *)addr;
		for (;;)
			*cp = data;
	}

	if (size == 4) {
		for (;;) {
			longp = (uint *)addr;
			i = length;
			while (i-- > 0)
				*longp++ = data;
		}
	}
	if (size == 2) {
		for (;;) {
			shortp = (ushort *)addr;
			i = length;
			while (i-- > 0)
				*shortp++ = data;
		}
	}
	for (;;) {
		cp = (u_char *)addr;
		i = length;
		while (i-- > 0)
			*cp++ = data;
	}
}
#endif /* CONFIG_LOOPW */

void print_cache(unsigned int ori_addr){

   unsigned int cache_value_DatReg0;
   unsigned int cache_value_DatReg1;
   unsigned int addr;

   extern int dcache_status(void);

   smp_printf("mpidr(%d) cache(%d) \n", GET_MPIDR(), dcache_status());
   
   ori_addr &= ~(3<<30); // clean bit 30 and bit 31.

   addr = ori_addr;
   asm(
      "mcr p15, 6, %[t_addr], c15, c4, 0\n\t"
      "mrc p15, 6, r1, c15, c0, 0\n\t"
      "mrc p15, 6, r2, c15, c0, 1\n\t"
      "mov %0, r1\n\t"
      "mov %1, r2" : "=r" (cache_value_DatReg0),  "=r" (cache_value_DatReg1) : [t_addr] "r" (addr)
   );
   smp_printf ("\nc_way0(DataRegister0):%08x\n", cache_value_DatReg0);
   smp_printf ("c_way0(DataRegister1):%08x\n", cache_value_DatReg1);

   addr = ori_addr | 0x40000000;
   asm(
      "mcr p15, 6, %[t_addr], c15, c4, 0\n\t"
      "mrc p15, 6, r1, c15, c0, 0\n\t"
      "mrc p15, 6, r2, c15, c0, 1\n\t"
      "mov %0, r1\n\t"
      "mov %1, r2" : "=r" (cache_value_DatReg0),  "=r" (cache_value_DatReg1) : [t_addr] "r" (addr)
   );
   smp_printf ("c_way1(DataRegister0):%08x\n", cache_value_DatReg0);
   smp_printf ("c_way1(DataRegister1):%08x\n", cache_value_DatReg1);

   addr = ori_addr | 0x80000000;
   asm(
      "mcr p15, 6, %[t_addr], c15, c4, 0\n\t"
      "mrc p15, 6, r1, c15, c0, 0\n\t"
      "mrc p15, 6, r2, c15, c0, 1\n\t"
      "mov %0, r1\n\t"
      "mov %1, r2" : "=r" (cache_value_DatReg0),  "=r" (cache_value_DatReg1) : [t_addr] "r" (addr)
   );
   smp_printf ("c_way2(DataRegister0):%08x\n", cache_value_DatReg0);
   smp_printf ("c_way2(DataRegister1):%08x\n", cache_value_DatReg1);

   addr = ori_addr | 0xC0000000;
   asm(
      "mcr p15, 6, %[t_addr], c15, c4, 0\n\t"
      "mrc p15, 6, r1, c15, c0, 0\n\t"
      "mrc p15, 6, r2, c15, c0, 1\n\t"
      "mov %0, r1\n\t"
      "mov %1, r2" : "=r" (cache_value_DatReg0),  "=r" (cache_value_DatReg1) : [t_addr] "r" (addr)
   );
   smp_printf ("c_way3(DataRegister0):%08x\n", cache_value_DatReg0);
   smp_printf ("c_way3(DataRegister1):%08x\n\n", cache_value_DatReg1);

}

typedef struct _region_s {
	unsigned long start;
	unsigned long end;
} REGION;

/* bypass u-boot context region */
extern unsigned int __image_copy_start, __stack_start, __bss_end__;
	

REGION reserved_region[] =
{
	/* bypass 1.05M context region */
	{0, 0x108000},

	/* bypass u-boot context region */
	{(unsigned long)&__image_copy_start ,  (((unsigned long)&__bss_end__)+0x200000)},
	
#ifdef CONFIG_SYS_SUPPORT_MULTICORE
//#define CONFIG_SYS_INIT_CORE1_SP_ADDR  0x02100000
//#define CONFIG_SYS_INIT_CORE2_SP_ADDR  0x02200000
//#define CONFIG_SYS_INIT_CORE3_SP_ADDR  0x02300000

	/* bypass uboot cpu 1/2/3/4 stack */
	{(unsigned long)CONFIG_SYS_INIT_CORE1_SP_ADDR-0x100000, (unsigned long)CONFIG_SYS_INIT_CORE4_SP_ADDR},
#endif //#ifdef CONFIG_SYS_SUPPORT_MULTICORE

	/* bypass K2L 0x00c00000~0x0f000000 */
	{(unsigned long)0x0b000000 , (unsigned long)0x10000000}, 

	/* bypass uboot non-cache working region, 0x0E800000~0x0E900000 */
	{(unsigned long)0x0e800000, (unsigned long)0x0e900000},

	/* bypass UBOOT_HEAP_RELOCATION_ADDR region, 0x14C00000~0x14C90000 */
	{(unsigned long)0x14c00000, (unsigned long)0x14c90000}, 

	/* bypass 0x16000000 unmapped */
	{(unsigned long)0x16000000, (unsigned long)0x17800000},

	/* bypass K2L 0x18000000~0x180c0000 */
	{(unsigned long)0x18000000, (unsigned long)0x20000000},

	/* bypass rbus region */
	{(unsigned long)0x18000000, (unsigned long)0x18000000+ 0x200000},
	
	/* bypass SMP CORE stack region, 0x1C000000~0x1C006000  */
	{(unsigned long)0x1c000000, (unsigned long)0x1c200000},
	
	/* bypass u-boot heap/stack region */
	/* fixme, I dont know stack buttom, and head top. so... */
	{(((unsigned long)&__stack_start) - 0x4000000 ), (((unsigned long)&__stack_start) + (0x4000000))},
};

#define RESERVED_REGION_SIZE (sizeof(reserved_region)/sizeof(REGION))



/* return a starting point of free space, upper than addr */
unsigned long find_free_start (unsigned long addr)
{
	unsigned long t, f;
        int i;
	
	/* find the minimal right-edge of reserved region whtch is larger than addr. */
	t = addr;
        do {
                f = t;
                for (i=0; i< RESERVED_REGION_SIZE; i++) {
                        unsigned long l = reserved_region[i].start;
                        unsigned long r = reserved_region[i].end;
//		smp_printf("find_free_end(%lx): r %8lx  t %8lx \n", addr, r, t);

                        /* if reserved_region cross over addr, take the !Dr!| side. */
                        if ( (l <= t) && (t < r))
                                t = r;
                }
        } while (f != t);  //if t is picked from a cover region, loop again.

//	smp_printf("find_free_end(%lx):  %8lx \n", addr, t);
	return t;	
}

/* assert addr is within a free space, return end point of free space. */
unsigned long find_free_end (unsigned long addr)
{
	unsigned long t;
	int i;

//	assert(!(t = reserved_to(addr)));
//	smp_printf("find_free_end(%lx): assert %8lx \n", addr, t);

	/* find the minimal right-edge of reserved region whtch is larger than addr. */
	t = ULONG_MAX;
	for (i=0; i< RESERVED_REGION_SIZE; i++) {
		unsigned long l = reserved_region[i].start;
//		smp_printf("find_free_end(%lx): r %8lx  t %8lx \n", addr, r, t);

                 /* take the min of all !Dl!|side of reserved_region that is bigger than addr. */    
		if ( (l > addr) && (l<t))
			t = l;
	}

//	smp_printf("find_free_end(%lx):  %8lx \n", addr, t);
	return t;	
}


unsigned long build_free_region(REGION *f, unsigned long start, unsigned long end)
{
	unsigned long l=start, r=end;
#ifdef CONFIG_SYS_SUPPORT_MULTICORE
	int i=0;
#endif	

	while (start<end) {
		l = find_free_start(start);
		if(l > end)
			break;

		r = find_free_end(l);
		if (r > end)
			r = end;

		f->start = l;
		f->end = r;
#ifdef CONFIG_SYS_SUPPORT_MULTICORE		
		if(cop_mtest_log_en)
		{
			smp_printf("free %d: %8lx %8lx \n", i++, l, r);
		}
#endif		
		f++;

		start = r;
	}

	f->start = 0;
	f->end = 0;
	return 0;
}

int do_build_free (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	REGION free_region[RESERVED_REGION_SIZE+1];


	return build_free_region(free_region, 0, 0x60000000);
}


int march_rDwdrd(unsigned long start,
		 unsigned long end,
		 unsigned int pattern, int compl)
{
	int s1, s2, s3;
	int t1, t2, t3;
	int ret = 0;

	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x, compl(%d)) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern, compl);
	}
	
	assert( start != end);
        
	__asm__ volatile  (
		/* if (start < end) */
		"1: \n\t"
		"cmp %[s], %[e] \n\t"
		"beq 8f \n\t"


		/* march element */
		"cmp %[compl], #0 \n\t"
		"moveq %[s2], %[p]  \n\t"
		"mvnne %[s2], %[p]  \n\t"

		/* read */
		"ldr %[t2], [%[s], #0] \n\t"

		/* validate */
		"cmp %[s2], %[t2] \n\t"
		"bne 9f \n\t"

		/* write inverse march element */
		"mvN %[s2], %[s2]  \n\t"
		"str %[s2],  [%[s], #0] \n\t"

		/* read */
		"ldr %[t2], [%[s], #0] \n\t"

		/* validate */
		"cmp %[s2], %[t2] \n\t"
		"bne 9f \n\t"

		/* repeat next */
		"cmp %[s], %[e] \n\t"
		"addlt %[s], #4 \n\t"
		"subgt %[s], #4 \n\t"

		"addlt %[p], #1 \n\t"
		"subgt %[p], #1 \n\t"
		"b 1b \n\t"

		
		/* pass */
		"8:  \n\t"
		"mov %[ret], #0 \n\t"
		"b 999f \n\t" 

		/* fail */
		"9: \n\t"
		"mov %[ret], #1 \n\t"

		/* end */
		"999: \n\t"
		: [s] "+r" (start), [ret] "=r" (ret),
		  [t1] "=r"(t1), [t2] "=r"(t2), [t3] "=r"(t3), 
		  [s1] "=r"(s1), [s2] "=r"(s2), [s3] "=r"(s3),
		  [e]"+r"(end), [p]"+r"(pattern),
		  [compl] "+r" (compl)
		:
		:
		);

	if (ret == 0)
		return 0;
	

#if (CONFIG_FT == 0)
	smp_printf ("%x %s @ 0x%08X:"
                    "found %08lX, expected %08lX \n",
                    get_MPIDR(), __FUNCTION__, (uint)(uintptr_t)start,
                    (unsigned long)t2, (unsigned long)s2);

	//print_cache(start);
#endif
	
	return start;
}

int march_rDwd(unsigned long start,
	       unsigned long end,
	       unsigned int pattern, int compl)
{
	int s2;
	int t2;
	int ret = 0;

	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x, compl(%d)) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern, compl);
	}
	
	assert( start != end);
		

	__asm__ volatile  (
		/* if (start < end) */
		"1: \n\t"
		"cmp %[s], %[e] \n\t"
		"beq 8f \n\t"


		/* march element */
		"cmp %[compl], #0 \n\t"
		"moveq %[s2], %[p]  \n\t"
		"mvnne %[s2], %[p]  \n\t"

		/* read */
		"ldr %[t2], [%[s], #0] \n\t"

		/* validate */
		"cmp %[s2], %[t2] \n\t"
		"bne 9f \n\t"

		/* write inverse march element */
		"mvN %[s2], %[s2]  \n\t"
		"str %[s2],  [%[s], #0] \n\t"

		/* repeat next */
		"cmp %[s], %[e] \n\t"
		"addlt %[s], #4 \n\t"
		"subgt %[s], #4 \n\t"

		"addlt %[p], #1 \n\t"
		"subgt %[p], #1 \n\t"
		"b 1b \n\t"

		/* pass */
		"8:  \n\t"
		"mov %[ret], #0 \n\t"
		"b 999f \n\t" 

		/* fail */
		"9: \n\t"
		"mov %[ret], #1 \n\t"

		/* end */
		"999: \n\t"
		: [s] "+r" (start), [ret] "=r" (ret),
		  [t2] "=r"(t2), 
		  [s2] "=r"(s2),
		  [e]"+r"(end), [p]"+r"(pattern),
		  [compl] "+r" (compl)
		:
		:
		);

	if (ret == 0)
		return 0;
	

#if (CONFIG_FT == 0)
	smp_printf ("%x %s @ 0x%08X:"
		"found %08lX, expected %08lX \n",
                    get_MPIDR(), __FUNCTION__, (uint)(uintptr_t)start,
		(unsigned long)t2, (unsigned long)s2);

	//print_cache(start);
#endif
	
	return start;
}

int march_rD(unsigned long start,
	     unsigned long end,
	     unsigned int pattern, int compl)
{
	int s2;
	int t2;
	int ret = 0;

	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x, compl(%d)) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern, compl);
	}
	
	assert( start != end);
	

	__asm__ volatile  (
		/* if (start < end) */
		"1: \n\t"
		"cmp %[s], %[e] \n\t"
		"beq 8f \n\t"

		/* march element */
		"cmp %[compl], #0 \n\t"
		"moveq %[s2], %[p]  \n\t"
		"mvnne %[s2], %[p]  \n\t"

		/* read */
		"ldr %[t2], [%[s], #0] \n\t"

		/* validate */
		"cmp %[s2], %[t2] \n\t"
		"bne 9f \n\t"

		/* repeat next */
		"cmp %[s], %[e] \n\t"
		"addlt %[s], #4 \n\t"
		"subgt %[s], #4 \n\t"

		"addlt %[p], #1 \n\t"
		"subgt %[p], #1 \n\t"
		"b 1b \n\t"

		/* pass */
		"8:  \n\t"
		"mov %[ret], #0 \n\t"
		"b 999f \n\t" 

		/* fail */
		"9: \n\t"
		"mov %[ret], #1 \n\t"

		/* end */
		"999: \n\t"
		: [s] "+r" (start), [ret] "=r" (ret),
		  [t2] "=r"(t2),
		  [s2] "=r"(s2),
		  [e]"+r"(end), [p]"+r"(pattern),
		  [compl] "+r" (compl)
		:
		: 
		);

	if (ret == 0)
		return 0;
	

#if (CONFIG_FT == 0)
	smp_printf ("%x %s @ 0x%08X:"
		"found %08lX, expected %08lX \n",
                    get_MPIDR(), __FUNCTION__, (uint)(uintptr_t)start,
		(unsigned long)t2, (unsigned long)s2);

	//print_cache(start);
#endif
	
	return start;
}


int march_wD(unsigned long start,
	     unsigned long end,
	     unsigned int pattern, int compl)
{
	int s2;
	int t2;
	int ret = 0;
	
	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x, compl(%d)) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern, compl);
	}
	
	assert( start != end);
	

	__asm__ volatile  (
		/* if (start < end) */
		"1: \n\t"
		"cmp %[s], %[e] \n\t"
		"beq 8f \n\t"

		/* march element */
		"cmp %[compl], #0 \n\t"
		"moveq %[s2], %[p]  \n\t"
		"mvnne %[s2], %[p]  \n\t"

		/* write */
		"str %[s2],  [%[s], #0] \n\t"

		/* repeat next */
		"cmp %[s], %[e] \n\t"
		"addlt %[s], #4 \n\t"
		"subgt %[s], #4 \n\t"

		"addlt %[p], #1 \n\t"
		"subgt %[p], #1 \n\t"
		"b 1b \n\t"


		/* pass */
		"8:  \n\t"
		"mov %[ret], #0 \n\t"
		"b 999f \n\t" 

		/* fail */
		"9: \n\t"
		"mov %[ret], #1 \n\t"

		/* end */
		"999: \n\t"
		: [s] "+r" (start), [ret] "=r" (ret),
		  [t2] "=r"(t2),  
		  [s2] "=r"(s2),
		  [e]"+r"(end), [p]"+r"(pattern),
		  [compl] "+r" (compl)
		:
		:
		);

	if (ret == 0)
		return 0;
	

#if (CONFIG_FT == 0)
	smp_printf ("%x %s @ 0x%08X:"
		"found %08lX, expected %08lX \n",
                    get_MPIDR(), __FUNCTION__, (uint)(uintptr_t)start,
		(unsigned long)t2, (unsigned long)s2);

	//print_cache(start);
#endif
	
	return start;
}

#define FORWARD(from, to, cur, pattern)		\
	do {					\
		int step;			\
		step = (cur) - (from);		\
		from += step;			\
		pattern += step/4;	\
	} while(0)

#define BACKWARD(from, to, cur, pattern)	\
	do {					\
		int step;			\
		step = (from) - (cur);		\
		from -= step;			\
		pattern -= step/4;		\
	} while(0)




/* march C- : x{w0}, u{r0, w1}, u{r1, w0}, d{r0, w1}, d{r1, w0}, x{r0} 
 */
int march_Cm(unsigned long start,
		  unsigned long end,
		  unsigned int pattern)
{

	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern);
	}
	
	if (start  == 0)
		start++;
	
#if (CONFIG_FT != 0)
	int ret;
	if ((ret = march_wD(start, end, pattern, 0)) ||
	    (ret = march_rDwd(start, end, pattern, 0)) ||
	    (ret = march_rDwd(start, end, pattern, 1)) ||
	    (ret = march_rDwd(end-1, start-1, pattern+(end-start), 0)) ||
	    (ret = march_rDwd(end-1, start-1, pattern+(end-start), 1)) ||
	    (ret = march_rD(start, end, pattern, 0)))
		return ret;
		
	return 0;	
#else

	int errs = 0;
	unsigned int s, e, p, ret;
	

	s = start;
	e = end;
	p = pattern;
	while ((ret = march_wD(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs ++;
	}

		
	s = start;
	e = end;
	p = pattern;
	while ((ret = march_rDwd(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs ++;
	}


	s = start;
	e = end;
	p = pattern;
	while ((ret = march_rDwd(s, e, p, 1))) {
		FORWARD(s, e, ret+4, p);
		errs++;
	}

	s = start-4;
	e = end-4;
	p = pattern + (e-s)/4 -1;
	while((ret = march_rDwd(e, s, p, 0))) {
		BACKWARD(e, s, ret - 4, p);
		errs++;
	}

	s = start-4;
	e = end-4;
	p = pattern + (e-s)/4 -1;
	while ((ret = march_rDwd(e, s, p, 1))) {
		BACKWARD(e, s, ret-4, p);
		errs++;
	}

	s = start;
	e = end;
	p = pattern;
	while ((ret = march_rD(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs++;
	}

	return errs;
	    
	
#endif
}


/* march Y: x{w0}, u{r0, w1, r1}, d{r1, w0, r0}, x{r0} 
 */
int march_Y(unsigned long start,
	    unsigned long end,
	    unsigned int pattern)
{
	if(cop_mtest_log_en) 
	{
		smp_printf("\n%x %s(%08lx, %08lx, %08x) \n",
                   get_MPIDR(), __FUNCTION__, start, end, pattern);
	}
	
	if (start  == 0)
		start++;

#if (CONFIG_FT != 0)
	int ret;
	if ((ret = march_wD(start, end, pattern, 0)) ||
	    (ret = march_rDwdrd(start, end, pattern, 0)) ||
	    (ret = march_rDwdrd(end-1, start-1, pattern+(end-start), 1)) ||
	    (ret = march_rD(start, end, pattern, 0)))
		return ret;
		
	return 0;	
#else

	int errs = 0;
	unsigned int s, e, p, ret;

	s = start;
	e = end;
	p = pattern;
	while ((ret = march_wD(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs ++;
	}

		
	s = start;
	e = end;
	p = pattern;
	while ((ret = march_rDwdrd(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs ++;
	}


	s = start -4;
	e = end - 4;
	p = pattern + (e -s )/4 -1;
	while ((ret = march_rDwdrd(e, s, p, 1))) {
		BACKWARD(e, s, ret-4, p);
		errs++;
	}

	s = start;
	e = end;
	p = pattern;
	while ((ret = march_rD(s, e, p, 0))) {
		FORWARD(s, e, ret+4, p);
		errs++;
	}

	return errs;
	    
	
#endif
}

#ifdef CONFIG_CMD_COPROC

int mem_dbus_test(unsigned long start,
		  unsigned long end,
		  unsigned int pattern)
{
	int s1, s2, s3;
	int t1, t2, t3;
	int ret = 0;
	
	__asm__ volatile  (
		/* if (start < end) */
		"1: \n\t"
		"cmp %[s], %[e] \n\t"
		"bge pass \n\t"

		/* bypass the reserved space */
/*		"mov r0, %[s] \n\t"
		"bl  reserved_to \n\t"
		"cmp r0, #0 \n\t"
		"beq 2f \n\t"
		"mov %[s], r0 \n\t"
		"b 1b \n\t"
*/
		/* read pre-/post- word */
		"2: \n\t"
		"ldr %[s1], [%[s], #-4] \n\t"
		"ldr %[s3], [%[s], #+4] \n\t"

		/* write */
		"mov %[s2], %[p]  \n\t"
		"str %[s2],  [%[s], #0] \n\t"

		/* read */
		"ldr %[t3], [%[s], #+4] \n\t"
		"ldr %[t2], [%[s], #0] \n\t"
		"ldr %[t1], [%[s], #-4] \n\t"

		/* validate */
		"cmp %[s1], %[t1] \n\t"
		"bne fail \n\t"
		"cmp %[s2], %[t2] \n\t"
		"bne fail \n\t"
		"cmp %[s3], %[t3] \n\t"
		"bne fail \n\t"

		/* write inverse */
		"mvN %[s2], %[s2]  \n\t"
		"str %[s2],  [%[s], #0] \n\t"

		/* read */
		"ldr %[t3], [%[s], #+4] \n\t"
		"ldr %[t2], [%[s], #0] \n\t"
		"ldr %[t1], [%[s], #-4] \n\t"

		/* validate */
		"cmp %[s1], %[t1] \n\t"
		"bne fail \n\t"
		"cmp %[s2], %[t2] \n\t"
		"bne fail \n\t"
		"cmp %[s3], %[t3] \n\t"
		"bne fail \n\t"

		/* repeat next */
		"add %[s], #4 \n\t"
		"add %[p], #1 \n\t"
		"b 1b \n\t"

		/* pass */
		"pass:  \n\t"
		"mov %[ret], #0 \n\t"
		"b 999f \n\t" 

		/* fail */
		"fail: \n\t"
		"mov %[ret], #1 \n\t"

		/* end */
		"999: \n\t"
		: [s] "+r" (start), [ret] "=r" (ret),
		  [t1] "=r"(t1), [t2] "=r"(t2), [t3] "=r"(t3), 
		  [s1] "=r"(s1), [s2] "=r"(s2), [s3] "=r"(s3),
		  [e]"+r"(end), [p]"+r"(pattern)
		:
		: "r0"
		);

	if (ret == 0)
		return 0;
	

#if (CONFIG_FT == 0)
	printf ("\nmem_dbus_test @ 0x%08X:\n"
		"found %08lX, expected %08lX \n"
		"found %08lX, expected %08lX \n"
		"found %08lX, expected %08lX \n",
		(uint)(uintptr_t)start,
		(unsigned long)t1, (unsigned long)s1, (unsigned long)t2,
                (unsigned long)s2, (unsigned long)t3, (unsigned long)s3);

	//print_cache(start);
#endif
	
	return start;
}

int mem_test(vu_long *start, vu_long *end, int iteration_limit, vu_long pattern)
{
	vu_long	*addr __attribute__((unused));
	ulong	val __attribute__((unused));
	ulong	errs = 0;
	int iterations;

#if defined(CONFIG_SYS_ALT_MEMTEST)
	vu_long len;
	vu_long offset;
	vu_long test_offset;
	vu_long temp;
	vu_long anti_pattern;
	vu_long num_words;
#if defined(CONFIG_SYS_MEMTEST_SCRATCH)
	vu_long *dummy = (vu_long*)CONFIG_SYS_MEMTEST_SCRATCH;
#else
	vu_long *dummy = 0; /* yes, this is address 0x0, not NULL */
#endif
	int j;

	static const ulong bitpattern[] = {
		0x00000001, /* single bit */
		0x00000003, /* two adjacent bits */
		0x00000007, /* three adjacent bits */
		0x0000000F, /* four adjacent bits */
		0x00000005, /* two non-adjacent bits */
		0x00000015, /* three non-adjacent bits */
		0x00000055, /* four non-adjacent bits */
		0xaaaaaaaa, /* alternating 1/0 */
	};
#else
	ulong	incr;
#endif

#if 0

	// for testing
	for (addr = 0x600000; addr<=0x600100; addr ++) {
		smp_printf("addr %x: %x", addr, rtd_inl(addr));
		print_cache(addr);
	}
#endif

	//invalidate_dcache_range(0x600000, 0x620000);
	//invalidate_icache_all();

	//flush_dcache_all();
	//invalidate_dcache_all();

#if defined(CONFIG_SYS_ALT_MEMTEST)
	smp_printf ("Testing %08x ... %08x:\n", (uint)start, (uint)end);
	debug("%s:%d: start 0x%p end 0x%p\n",
		__FUNCTION__, __LINE__, start, end);

	for (iterations=0;;iterations++) {
		if (ctrlc()) {
			putc ('\n');
			return 1;
		}


		if ((iteration_limit && !(iterations < iteration_limit))
			|| (((!iteration_limit) && (ctrlc())))) // Press ctrl-c to escape repeat mode
			smp_printf("Tested %d iteration(s) with %lu errors.\n",
				iterations, errs);
			return errs != 0;
		}

		smp_printf("Iteration: %6d\r", iterations);
		debug("\n");


		/*
		 * Data line test: write a pattern to the first
		 * location, write the 1's complement to a 'parking'
		 * address (changes the state of the data bus so a
		 * floating bus doen't give a false OK), and then
		 * read the value back. Note that we read it back
		 * into a variable because the next time we read it,
		 * it might be right (been there, tough to explain to
		 * the quality guys why it prints a failure when the
		 * "is" and "should be" are obviously the same in the
		 * error message).
		 *
		 * Rather than exhaustively testing, we test some
		 * patterns by shifting '1' bits through a field of
		 * '0's and '0' bits through a field of '1's (i.e.
		 * pattern and ~pattern).
		 */
		addr = start;
		for (j = 0; j < sizeof(bitpattern)/sizeof(bitpattern[0]); j++) {
			val = bitpattern[j];
			for(; val != 0; val <<= 1) {
			*addr  = val;
			*dummy	= ~val; /* clear the test data off of the bus */
			readback = *addr;
			if(readback != val) {
				smp_printf ("FAILURE (data line): "
				"expected %08lx, actual %08lx\n",
					  val, readback);
				errs++;
				if (ctrlc()) {
				putc ('\n');
				return 1;
				}
			}
			*addr  = ~val;
			*dummy	= val;
			readback = *addr;
			if(readback != ~val) {
				smp_printf ("FAILURE (data line): "
				"Is %08lx, should be %08lx\n",
					readback, ~val);
				errs++;
				if (ctrlc()) {
				putc ('\n');
				return 1;
				}
			}
			}
		}

		/*
		 * Based on code whose Original Author and Copyright
		 * information follows: Copyright (c) 1998 by Michael
		 * Barr. This software is placed into the public
		 * domain and may be used for any purpose. However,
		 * this notice must not be changed or removed and no
		 * warranty is either expressed or implied by its
		 * publication or distribution.
		 */

		/*
		 * Address line test
		 *
		 * Description: Test the address bus wiring in a
		 *				memory region by performing a walking
		 *				1's test on the relevant bits of the
		 *				address and checking for aliasing.
		 *				This test will find single-bit
		 *				address failures such as stuck -high,
		 *				stuck-low, and shorted pins. The base
		 *				address and size of the region are
		 *				selected by the caller.
		 *
		 * Notes:	For best results, the selected base
		 *				address should have enough LSB 0's to
		 *				guarantee single address bit changes.
		 *				For example, to test a 64-Kbyte
		 *				region, select a base address on a
		 *				64-Kbyte boundary. Also, select the
		 *				region size as a power-of-two if at
		 *				all possible.
		 *
		 * Returns: 	0 if the test succeeds, 1 if the test fails.
		 */
		len = ((ulong)end - (ulong)start)/sizeof(vu_long);
		pattern = (vu_long) 0xaaaaaaaa;
		anti_pattern = (vu_long) 0x55555555;

		debug("%s:%d: length = 0x%.8lx\n",
			__FUNCTION__, __LINE__,
			len);
		/*
		 * Write the default pattern at each of the
		 * power-of-two offsets.
		 */
		for (offset = 1; offset < len; offset <<= 1) {
			start[offset] = pattern;
		}

		/*
		 * Check for address bits stuck high.
		 */
		test_offset = 0;
		start[test_offset] = anti_pattern;

		for (offset = 1; offset < len; offset <<= 1) {
			temp = start[offset];
			if (temp != pattern) {
			smp_printf ("\nFAILURE: Address bit stuck high @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx\n",
				(ulong)&start[offset], pattern, temp);
			errs++;
			if (ctrlc()) {
				putc ('\n');
				return 1;
			}
			}
		}
		start[test_offset] = pattern;
		WATCHDOG_RESET();

		/*
		 * Check for addr bits stuck low or shorted.
		 */
		for (test_offset = 1; test_offset < len; test_offset <<= 1) {
			start[test_offset] = anti_pattern;

			for (offset = 1; offset < len; offset <<= 1) {
			temp = start[offset];
			if ((temp != pattern) && (offset != test_offset)) {
				smp_printf ("\nFAILURE: Address bit stuck low or shorted @"
				" 0x%.8lx: expected 0x%.8lx, actual 0x%.8lx\n",
				(ulong)&start[offset], pattern, temp);
				errs++;
				if (ctrlc()) {
				putc ('\n');
				return 1;
				}
			}
			}
			start[test_offset] = pattern;
		}

		/*
		 * Description: Test the integrity of a physical
		 *		memory device by performing an
		 *		increment/decrement test over the
		 *		entire region. In the process every
		 *		storage bit in the device is tested
		 *		as a zero and a one. The base address
		 *		and the size of the region are
		 *		selected by the caller.
		 *
		 * Returns: 	0 if the test succeeds, 1 if the test fails.
		 */
		num_words = ((ulong)end - (ulong)start)/sizeof(vu_long) + 1;

		/*
		 * Fill memory with a known pattern.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
			WATCHDOG_RESET();
			start[offset] = pattern;
		}

		/*
		 * Check each location and invert it for the second pass.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
			WATCHDOG_RESET();
			temp = start[offset];
			if (temp != pattern) {
			smp_printf ("\nFAILURE (read/write) @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx)\n",
				(ulong)&start[offset], pattern, temp);
			errs++;
			if (ctrlc()) {
				putc ('\n');
				return 1;
			}
			}

			anti_pattern = ~pattern;
			start[offset] = anti_pattern;
		}

		/*
		 * Check each location for the inverted pattern and zero it.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
			WATCHDOG_RESET();
			anti_pattern = ~pattern;
			temp = start[offset];
			if (temp != anti_pattern) {
			smp_printf ("\nFAILURE (read/write): @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx)\n",
				(ulong)&start[offset], anti_pattern, temp);
			errs++;
			if (ctrlc()) {
				putc ('\n');
				return 1;
			}
			}
			start[offset] = 0;
		}
	}

#else /* The original, quickie test */
	incr = 1;

	if ((GET_MPIDR() == 0) && (!iteration_limit) && cop_mtest_log_en)
		smp_printf("\n Repeat Mode \n");

	for (iterations = 0;;++iterations) {
		if ((iteration_limit && !(iterations < iteration_limit))
			|| (((!iteration_limit) && (ctrlc())))) // Press ctrl-c to escape repeat mode
		{
			if (((GET_MPIDR() == 0) && !(GET_MPIDR() & 0x100) && cop_mtest_log_en)
				|| (!iteration_limit)) // repeat mode shows interations
				smp_printf("%x Tested %d iteration(s) with %lu errors.\n",
                                           get_MPIDR(), iterations, errs);
			return (errs != 0);
		}

		if ((GET_MPIDR() == 0) && cop_mtest_log_en) smp_printf("\n%d \n", iterations);
		
		if (((GET_MPIDR() == 0) && !(GET_MPIDR() & 0x100)) && ctrlc()) {
			putc ('\n');
			return 1;
		}

		{
			REGION free_region[RESERVED_REGION_SIZE+1];
			REGION *f = (REGION *)free_region;

			build_free_region(free_region, (unsigned long)start, (unsigned long)end);
			
			if((f->end == 0) && (f->start == 0) && (!iterations)) // no free region
			{
				smp_printf("No Free Region for mtest (0x%x)\n", GET_MPIDR());
				return 0; 
			}
#if 1
			while (f->end) {
				errs += (march_Cm(f->start, f->end, pattern));
				
				f++;
			}

			f = (REGION *)free_region;
			while (f->end) {
				errs += (march_Y(f->start, f->end, pattern));
				
				f++;
			}
#else			
			while (f->end) {
				addr = f->start;

				smp_printf ("iter(%d), mem_dbus_test(%8lx, %8lx, %8lx) \n", iterations, addr, f->end, pattern);
				
				while (addr = mem_dbus_test(addr, f->end, pattern)) {
#if (CONFIG_FT == 0)
					if (addr < end) {
						addr++;
						errs++;
					}
				
					if (ctrlc()) {
						putc ('\n');
						break;
					}
#else
					return (errs != 0);
#endif				
				}
				
				f++;
			}
#endif			
			
		}

		/*
		 * Flip the pattern each time to make lots of zeros and
		 * then, the next time, lots of ones.  We decrement
		 * the "negative" patterns and increment the "positive"
		 * patterns to preserve this feature.
		 */
		if(pattern & 0x80000000) {
			pattern = -pattern; /* complement & increment */
		}
		else {
			pattern = ~pattern;
		}
		incr = -incr;
	}
#endif
	return 0;	/* not reached */
}
#endif	//CONFIG_CMD_COPROC

/*
 * Perform a memory test. A more complete alternative test can be
 * configured using CONFIG_SYS_ALT_MEMTEST. The complete test loops until
 * interrupted by ctrl-c or by a failure of one of the sub-tests.
 */
int do_mem_mtest (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	vu_long	*addr, *start, *end;
	ulong	val;
	ulong	readback;
	ulong	errs = 0;
	int iterations = 1;
	int iteration_limit;

#if defined(CONFIG_SYS_ALT_MEMTEST)
	vu_long	len;
	vu_long	offset;
	vu_long	test_offset;
	vu_long	pattern;
	vu_long	temp;
	vu_long	anti_pattern;
	vu_long	num_words;
#if defined(CONFIG_SYS_MEMTEST_SCRATCH)
	vu_long *dummy = (vu_long*)CONFIG_SYS_MEMTEST_SCRATCH;
#else
	vu_long *dummy = 0;	/* yes, this is address 0x0, not NULL */
#endif
	int	j;

	static const ulong bitpattern[] = {
		0x00000001,	/* single bit */
		0x00000003,	/* two adjacent bits */
		0x00000007,	/* three adjacent bits */
		0x0000000F,	/* four adjacent bits */
		0x00000005,	/* two non-adjacent bits */
		0x00000015,	/* three non-adjacent bits */
		0x00000055,	/* four non-adjacent bits */
		0xaaaaaaaa,	/* alternating 1/0 */
	};
#else
	ulong	incr;
	ulong	pattern;
#endif

	if (argc == 1) // print usage
		return CMD_RET_USAGE;

	if (argc > 1)
		start = (ulong *)simple_strtoul(argv[1], NULL, 16);
	else
		start = (ulong *)CONFIG_SYS_MEMTEST_START;

	if (argc > 2)
		end = (ulong *)simple_strtoul(argv[2], NULL, 16);
	else
		end = (ulong *)(CONFIG_SYS_MEMTEST_END);

	if (argc > 3)
		pattern = (ulong)simple_strtoul(argv[3], NULL, 16);
	else
		pattern = 0;

	if (argc > 4)
		iteration_limit = (ulong)simple_strtoul(argv[4], NULL, 16);
	else
		iteration_limit = 0;

#if defined(CONFIG_SYS_ALT_MEMTEST)
	smp_printf ("Testing %08x ... %08x:\n", (uint)start, (uint)end);
	debug("%s:%d: start 0x%p end 0x%p\n",
		__FUNCTION__, __LINE__, start, end);

	for (;;) {
		if (ctrlc()) {
			putc ('\n');
			return 1;
		}


		if (iteration_limit && iterations > iteration_limit) {
			smp_printf("Tested %d iteration(s) with %lu errors.\n",
				iterations-1, errs);
			return errs != 0;
		}

		smp_printf("Iteration: %6d\r", iterations);
		debug("\n");
		iterations++;

		/*
		 * Data line test: write a pattern to the first
		 * location, write the 1's complement to a 'parking'
		 * address (changes the state of the data bus so a
		 * floating bus doen't give a false OK), and then
		 * read the value back. Note that we read it back
		 * into a variable because the next time we read it,
		 * it might be right (been there, tough to explain to
		 * the quality guys why it prints a failure when the
		 * "is" and "should be" are obviously the same in the
		 * error message).
		 *
		 * Rather than exhaustively testing, we test some
		 * patterns by shifting '1' bits through a field of
		 * '0's and '0' bits through a field of '1's (i.e.
		 * pattern and ~pattern).
		 */
		addr = start;
		for (j = 0; j < sizeof(bitpattern)/sizeof(bitpattern[0]); j++) {
		    val = bitpattern[j];
		    for(; val != 0; val <<= 1) {
			*addr  = val;
			*dummy  = ~val; /* clear the test data off of the bus */
			readback = *addr;
			if(readback != val) {
			    smp_printf ("FAILURE (data line): "
				"expected %08lx, actual %08lx\n",
					  val, readback);
				print_cache(addr);
			    errs++;
			    if (ctrlc()) {
				putc ('\n');
				return 1;
			    }
			}
			*addr  = ~val;
			*dummy  = val;
			readback = *addr;
			if(readback != ~val) {
			    smp_printf ("FAILURE (data line): "
				"Is %08lx, should be %08lx\n",
					readback, ~val);
				print_cache(addr);
			    errs++;
			    if (ctrlc()) {
				putc ('\n');
				return 1;
			    }
			}
		    }
		}

		/*
		 * Based on code whose Original Author and Copyright
		 * information follows: Copyright (c) 1998 by Michael
		 * Barr. This software is placed into the public
		 * domain and may be used for any purpose. However,
		 * this notice must not be changed or removed and no
		 * warranty is either expressed or implied by its
		 * publication or distribution.
		 */

		/*
		 * Address line test
		 *
		 * Description: Test the address bus wiring in a
		 *              memory region by performing a walking
		 *              1's test on the relevant bits of the
		 *              address and checking for aliasing.
		 *              This test will find single-bit
		 *              address failures such as stuck -high,
		 *              stuck-low, and shorted pins. The base
		 *              address and size of the region are
		 *              selected by the caller.
		 *
		 * Notes:	For best results, the selected base
		 *              address should have enough LSB 0's to
		 *              guarantee single address bit changes.
		 *              For example, to test a 64-Kbyte
		 *              region, select a base address on a
		 *              64-Kbyte boundary. Also, select the
		 *              region size as a power-of-two if at
		 *              all possible.
		 *
		 * Returns:     0 if the test succeeds, 1 if the test fails.
		 */
		len = ((ulong)end - (ulong)start)/sizeof(vu_long);
		pattern = (vu_long) 0xaaaaaaaa;
		anti_pattern = (vu_long) 0x55555555;

		debug("%s:%d: length = 0x%.8lx\n",
			__FUNCTION__, __LINE__,
			len);
		/*
		 * Write the default pattern at each of the
		 * power-of-two offsets.
		 */
		for (offset = 1; offset < len; offset <<= 1) {
			start[offset] = pattern;
		}

		/*
		 * Check for address bits stuck high.
		 */
		test_offset = 0;
		start[test_offset] = anti_pattern;

		for (offset = 1; offset < len; offset <<= 1) {
		    temp = start[offset];
		    if (temp != pattern) {
			smp_printf ("\nFAILURE: Address bit stuck high @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx\n",
				(ulong)&start[offset], pattern, temp);
			print_cache(&start[offset]);
			errs++;
			if (ctrlc()) {
			    putc ('\n');
			    return 1;
			}
		    }
		}
		start[test_offset] = pattern;
		WATCHDOG_RESET();

		/*
		 * Check for addr bits stuck low or shorted.
		 */
		for (test_offset = 1; test_offset < len; test_offset <<= 1) {
		    start[test_offset] = anti_pattern;

		    for (offset = 1; offset < len; offset <<= 1) {
			temp = start[offset];
			if ((temp != pattern) && (offset != test_offset)) {
			    smp_printf ("\nFAILURE: Address bit stuck low or shorted @"
				" 0x%.8lx: expected 0x%.8lx, actual 0x%.8lx\n",
				(ulong)&start[offset], pattern, temp);
				print_cache(&start[offset]);
			    errs++;
			    if (ctrlc()) {
				putc ('\n');
				return 1;
			    }
			}
		    }
		    start[test_offset] = pattern;
		}

		/*
		 * Description: Test the integrity of a physical
		 *		memory device by performing an
		 *		increment/decrement test over the
		 *		entire region. In the process every
		 *		storage bit in the device is tested
		 *		as a zero and a one. The base address
		 *		and the size of the region are
		 *		selected by the caller.
		 *
		 * Returns:     0 if the test succeeds, 1 if the test fails.
		 */
		num_words = ((ulong)end - (ulong)start)/sizeof(vu_long) + 1;

		/*
		 * Fill memory with a known pattern.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
			WATCHDOG_RESET();
			start[offset] = pattern;
		}

		/*
		 * Check each location and invert it for the second pass.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
		    WATCHDOG_RESET();
		    temp = start[offset];
		    if (temp != pattern) {
			smp_printf ("\nFAILURE (read/write) @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx)\n",
				(ulong)&start[offset], pattern, temp);
			print_cache(&start[offset]);
			errs++;
			if (ctrlc()) {
			    putc ('\n');
			    return 1;
			}
		    }

		    anti_pattern = ~pattern;
		    start[offset] = anti_pattern;
		}

		/*
		 * Check each location for the inverted pattern and zero it.
		 */
		for (pattern = 1, offset = 0; offset < num_words; pattern++, offset++) {
		    WATCHDOG_RESET();
		    anti_pattern = ~pattern;
		    temp = start[offset];
		    if (temp != anti_pattern) {
			smp_printf ("\nFAILURE (read/write): @ 0x%.8lx:"
				" expected 0x%.8lx, actual 0x%.8lx)\n",
				(ulong)&start[offset], anti_pattern, temp);
			print_cache(&start[offset]);
			errs++;
			if (ctrlc()) {
			    putc ('\n');
			    return 1;
			}
		    }
		    start[offset] = 0;
		}
	}

#else /* The original, quickie test */
	incr = 1;
	smp_printf("Start address: 0x%08X.\n  End address: 0x%08X.\n", (uint)start, (uint)end);
	for (;;) {
		if (ctrlc()) {
			putc ('\n');
			return 1;
		}

		if (iteration_limit && iterations > iteration_limit) {
			smp_printf("Tested %d iteration(s) with %lu errors.\n",
				iterations-1, errs);
			return errs != 0;
		}
		++iterations;

		smp_printf ("\rPattern %08lX  Writing..."
			"%12s"
			"\b\b\b\b\b\b\b\b\b\b",
			pattern, "");

		for (addr=start,val=pattern; addr<end; addr++) {
			WATCHDOG_RESET();
			*addr = val;
			val  += incr;
			//for debug
			//if (((uint)addr&(0x10000-1)) == 0)
			//	smp_printf("address: 0x%08X.\n",  (uint)addr);
		}

		puts ("Reading...");

		for (addr=start,val=pattern; addr<end; addr++) {
			WATCHDOG_RESET();
			readback = *addr;
			if (readback != val) {
				smp_printf ("\nMem error @ 0x%08X: "
					"found %08lX, expected %08lX\n",
					(uint)(uintptr_t)addr, readback, val);
				print_cache((unsigned int)addr);
				errs++;
				if (ctrlc()) {
					putc ('\n');
					return 1;
				}
			}
			val += incr;
		}

		/*
		 * Flip the pattern each time to make lots of zeros and
		 * then, the next time, lots of ones.  We decrement
		 * the "negative" patterns and increment the "positive"
		 * patterns to preserve this feature.
		 */
		if(pattern & 0x80000000) {
			pattern = -pattern;	/* complement & increment */
		}
		else {
			pattern = ~pattern;
		}
		incr = -incr;
		
		smp_printf ("iteration %d\n", iterations);
	}
#endif
	return 0;	/* not reached */
}

#if defined(CONFIG_CMD_MM) || defined(CONFIG_CMD_NM)
/* Modify memory.
 *
 * Syntax:
 *	mm{.b, .w, .l} {addr}
 *	nm{.b, .w, .l} {addr}
 */
static int
mod_mem(cmd_tbl_t *cmdtp, int incrflag, int flag, int argc, char * const argv[])
{
	ulong	addr, i;
	int	nbytes, size;

	if (argc != 2)
		return CMD_RET_USAGE;

#ifdef CONFIG_BOOT_RETRY_TIME
	reset_cmd_timeout();	/* got a good command to get here */
#endif
	/* We use the last specified parameters, unless new ones are
	 * entered.
	 */
	addr = mm_last_addr;
	size = mm_last_size;

	if ((flag & CMD_FLAG_REPEAT) == 0) {
		/* New command specified.  Check for a size specification.
		 * Defaults to long if no or incorrect specification.
		 */
		if ((size = cmd_get_data_size(argv[0], 4)) < 0)
			return 1;

		/* Address is specified since argc > 1
		*/
		addr = simple_strtoul(argv[1], NULL, 16);
		addr += base_address;
	}

#ifdef CONFIG_HAS_DATAFLASH
	if (addr_dataflash(addr)){
		puts ("Can't modify DataFlash in place. Use cp instead.\n\r");
		return 0;
	}
#endif

#ifdef CONFIG_BLACKFIN
	if (addr_bfin_on_chip_mem(addr)) {
		puts ("Can't modify L1 instruction in place. Use cp instead.\n\r");
		return 0;
	}
#endif

	/* Print the address, followed by value.  Then accept input for
	 * the next value.  A non-converted value exits.
	 */
	do {
		printf("%08lx:", addr);
		if (size == 4)
			printf(" %08x", *((uint   *)addr));
		else if (size == 2)
			printf(" %04x", *((ushort *)addr));
		else
			printf(" %02x", *((u_char *)addr));

		nbytes = readline (" ? ");
		if (nbytes == 0 || (nbytes == 1 && console_buffer[0] == '-')) {
			/* <CR> pressed as only input, don't modify current
			 * location and move to next. "-" pressed will go back.
			 */
			if (incrflag)
				addr += nbytes ? -size : size;
			nbytes = 1;
#ifdef CONFIG_BOOT_RETRY_TIME
			reset_cmd_timeout(); /* good enough to not time out */
#endif
		}
#ifdef CONFIG_BOOT_RETRY_TIME
		else if (nbytes == -2) {
			break;	/* timed out, exit the command	*/
		}
#endif
		else {
			char *endp;
			i = simple_strtoul(console_buffer, &endp, 16);
			nbytes = endp - console_buffer;
			if (nbytes) {
#ifdef CONFIG_BOOT_RETRY_TIME
				/* good enough to not time out
				 */
				reset_cmd_timeout();
#endif
				if (size == 4)
					*((uint   *)addr) = i;
				else if (size == 2)
					*((ushort *)addr) = i;
				else
					*((u_char *)addr) = i;
				if (incrflag)
					addr += size;
			}
		}
	} while (nbytes);

	mm_last_addr = addr;
	mm_last_size = size;
	return 0;
}
#endif

#ifdef CONFIG_CMD_CRC32

#ifndef CONFIG_CRC32_VERIFY

int do_mem_crc (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong addr, length;
	ulong crc;
	ulong *ptr;

	if (argc < 3)
		return CMD_RET_USAGE;

	addr = simple_strtoul (argv[1], NULL, 16);
	addr += base_address;

	length = simple_strtoul (argv[2], NULL, 16);

	crc = crc32_wd (0, (const uchar *) addr, length, CHUNKSZ_CRC32);

	printf ("CRC32 for %08lx ... %08lx ==> %08lx\n",
			addr, addr + length - 1, crc);

	if (argc > 3) {
		ptr = (ulong *) simple_strtoul (argv[3], NULL, 16);
		*ptr = crc;
	}

	return 0;
}

#else	/* CONFIG_CRC32_VERIFY */

int do_mem_crc (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong addr, length;
	ulong crc;
	ulong *ptr;
	ulong vcrc;
	int verify;
	int ac;
	char * const *av;

	if (argc < 3) {
usage:
		return CMD_RET_USAGE;
	}

	av = argv + 1;
	ac = argc - 1;
	if (strcmp(*av, "-v") == 0) {
		verify = 1;
		av++;
		ac--;
		if (ac < 3)
			goto usage;
	} else
		verify = 0;

	addr = simple_strtoul(*av++, NULL, 16);
	addr += base_address;
	length = simple_strtoul(*av++, NULL, 16);

	crc = crc32_wd (0, (const uchar *) addr, length, CHUNKSZ_CRC32);

	if (!verify) {
		printf ("CRC32 for %08lx ... %08lx ==> %08lx\n",
				addr, addr + length - 1, crc);
		if (ac > 2) {
			ptr = (ulong *) simple_strtoul (*av++, NULL, 16);
			*ptr = crc;
		}
	} else {
		vcrc = simple_strtoul(*av++, NULL, 16);
		if (vcrc != crc) {
			printf ("CRC32 for %08lx ... %08lx ==> %08lx != %08lx ** ERROR **\n",
					addr, addr + length - 1, crc, vcrc);
			return 1;
		}
	}

	return 0;

}
#endif	/* CONFIG_CRC32_VERIFY */

#endif

/**************************************************/
U_BOOT_CMD(
	md,	3,	1,	do_mem_md,
	"memory display",
	"[.b, .w, .l] address [# of objects]"
);

#ifdef CONFIG_CMD_MM
U_BOOT_CMD(
	mm,	2,	1,	do_mem_mm,
	"memory modify (auto-incrementing address)",
	"[.b, .w, .l] address"
);
#endif

#ifdef CONFIG_CMD_NM
U_BOOT_CMD(
	nm,	2,	1,	do_mem_nm,
	"memory modify (constant address)",
	"[.b, .w, .l] address"
);
#endif

U_BOOT_CMD(
	mw,	4,	1,	do_mem_mw,
	"memory write (fill)",
	"[.b, .w, .l] address value [count]"
);

#ifdef CONFIG_CMD_CP
U_BOOT_CMD(
	cp,	4,	1,	do_mem_cp,
	"memory copy",
	"[.b, .w, .l] source target count"
);
#endif

#ifdef CONFIG_CMD_CMP
U_BOOT_CMD(
	cmp,	4,	1,	do_mem_cmp,
	"memory compare",
	"[.b, .w, .l] addr1 addr2 count"
);
#endif

#ifdef CONFIG_CMD_CRC32

#ifndef CONFIG_CRC32_VERIFY

U_BOOT_CMD(
	crc32,	4,	1,	do_mem_crc,
	"checksum calculation",
	"address count [addr]\n    - compute CRC32 checksum [save at addr]"
);

#else	/* CONFIG_CRC32_VERIFY */

U_BOOT_CMD(
	crc32,	5,	1,	do_mem_crc,
	"checksum calculation",
	"address count [addr]\n    - compute CRC32 checksum [save at addr]\n"
	"-v address count crc\n    - verify crc of memory area"
);

#endif	/* CONFIG_CRC32_VERIFY */

#endif

#ifdef CONFIG_CMD_BASE
U_BOOT_CMD(
	base,	2,	1,	do_mem_base,
	"print or set address offset",
	"\n    - print address offset for memory commands\n"
	"base off\n    - set address offset for memory commands to 'off'"
);
#endif /* CONFIG_CMD_BASE */

#ifdef CONFIG_CMD_LOOP
U_BOOT_CMD(
	loop,	3,	1,	do_mem_loop,
	"infinite loop on address range",
	"[.b, .w, .l] address number_of_objects"
);
#endif /* CONFIG_CMD_LOOP */

#ifdef CONFIG_LOOPW
U_BOOT_CMD(
	loopw,	4,	1,	do_mem_loopw,
	"infinite write loop on address range",
	"[.b, .w, .l] address number_of_objects data_to_write"
);
#endif /* CONFIG_LOOPW */

U_BOOT_CMD(
	mtest,	5,	1,	do_mem_mtest,
	"simple RAM read/write test",
	"[start [end [pattern [iterations]]]]"
);



U_BOOT_CMD(
	build_free, 5,	1,	do_build_free,
	"build free region table",
	"build_free"
);


#ifdef CONFIG_MX_CYCLIC
U_BOOT_CMD(
	mdc,	4,	1,	do_mem_mdc,
	"memory display cyclic",
	"[.b, .w, .l] address count delay(ms)"
);

U_BOOT_CMD(
	mwc,	4,	1,	do_mem_mwc,
	"memory write cyclic",
	"[.b, .w, .l] address value delay(ms)"
);
#endif /* CONFIG_MX_CYCLIC */
