/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Misc functions
 */
#include <common.h>
#include <command.h>
#ifdef CONFIG_REALTEK_EERPOM_TV010_A
#include "../drivers/eeprom/EEP_TV010_A/eeprom.h"
#endif
#include <drivers/i2c/rtk_i2c_lib.h>
#include <asm-generic/gpio.h>
#include <asm/io.h>
#include <platform_lib/board/gpio.h>
#include <linux/string.h>
DECLARE_GLOBAL_DATA_PTR;

#ifndef name_to_gpio
#define name_to_gpio(name) simple_strtoul(name, NULL, 10)
#endif

enum gpio_cmd {
    GPIO_MISC_GET,
    GPIO_MISC_SET,
    GPIO_ISO_GET,
	GPIO_ISO_SET,
	GPIO_HELP,
};

extern void SHOW_GPIO_HELP(void);
extern void enable_console(void);
extern void disable_console(void);
extern unsigned long strtoul(	const char *str, char **endptr, int base);

#ifdef CONFIG_CMD_SLEEP
int do_sleep (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong start = get_timer(0);
	ulong delay;

	if (argc != 2)
		return CMD_RET_USAGE;

	delay = simple_strtoul(argv[1], NULL, 10) * CONFIG_SYS_HZ;

	while (get_timer(start) < delay) {
		if (ctrlc ())
			return (-1);

		udelay (100);
	}

	return 0;
}

U_BOOT_CMD(
	sleep ,    2,    1,     do_sleep,
	"delay execution for some time",
	"N\n"
	"    - delay execution for N seconds (N is _decimal_ !!!)"
);
#endif

#if 0
int do_gpiotest (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int gpio;
    enum gpio_cmd sub_cmd = 5; 
    ulong value;
	int gpio_on = 0;
    char *str_cmd,*str_type,*str_gpio = NULL;
	str_cmd = argv[1];
	str_type = argv[2];
	if(argc >=4)
        str_gpio = argv[3];

    if(argc >= 5)
        gpio_on = simple_strtoul(argv[4], NULL, 10);
    /* parse the behavior */
    if(strcmp(str_cmd,"mis") == 0)
    {
        if(strcmp(str_type,"g") == 0)
            sub_cmd = GPIO_MISC_GET;
        else if(strcmp(str_type,"s") == 0)
            sub_cmd = GPIO_MISC_SET;
    }
    else if( strcmp(str_cmd,"iso") == 0)
    {
        if(strcmp(str_type,"g") == 0)
            sub_cmd = GPIO_ISO_GET;
        else if(strcmp(str_type,"s") == 0)
            sub_cmd = GPIO_ISO_SET;
    }
    else if( strcmp(str_cmd,"help") == 0)
    {
    	sub_cmd = GPIO_HELP;
    }
    else
		goto show_usage;

	/* turn the gpio name into a gpio number */
    gpio = name_to_gpio(str_gpio);

    if (gpio < 0)
        goto show_usage;

    /* finally, let's do it: set direction and exec command */
        switch (sub_cmd) {
            case GPIO_MISC_GET:    value = getGPIO(gpio); break;
            case GPIO_MISC_SET:    value = setGPIO(gpio,gpio_on); break;
            case GPIO_ISO_GET:     value = getISOGPIO(gpio); break;
            case GPIO_ISO_SET:     value = setISOGPIO(gpio,gpio_on); break;
            case GPIO_HELP:        SHOW_GPIO_HELP(); goto show_info;
            default:          goto show_usage;
        }

    if( argc >4)
    {
        if(value == (-1))
            printf("Set %s gpio:%s on:%i Failed!!\n",str_cmd,str_gpio,gpio_on);
        else if(value == 0)
            printf("Set %s gpio:%s on:%i Success\n",str_cmd,str_gpio,gpio_on);
    }
    else
    {
        if(value == (-1))
            printf("Get %s gpio:%s value:%lu Failed!!\n",str_cmd,str_gpio,value);
        else
            printf("Get %s gpio:%s value:%lu Success\n",str_cmd,str_gpio,value);
    }
		return 0;
 show_usage:
        return CMD_RET_USAGE;
 show_info:
        return 0;
}

U_BOOT_CMD(
        gpio ,    5,    0,     do_gpiotest,
        "GPIO command description/usage",
        "Usage:\n"
        "\tgpio mis g <num> - Get MISC GPIO\n"
		"\tExample: gpio mis g 80\n"
		"\tgpio mis s <num> <value> - Set MISC GPIO\n"
		"\tExample: gpio mis s 80 1\n"
		"\tgpio iso g <num> - Get ISO GPIO\n"
		"\tExample: gpio iso g 10\n"
		"\tgpio iso s <num> <value> - Set ISO GPIO\n"
		"\tExample: gpio iso s 10 1\n"
);
#endif

#if defined(CONFIG_REALTEK_EERPOM_TV010_A)
/************************************************************************
 *  Implementation : Public functions
 ************************************************************************/
static unsigned int mhexstr2int(char *s)
{
	return (unsigned int) strtoul (s, (char **) 0, 16);
}

static UINT32 eep_parse_main(char eep_address_index, char* hex_string)
{
	UINT8 read_buf[32];
	UINT8 hex_buf[3];
	bool isStr = false;
	int i = 0;
	hex_buf[2] = 0;
	memset(read_buf, 0, 32);

	if (hex_string == NULL) {
		switch(eep_address_index) {
			case 'a':
				getMacAddress(read_buf);
				printf("a=0x%02x%02x%02x%02x%02x%02x\n",read_buf[0],read_buf[1],read_buf[2],read_buf[3],read_buf[4], read_buf[5]);
				break;
			case 'b':
				getUartMode(read_buf);
				printf("b=0x%02x\n",read_buf[0]);
				break;
			case 'c':
				setCustomIrCode(0x0E);
				printf("Reset the custom ir code ok.\n");
				break;
		#if defined(Config_Board_TV010_R8)
			case 'd':
				getOledSysLock(read_buf);
				printf("OLED system lock flag =0x%02x\n",read_buf[0]);
				break;
		#endif
			default:
				break;
		}
	}
	else {
		if (hex_string[0] == '0' && hex_string[1] == 'x') {
			printf("input is hex = %s\n", hex_string);
			hex_string = hex_string+2;
			for (i = 0; i < 32 || i < strlen(hex_string); i++) {
				hex_buf[0] = *hex_string;
				hex_buf[1] = *(hex_string+1);
				read_buf[i] = mhexstr2int((char*)hex_buf);
				hex_string = hex_string+2;
			}
		}
		else {
			isStr = true;
			printf("input is string = %s\n", hex_string);
		}

		switch(eep_address_index) {
			case 'a':
				if (!isStr)
					setMacAddress((UINT8*)read_buf);
				break;
			case 'b':
				if (!isStr)
					setUartMode(read_buf[0]);
				break;
			case 'c':
				printf("Please use \"eep c\" to reset the custom ir code.\n");
				break;
		#if defined(Config_Board_TV010_R8)
			case 'd':
				if (!isStr)
					setOledSysLock(read_buf[0]);
				break;
		#endif
			default:
				break;
		}
	}
	return OK;

}

int do_eeprom (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *pch;
	char *pch2;

	if( argc > 3 ) {
		printf("eep [<eep_name>] [<assigned_value>]\n");
	    return OK;
	}

	if( argc == 1 ) {
		printf("eep [<eep_name>] [<assigned_value>]\n");
		printf("<eep_name>\n");
		printf("  a\t EEP_MAC\n");
		printf("  b\t FAC_UART\n");
		printf("  c\t reset ir custome code\n");
	#if defined(Config_Board_TV010_R8)
		printf("  d\t clear oled system lock\n");
	#endif
		printf("\n\n");
		printf("example read: eep a\n");
		printf("example write: eep a 0x001a9a6688ee\n");
		printf("example write: eep b 0x0\n");
	    return OK;
	}

	if( argc >= 2) {
		pch = argv[1];
		if (*pch!='a' && *pch!='b' && *pch!='c'
	#if defined(Config_Board_TV010_R8)
			&& *pch!='d'
	#endif
		) {
	#if defined(Config_Board_TV010_R8)
			printf("ERROR!argv[1] is not a/b/c/d\n");
	#else
			printf("ERROR!argv[1] is not a/b/c\n");
	#endif
			return OK;
		}
	}

	if( argc == 2) {
		if (pch != NULL) {
			eep_parse_main(*pch, NULL);
		}
	}

	if( argc == 3) {
		pch2 = argv[2];
		if (pch != NULL) {
			eep_parse_main(*pch, pch2);
		}
	}
	return OK;
}

U_BOOT_CMD(
        eep, 3, 1,    do_eeprom,
        "eeprom write special area and set value to it",
        " [<eep_name>]  [<assigned_value>]\n"
        "    - just type eep will show usage examples"
);
#endif

#if defined(CONFIG_BOARD_TV005_L4300_RTD2993_EMMC) || defined(CONFIG_BOARD_TV005_L4300_RTD2993_EMMC_V2)
#include <asm/arch/system.h>
int rtk_plat_do_etherfix(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc != 1)
                return CMD_RET_USAGE;

	rtd_outl(0xb801605c,0x841f0a46);// change Page to a46
	mdelay(100);
	rtd_outl(0xb801605c,0x84140003);// set fuse_rdy = 1 , set ext_ini_done = 1, to skip ext_ini
	mdelay(10);
	rtd_outl(0xb801605c,0x04140000);// read fuse_redy
	mdelay(10);
	rtd_inl(0xb801605c);

	rtd_outl(0xb801605c,0x841f0a42);// change Page to a42
	mdelay(100);

	rtd_outl(0xb801605c,0x04100000);// polling rg_phy_status = 1
	mdelay(10);
	rtd_inl(0xb801605c);
	mdelay(10);

	rtd_outl(0xb801605c,0x04010000);// read link status
	mdelay(100);
	rtd_outl(0xb801605c,0x04010000);// read link status
	mdelay(100);
	rtd_inl(0xb801605c);

	printf("Ether Fix for Tv005 L4300 10/100 Ether Phy\n");

        return 0;
}

U_BOOT_CMD(
        etherfix, 1, 0,    rtk_plat_do_etherfix,
	"Ether Fix for Tv005 L4300 10/100 Ether Phy",
	"\n"
        "    - Run Ether Fix for Tv005 L4300 10/100 Ether Phy"
);


#include <platform_lib/board/pcb_mgr.h>
#ifndef IS_FPGA
#include <platform_lib/board/pcb.h>
#endif

#ifdef CONFIG_CUSTOMIZE_FUNC_REGISTER // SoftDD rev0
#include <common.h>
#include <watchdog.h>
int count_pwm_led = 0;
int skip_condition= 32; // 32ms
ulong pwm_led_currTime = 0;
ulong pwm_led_saveTime = 0;

#define LED_DEBUG 0
void tv005_update_led(void)
{
        int i, T_scale=32, X; // 32ms/32000us
        ulong T_up_GR=0x1F, T_down_GR=0x3E, T_max_G=0x3E, T_max_R=0x3E;
        ulong L_min_G=0x00, L_min_R=0x00, L_typ_G=0x34, L_typ_R=0x34, L_max_G=0xFB, L_max_R=0xFB;

        X=(int)(T_scale/skip_condition);
        i=count_pwm_led;
        pwm_led_currTime = get_timer_masked();
        if ( pwm_led_saveTime != 0)
        {
            if ( (pwm_led_currTime - pwm_led_saveTime) < skip_condition )
            {
                #if LED_DEBUG
                printf("The gap is too short... skip !!\n");
                #endif
                goto skip;
            }
            #if LED_DEBUG
            printf("pwm_led_currTime - pwm_led_saveTime = %8u ms\n", pwm_led_currTime - pwm_led_saveTime);
            #endif
        }
        //Stage I
        //for (i=0; i< T_up_GR*X; i++)
        if ( i < T_up_GR*X )
        {
            IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_min_G+(int)((L_max_G-L_min_G)*i/T_up_GR/X)));
            //mdelay((int)(T_scale));
            #if LED_DEBUG
            printf("Stage I: T_up_GR %d (%d)\n", i,  (int)(L_min_G+(int)((L_max_G-L_min_G)*i/T_up_GR/X)));
            #endif
        }
        //Stage II
        // flat area
        else if ( (i >= T_up_GR*X) && (i < (T_up_GR+T_max_G)*X) )
        {
            IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
            //mdelay(T_max_G*T_scale);
            #if LED_DEBUG
            printf("Stage II: T_max_G %d (%d)\n", i, L_max_G);
            #endif
        }
        //Stage III
        //for (i=0; i< T_down_GR*X; i++)
        else if ( (i >= (T_up_GR+T_max_G)*X) && (i < (T_up_GR+T_max_G+T_down_GR)*X) )
        {
            IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_max_G-(int)((L_max_G-L_typ_G)*(i-((T_up_GR+T_max_G)*X))/T_down_GR/X)));
            //mdelay((int)(T_scale));
            #if LED_DEBUG
            printf("Stage III: T_max_G %d (%d)\n", i, (int)(L_max_G-(int)((L_max_G-L_typ_G)*(i-((T_up_GR+T_max_G)*X))/T_down_GR/X)));
            #endif
        }
        //Stage Last
        else
        {
            IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
            #if LED_DEBUG
            printf("Stage End: L_typ_G %d (%d)\n", i, L_typ_G);
            #endif
        }
        count_pwm_led++;
skip:
        pwm_led_saveTime = get_timer_masked();
}

void tv005_finish_led(void)
{
        int j, start=count_pwm_led, T_scale=32, X;
        ulong T_up_GR=0x1F, T_down_GR=0x3E, T_max_G=0x3E;

        X=(int)(T_scale/skip_condition);
        for (j=start; j <= (T_up_GR+T_max_G+T_down_GR)*X; j++)
        {
            tv005_update_led();
            mdelay(skip_condition);
        }
}
#else // SoftDD rev0 -> rev2
int count_pwm_led = 0;
void tv005_update_led(void)
{
	if (count_pwm_led == 0)
		IO_PWM_SetDuty(PIN_GREEN_LED, 0xFF);
	count_pwm_led = 1;
}

void tv005_finish_led(void)
{
        int j, T_scale=100; // 100ms
	ulong L_dim_G[14] = { 0xF3, 0xE6, 0xD9, 0xCC, 0xC0, 0xB3, 0xA6, 0x99, 0x8C, 0x80, 0x73, 0x66, 0x59, 0x4C };

        IO_PWM_SetDuty(PIN_RED_LED, 0);
        for (j=0; j < 14; j++) // total about 100ms x 13 = 1.3 seconds
        {
            IO_PWM_SetDuty(PIN_GREEN_LED, L_dim_G[j]);
            mdelay(T_scale);
        }
}
#endif // CONFIG_CUSTOMIZE_FUNC_REGISTER

int rtk_plat_do_panelbl(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong freq;
	ulong duty;
	unsigned long long value = 0;

	if (argc != 3)
                return CMD_RET_USAGE;

     	freq = simple_strtoul(argv[1], NULL, 16);
	duty = simple_strtoul(argv[2], NULL, 16);

	printf("Panel BackLight PWM SetFreq=%d, SetDuty=%d\n", freq, duty);
	if (pcb_mgr_get_enum_info_byname("PIN_BL_ADJ", &value) == 0)
	{
		IO_PWM_SetFreq(value, freq);
		IO_PWM_SetDuty(value, duty);
	}

        return 0;
}

U_BOOT_CMD(
        panelbl, 3, 0,    rtk_plat_do_panelbl,
        "Panel BackLight PWM Setting",
        "[Frequency] [Duty Cycle]\n"
        "    - Change Panel BlackLight by Frequency and Duty Cycle"
);

int rtk_plat_do_ledr(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        ulong freq;
        ulong duty;

	if (argc != 3)
                return CMD_RET_USAGE;

        freq = simple_strtoul(argv[1], NULL, 16);
        duty = simple_strtoul(argv[2], NULL, 16);

	printf("Red LED PWM SetFreq=%d, SetDuty=%d\n", freq, duty);
	IO_PWM_SetFreq(PIN_RED_LED, freq);
	IO_PWM_SetDuty(PIN_RED_LED, duty);

        return 0;
}

U_BOOT_CMD(
        ledr, 3, 0,    rtk_plat_do_ledr,
        "Red LED PWM Setting",
        "[Frequency] [Duty Cycle]\n"
        "    - Change Red LED by Frequency and Duty Cycle"
);

int rtk_plat_do_ledg(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        ulong freq;
        ulong duty;

        if (argc != 3)
                return CMD_RET_USAGE;

        freq = simple_strtoul(argv[1], NULL, 16);
        duty = simple_strtoul(argv[2], NULL, 16);

        printf("Green LED PWM SetFreq=%d, SetDuty=%d\n", freq, duty);
        IO_PWM_SetFreq(PIN_GREEN_LED, freq);
        IO_PWM_SetDuty(PIN_GREEN_LED, duty);

        return 0;
}

U_BOOT_CMD(
        ledg, 3, 0,    rtk_plat_do_ledg,
        "Green LED PWM Setting",
        "[Frequency] [Duty Cycle]\n"
        "    - Change Green LED by Frequency and Duty Cycle"
);

int rtk_plat_do_sdd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        int ret = 0, i, T_scale=32, X=1; // 32ms/32000us
        ulong T_up_GR=0x1F, T_down_GR=0x3E, T_max_G=0x3E, T_max_R=0x3E;
        ulong L_min_G=0x00, L_min_R=0x00, L_typ_G=0x34, L_typ_R=0x34, L_max_G=0xFB, L_max_R=0xFB;

	printf("T_up_GR=0x%02x, T_down_GR=0x%02x, T_max_G=0x%02x, T_max_R=0x%02x\n", T_up_GR, T_down_GR, T_max_G, T_max_R);
	printf("L_min_G=0x%02x, L_min_R=0x%02x, L_typ_G=0x%02x, L_typ_R=0x%02x, L_max_G=0x%02x, L_max_R=0x%02x\n", L_min_G, L_min_R, L_typ_G, L_typ_R, L_max_G, L_max_R);
        if (argc != 3)
                return CMD_RET_USAGE;

        if (argv[1][0] == 'a' && argv[1][1] == 'c' )
	{
		if ( simple_strtoul(argv[2], NULL, 16) == 1 )	// ac on
		{
			//IO_PWM_SetFreq(PIN_RED_LED, 200);
			//IO_PWM_SetDuty(PIN_RED_LED, 0);
			//IO_PWM_SetFreq(PIN_GREEN_LED, 200);
			//IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			//mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			for (i=0; i< T_up_GR*X; i++)
			{
				IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_min_G+(int)((L_max_G-L_min_G)*i/T_up_GR/X)));
				mdelay((int)(T_scale));
			}
			IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
			mdelay(T_max_G*T_scale);
			for (i=0; i< T_down_GR*X; i++)
			{
				IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_max_G-(int)((L_max_G-L_typ_G)*i/T_down_GR/X)));
				mdelay((int)(T_scale));
			}
			IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
			//mdelay(T_down_GR*T_scale);// 32 ms x T_down_GR about 2 sec
		}
		else if ( simple_strtoul(argv[2], NULL, 16) == 0 )	// ac off
		{
			IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
			IO_PWM_SetDuty(PIN_RED_LED, L_typ_R);
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			IO_PWM_SetDuty(PIN_RED_LED, 0);
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			mdelay(T_max_G*T_scale);// 32 ms x T_max_G about 2 sec
			mdelay(T_down_GR*T_scale);// 32 ms x T_down_GR about 2 sec
		}
		else
		{
			return CMD_RET_USAGE;
		}
	}
	else if (argv[1][0] == 'd' && argv[1][1] == 'c' )
        {
                if ( simple_strtoul(argv[2], NULL, 16) == 1 ) // dc on
                {
			//IO_PWM_SetDuty(PIN_RED_LED, L_typ_R);
			//IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			//mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			//IO_PWM_SetFreq(PIN_RED_LED, 200);
			//IO_PWM_SetDuty(PIN_RED_LED, 0);
			//IO_PWM_SetFreq(PIN_GREEN_LED, 200);
			//IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			//mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			for (i=0; i< T_up_GR*X; i++)
			{
				IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_min_G+(int)((L_max_G-L_min_G)*i/T_up_GR/X)));
				mdelay((int)(T_scale));
			}
			IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
			mdelay(T_max_G*T_scale);
			for (i=0; i< T_down_GR*X; i++)
			{
				IO_PWM_SetDuty(PIN_GREEN_LED, (int)(L_max_G-(int)((L_max_G-L_typ_G)*i/T_down_GR/X)));
				mdelay((int)(T_scale));
			}
			IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
			//mdelay(T_down_GR*T_scale);// 32 ms x T_down_GR about 2 sec

                }
                else if ( simple_strtoul(argv[2], NULL, 16) == 0 )    // dc off
                {
			//IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
			IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
			IO_PWM_SetDuty(PIN_RED_LED, 0);
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			IO_PWM_SetDuty(PIN_RED_LED, 0);
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec // As short as possible
			IO_PWM_SetDuty(PIN_GREEN_LED, 0);
			IO_PWM_SetDuty(PIN_RED_LED, L_typ_R);
			mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
			mdelay(T_max_G*T_scale);// 32 ms x T_max_G about 2 sec
			mdelay(T_down_GR*T_scale);// 32 ms x T_down_GR about 2 sec
                }
                else
                {
                        return CMD_RET_USAGE;
                }
        }
	else if (argv[1][0] == 'u' && argv[1][1] == 'p' )
	{
                if ( simple_strtoul(argv[2], NULL, 16) == 0 )    // software update failure
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                        mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
                        IO_PWM_SetDuty(PIN_GREEN_LED, 0);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                        mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec // As short as possible
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
                        IO_PWM_SetDuty(PIN_RED_LED, L_max_R);
                        mdelay(3000);// 3 sec Period of SW Update
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
                        IO_PWM_SetDuty(PIN_RED_LED, L_max_R);
                }
                else if ( simple_strtoul(argv[2], NULL, 16) == 1 )    // software update success
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_typ_G);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                        mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec
                        IO_PWM_SetDuty(PIN_GREEN_LED, 0);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                        mdelay(T_up_GR*T_scale);// 32 ms x T_up_GR about 1 sec // As short as possible
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
                        IO_PWM_SetDuty(PIN_RED_LED, L_max_R);
                        mdelay(3000);// 3 sec Period of SW Update
                        IO_PWM_SetDuty(PIN_GREEN_LED, 0);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                }
                else
                {
                        return CMD_RET_USAGE;
                }

	}
	else if (argv[1][0] == 'r' && argv[1][1] == 'g' )
	{
                if ( simple_strtoul(argv[2], NULL, 16) == 0 )    // all off
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, 0);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                }
                else if ( simple_strtoul(argv[2], NULL, 16) == 1 )    // GREEN on; RED off
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
                        IO_PWM_SetDuty(PIN_RED_LED, 0);
                }
                else if ( simple_strtoul(argv[2], NULL, 16) == 2 )    // RED on; GREEN off
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, 0);
                        IO_PWM_SetDuty(PIN_RED_LED, L_max_R);
                }
                else if ( simple_strtoul(argv[2], NULL, 16) == 3 )    // RED on; GREEN on
                {
                        IO_PWM_SetDuty(PIN_GREEN_LED, L_max_G);
                        IO_PWM_SetDuty(PIN_RED_LED, L_max_R);
                }
                else
                {
                        return CMD_RET_USAGE;
                }
	}
	else
	{
        	return CMD_RET_USAGE;
	}

        return ret;
}

U_BOOT_CMD(
        sdd, 3, 0,    rtk_plat_do_sdd,
        "Software Design Data Running",
        "[ac|dc|up] [0|1]\n"
	"    - Software DD Demo for dc on/off, ac on/off and software update fail/success"
);

int do_gpiotest (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        ulong id_addr;

        if (argc != 2)
                return CMD_RET_USAGE;

        id_addr = simple_strtoul(argv[1], NULL, 16);
        printf("id_addr=%02x\n\n", id_addr);

	if (id_addr == 0)
	{
		IO_Set( PIN_AMP_SHDN, 0 );
	}
	else
	{
		IO_Set( PIN_AMP_SHDN, 1 );
	}
	return 0;

}

U_BOOT_CMD(
        gpiotest ,    2,    0,     do_gpiotest,
        "gpio test",
        "N\n"
        "    - run gpio test"
);

int do_ampvol (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        ulong val;

        if (argc != 2)
                return CMD_RET_USAGE;

        val = simple_strtoul(argv[1], NULL, 16);
        printf("val=%02x\n\n", val);

	// Setting of volume Default = FFh
	amp_IO(AMP_I2C, 0x40, 0x11, val);
	amp_IO(AMP_I2C, 0x40, 0x13, val);
	amp_IO(AMP_I2C, 0x40, 0x15, val);
	amp_IO(AMP_I2C, 0x40, 0x17, val);

        return 0;
}

U_BOOT_CMD(
        ampvol ,    2,    0,     do_ampvol,
        "AMP Volumn test",
        "N\n"
        "    - run AMP Volumn test"
);

int do_ampset (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        ulong addr, val;

        if (argc != 3)
                return CMD_RET_USAGE;

        addr = simple_strtoul(argv[1], NULL, 16);
        printf("addr=%02x\n\n", addr);
        val = simple_strtoul(argv[2], NULL, 16);
        printf("val=%02x\n\n", val);

	// Setting of volume Default = FFh
	amp_IO(2, 0x40, addr, val);

        return 0;
}

U_BOOT_CMD(
        ampset ,    3,    0,     do_ampset,
        "AMP I2C config test",
        "N\n"
        "    - run AMP I2C config test"
);

int do_dtvrst (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        printf("DTV PIN_NRST reset!! \n");
        IO_Set(PIN_DTV_NRST, 1 );
        mdelay(1);
        IO_Set(PIN_DTV_NRST, 0 );
        mdelay(1);
        IO_Set(PIN_DTV_NRST, 1 );

	return 0;
}

U_BOOT_CMD(
        dtvrst, 1, 0,    do_dtvrst,
        "Reset DVT demod for Tv005 L4300",
        "\n"
        "    - Reset DVT demod for Tv005 L4300"
);
#endif

void display_boottime_log(char *pString)
{
	char*	timelog = getenv("timelog");
	int is_silent_mode = 0;
	unsigned long first_boot_time;
	unsigned long current_time;

	if(!strcmp("on", timelog))
	{
		first_boot_time = get_firstboot_time_us();
		current_time = get_boot_time_us();

		if (gd->flags & GD_FLG_SILENT)
			is_silent_mode = 1;
		else
			is_silent_mode = 0;

		if (is_silent_mode)
			enable_console();
		printf("\033[0;31m[time][%s] 1stboot takes %d.%03dms, boot takes %d.%03dms \033[0m\n", pString,
			(unsigned int)(first_boot_time/1000), (unsigned int)(first_boot_time%1000), (unsigned int)(current_time/1000),(unsigned int)(current_time%1000));

		if (is_silent_mode)
			disable_console();
	}
}

#ifdef CONFIG_CMD_PWMINVERT
int do_pwminvert_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int		changed = 0;
	char*	pwminvert = getenv("pwminvert");

	if(pwminvert == NULL)
	{
		setenv("pwminvert", "off");
		changed = 1;
	}

	printf("current pwminvert : %s\n", pwminvert);

	if(argc == 2)
	{
		char* input = argv[1];

		if(strcmp(input, pwminvert))
		{
			printf("change the pwminvert to %s\n", input);
			setenv("pwminvert", input);
			changed = 1;
		}
	}

	if(changed)
		saveenv();

	return 0;
}


U_BOOT_CMD(
    pwminvert,	2,	0,	do_pwminvert_cmd,
    "set the pwminvert [on|off]\n",
    "[on|off]\n"
);
#endif

int do_timelog_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int		changed = 0;
	char*	timelog = getenv("timelog");

	if(timelog == NULL)
	{
		setenv("timelog", "off");
		changed = 1;
	}

	printf("current timelog : %s\n", timelog);

	if((argc == 2) && (timelog != NULL)) //[CID-313979] avoid passing null pointer to strcmp
	{
		char* input = argv[1];

		if(strcmp(input, timelog))
		{
			printf("change the timelog to %s\n", input);
			setenv("timelog", input);
			changed = 1;
		}
	}

	if(changed)
		saveenv();

	return 0;
}


U_BOOT_CMD(
    timelog,	2,	0,	do_timelog_cmd,
    "set the bootloader time log [on|off]\n",
    "[on|off]\n"
);

int do_print(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    int changed = 0;
    char* print = getenv("print");

    if(print == NULL)
    {
        setenv("print", "on");
        print = getenv("print");
        changed = 1;
    }
    printf("current print status : %s\n", print);

    if(argc == 2)
    {
        char* input = argv[1];

        if(!strcmp("on", input) || !strcmp("off", input))
        {
            if(strcmp(input, print))
            {
                printf("change the print status to %s\n", input);
                setenv("print", input);
                changed = 1;
            }
            if((!strcmp("off", input)) && changed == 1)
                setenv("timelog", "on");
        }
    }

    if(changed)
        saveenv();

    return 0;
}

U_BOOT_CMD(
    print,    2,    0,    do_print,
    "print\t- turn on/off print message.\n",
    "print \n"
);

#ifdef CONFIG_CMD_VSYNCAPAD
void set_vsync_apad_status(void)
{
        char * vsyncapad = getenv("vsyncapad");
        if(vsyncapad != NULL)
        {
               if(strcmp("I2S", vsyncapad)==0)
               {
						rtd_maskl(0xb8000820, ~(((0x1)<<1) | ((0xF)<<28)), ((0x1&0x1)<<1) | (0xA<<28));
						rtd_clearbits(0xb806021c, _BIT26);	// gpio 28 pull down disable	
               }
					else {
						rtd_maskl(0xb8000820, ~(((0x1)<<1) | ((0xF)<<28)), (0xC<<28));
						rtd_setbits(0xb806021c, _BIT26);		//gpio 28 pull down enable
					}
        }
        return;
}


int do_vsyncapad_control_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
        int             changed = 0;
        char*   vsyncapad = getenv("vsyncapad");

        if(vsyncapad == NULL)
        {
                setenv("vsyncapad", "VGA");
                changed = 1;
                vsyncapad = getenv("vsyncapad");
        }

        printf("current vsyncapad : %s\n", vsyncapad);

        if(argc == 2)
        {
                char* input = argv[1];

                if(strcmp(input, vsyncapad))
                {
                        printf("change the vsyncapad to %s\n", input);
                        setenv("vsyncapad", input);
                        changed = 1;
                }
        }

        if(changed)
                saveenv();

        return 0;
}

U_BOOT_CMD(
                vsyncapad,      2,      0,      do_vsyncapad_control_cmd,
                "vsyncapad [I2S|VGA]\n",
                "\n"
);
#endif

void set_uart2rbus_status(void)
{
#define DDC_CTRL_DDC_U2R_OFF 0
#define DDC_CTRL_DDC_U2R_USE_DDC0 1
#define DDC_CTRL_DDC_U2R_USE_DDC1 2
#define DDC_CTRL_DDC_U2R_USE_DDC2 3
#define DDC_U2R_DISABLE DDC_CTRL_DDC_U2R_OFF
#define DDC_U2R_ENABLE DDC_CTRL_DDC_U2R_USE_DDC1

	char* uart2rbus = getenv("uart2rbus");
	if(uart2rbus != NULL)
	{
		// enable uart2rbus, set ddc_u2r_en (0xb8061010[2:1]) to ddc0/ddc1/ddc2
		if (strncmp("on_2", uart2rbus, 4) == 0)
		{
			rtd_maskl(0xb8061010, ~(0x00000006), (DDC_CTRL_DDC_U2R_USE_DDC2) << 1);
		}
		else if (strncmp("on_1", uart2rbus, 4) == 0)
		{
			rtd_maskl(0xb8061010, ~(0x00000006), (DDC_CTRL_DDC_U2R_USE_DDC1) << 1);
		}
		else if (strncmp("on_0", uart2rbus, 4) == 0)
		{
			rtd_maskl(0xb8061010, ~(0x00000006), (DDC_CTRL_DDC_U2R_USE_DDC0) << 1);
		}
		else if (strncmp("on", uart2rbus, 2) == 0) // default setting
		{
			rtd_maskl(0xb8061010, ~(0x00000006), (DDC_U2R_ENABLE) << 1);
		}
	}
	return;
}

int do_uart2rbus_control_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int changed = 0;
	char* uart2rbus = getenv("uart2rbus");

	if(uart2rbus == NULL)
	{
		setenv("uart2rbus", "off");
		changed = 1;
		uart2rbus = getenv("uart2rbus");
	}

	printf("current uart2rbus : %s\n", uart2rbus);
	printf("  options: off, on, on_0, on_1, on_2\n");

	if(argc == 2)
	{
		char* input = argv[1];
		if(strcmp(input, uart2rbus))
		{
			printf("change the uart2rbus to %s\n", input);
			setenv("uart2rbus", input);
			changed = 1;
		}
	}

	if(changed)
		saveenv();

	return 0;
}

U_BOOT_CMD(
		uart2rbus, 2, 0, do_uart2rbus_control_cmd,
		"uart2rbus [off|on|on_0|on_1|on_2]\n",
		"\n"
);
