/*
 * (C) Copyright 2001
 * Gerald Van Baren, Custom IDEAS, vanbaren@cideas.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <command.h>
#include <environment.h>
#include <malloc.h>
#include <asm/byteorder.h>
#include <asm/io.h>
#include "tp_reg.h"

#define TP_DEBUG 0
#define TP_LOAD_TS_BY_PROBE_SCRIPT 1

typedef struct {
	unsigned long cntl;
	unsigned long frmcfg;

	// counter
	unsigned long cnt;
	unsigned long drp_cnt;
	unsigned long err_cnt;

	// m2m buffer
	unsigned long m2m_ctrl;
	unsigned long m2m_base;
	unsigned long m2m_limit;
	unsigned long m2m_rp;
	unsigned long m2m_wp;

}TP_FRAMER_REG;

typedef struct {
	unsigned long cntl;
	unsigned long frmcfg;

	// counter
	unsigned long cnt;
	unsigned long drp_cnt;
	unsigned long intr;

	// m2m buffer
	unsigned long m2m_ctrl;
	unsigned long m2m_base;
	unsigned long m2m_limit;
	unsigned long m2m_rp;
	unsigned long m2m_wp;
//	unsigned long mtp_frc_ctrl;
//	unsigned long mtp_frc_en_bit;
//
//	// sync detection
//	unsigned long detect;
//	unsigned long detect_ctrl;
}TP_P_REG;


TP_FRAMER_REG g_tp_framer[] =
{
	// TP0
	{
		.cntl 	   = TP_TF0_CNTL,
		.frmcfg    = TP_TF0_FRMCFG,
		.cnt 	   = TP_TF0_CNT,
		.drp_cnt   = TP_TF0_DRP_CNT,
		.err_cnt   = TP_TF0_ERR_CNT,
		.m2m_ctrl  = TP0_M2M_RING_CTRL,
		.m2m_base  = TP0_M2M_RING_BASE,
		.m2m_limit = TP0_M2M_RING_LIMIT,
		.m2m_rp    = TP0_M2M_RING_RP,
		.m2m_wp    = TP0_M2M_RING_WP,
	},
	// TP1
	{
		.cntl 	   = TP_TF1_CNTL,
		.frmcfg    = TP_TF1_FRMCFG,
		.cnt 	   = TP_TF1_CNT,
		.drp_cnt   = TP_TF1_DRP_CNT,
		.err_cnt   = TP_TF1_ERR_CNT,
		.m2m_ctrl  = TP1_M2M_RING_CTRL,
		.m2m_base  = TP1_M2M_RING_BASE,
		.m2m_limit = TP1_M2M_RING_LIMIT,
		.m2m_rp    = TP1_M2M_RING_RP,
		.m2m_wp    = TP1_M2M_RING_WP,
	},
	// TP2
	{
		.cntl 	   = TP_TF2_CNTL,
		.frmcfg    = TP_TF2_FRMCFG,
		.cnt 	   = TP_TF2_CNT,
		.drp_cnt   = TP_TF2_DRP_CNT,
		.err_cnt   = TP_TF2_ERR_CNT,
		.m2m_ctrl  = TP2_M2M_RING_CTRL,
		.m2m_base  = TP2_M2M_RING_BASE,
		.m2m_limit = TP2_M2M_RING_LIMIT,
		.m2m_rp    = TP2_M2M_RING_RP,
		.m2m_wp    = TP2_M2M_RING_WP,
	},
	// TP3
	{
		.cntl 	   = TP_TF3_CNTL,
		.frmcfg    = TP_TF3_FRMCFG,
		.cnt 	   = TP_TF3_CNT,
		.drp_cnt   = TP_TF3_DRP_CNT,
		.err_cnt   = TP_TF2_ERR_CNT,
		.m2m_ctrl  = TP3_M2M_RING_CTRL,
		.m2m_base  = TP3_M2M_RING_BASE,
		.m2m_limit = TP3_M2M_RING_LIMIT,
		.m2m_rp    = TP3_M2M_RING_RP,
		.m2m_wp    = TP3_M2M_RING_WP,
	},
};

TP_P_REG g_tpp_framer[] =
{
	// TP0_P
	{
		.cntl 	        = TP_TF0_P_CNTL,
		.frmcfg         = TP_TF0_P_FRMCFG,
		.cnt 	        = TP_TF0_P_CNT,
		.drp_cnt        = TP_TF0_P_DRP_CNT,
		.intr           = TP_TF0_P_INT,

		.m2m_ctrl       = TP0_P_M2M_RING_CTRL,
		.m2m_base       = TP0_P_M2M_RING_BASE,
		.m2m_limit      = TP0_P_M2M_RING_LIMIT,
		.m2m_rp         = TP0_P_M2M_RING_RP,
		.m2m_wp         = TP0_P_M2M_RING_WP,
//		.mtp_frc_ctrl   = TP_FR_CTRL,
//		.mtp_frc_en_bit = TP_FR_CTRL_TP0_EN,

//		.detect         = TP_TF0_DETECT_1,
//		.detect_ctrl    = TP_TF0_DETECT_CTRL,
	},
	// TP1_P
	{
		.cntl	   = TP_TF1_P_CNTL,
		.frmcfg    = TP_TF1_P_FRMCFG,
		.cnt	   = TP_TF1_P_CNT,
		.drp_cnt   = TP_TF1_P_DRP_CNT,
		.intr      = TP_TF1_P_INT,

		.m2m_ctrl  = TP1_P_M2M_RING_CTRL,
		.m2m_base  = TP1_P_M2M_RING_BASE,
		.m2m_limit = TP1_P_M2M_RING_LIMIT,
		.m2m_rp    = TP1_P_M2M_RING_RP,
		.m2m_wp    = TP1_P_M2M_RING_WP,
//		.mtp_frc_ctrl = TP_FR_CTRL,
//		.mtp_frc_en_bit = TP_FR_CTRL_TP0_EN,

//		.detect      = TP_TF1_DETECT_1,
//		.detect_ctrl = TP_TF1_DETECT_CTRL,
	},
	// TP2_P
	{
		.cntl	   = TP_TF2_P_CNTL,
		.frmcfg    = TP_TF2_P_FRMCFG,
		.cnt	   = TP_TF2_P_CNT,
		.drp_cnt   = TP_TF2_P_DRP_CNT,
		.intr      = TP_TF2_P_INT,

		.m2m_ctrl  = TP2_P_M2M_RING_CTRL,
		.m2m_base  = TP2_P_M2M_RING_BASE,
		.m2m_limit = TP2_P_M2M_RING_LIMIT,
		.m2m_rp    = TP2_P_M2M_RING_RP,
		.m2m_wp    = TP2_P_M2M_RING_WP,
//		.mtp_frc_ctrl = TP_FR_CTRL,
//		.mtp_frc_en_bit = TP_FR_CTRL_TP0_EN,

//		.detect      = TP_TF2_DETECT_1,
//		.detect_ctrl = TP_TF2_DETECT_CTRL,
	},
};

/********************************************************************************
                                TP GET/SET marcos
*********************************************************************************/
// FRMCFG
#define SET_TP_CNTL(i, val)             rtd_outl(g_tp_framer[i].cntl, val)
#define GET_TP_CNTL(i)                  rtd_inl(g_tp_framer[i].cntl)

#define SET_TP_FRMCFG(i, val)           rtd_outl(g_tp_framer[i].frmcfg, val)
#define GET_TP_FRMCFG(i)                rtd_inl(g_tp_framer[i].frmcfg)

#define SET_TP_CNT(i, val)       		rtd_outl(g_tp_framer[i].cnt, val)
#define GET_TP_CNT(i)                 	rtd_inl(g_tp_framer[i].cnt)

#define SET_TP_DRP_CNT(i, val)       	rtd_outl(g_tp_framer[i].drp_cnt, val)
#define GET_TP_DRP_CNT(i)            	rtd_inl(g_tp_framer[i].drp_cnt)

#define SET_TP_ERR_CNT(i, val)      	rtd_outl(g_tp_framer[i].err_cnt, val)
#define GET_TP_ERR_CNT(i)           	rtd_inl(g_tp_framer[i].err_cnt)

#define SET_MTP_RING_CTRL(i, val)       rtd_outl(g_tp_framer[i].m2m_ctrl, val)
#define GET_MTP_RING_CTRL(i)            rtd_inl(g_tp_framer[i].m2m_ctrl)

#define SET_MTP_RING_BASE(i, val)       rtd_outl(g_tp_framer[i].m2m_base, val)
#define GET_MTP_RING_BASE(i)            rtd_inl(g_tp_framer[i].m2m_base)

#define SET_MTP_RING_LIMIT(i, val)      rtd_outl(g_tp_framer[i].m2m_limit, val)
#define GET_MTP_RING_LIMIT(i)           rtd_inl(g_tp_framer[i].m2m_limit)

#define SET_MTP_RING_RP(i, val)         rtd_outl(g_tp_framer[i].m2m_rp, val)
#define GET_MTP_RING_RP(i)              rtd_inl(g_tp_framer[i].m2m_rp)

#define SET_MTP_RING_WP(i, val)         rtd_outl(g_tp_framer[i].m2m_wp, val)
#define GET_MTP_RING_WP(i)              rtd_inl(g_tp_framer[i].m2m_wp)

// TPP FRMCFG
#define SET_TPP_CNTL(i, val)			rtd_outl(g_tpp_framer[i].cntl, val)
#define GET_TPP_CNTL(i)					rtd_inl(g_tpp_framer[i].cntl)

#define SET_TPP_FRMCFG(i, val)			rtd_outl(g_tpp_framer[i].frmcfg, val)
#define GET_TPP_FRMCFG(i)				rtd_inl(g_tpp_framer[i].frmcfg)

#define SET_TPP_CNT(i, val)				rtd_outl(g_tpp_framer[i].cnt, val)
#define GET_TPP_CNT(i)                 	rtd_inl(g_tpp_framer[i].cnt)

#define SET_TPP_DRP_CNT(i, val)       	rtd_outl(g_tpp_framer[i].drp_cnt, val)
#define GET_TPP_DRP_CNT(i)            	rtd_inl(g_tpp_framer[i].drp_cnt)

#define SET_TPP_THRESHOLD(val)          rtd_outl(TP_P_THRESHOLD, val)
#define SET_TPP_FULLNESS(val)           rtd_outl(TP_P_FULLNESS, val)

#define SET_TPP_MTP_RING_CTRL(i, val)   rtd_outl(g_tpp_framer[i].m2m_ctrl, val)
#define GET_TPP_MTP_RING_CTRL(i)        rtd_inl(g_tpp_framer[i].m2m_ctrl)

#define SET_TPP_MTP_RING_BASE(i, val)   rtd_outl(g_tpp_framer[i].m2m_base, val)
#define GET_TPP_MTP_RING_BASE(i)        rtd_inl(g_tpp_framer[i].m2m_base)

#define SET_TPP_MTP_RING_LIMIT(i, val)  rtd_outl(g_tpp_framer[i].m2m_limit, val)
#define GET_TPP_MTP_RING_LIMIT(i)       rtd_inl(g_tpp_framer[i].m2m_limit)

#define SET_TPP_MTP_RING_RP(i, val)     rtd_outl(g_tpp_framer[i].m2m_rp, val)
#define GET_TPP_MTP_RING_RP(i)          rtd_inl(g_tpp_framer[i].m2m_rp)

#define SET_TPP_MTP_RING_WP(i, val)     rtd_outl(g_tpp_framer[i].m2m_wp, val)
#define GET_TPP_MTP_RING_WP(i)          rtd_inl(g_tpp_framer[i].m2m_wp)

// SYNC REPLACE
#define SET_TPP_SYNC_REPLACE(tp_id,sync)  rtd_outl(TP_PID_PART, val)

// PID
#define SET_TP_PID_PART(val)            rtd_outl(TP_PID_PART, val)
#define SET_TP_PID_CTRL(val)            rtd_outl(TP_PID_CTRL, val)
#define SET_TP_PID_DATA(val)            rtd_outl(TP_PID_DATA, val)
#define SET_TP_PID_DATA2(val)           rtd_outl(TP_PID_DATA2, val)
#define SET_TP_PID_DATA3(val)           rtd_outl(TP_PID_DATA3, val)

#define GET_TP_PID_PART()               rtd_inl(TP_PID_PART)
#define GET_TP_PID_CTRL()               rtd_inl(TP_PID_CTRL)
#define GET_TP_PID_DATA()               rtd_inl(TP_PID_DATA)
#define GET_TP_PID_DATA2()              rtd_inl(TP_PID_DATA2)
#define GET_TP_PID_DATA3()              rtd_inl(TP_PID_DATA3)

// BUFF
#define SET_TP_RING_CTRL(val)           rtd_outl(TP_RING_CTRL, val)
#define SET_TP_RING_LIMIT(val)          rtd_outl(TP_RING_LIMIT, val)
#define SET_TP_RING_BASE(val)           rtd_outl(TP_RING_BASE, val)
#define SET_TP_RING_RP(val)             rtd_outl(TP_RING_RP, val)
#define SET_TP_RING_WP(val)             rtd_outl(TP_RING_WP, val)

#define GET_TP_RING_CTRL(val)           rtd_inl(TP_RING_CTRL)
#define GET_TP_RING_LIMIT(val)          rtd_inl(TP_RING_LIMIT)
#define GET_TP_RING_BASE(val)           rtd_inl(TP_RING_BASE)
#define GET_TP_RING_RP(val)             rtd_inl(TP_RING_RP)
#define GET_TP_RING_WP(val)             rtd_inl(TP_RING_WP)

// TPP BUFF
#define SET_TP_P_RING_CTRL(val)           rtd_outl(TP_P_RING_CTRL, val)
#define SET_TP_P_RING_LIMIT(val)          rtd_outl(TP_P_RING_LIMIT, val)
#define SET_TP_P_RING_BASE(val)           rtd_outl(TP_P_RING_BASE, val)
#define SET_TP_P_RING_RP(val)             rtd_outl(TP_P_RING_RP, val)
#define SET_TP_P_RING_WP(val)             rtd_outl(TP_P_RING_WP, val)

#define GET_TP_P_RING_CTRL(val)           rtd_inl(TP_P_RING_CTRL)
#define GET_TP_P_RING_LIMIT(val)          rtd_inl(TP_P_RING_LIMIT)
#define GET_TP_P_RING_BASE(val)           rtd_inl(TP_P_RING_BASE)
#define GET_TP_P_RING_RP(val)             rtd_inl(TP_P_RING_RP)
#define GET_TP_P_RING_WP(val)             rtd_inl(TP_P_RING_WP)


// TPO
#define SET_TPO_FRMCFG(val)             rtd_outl(TP_TF_OUT_FRMCFG, val)
#define GET_TPO_FRMCFG()                rtd_inl(TP_TF_OUT_FRMCFG)

#define SET_TPO_CTRL(val)               rtd_outl(TP_TF_OUT_CTRL, val)
#define GET_TPO_CTRL()                  rtd_inl(TP_TF_OUT_CTRL)
#define SET_TPO_CLK_SEL(val)            rtd_outl(TPO_CLK_SEL, val)
#define GET_TPO_CLK_SEL(val)            rtd_inl(TPO_CLK_SEL)

/********************************************************************************
                         PID function
*********************************************************************************/
int tp_pid_set_param(unsigned char id, unsigned long data[3])
{
    if (id>=MAX_TP_PID)
    {
        printf("Invalid PID Id - %d\n", id);
        return -1;
    }

	// TODO: need clear it
	rtd_outl(0xb80140d8,data[0]);
	rtd_outl(0xb80140dc,data[1]);
	rtd_outl(0xb80140d4,0x00000080| id);

#if TP_DEBUG
	printf("PID[%d] = %08lx, %08lx, %08lx\n", id, data[0], data[1], data[2]);
#endif
    /*
    SET_TP_PID_DATA(data[0]);
    SET_TP_PID_DATA2(data[1]);
    SET_TP_PID_DATA3(data[2]);
    SET_TP_PID_CTRL(TP_PID_CTRL_R_W(1) | TP_PID_CTRL_IDX(id));
*/
    return 0;
}

int tp_pid_get_param(unsigned char id, unsigned long data[3])
{
    if (id>=MAX_TP_PID)
    {
        printf("Invalid PID Id - %d\n", id);
        return -1;
    }

    SET_TP_PID_CTRL(TP_PID_CTRL_R_W(0) | TP_PID_CTRL_IDX(id));

    data[0] = GET_TP_PID_DATA();
    data[1] = GET_TP_PID_DATA2();
    data[2] = GET_TP_PID_DATA3();

#if TP_DEBUG
    printf("PID[%d] = %08lx, %08lx, %08lx\n", id, data[0], data[1], data[2]);
#endif
    return 0;
}

int tp_pid_reset(unsigned char id)
{
    unsigned long data[] = {0x00000000, 0x00000020, 0x00000000};
    //printf("Reset PID %d\n", id);
	tp_pid_set_param(id, data);
    return  0;
}


/********************************************************************************
                         Get/Set function
*********************************************************************************/
/***** TP buffer *****/
int tp_buff_set_boundrary(
    unsigned char   id,
    unsigned long   base,
    unsigned long   limit,
    unsigned long   rp,
    unsigned long   wp
    )
{
    if (id>=MAX_TP_RING_BUFFER)
    {
        printf("Invalid Buffer Id - %d\n", id);
        return -1;
    }

    SET_TP_RING_LIMIT(limit);
    SET_TP_RING_BASE(base);
    SET_TP_RING_RP(rp);
    SET_TP_RING_WP(wp);

    SET_TP_RING_CTRL(TP_RING_CTRL_WM(0x00) |    // write mask = no mask
                     TP_RING_CTRL_R_W(1)   |    // Write
                     TP_RING_CTRL_IDX(id));   // Index

	//tp_buff_dump_status(id);
    return 0;
}

int tp_buff_get_boundrary(
    unsigned char   id,
    unsigned long*  p_base,
    unsigned long*  p_limit,
    unsigned long*  p_rp,
    unsigned long*  p_wp
    )
{
    if (id>=MAX_TP_RING_BUFFER)
    {
        printf("Invalud Buffer Id - %d\n", id);
        return -1;
    }

    *p_limit = 0xFFFFFFFF;
    *p_base = 0xFFFFFFFF;
    *p_rp = 0xFFFFFFFF;
    *p_wp = 0xFFFFFFFF;


    SET_TP_RING_CTRL(TP_RING_CTRL_WM(0x00) |    // write mask = no mask
                     TP_RING_CTRL_R_W(0)   |    // read
                     TP_RING_CTRL_IDX(id));     // Index
    mdelay(1);

    *p_limit = GET_TP_RING_LIMIT();
    *p_base = GET_TP_RING_BASE();
    *p_rp = GET_TP_RING_RP();
    *p_wp = GET_TP_RING_WP();

    return 0;
}

/***** TP_P buffer *****/
int tp_p_buff_set_boundrary(
    unsigned char   id,
    unsigned long   base,
    unsigned long   limit,
    unsigned long   rp,
    unsigned long   wp
    )
{
    if (id>=MAX_TP_P_RING_BUFFER)
    {
        printf("Invalid Buffer Id of TP_P - %d\n", id);
        return -1;
    }

    SET_TP_P_RING_LIMIT(limit);
    SET_TP_P_RING_BASE(base);
    SET_TP_P_RING_RP(rp);
    SET_TP_P_RING_WP(wp);

    SET_TP_P_RING_CTRL(TP_RING_CTRL_WM(0x00) |    // write mask = no mask
                       TP_RING_CTRL_R_W(1)   |    // Write
                       TP_RING_CTRL_IDX(id));   // Index

    // tp_p_buff_dump_status(id);
    return 0;
}

int tp_p_buff_get_boundrary(
    unsigned char   id,
    unsigned long*  p_base,
    unsigned long*  p_limit,
    unsigned long*  p_rp,
    unsigned long*  p_wp
    )
{
    if (id>=MAX_TP_P_RING_BUFFER)
    {
        printf("Invalud Buffer Id of TP_P_MTP - %d\n", id);
        return -1;
    }

    *p_limit = 0xFFFFFFFF;
    *p_base = 0xFFFFFFFF;
    *p_rp = 0xFFFFFFFF;
    *p_wp = 0xFFFFFFFF;


    SET_TP_P_RING_CTRL(TP_RING_CTRL_WM(0x00) |    // write mask = no mask
                       TP_RING_CTRL_R_W(0)   |    // read
                       TP_RING_CTRL_IDX(id));     // Index
    mdelay(1);

    *p_limit = GET_TP_P_RING_LIMIT();
    *p_base = GET_TP_P_RING_BASE();
    *p_rp = GET_TP_P_RING_RP();
    *p_wp = GET_TP_P_RING_WP();

    return 0;
}
/***** MTP buffer *****/
int mtp_set_boundrary(
    unsigned char   id,
    unsigned long   base,
    unsigned long   limit,
    unsigned long   rp,
    unsigned long   wp
    )
{
    if (id>=MAX_MTP_RING_BUFFER)
    {
        printf("Invalid Buffer Id - %d\n", id);
        return -1;
    }

    SET_MTP_RING_LIMIT(id, limit);
    SET_MTP_RING_BASE(id, base);
    SET_MTP_RING_RP(id, rp);
    SET_MTP_RING_WP(id, wp);

    return 0;
}

int mtp_get_boundrary(
    unsigned char   id,
    unsigned long*  p_base,
    unsigned long*  p_limit,
    unsigned long*  p_rp,
    unsigned long*  p_wp
    )
{
    if (id>=MAX_MTP_RING_BUFFER)
    {
        printf("Invalud Buffer Id - %d\n", id);
        return -1;
    }

    *p_limit = GET_MTP_RING_LIMIT(id);
    *p_base  = GET_MTP_RING_BASE(id);
    *p_rp    = GET_MTP_RING_RP(id);
    *p_wp    = GET_MTP_RING_WP(id);

    return 0;
}

/***** TP_P MTP buff *****/
int tpp_mtp_set_boundrary(
    unsigned char   id,
    unsigned long   base,
    unsigned long   limit,
    unsigned long   rp,
    unsigned long   wp
    )
{
    if (id>=MAX_TP_P_MTP_RING_BUFFER)
    {
        printf("Invalid Buffer Id of TP_P MTP - %d\n", id);
        return -1;
    }

    SET_TPP_MTP_RING_LIMIT(id, limit);
    SET_TPP_MTP_RING_BASE(id, base);
    SET_TPP_MTP_RING_RP(id, rp);
    SET_TPP_MTP_RING_WP(id, wp);

	//tpp_mtp_dump_status(id);
    return 0;
}

int tpp_mtp_get_boundrary(
    unsigned char   id,
    unsigned long*  p_base,
    unsigned long*  p_limit,
    unsigned long*  p_rp,
    unsigned long*  p_wp
    )
{
    if (id>=MAX_TP_P_MTP_RING_BUFFER)
    {
        printf("Invalud Buffer Id of TP_P MTP - %d\n", id);
        return -1;
    }

    *p_limit = GET_TPP_MTP_RING_LIMIT(id);
    *p_base  = GET_TPP_MTP_RING_BASE(id);
    *p_rp    = GET_TPP_MTP_RING_RP(id);
    *p_wp    = GET_TPP_MTP_RING_WP(id);

    return 0;
}
/********************************************************************************
                         Dump status/data function
*********************************************************************************/
/***** tp register status *****/
void tp_dump_status(unsigned char id)
{
	printf("TP[%d] : CNTL=%08x, FRMCFG=%08x, CNT=%08x/%08x/%08x\n",
	     id, GET_TP_CNTL(id), GET_TP_FRMCFG(id), GET_TP_CNT(id), GET_TP_DRP_CNT(id), GET_TP_ERR_CNT(id));
}

void tp_p_dump_status(unsigned char id)
{
	printf("TP[%d]_P : CNTL=%08x, FRMCFG=%08x, CNT=%08x/%08x\n",
	     id, GET_TPP_CNTL(id), GET_TPP_FRMCFG(id), GET_TPP_CNT(id), GET_TPP_DRP_CNT(id));
}

/***** tp buff status *****/
int tp_buff_dump_status(unsigned char id)
{
	unsigned long base, limit, rp, wp;

	if (tp_buff_get_boundrary(id, &base, &limit, &rp, &wp)<0)
		return -1;

	printf("TP BUFF[%d] = %08lx/%08lx/%08lx/%08lx\n", id, base, rp, wp, limit);
	return 0;
}
/***** tp_p buff status *****/
int tp_p_buff_dump_status(unsigned char id)
{
	unsigned long base, limit, rp, wp;

	if (tp_p_buff_get_boundrary(id, &base, &limit, &rp, &wp)<0)
		return -1;

	printf("TP_P BUFF[%d] = %08lx/%08lx/%08lx/%08lx\n", id, base, rp, wp, limit);
	return 0;
}
/***** mtp buff status *****/
void mtp_dump_status(unsigned char id)
{
    printf("MTP[%d] : CTRL=%02x, {%08x/%08x/%08x/%08x}\n",
	       id, GET_MTP_RING_CTRL(id), GET_MTP_RING_BASE(id),
		   GET_MTP_RING_RP(id), GET_MTP_RING_WP(id), GET_MTP_RING_LIMIT(id));
}
/***** tp_p mtp buff status *****/
void tpp_mtp_dump_status(unsigned char id)
{
    printf("TP_P_MTP[%d] : CTRL=%08x, {%08x/%08x/%08x/%08x}\n",
	       id, GET_TPP_MTP_RING_CTRL(id), GET_TPP_MTP_RING_BASE(id),
		   GET_TPP_MTP_RING_RP(id), GET_TPP_MTP_RING_WP(id), GET_TPP_MTP_RING_LIMIT(id));
}
/***** tp CNT/EROP/ERR register status ****/
void dump_tp_info( int tp_id )
{
	printf("TP0 CNT=%d\n", GET_TP_CNT(tp_id));
	printf("TP0 DRP_CNT=%d\n", GET_TP_DRP_CNT(tp_id));
	printf("TP0 ERR_CNT=%d\n", GET_TP_ERR_CNT(tp_id));
}
/***** tpp CNT/EROP/ERR register status ****/
void dump_tpp_info( int tpp_id )
{
	printf("TPP%d TPP_C=%d\n", tpp_id, GET_TPP_CNT(tpp_id));
	printf("TPP%d DRP_CNT=%d\n", tpp_id, GET_TPP_DRP_CNT(tpp_id));
}
/********************************************************************************
                         Init/Reset function
*********************************************************************************/
/***** TP framer *****/
static int tp_framer_reset(unsigned char id)
{
	const unsigned long CntlInit     = TP_TF_CNTL_STRM_ID_EN(0)  |
                                       TP_TF_CNTL_BUF_RDY_CTL(1) |
                                       TP_TF_CNTL_PSC_EN(0)      |
                                       TP_TF_CNTL_PES_EN(0)      |
                                       TP_TF_CNTL_TSC_EN(0)      |
                                       TP_TF_CNTL_TB(0)          |
                                       TP_TF_CNTL_BUSY(0)        |
                                       TP_TF_CNTL_MODE(0)        |
                                       TP_TF_CNTL_DU_EN(0)       |
                                       TP_TF_CNTL_DE_EN(0)       |
                                       TP_TF_CNTL_XT_EN(0)       |
                                       TP_TF_CNTL_PID_EN(0)      |
                                       TP_TF_CNTL_NULL_EN(1)     |          // Discard Null Packet
                                       TP_TF_CNTL_TRERR_EN(0)    |
                                       TP_TF_CNTL_SYNC_EN(1)     |          // Discard not sync packet
									   TP_TF_CNTL_RST_EN(0);

    const unsigned long FrameCfgInit = TP_TF_FRMCFG_SYNC_BYTE(0x47) |      // sync byte=0x47
                                       TP_TF_FRMCFG_DROPNO(1)       |
                                       TP_TF_FRMCFG_LOCKNO(3)       |		// lock stream by get third sync byte
                                       TP_TF_FRMCFG_PACKET_SIZE(0)  |       // 00:188B 01:192 10:204
                                       TP_TF_FRMCFG_DATA_ORDER(0)   |       // data order = bit 0 is first bit
                                       TP_TF_FRMCFG_FRM_EN(0)       |       // disable framer
                                       TP_TF_FRMCFG_FORCEDROP(0)    |
                                       TP_TF_FRMCFG_SYNCMODE(0x00)  |       // external input:1F
                                       TP_TF_FRMCFG_SERIAL(0)       |       // serial mode
                                       TP_TF_FRMCFG_DATAPIN(0)      |
                                       TP_TF_FRMCFG_ERR_POL(0)      |       // active high
                                       TP_TF_FRMCFG_SYNC_POL(0)     |       // active high
                                       TP_TF_FRMCFG_VAL_POL(0)      |       // active high
                                       TP_TF_FRMCFG_CLK_POL(0);            // active high

	if (id>MAX_TP_COUNT)
    {
        printf("reset tp failed, invalid id - %d\n", id);
		return -1;
	}

    //printf("Reset TP Framer %d\n", id);
    SET_TP_CNTL(id, 0x1FFFFE);          // clear CNTL
    SET_TP_CNTL(id, CntlInit | TP_TF_CTRL_ERR_FIX_EN_BIT | TP_TF_CNTL_WRITE_DATA(1));
    SET_TP_FRMCFG(id, FrameCfgInit);

	SET_TP_CNT(id, 0);
	SET_TP_DRP_CNT(id, 0);
	SET_TP_ERR_CNT(id, 0);

    return 0;
}
/***** TP_P framer *****/
static int tpp_framer_reset(unsigned char id)
{
	const unsigned long CntlInit     = TP_TF_CNTL_CI_MODE_EN(1)  | // use sync replace this bit must be enable
									   TP_TF_CTRL_ERR_FIX_EN(0)  |
									   TP_TF_CNTL_STRM_ID_EN(0)  |
                                       TP_TF_CNTL_BUF_RDY_CTL(1) |
                                       TP_TF_CNTL_PSC_EN(0)      |
                                       TP_TF_CNTL_PES_EN(0)      |
                                       TP_TF_CNTL_TSC_EN(0)      |
                                       TP_TF_CNTL_TB(0)          |
                                       TP_TF_CNTL_BUSY(0)        |
                                       TP_TF_CNTL_MODE(0)        |
                                       TP_TF_CNTL_DU_EN(0)       |
                                       TP_TF_CNTL_DE_EN(0)       |
                                       TP_TF_CNTL_XT_EN(0)       |
                                       TP_TF_CNTL_PID_EN(0)      |
                                       TP_TF_CNTL_NULL_EN(0)     |          // Discard Null Packet
                                       TP_TF_CNTL_TRERR_EN(0)    |
                                       TP_TF_CNTL_SYNC_EN(0)     |          // Discard not sync packet
                                       TP_TF_CNTL_RST_EN(0);

    const unsigned long FrameCfgInit = TP_TF_FRMCFG_SYNC_BYTE(0x47)|       // sync byte = 0x47
                                       TP_TF_FRMCFG_DROPNO(1)      |
                                       TP_TF_FRMCFG_LOCKNO(3)      |       // lock stream by get third sync byte
                                       TP_TF_FRMCFG_PACKET_SIZE(0) |       // packet size = 188B
                                       TP_TF_FRMCFG_DATA_ORDER(0)  |       // data order = bit 0 is first bit
                                       TP_TF_FRMCFG_FRM_EN(0)      |       // disable framer
                                       TP_TF_FRMCFG_FORCEDROP(0)   |
                                       TP_TF_FRMCFG_SYNCMODE(0x00) |       // external input:1F
                                       TP_TF_FRMCFG_SERIAL(0)      |       // 0:paralle mode 1:serial mode
                                       TP_TF_FRMCFG_DATAPIN(0)     |
                                       TP_TF_FRMCFG_ERR_POL(0)     |       // active high
                                       TP_TF_FRMCFG_SYNC_POL(0)    |       // active high
                                       TP_TF_FRMCFG_VAL_POL(0)     |       // active high
                                       TP_TF_FRMCFG_CLK_POL(0);            // active high

	if (id>2)
	{
		printf("reset tpp failed, invalid id - %d\n", id);
		return -1;
	}

	// printf("Reset TP%d_P Framer ... \n", id);

	SET_TPP_CNTL(id, 0x1FFFFE);          // clear CNTL
	// printf("Clear CNTL to TP%d_P_CRTL=0x%p\n", id, GET_TPP_CNTL(id) );
	SET_TPP_CNTL(id, CntlInit | TP_TF_CNTL_WRITE_DATA(1));
	SET_TPP_FRMCFG(id, FrameCfgInit);
	SET_TPP_CNT(id, 0);
	SET_TPP_DRP_CNT(id, 0);

	// printf("TP%d_P_FRMCGF=0x%p TP%d_P_CRTL=0x%p\n", id, GET_TPP_FRMCFG(id), id, GET_TPP_CNTL(id) );

	return 0;
}

static int tpo_reset(void)
{
	const unsigned long TPOCntlInit  =  TP_OUT_SHARE_MODE(TPOUT_SHARE_SW_MODE) |
										TP_OUT_SHARE_EN(0)			           |
										TP_OUT_STOP_MODE(TP_OUT_STOP_PACKET)   |
										TP_OUT_SYNC_DUR(TP_OUT_SYNC_DUR_8)	   |
										TP_OUT_CTRL_DATA_SEL(TP_OUT_CTRL_DATA_SEL_TP);
										
    //printf("Reset TPO %d\n", id);
	SET_TPO_FRMCFG(0);                // clean tpo
    SET_TPO_CTRL(TPOCntlInit);

    return 0;
}

/****** TP buffer *****/
int tp_buff_reset(unsigned char id)
{
    // printf("Reset Ring Buffer %d\n", id);
    return tp_buff_set_boundrary(id, 0, 0, 0, 0);
}
/****** TP_P buffer *****/
int tp_p_buff_reset(unsigned char id)
{
    // printf("Reset TP_P_%d Ring Buffer \n", id);
    return tp_p_buff_set_boundrary(id, 0, 0, 0, 0);
}
/****** MTP buffer *****/
int mtp_reset(unsigned char id)
{
    printf("Reset MTP %d\n", id);
    SET_MTP_RING_CTRL(id, 0);
    return mtp_set_boundrary(id, 0, 0, 0, 0);
}
/***** TPP MTP buffer *****/
int tpp_mtp_reset(unsigned char id)
{
    //printf("Reset TP_P MTP %d\n", id);
    SET_TPP_MTP_RING_CTRL(id, 0);
    return tpp_mtp_set_boundrary(id, 0, 0, 0, 0);
}
/***** TP reset *****/
int tp_reset(void)
{
	int i;
	// TP Framer Init
	for (i=0; i<MAX_TP_COUNT; i++)
		tp_framer_reset(i);

	// TP Ring buffer initialization
	for (i=0; i<MAX_TP_RING_BUFFER; i++)
		tp_buff_reset(i);

	//for (i=0; i<3; i++)
	//	mtp_reset(i);

	// PID filiter initialization
	for (i=0; i<96; i++)
		tp_pid_reset(i);

	return 0;
}
/***** TPP reset *****/
int tp_p_reset(void)
{
	int i;

	// TPP Framer Init
	for (i=0; i<MAX_TP_P_COUNT; i++)
		tpp_framer_reset(i);
	
	// set tpp sync replace
	rtd_outl(TP_P_SYNC_RPLACE, 0x00000047);
	// TP_P Ring buffer initialization
	for (i=0; i<MAX_TP_P_RING_BUFFER; i++)
		tp_p_buff_reset(i);

	for (i=0; i<3; i++)
		tpp_mtp_reset(i);

	printf("[TP_DDTS] tp_p_reset (tpp_framer,tp_p_buffer,tpp_mtp!\n");

	return 0;
}



/********************************************************************************
                         Read/Release data function
*********************************************************************************/
/***** TP buffer *****/
int tp_buff_read(unsigned char id, unsigned long* p_data, unsigned long* p_len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tp_buff_get_boundrary(id, &base, &limit, &rp, &wp);

	*p_data = rp;

	if (wp >=rp)
		*p_len = wp -rp;
	else
		*p_len = rp - wp;

	return (*p_len) ? 0 : -1;
}

int tp_buff_release(unsigned char id, unsigned long data, unsigned long len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tp_buff_get_boundrary(id, &base, &limit, &rp, &wp);

	if (data != rp)
    {
		printf("release data failed, data  %08lx != rp %08lx\n", data, rp);
		tp_buff_dump_status(id);
		return -1;
	}

	data += len;
	if (data >= limit)
		data = base;

	printf("rp = %08lx -> %08lx\n", rp, data);

	rp = data;

	tp_buff_set_boundrary(id, base, limit, rp, wp);

	return 0;
}

/***** TP_P buffer *****/
int tp_p_buff_read(unsigned char id, unsigned long* p_data, unsigned long* p_len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tp_p_buff_get_boundrary(id, &base, &limit, &rp, &wp);

	*p_data = rp;

	if (wp >=rp)
		*p_len = wp -rp;
	else
		*p_len = rp - wp;

	return (*p_len) ? 0 : -1;
}

int tp_p_buff_release(unsigned char id, unsigned long data, unsigned long len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tp_p_buff_get_boundrary(id, &base, &limit, &rp, &wp);

	if (data != rp)
	{
		printf("release data failed, data  %08lx != rp %08lx\n", data, rp);
		tp_p_buff_dump_status(id);
		return -1;
	}

	data += len;
	if (data >= limit)
		data = base;

	printf("tp_p_buffer release : rp = %08lx -> %08lx\n", rp, data);

	rp = data;

	tp_p_buff_set_boundrary(id, base, limit, rp, wp);

	return 0;
}

/***** TPP MTP buffer *****/
int tpp_mtp_read(unsigned char id, unsigned long* p_data, unsigned long* p_len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tpp_mtp_get_boundrary(id, &base, &limit, &rp, &wp);

	*p_data = rp;

	if (wp >=rp)
		*p_len = wp -rp;
	else
		*p_len = rp - wp;

	return (*p_len) ? 0 : -1;
}

int tpp_mtp_release(unsigned char id, unsigned long data, unsigned long len)
{
	unsigned long base;
	unsigned long limit;
	unsigned long rp;
	unsigned long wp;

	tpp_mtp_get_boundrary(id, &base, &limit, &rp, &wp);

	if (data != rp)
    {
		printf("release data failed, data  %08lx != rp %08lx\n", data, rp);
		tpp_mtp_dump_status(id);
		return -1;
	}

	data += len;
	if (data >= limit)
		data = base;

	printf("rp = %08lx -> %08lx\n", rp, data);

	rp = data;

	tpp_mtp_set_boundrary(id, base, limit, rp, wp);

	return 0;
}




/********************************************************************************
                         DDTS function
*********************************************************************************/
// eva change buffer size
//#define DDR_BUFFER_SIZE    (188 * 1024 * 5)
#define DDR_BUFFER_BASE        0x02000000
#if TP_LOAD_TS_BY_PROBE_SCRIPT
#define MTP_BUFFER_BASE        0x00030000
#define DDR_BUFFER_SIZE        (188 * 1024 * 42)
#else
#define MTP_BUFFER_BASE        0x04000000
#define DDR_BUFFER_SIZE        (188 * 1024 * 100)
#endif
#define TP_DDR_BUFFER_BASE     0x06000000

void _load_ts(unsigned char* p_data, unsigned long size)
{
    int cnt = 0;
	printf("load_ts...");

	while(size >=188)
    {
		#if 1
		p_data[0] = 0x47;
		p_data[1] = 0x00;
		p_data[2] = 0x01;
		p_data[3] = 0x10 | (cnt & 0xF);
		#else
		p_data[0] = 0x47;
		p_data[1] = 0xff;
		p_data[2] = 0xff;
		p_data[3] = 0xff;
		#endif
		memset(p_data + 4, (unsigned char) cnt, 184);
		cnt++;
		p_data += 188;
		size   -= 188;
		
		if (cnt % 20)
			printf(".");
		
	}
	
	printf("finish! size=%d cnt=%d\n", size, cnt);
}

void _dump_ts(unsigned char* p_data, unsigned long size)
{
	while(size >=188)
    {
		printf("TSP[%p] = %02x %02x %02x %02x %02x %02x ....\n",
				p_data, p_data[0], p_data[1], p_data[2], p_data[3], p_data[4], p_data[5]);
		p_data += 188;
		size   -= 188;
	}
}

#if 0
int tpout_set_mode( int mode, int master, int out_stop_mode )
{
	// mode = 0 --> s/w mode, mode = 1 --> h/w mode
    // master = 0 --> tp is master, master = 1 --> pcmcia is master
	// out_stop_mode = 0 --> stop right away, out_stop_mode = 1 --> stop tp when get a packet finish
	
	// debug output
	if ( mode )
		printf("[TPO] h/w mode\n");
	else
		printf("[TPO] s/w mode\n");
	
	if ( master )
		printf("[TPO] PCMCIA is the master control\n");
	else
		printf("[TPO] TP is the master control\n");
	
	if ( out_stop_mode )
		printf("[TPO] stop TP right away!\n");
	else
		printf("[TPO] stop TP after get a packet finish!\n");
	
	if ( out_step_mode == 0 )
		SET_TPO_CTRL( GET_TPO_CTRL() & ~TP_OUT_STOP_MODE(TP_OUT_STOP_RIGHT_AWAY));
	else
		SET_TPO_CTRL( GET_TPO_CTRL() | TP_OUT_STOP_MODE(TP_OUT_STOP_PACKET));

	if ( mode ) // hw mode
	{
		
	}
	else // sw mode
	{
		if( master ) // PCMCIA
		{
			SET_TPO_CTRL( GET_TPO_CTRL() & ~TP_OUT_SHARE_MODE(TPOUT_SHARE_SW_MODE));
			SET_TPO_CTRL( GET_TPO_CTRL() | TP_OUT_CTRL_DATA_SEL(TP_OUT_CTRL_DATA_SEL_PCMCIA));
		}
		else
		{
			SET_TPO_CTRL( GET_TPO_CTRL() | TP_OUT_SHARE_MODE(TPOUT_SHARE_HW_MODE));
			SET_TPO_CTRL( GET_TPO_CTRL() & ~TP_OUT_CTRL_DATA_SEL(TP_OUT_CTRL_DATA_SEL_TP));
		}
	}
/*
	if ( out_share_en == 0 )
		SET_TPO_CTRL( GET_TPO_CTRL() & ~TP_OUT_SHARE_EN_BIT);
	else
		SET_TPO_CTRL( GET_TPO_CTRL() | TP_OUT_SHARE_EN_BIT);
*/

#if 1 //TP_DEBUG
	printf("TPO CTRL 0x08%x\n", GET_TPO_CTRL());
#endif
	return 0;
}
#endif

#if 0
static int do_mtp_in_test()
{
	unsigned char tp_idx = 0;
	unsigned char buff_idx = 0;
	int i;
	int rx_cnt = 0;
	int wait_cnt = 0;

    tp_reset();
	mtp_set_boundrary(tp_idx, MTP_BUFFER_BASE, MTP_BUFFER_BASE + DDR_BUFFER_SIZE, MTP_BUFFER_BASE, MTP_BUFFER_BASE + DDR_BUFFER_SIZE);
	tp_buff_set_boundrary(buff_idx, DDR_BUFFER_BASE,  DDR_BUFFER_BASE + DDR_BUFFER_SIZE, DDR_BUFFER_BASE, DDR_BUFFER_BASE);

	SET_TP_CNTL(tp_idx, (1<<15) |TP_TF_CNTL_MODE(1) | TP_TF_CNTL_WRITE_DATA(1));  // set to memory mode
	SET_TP_FRMCFG(tp_idx, GET_TP_FRMCFG(tp_idx) | TP_TF_FRMCFG_FRM_EN(1));   // frame enable
	printf("Start TP Framer %d\n", tp_idx);

	_load_ts((unsigned char*) MTP_BUFFER_BASE, DDR_BUFFER_SIZE);

	SET_MTP_RING_CTRL(tp_idx, 0x5);
	printf("Set to mem mode CNTL to TP%d_CRTL=0x%p\n", tp_idx, GET_TP_CNTL(tp_idx) );
	printf("Frame enable TP_FRMCF to TP%d_FRMCFG=0x%p\n", tp_idx, GET_TP_FRMCFG(tp_idx) );
	for (i=0; i<50; i++)
    {
		unsigned char* data;
		unsigned long len;
		tp_dump_status(0);
#if 0
		if (tp_buff_read(buff_idx, (unsigned long*) &data, &len)==0)
	    {
			int j;
			for (j=0; j<len; j+=188)
		    {
				printf("TSP[%d] addr=%08x, %02x %02x %02x %02x %02x %02x %02x ...\n",
						rx_cnt++, data, data[j], data[j+1], data[j+2], data[j+3], data[j+4] , data[j+5], data[j+6]);

			}
			wait_cnt = 0;

			tp_buff_release(buff_idx, data, len);
		}
		else
		{
			wait_cnt++;
		}
#endif
		if (wait_cnt >= 100)
		{
			tp_dump_status(tp_idx);
			mtp_dump_status(tp_idx);
			tp_buff_dump_status(buff_idx);
			wait_cnt = 0;
		}

		mdelay(10);
	}

	SET_MTP_RING_CTRL(tp_idx, 0x4);
	printf("Stop TP Framer %d\n", tp_idx);
	SET_TP_FRMCFG(tp_idx, GET_TP_FRMCFG(tp_idx) & ~TP_TF_FRMCFG_FRM_EN(1));   // frame enable

    return 0;
}
#endif
// eva
static int do_mtp_in_to_tpp_test(unsigned char tp_idx, unsigned char tp_p_idx)
{
	int i; // cunter
	unsigned char buff_idx = 0; // tp ring buff index, default set to 0
	
	printf("========================================\n");
	printf("=========== TPP DDTS START!=============\n");
	printf("========================================\n");
	printf("=====Path:MTP->TP%d_P->TPO==========\n", tp_p_idx);
	printf("========================================\n");
	
	// Init
	tp_reset();
	tp_p_reset();
	mtp_reset(tp_idx);
	tpo_reset();

#if TP_LOAD_TS_BY_PROBE_SCRIPT 
	// load ts from probe script
	// load d:\kevin_wang\tsp1_ref.ts 0x30000
#else
	// load ts to MTP
	_load_ts((unsigned char*) MTP_BUFFER_BASE, DDR_BUFFER_SIZE);
	mdelay(100);
#endif
	
	tpp_mtp_set_boundrary(tp_p_idx, MTP_BUFFER_BASE, MTP_BUFFER_BASE + DDR_BUFFER_SIZE  + 376, MTP_BUFFER_BASE, MTP_BUFFER_BASE + DDR_BUFFER_SIZE);
	tpp_mtp_dump_status(tp_p_idx);

	tp_p_buff_set_boundrary(buff_idx, DDR_BUFFER_BASE,  DDR_BUFFER_BASE + DDR_BUFFER_SIZE + 376, DDR_BUFFER_BASE, DDR_BUFFER_BASE);
	tp_p_buff_dump_status(buff_idx);

	tp_buff_set_boundrary(tp_idx, TP_DDR_BUFFER_BASE,  TP_DDR_BUFFER_BASE + DDR_BUFFER_SIZE + 376, TP_DDR_BUFFER_BASE, TP_DDR_BUFFER_BASE);
	tp_buff_dump_status(tp_idx);
	
	// TPP setting
	printf("Original_TP%d_P_CRTL=0x%08x\n", tp_p_idx, GET_TPP_CNTL(tp_p_idx) );
	// set ci_mode(because use tpp must be ci_mode) and mode to 1, then tpp will start to work.
	SET_TPP_CNTL(tp_p_idx, TP_TF_CNTL_MODE(1) | TP_TF_CNTL_CI_MODE_EN(1)| TP_TF_CNTL_WRITE_DATA(1));  // set to memory mode
	printf("Memory_modeTP%d_P_CRTL=0x%08x\n", tp_p_idx, GET_TPP_CNTL(tp_p_idx) );

	printf("Original_TP%d_P_FRMCFG=0x%08x\n", tp_p_idx, GET_TPP_FRMCFG(tp_p_idx) );
	SET_TPP_FRMCFG(tp_p_idx, GET_TPP_FRMCFG(tp_p_idx) | TP_TF_FRMCFG_FRM_EN(1));   // frame enable -- tp_p will start to work
	printf("Frame_enable_TP%d_P_FRMCFG=0x%08x\n", tp_p_idx, GET_TPP_FRMCFG(tp_p_idx) );

	// TP OUT
	printf("Original_TPO_FRMCFG=0x%08x\n", GET_TPO_FRMCFG() );
	SET_TPO_FRMCFG(TPOUT_IN_TPP_OUT_EN(1) | TPOUT_SRC(TPOUT_SRC_MEM)); // in_tpp_out_en, suppose data_src wiil change to tpp if set to tpp out en
	printf("TPOENABLE_TPO_FRMCFG=0x%08x\n", GET_TPO_FRMCFG() );
	
	printf("Original_TPOUT_CTRL=0x%08x\n", GET_TPO_CTRL() );
	SET_TPO_CTRL(TP_OUT_SHARE_MODE(TPOUT_SHARE_SW_MODE) | TP_OUT_SHARE_EN(0));
	printf("MEM_TPPO_TPOUT_CTRL=0x%08x\n", GET_TPO_CTRL() );
	
	//tpout_set_mode(0, 0, 1); // sw mode, tp master, stop right away

#if 0 // TPO->TP
	// TP0 setting
	SET_TP_CNTL(tp_idx, TP_TF_CNTL_BUF_RDY_CTL(1) | TP_TF_CNTL_SRC_SEL_EN(0)|TP_TF_CNTL_CI_MODE_EN(1)| TP_TF_CNTL_WRITE_DATA(1));  // set to data from external interface (PORT_0)
	rtd_outl(TP_SYNC_RPLACE, 0x47);
	printf("TP%d_CRTL=0x%08x\n", tp_idx, GET_TP_CNTL(tp_idx) );
	SET_TP_FRMCFG(tp_idx, GET_TP_FRMCFG(tp_idx) | TP_TF_FRMCFG_SERIAL(0)| TP_TF_FRMCFG_FRM_EN(1)|TP_TF_FRMCFG_CLK_POL(1));   // frame enable
	printf("TP%d_P_FRMCFG=0x%08x\n", tp_idx, GET_TP_FRMCFG(tp_idx) );
#endif

	// turn on
	SET_TPP_MTP_RING_CTRL(tp_p_idx, 0x7);         // enable MTP to TPP(in_go) and TPP ddr OUT(go)
	printf("tpp mtp 0x%08x\n",GET_TPP_MTP_RING_CTRL(tp_p_idx));
	// print ts data from tp_p buffer
	printf("------- Print TSP from tp_p buffer -------\n");
	for (i=0; i<100; i++)
	{
		#if 1
		// keep track the status
		printf("------trace line[%d]-----\n", i);
		printf("TPO FRMCFG =0x%08x\n", GET_TPO_FRMCFG() );
		printf("TPO CTRL =0x%08x\n", GET_TPO_CTRL() );
		//printf("TPP MTP CTRL =0x%08x\n", GET_TPP_MTP_RING_CTRL(tp_p_idx) );
		tpp_mtp_dump_status(tp_p_idx);
		tp_p_dump_status(tp_p_idx);
		tp_p_buff_dump_status(buff_idx);
		//tp_dump_status(tp_idx);
		//tp_buff_dump_status(tp_idx);
		//dump_tp_info(0);
		//dump_tpp_info(0);
		#endif

		#if 1 // liangliang pcmcia code
		if (i%20==0)
		{
			mdelay(2);
			// stop tp out right away
			//tpout_set_mode(0);  /*Need to modify, only need to set out stop mode.*/
			printf("=======$ TP OUT STOP! mode=stop right away $======\n");

			/*auto_pinmux_en:1: auto pin mux, 0: no auto pin mux*/
			/*hw_mode_en:1: HW mode, 0: SW mode*/
			//extern void pcmcia_fpga_test(int auto_pinmux_en, int hw_mode_en);/*Just for convience*/
			//pcmcia_fpga_test(1, 0);

			//mdelay(10);
			//tpout_set_mode(0);/*Need to modify, only need to set out stop mode.*/

			printf("=======$ TP OUT START! $======\n");
		}

		#endif



		mdelay(10);
	}

	// turn off(only off tpp ddr)
	//SET_TPP_MTP_RING_CTRL(tp_p_idx, 0x4);
	//printf("==========Stop TP_P Framer %d==========\n", tp_p_idx);

	//SET_TPP_FRMCFG(tp_p_idx, GET_TPP_FRMCFG(tp_p_idx) & ~TP_TF_FRMCFG_FRM_EN(1));   // tpp frame disable
	//printf("Frame enable TPP_FRMCF to TP%d_P_FRMCFG=0x%08x\n", tp_p_idx, GET_TPP_FRMCFG(tp_p_idx) );
	// TODO: TP SET_TP_FRMCFG(tp_idx, GET_TP_FRMCFG(tp_idx) & ~TP_TF_FRMCFG_FRM_EN(1));   // tp frame disable
	//printf("Frame enable TP_FRMCF to TP%d_P_FRMCFG=0x%08x\n", tp_idx, GET_TP_FRMCFG(tp_idx) );

	printf("========================================\n");
	printf("=========== TPP DDTS END!=============\n");
	printf("========================================\n");
	
	tp_p_dump_status(tp_p_idx);
	tp_p_buff_dump_status(tp_p_idx);
	tp_buff_dump_status(tp_idx);
    return 0;
}



static int do_tp_ddts(cmd_tbl_t * cmdtp, int flag, int argc, char * const argv[])
{

	/*intf("argc = %d\n", argc);
	if (argc <2)
		return CMD_RET_USAGE;
    */
	// kevin
	//do_mtp_in_test();
	// eva
	do_mtp_in_to_tpp_test(0,0);
	return 0;
}


U_BOOT_CMD(
	tp_ddts,	    // name
	6, 		        // max arguments
	1, 		        // repeatable
	do_tp_ddts,	    // cmd
	"TP DDTS",      // usage
	"tp_ddts [test number]\n"
);
