#include <common.h>
#include <asm/arch/io.h>
#include <platform_lib/board/pcb.h>
#include <platform_lib/board/pcb_bga_library.h>
#include <flashdev_s.h>
#if defined(Config_Boot_Flash_Type_is_SPI)
#include <u-boot/md5.h>

typedef struct fsw_ev {
	unsigned long long	header;		/* # Magic header 8 bytes */
	unsigned long long	board_ev;	/* Board_EV 8 bytes */
	unsigned long long	enable_rs232c;	/* enable_rs232c 8 bytes */
	unsigned long long	b2b_tuner_disable;	/* b2b_tuner_disable 8 bytes */
	unsigned long long	disable_composite;	/* disable_composite 8 bytes */
	unsigned long long	dtv_satellite;	/* dtv_satellite 8 bytes */
	uint8_t	reserved[960];	/* reserved 968*/
	uint8_t	hash[16];		/* md5 */
} fsw_ev_t;
#endif

extern unsigned int env_setup_env_pcb(void);
extern int CusApi_env_readXmlFile(const char* pFileName, const char *pItemName, char *pParameter, size_t parameterLen);
extern int CusApi_env_readXmlFile_FSW(const char* pFileName, const char *pItemName, char *pParameter, size_t parameterLen);
#define MODEL_ENV_FILE_NAME		"tmp/factory/Model_Env.xml"
#define EARC_ARC_HW_VAR "earc_arc_hw_var"
#define MIC_HW_VARIATION "mic_hw_variation"
#define ENABLE_BUILTIN_MICROPHONE "enable_builtin_microphone"
#define BUILTIN_MIC "builtin_mic"
#define ENABLE_RS232C "enable_rs232c"
#define B2B_TUNER_DISABLE "b2b_tuner_disable"
#define DISABLE_COMPOSITE "disable_composite"
#define DTV_SATELLITE "dtv_satellite"
#define DDR_SS_SETTING "ddr_ss_setting"

#define BOARD_EV_FILE_NAME		"/BoardEV.xml"
extern bool fsw_emmc_bootup;
extern bool fsw_spi_bootup;

void set_EARC_ARC_pinmux(void)
{
	//earc_arc_hw_var define:
	//0x00: set pin mux as ARC(SPDIF)
	//0x01: set pin mux as GPIO
	//0x02: set pin mux as GPIO
	//NULL: set pin mux as ARC(SPDIF)
	//default setting is ARC(SPDIF) defined in pcb_tv002_bga_Vulcan3
	char retStr[6]={0};
	if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, EARC_ARC_HW_VAR,retStr, sizeof(retStr)) != 0) {
		printf ("Failed to load %s from Model_Env.xml, keep default pin mux!\n", EARC_ARC_HW_VAR);
	}
	else {
		if (strcmp("0x00", retStr) != 0) {//no mount ARC circuit, ARC pin need to be set as GIPO
			rtd_maskl(0xB8060214, 0xFFFFFF00, (((0xF << 4) | (BGA_PIN_609_UP << 3) | (BGA_PIN_609_PD << 2) |(BGA_PIN_609_DS << 1) | (BGA_PIN_609_SR)) << 0));
			//printf ("%s is %s, set as GIPO\n", EARC_ARC_HW_VAR, retStr);
		}
		else {
			printf ("%s is 0x00, keep default pin mux\n", EARC_ARC_HW_VAR);
		}
	}
}

void FFM_LED_RollBack(void){

	#define PIN_PWM_LED_TALLY_GPIO_INFO_APR	PCB_PINMUX_GPIO_INFO(0xB8060200, 16, 19, 0xF, 0xF, PCB_PIN_TYPE_GPIO, 1, 15)
	#define PIN_PWM_LED_R_GPIO_INFO_APR		PCB_PINMUX_GPIO_INFO(0xB806020C, 28, 31, 0xF, 0xF, PCB_PIN_TYPE_GPIO, 1, 4)
	#define PIN_PWM_LED_G_GPIO_INFO_APR		PCB_PINMUX_GPIO_INFO(0xB8060210, 4, 7, 0xF, 0xF, PCB_PIN_TYPE_GPIO, 1, 11)
	#define PIN_PWM_LED_B_GPIO_INFO_APR		PCB_PINMUX_GPIO_INFO(0xB8060208, 20, 23, 0xF, 0xF, PCB_PIN_TYPE_GPIO, 1, 1)

	//tally led(PWM -> GPIO) 0xB8060200[19:16]; //F: ISO_MIS_GPIO_bit15,<I/O>
	rtd_maskl(0xB8060200, 0xFFF0FFFF, (0xF << 16));

	//R LED(PWM -> GPIO)0xB806020C[31:28];
	rtd_maskl(0xB806020C, 0x0FFFFFFF, (0xF  << 28));

	//G LED(PWM -> GPIO) 0xB8060210[7:4]
	rtd_maskl(0xB8060210, 0xFFFFFF0F, (0xF << 4));

	//B LED(PWM -> GPIO) 0xB8060208[23:20]
	rtd_maskl(0xB8060208, 0xFF0FFFFF, (0xF << 20));

	pcb_mgr_set_enum_info_byname("PIN_PWM_LED_TALLY_GPIO_INFO", PIN_PWM_LED_TALLY_GPIO_INFO_APR);
	pcb_mgr_set_enum_info_byname("PIN_PWM_LED_R_GPIO_INFO", PIN_PWM_LED_R_GPIO_INFO_APR);
	pcb_mgr_set_enum_info_byname("PIN_PWM_LED_G_GPIO_INFO", PIN_PWM_LED_G_GPIO_INFO_APR);
	pcb_mgr_set_enum_info_byname("PIN_PWM_LED_B_GPIO_INFO", PIN_PWM_LED_B_GPIO_INFO_APR);

}


void set_ddr_ssc(void){ //from 0.5% to 1% for 1333MHz

	//rtd_maskl(0xb80be01c, 0xfffffffd, 0x00000000); //[1]En_ssc=1
	//rtd_maskl(0xb80be028, 0xfbffffff, 0x00000000); //[26]oc_en=0
	rtd_maskl(0x180be024, 0xf800ffff, 0x02F60000); // dpi_f_code_t = 758
	rtd_maskl(0x180be028, 0xffff00ff, 0x00002E00); // dpi_n_code_t = 46
	//target=1333, SSCG=3%,60KHz,DDR3
	rtd_maskl(0x180be01c, 0xfffffff7, 0x00000008); //[3]ssc_flag_init=1
	rtd_maskl(0x180be028, 0xfbffffff, 0x00000000); //[26]oc_en=0
	rtd_maskl(0x180be028, 0xffffff00, 0x0000002D); //[7:0]Ncode_ssc=45
	rtd_maskl(0x180be024, 0xfffff800, 0x00000703); //[10:0]Fcode_ssc=1795
	rtd_maskl(0x180be020, 0xff8fffff, 0x00400000); //[22:20]dot_gran=4
	rtd_maskl(0x180be020, 0xfff80000, 0x000001A7); //[18:0]Gran_set=423(30k)
	rtd_maskl(0x180be028, 0xfbffffff, 0x04000000); //[26]oc_en=1
	rtd_maskl(0x180be028, 0xfbffffff, 0x00000000); //[26]oc_en=0
	rtd_maskl(0x180be01c, 0xfffffffd, 0x00000002); //[1]En_ssc=1

	printf ("set_ddr_ssc(0x%x, 0x%x, 0x%x)\n", rtd_inl(0x180be028),rtd_inl(0x180be024),rtd_inl(0x180be020));
}


#if defined(Config_Boot_Flash_Type_is_SPI)
#define FSW_EV 1024
#define DATA_LEN 1008
#endif

void set_FFM_pinmux(void){
	//default setting is defined in pcb_tv002_bga_Vulcan3
	char retStr[16]={0};
	char mic_hw_variation = 0x00;
	bool enable_builtin_mic = false;
	bool b2b_tuner_disable = false;
	bool enable_rs232c = false;
	bool disable_composite = false;
	bool dtv_satellite = false;
	char ddr_ss_setting = 0x01;

	if(fsw_emmc_bootup == 0 && fsw_spi_bootup == 0)
	{
		setenv(BUILTIN_MIC, "false");
		if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, MIC_HW_VARIATION,retStr, sizeof(retStr)) != 0) {
			printf ("Failed to read %s!\n", MIC_HW_VARIATION);
		}else{
			if (strcmp("0x00", retStr) == 0){
				mic_hw_variation = 0x00;
			}
			else if (strcmp("0x01", retStr) == 0){
				mic_hw_variation = 0x01;
				if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, ENABLE_BUILTIN_MICROPHONE,retStr, sizeof(retStr)) != 0) {
					printf ("Failed to read %s!\n", ENABLE_BUILTIN_MICROPHONE);
				}else{
					if (strcmp("true", retStr) == 0){
						enable_builtin_mic = true;
						setenv(BUILTIN_MIC, "true");
					}
				}
			}
		}
		// B2B pinmux
		if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, B2B_TUNER_DISABLE,retStr, sizeof(retStr)) != 0) {
			printf ("Failed to read %s!\n", B2B_TUNER_DISABLE);
		}else{
			if (strcmp("true", retStr) == 0){
				b2b_tuner_disable = true;
			}
		}
		// B2B pinmux
		if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, ENABLE_RS232C,retStr, sizeof(retStr)) != 0) {
			printf ("Failed to read %s!\n", ENABLE_RS232C);
		}else{
			if (strcmp("true", retStr) == 0){
				enable_rs232c = true;
			}
		}
		// B2C pinmux
		if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, DISABLE_COMPOSITE,retStr, sizeof(retStr)) != 0) {
			printf ("Failed to read %s!\n", DISABLE_COMPOSITE);
		}else{
			if (strcmp("true", retStr) == 0){
				disable_composite = true;
			}
		}
		// B2C pinmux
		if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, DTV_SATELLITE,retStr, sizeof(retStr)) != 0) {
			printf ("Failed to read %s!\n", DTV_SATELLITE);
		}else{
			if (strcmp("1", retStr) == 0){
				dtv_satellite = true;
			}
		}
	}
	else
	{
		// FSW flow
		unsigned long long board_ev = 0x10;
#if defined(Config_Boot_Flash_Type_is_EMMC)
		if(fsw_emmc_bootup == 1)
		{
			// read BoardEV.xml
			if(CusApi_env_readXmlFile_FSW(BOARD_EV_FILE_NAME, "board_ev", retStr, sizeof(retStr)) != 0) {
				printf ("Failed to read %s!\n", BOARD_EV_FILE_NAME);
			}else{
				if (strcmp("0x10", retStr) == 0){
					board_ev = 0x10;
				}else if (strcmp("0x11", retStr) == 0){
					board_ev = 0x11;
				}else if (strcmp("0x12", retStr) == 0){
					board_ev = 0x12;
				}
				//printf("[%s:%d] board_ev = 0x%x\n", __FILE__, __LINE__, board_ev);
			}
			// read BoardEV.xml
			if(CusApi_env_readXmlFile_FSW(BOARD_EV_FILE_NAME, B2B_TUNER_DISABLE, retStr, sizeof(retStr)) != 0) {
				printf ("Failed to read %s!\n", BOARD_EV_FILE_NAME);
			}else{
				if (strcmp("true", retStr) == 0){
					b2b_tuner_disable = true;
				}
				//printf("[%s:%d] b2b_tuner_disable = 0x%x\n", __FILE__, __LINE__, b2b_tuner_disable);
			}
			// read BoardEV.xml
			if(CusApi_env_readXmlFile_FSW(BOARD_EV_FILE_NAME, ENABLE_RS232C, retStr, sizeof(retStr)) != 0) {
				printf ("Failed to read %s!\n", BOARD_EV_FILE_NAME);
			}else{
				if (strcmp("true", retStr) == 0){
					enable_rs232c = true;
				}
				//printf("[%s:%d] enable_rs232c = 0x%x\n", __FILE__, __LINE__, enable_rs232c);
			}
			// read BoardEV.xml
			if(CusApi_env_readXmlFile_FSW(BOARD_EV_FILE_NAME, DISABLE_COMPOSITE, retStr, sizeof(retStr)) != 0) {
				printf ("Failed to read %s!\n", BOARD_EV_FILE_NAME);
			}else{
				if (strcmp("true", retStr) == 0){
					disable_composite = true;
				}
				//printf("[%s:%d] disable_composite = 0x%x\n", __FILE__, __LINE__, disable_composite);
			}
			// read BoardEV.xml
			if(CusApi_env_readXmlFile_FSW(BOARD_EV_FILE_NAME, DTV_SATELLITE, retStr, sizeof(retStr)) != 0) {
				printf ("Failed to read %s!\n", BOARD_EV_FILE_NAME);
			}else{
				if (strcmp("true", retStr) == 0){
					dtv_satellite = true;
				}
				//printf("[%s:%d] b2b_tuner_disable = 0x%x\n", __FILE__, __LINE__, b2b_tuner_disable);
			}
		}
#elif defined(Config_Boot_Flash_Type_is_SPI)
		fsw_ev_t fsw_ev_data;
		if(fsw_spi_bootup == 1)
		{
			// read SPI offset
			// 0x3FEFC00 (0x4000000 - 0x1000)
			unsigned int SPIaddress = 0x3FEFC00;

			if(rtk_spi_nor_read(SPIaddress, FSW_EV, (unsigned char*) &fsw_ev_data) == -1)
			{
				/* Read error */
				printf("Get board_ev data from SPI failed!\n");
			}
			else
			{
				/* Read success */
				//printf("[trace code] Get board_ev data from SPI!\n");
				if (fsw_ev_data.header != 0x123456789ABCDEF)
				{
					printf("magic_header failed!\n");
				}
				else
				{
					// calculate md5
					uint8_t calculate_md5[16]={0};
					md5((unsigned char *)&fsw_ev_data, DATA_LEN, calculate_md5);

					if (memcmp(fsw_ev_data.hash, calculate_md5, 16)!=0)
					{
						printf("check SPI md5 failed!\n");
					}
					else
					{
						printf("check SPI md5 pass!\n");
						board_ev = fsw_ev_data.board_ev;	// board_ev
						b2b_tuner_disable = fsw_ev_data.b2b_tuner_disable;	// b2b_tuner_disable
						enable_rs232c = fsw_ev_data.enable_rs232c;	// enable_rs232c
						disable_composite = fsw_ev_data.disable_composite;	// disable_composite
						dtv_satellite = fsw_ev_data.dtv_satellite;	// dtv_satellite
					}
				}
			}
			printf("[%s:%d] board_ev = 0x%llx\n", __FILE__, __LINE__, board_ev);
		}
#endif
		if(board_ev == 0x10){
			mic_hw_variation = 0x00;
			enable_builtin_mic = false;
		}else if(board_ev == 0x11){
			mic_hw_variation = 0x01;
			enable_builtin_mic = false;
		}else if(board_ev == 0x12){
			mic_hw_variation = 0x01;
			enable_builtin_mic = true;
		}
		setenv("EV", "BoardEV");
	}

	if(CusApi_env_readXmlFile(MODEL_ENV_FILE_NAME, DDR_SS_SETTING,retStr, sizeof(retStr)) != 0) {
		printf ("Failed to read %s!\n", DDR_SS_SETTING);
	}else{
		if (strcmp("0x02", retStr) == 0){
			ddr_ss_setting = 0x02; //ddr sscg is 1%, and default is 0.5%
		}
	}

	printf("== FFM (%d, %d), B2B (%d, %d, %d), B2C (%d, %d) ==\n", mic_hw_variation, enable_builtin_mic, enable_rs232c, b2b_tuner_disable, ddr_ss_setting, disable_composite, dtv_satellite);

	if(ddr_ss_setting == 0x02)
		set_ddr_ssc();

	if(mic_hw_variation == 0x00){
		// vu3
		// AB4, AA6 dmic unused
		//AB4(I2C0_SCL_src0 -> GPIO) 0xB8060210[23:20]
		rtd_maskl(0xB8060210, 0xFF0FFFFF, (0xF<< 20));
		//AA6(I2C0_SDA_src0 -> GPIO) 0xB8060210[31:28]
		rtd_maskl(0xB8060210, 0x0FFFFFFF, (0xF  << 28));
		//M25(DMIC SCL -> GPIO) 0xB8060214[31:28];
		rtd_maskl(0xB8060214, 0x0FFFFFFF, (0xF << 28));
	}else{
		if(b2b_tuner_disable == false)
		{
			// vu3.1 tuner (default)
			//AB4, AA6 tuner
			//AB5, AB6 dmic
		}
		else
		{
			// vu3.2 B2B
			//AB4(I2C0_SCL_src0 -> GPIO) 0xB8060210[23:20]
			rtd_maskl(0xB8060210, 0xFF0FFFFF, (0xF<< 20));
			//AA6(I2C0_SDA_src0 -> GPIO) 0xB8060210[31:28]
			rtd_maskl(0xB8060210, 0x0FFFFFFF, (0xF  << 28));
			//AB5(dmic_clk_data_1_src0 -> GPIO) , 0xB806020C[15:12]; F: ISO_MIS_GPIO_bit06,<I/O>
			rtd_maskl(0xB806020C, 0xFFFF0FFF, (0xF  << 12));
			//AB6(dmic_clk_data_0_src0 -> I2C5_SDA_src0) , 0xB806020C[7:4]; B: I2C5_SDA_src0,<I/O>
			rtd_maskl(0xB806020C, 0xFFFFFF0F, (0xF  << 4));
		}
	}

	if(enable_builtin_mic == false){
		if(mic_hw_variation == 0x00 && b2b_tuner_disable == false){
			// use v3 tuner
			#define I2C_PORT_0_V31		PCB_HW_I2C(PCB_I2C_PHY_5, PCB_I2C_SUB_SRC_0) // AB5/AB6	Tuner I2C5 SDA/SCL AB5 default is I2C5_SCL_src0
			//AB5(dmic_clk_data_1_src0 -> I2C5_SCL_src0) , 0xB806020C[15:12]; B: I2C5_SCL_src0,<I/O>
			rtd_maskl(0xB806020C, 0xFFFF0FFF, (0xB  << 12));
			//AB6(dmic_clk_data_0_src0 -> I2C5_SDA_src0) , 0xB806020C[7:4]; B: I2C5_SDA_src0,<I/O>
			rtd_maskl(0xB806020C, 0xFFFFFF0F, (0xB  << 4));
			pcb_mgr_set_enum_info_byname("I2C_PORT_0", I2C_PORT_0_V31);
		}else{
			// unused dmic
			//AB5(dmic_clk_data_1_src0 -> GPIO) , 0xB806020C[15:12]; F: ISO_MIS_GPIO_bit06,<I/O>
			rtd_maskl(0xB806020C, 0xFFFF0FFF, (0xF  << 12));
			//AB6(dmic_clk_data_0_src0 -> I2C5_SDA_src0) , 0xB806020C[7:4]; B: I2C5_SDA_src0,<I/O>
			rtd_maskl(0xB806020C, 0xFFFFFF0F, (0xF  << 4));

			//M25(DMIC SCL, output low) 0xB8006030 bit0 disable clock function
			rtd_maskl(0xB8006030, 0xFFFFFFFE, 0x0);
		}
		//F3(ISDATA0 -> GPIO) 0xB8000830[31:28];
		rtd_maskl(0xB8000830, 0x0FFFFFFF, (0xF << 28));

		FFM_LED_RollBack();
	}
	else
	{
		if(b2b_tuner_disable == true){
			// vu3.2 B2B unuse tuner, dmic
			//AB4(I2C0_SCL_src0 -> GPIO) 0xB8060210[23:20]
			rtd_maskl(0xB8060210, 0xFF0FFFFF, (0xF<< 20));
			//AA6(I2C0_SDA_src0 -> GPIO) 0xB8060210[31:28]
			rtd_maskl(0xB8060210, 0x0FFFFFFF, (0xF  << 28));
			//AB5(dmic_clk_data_1_src0 -> GPIO) , 0xB806020C[15:12]; F: ISO_MIS_GPIO_bit06,<I/O>
			rtd_maskl(0xB806020C, 0xFFFF0FFF, (0xF  << 12));
			//AB6(dmic_clk_data_0_src0 -> I2C5_SDA_src0) , 0xB806020C[7:4]; B: I2C5_SDA_src0,<I/O>
			rtd_maskl(0xB806020C, 0xFFFFFF0F, (0xF  << 4));
		}
	}
	// B2B pinmux
	if(enable_rs232c == true){
		//AB13(ISO_MIS_GPIO_bit37 -> uart0_tx) , 0xB8060220[15:12]; 7: uart0_tx,<OD>
		rtd_maskl(0xB8060220, 0xFFFF0FFF, (0x7  << 12));
		//Y4(ISO_MIS_GPIO_bit54 -> uart0_rxdi_src8) , 0xB8060230[7:4]; 0: uart0_rxdi_src8,<I>
		rtd_maskl(0xB8060230, 0xFFFFFF0F, (0x0  << 4));
		rtd_maskl(0xB8060248, 0xFFFFFFF0, 0x8);	// 0: uart0_rxdi_src8,<I>
	}
	// B2C pinmux
	if(disable_composite == true){
		//K1 (VIN_10P -> MIS_GPIO_bit53) , 0xB8000838[27:24]; F: MIS_GPIO_bit53,<I/O>
		rtd_maskl(0xB8000838, 0xF0FFFFFF, (0xF  << 24));
		//K2 (VIN_a0n -> MIS_GPIO_bit54) , 0xB8000838[23:20]; F: MIS_GPIO_bit54,<I/O>
		rtd_maskl(0xB8000838, 0xFF0FFFFF, (0xF  << 20));
	}
	// B2C pinmux
	if(dtv_satellite == true){
		//P4 (MIS_GPIO_bit37 -> DEMOD_DSC_22_src1) , 0xB8000828[11:8]; A: DEMOD_DSC_22_src1,<I/O>
		rtd_maskl(0xB8000828, 0xFFFFF0FF, (0xA  << 8));
	}
	env_setup_env_pcb();
}

void CusApi_post_pinmux_init(void)
{
	set_EARC_ARC_pinmux();
	set_FFM_pinmux();
}

