#include <common.h>
#include <asm/arch/bootparam.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <exports.h>
#include <drivers/i2c/rtk_g2c_lib.h>
#include <drivers/i2c/rtk_i2c_lib.h>
#include <panel.h>
#include <asm/arch/panel_config_parameter.h>

#include <ppoverlay_reg.h>
#include <demura_reg.h>
#include <outputgamma_reg.h>
#include <od_reg.h>
#include <invoutputgamma_reg.h>
#include <pcid_reg.h>

#include "cus_common.h"

#define MLVDS_TCON_SETTING_FILENAME_IN_FACTORY FACTORY_HEADER_FILE_NAME"bin_panel/tcon_timming.bin"
#define VALC_FILENAME_IN_FACTORY FACTORY_HEADER_FILE_NAME"bin_panel/tcon_vac.bin"
#define OUTPUT_GAMMA_FILENAME_IN_FACTORY FACTORY_HEADER_FILE_NAME"bin_panel/tcon_gamma.bin"

#define Bin_Num_Output_InvOutput_Gamma 256

typedef enum {
	_PCID2_COLOR_R = 0,
	_PCID2_COLOR_G,
	_PCID2_COLOR_B,
	_PCID2_COLOR_ALL,
	_PCID2_COLOR_ERR
} DRV_pcid2_channel_t;

typedef struct _Tcon_Setting_Data{
       unsigned int type;//0:rtd_outl,1<A1><EA>ortd_maskl,2:rtd_inl,3:delay
       unsigned int addr;
       unsigned int value1;
       unsigned int value2;
}TCON_SETTING_DATA;

typedef struct _Tcon_Setting_Header {
       unsigned int Offset;
       unsigned int endCode;
}TCON_SETTING_HEADER;

typedef struct _VALC_Header {
       unsigned int Offset;
       unsigned int endCode;
}VALC_HEADER;

typedef struct _Output_Gamma_Header {
       unsigned int Offset;
       unsigned int endCode;
}OUTPUT_GAMMA_HEADER;


extern struct _PANEL_CONFIG_PARAMETER  default_panel_parameter;

extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);

//---TCL PM_IC & GAMMA_IC Access Function--->>>

//---GMT gamma & pmu common setting-------
//#define TCON_I2C			0x03	// gamma & pmu ic use i2c_3 access
#define CTRL_OUT_DISABLE    0x00
#define CTRL_OUT_ENABLE		0x02
#define CTRL_WR2NVM			0x08
#define CTRL_RESET			0x10
//----------------------------------------

//---gamma ic is GMT G1632---------
#define BOOT_GAMMA_FILE_NAME	    FACTORY_HEADER_FILE_NAME"bin_panel/tcon_p_gamma.bin"
#define G1632_DEV_ADDR              0x75	// gamma ic slave address
#define G1632_CTRL_ADDR			    0x00	// control sub address
#define G1632_VCOM_ADDR		        0x01	// vcom voltage
#define G1632_GAMMA_BANK_A_ADDR		0x02	// BANK_A sub address
#define G1632_GAMMA_BANK_B_ADDR		0x25	// BANK_B sub address


#define G1632_DATA_NUM          21		// total data number(1 vcom + 21 gamma)

//--------------------------

//---PM ic is GMT G2510S------------
#define BOOT_PMU_FILE_NAME	FACTORY_HEADER_FILE_NAME"bin_panel/tcon_pmu.bin"
#define G2510S_DEV_ADDR         0x70	// pmu ic slave address
#define G2510S_CTRL_ADDR        0x00	// control register address
#define G2510S_DATA_ADDR        0x10	// pum data start

#define G2510S_DATA_NUM         25		// total data number

//--------------------------

//GammaIC
unsigned char G1632_Data[70] =
{
	0x02,               //CTRL
	0x72,				//VCOM

	//BankA
	0x3e, 0xa3, 0xd6, 		//gamma 1/2
	0x33, 0xe2, 0xe3,		//gamma 3/4
	0x2b, 0x82, 0x12,		//gamma 5/6
	0x20, 0x61, 0xab,		//gamma 7/8
	0x19, 0xf1, 0x29,		//gamma 9/A
	0x0f, 0xb0, 0x97,		//gamma B/C
	0x02, 0x10, 0x0e,		//gamma D/E
	0x00, 0x00, 0x00,       //reserved
	0x00, 0x00, 0x00,       //reserved
	0x00, 0x00, 0x00,       //reserved
	0x00, 0x00, 0x00,       //reserved
		
	0x00,                   //reserved
	0x00,                   //reserved
	
	//BankB
	0x00, 0x00, 0x00, 		//gamma 1/2
	0x00, 0x00, 0x00,	    //gamma 3/4
	0x00, 0x00, 0x00,	    //gamma 5/6
	0x00, 0x00, 0x00,		//gamma 7/8
	0x00, 0x00, 0x00,		//gamma 9/A
	0x00, 0x00, 0x00,		//gamma B/C
	0x00, 0x00, 0x00,		//gamma D/E
	0x00, 0x00, 0x00,	    //reserved
	0x00, 0x00, 0x00,		//reserved
	0x00, 0x00, 0x00,       //reserved
	0x00, 0x00, 0x00,       //reserved
};

//PMIC
unsigned char G2510S_data[48] =
{
	0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	0x5e, 0x00, 0x00, 0x12, 0x00, 0x20, 0x00, 0xa7,
	0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0xc5, 0x00, 0x17, 0x0b, 0x00, 0x00, 0x20, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

static bool panel_i2c_write(unsigned char i2c_master,unsigned char dev_addr, unsigned char sub_addr, unsigned char w_num, unsigned char* p_buff)
{
	bool ret = false;
	unsigned char w_buff[w_num + 1];
	
	if(p_buff == NULL)
		return ret;

	w_buff[0] = sub_addr;
	
	for(int i = 1 ; i <= w_num ; i++)	
	{		
		w_buff[i] = *p_buff;		
		p_buff++;
	}	

	if(I2C_Write_EX(i2c_master, dev_addr, w_num+1, w_buff) == S_OK)
		ret = true;

	return ret;
}

static bool panel_i2c_read(unsigned char i2c_master,unsigned char dev_addr, unsigned char sub_addr, unsigned char r_num, unsigned char* p_buff)
{
	if(p_buff == NULL)
		return false;
	if(I2C_Read_EX(i2c_master, dev_addr, 1, &sub_addr, r_num, p_buff) != S_OK)
		return false;

	return true;
}

static bool Check_Reg_Data(unsigned char *pDes, unsigned char *pSrc,unsigned char num)
{
	bool bIsDiff = false;
		
	if(pDes == NULL || pSrc == NULL)
		return false;


	for(int i = 0; i < num;i++)
	{
		if(*(pDes+i) != *(pSrc+i))
		{
			printf("\n033[1;31m===pDes[0x%02x]:0x%02x pSrc[0x%02x]:0x%02x===\033[m\n",i,*(pDes+i),i,*(pSrc+i));
			bIsDiff = true;
			break;
		}
	}
	
	return bIsDiff;
}

static void G1632_get_data(void)
{
	char *dst_addr = NULL;
	int dst_length;
	
	if(factory_read(BOOT_GAMMA_FILE_NAME, &dst_addr, &dst_length))
	{
		printf("\033[1;31m>>>>>>>>>>>>>>>> can't find %s\033[m\n", BOOT_GAMMA_FILE_NAME);
	}
	else
	{
		if(dst_length == sizeof(G1632_Data))
		{
			memcpy(G1632_Data,dst_addr,dst_length);
		}
		else
		{
			printf("\033[1;31mG2510S_data file length is different\033[m\n");
		}
	}
}

static void G1632_i2c_cmd(void)
{
	unsigned char t_buff[G1632_DATA_NUM+1];
	
	G1632_get_data();
	
	if(panel_i2c_read(TCON_I2C,G1632_DEV_ADDR, G1632_VCOM_ADDR, (G1632_DATA_NUM+1), t_buff))
	{
		if(Check_Reg_Data(t_buff,&G1632_Data[G1632_VCOM_ADDR],(G1632_DATA_NUM+1)))
		{
			unsigned char ctrl_flag;
			
			printf("\n\033[1;31m===G1632 register is different from factory table===\033[m\n");
			ctrl_flag = CTRL_OUT_DISABLE;
			panel_i2c_write(TCON_I2C, G1632_DEV_ADDR, G1632_CTRL_ADDR, 1, &ctrl_flag);
			panel_i2c_write(TCON_I2C, G1632_DEV_ADDR, G1632_VCOM_ADDR, (G1632_DATA_NUM+1), &G1632_Data[G1632_VCOM_ADDR]);//VCOM+BANK_A
			IO_Set(PIN_PMU_WP, 0);		//disable WP only write nvm.
			ctrl_flag = CTRL_WR2NVM;
			panel_i2c_write(TCON_I2C, G1632_DEV_ADDR, G1632_CTRL_ADDR, 1, &ctrl_flag);
			mdelay(200);				// must delay 200 mini-seconds.
			IO_Set(PIN_PMU_WP, 1);		//enable WP
			ctrl_flag = CTRL_OUT_ENABLE;
			panel_i2c_write(TCON_I2C, G1632_DEV_ADDR, G1632_CTRL_ADDR, 1, &ctrl_flag);
		}
		else
		{
			printf("\n===G1632 register is the same as factory table===\n");
		}
	}
	else
	{
		printf("\n\033[1;31m===G1632 i2c_read error===\033[m\n");
	}
}

static void G2510S_get_data(void)
{
	char *dst_addr = NULL;
	int dst_length;
	
	if(factory_read(BOOT_PMU_FILE_NAME, &dst_addr, &dst_length))
	{
		printf("\033[1;31m>>>>>>>>>>>>>>>> can't find %s\033[m\n", BOOT_PMU_FILE_NAME);
	}
	else
	{
		if(dst_length == sizeof(G2510S_data))
		{
			memcpy(G2510S_data,dst_addr,dst_length);
		}
		else
		{
			printf("\033[1;31mG1632_Data file length is different\033[m\n");
		}

	}
}

static void G2510S_i2c_cmd(void)
{
	unsigned char t_buff[G2510S_DATA_NUM];
	
	G2510S_get_data();
	
	if(panel_i2c_read(TCON_I2C,G2510S_DEV_ADDR, G2510S_DATA_ADDR, G2510S_DATA_NUM, t_buff))
	{
		if(Check_Reg_Data(t_buff,&G2510S_data[G2510S_DATA_ADDR],G2510S_DATA_NUM))
		{
			unsigned char ctrl_flag;
			
			printf("\n\033[1;31m===G2510S register is different from factory table===\033[m\n");
			ctrl_flag = CTRL_OUT_DISABLE;
			panel_i2c_write(TCON_I2C, G2510S_DEV_ADDR, G2510S_CTRL_ADDR, 1, &ctrl_flag);
			IO_Set(PIN_PMU_WP, 0);		//disable WP both write dac register & nvm
			panel_i2c_write(TCON_I2C, G2510S_DEV_ADDR, G2510S_DATA_ADDR, G2510S_DATA_NUM, &G2510S_data[G2510S_DATA_ADDR]);
			ctrl_flag = CTRL_WR2NVM;
			panel_i2c_write(TCON_I2C, G2510S_DEV_ADDR, G2510S_CTRL_ADDR, 1, &ctrl_flag);
			mdelay(10);
			IO_Set(PIN_PMU_WP, 1);		//enable WP
			ctrl_flag = CTRL_OUT_ENABLE;
			panel_i2c_write(TCON_I2C, G2510S_DEV_ADDR, G2510S_CTRL_ADDR, 1, &ctrl_flag);
		}
		else
		{
			printf("\n===G2510S register is the same as factory table===\n");
		}
	}
	else
	{
		printf("\n\033[1;31m===G2510S i2c_read error===\033[m\n");
	}

}
//---TCL PM_IC & GAMMA_IC Access Function---<<<
#if 1
unsigned char VALC_Table[6][17][17]={
	//Red LUT1 Channel
	{
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
	},
	//Red LUT2 Channel
	{
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
	},
	//Green LUT1 Channel
	{
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
	},
	//Green LUT2 Channel
	{
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
	},
	//Blue LUT1 Channel
	{
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
		{0,	0,	0,	1,	2,	4,	8,	14,	23,	39,	60,	94,	125,	160,	192,	225,	255,	},
	},
	//Blue LUT2 Channel
	{
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
		{0,	20,	42,	65,	86,	108,	130,	152,	173,	194,	213,	227,	238,	245,	251,	254,	255,	},
	},
};

unsigned int OutputGamma_Table[512]=
{
	/* TBL R*/
	0x00001a1b,	0x00351a1a,	0x00691411,	0x008e1111,	0x00b0100d,	0x00cd0e0d,	0x00e80d0d,	0x01020c0d,	//R   1 ~   8
	0x011b0c0c,	0x01330d0c,	0x014c0d0d,	0x01660d0d,	0x01800d0e,	0x019b0e0e,	0x01b70e0e,	0x01d31112,	//R   9 ~  16
	0x01f61111,	0x02181415,	0x02411415,	0x026a1413,	0x02911413,	0x02b81413,	0x02df1413,	0x03061413,	//R  17 ~  24
	0x032d1313,	0x03531313,	0x03791213,	0x039e1213,	0x03c31212,	0x03e71212,	0x040b1312,	0x04301313,	//R  25 ~  32
	0x04561212,	0x047a1212,	0x049e1212,	0x04c21213,	0x04e71213,	0x050c1213,	0x05311212,	0x05551213,	//R  33 ~  40
	0x057a1212,	0x059e1213,	0x05c31213,	0x05e81213,	0x060d1213,	0x06321313,	0x06581313,	0x067e1413,	//R  41 ~  48
	0x06a51313,	0x06cb1313,	0x06f11313,	0x07171412,	0x073d1211,	0x07601211,	0x07831110,	0x07a41110,	//R  49 ~  56
	0x07c51011,	0x07e61010,	0x08061111,	0x08281111,	0x084a1110,	0x086b1110,	0x088c0e0f,	0x08a90f0f,	//R  57 ~  64
	0x08c70e0e,	0x08e30e0d,	0x08fe0e0d,	0x09190d0d,	0x09330e0d,	0x094e0d0e,	0x09690e0e,	0x09850e0d,	//R  65 ~  72
	0x09a00e0d,	0x09bb0e0d,	0x09d60e0e,	0x09f20e0e,	0x0a0e0e0f,	0x0a2b0e0f,	0x0a481010,	0x0a681010,	//R  73 ~  80
	0x0a881011,	0x0aa91010,	0x0ac9100f,	0x0ae81010,	0x0b080e0f,	0x0b250f0f,	0x0b430e0f,	0x0b600e0f,	//R  81 ~  88
	0x0b7d0e0e,	0x0b990e0e,	0x0bb5100f,	0x0bd41010,	0x0bf41111,	0x0c161011,	0x0c370e0e,	0x0c530e0f,	//R  89 ~  96
	0x0c700f10,	0x0c8f0f10,	0x0cae1010,	0x0cce1010,	0x0cee0e0f,	0x0d0b0f0f,	0x0d290f10,	0x0d480f10,	//R  97 ~ 104
	0x0d671010,	0x0d871010,	0x0da71010,	0x0dc7100f,	0x0de60e0d,	0x0e010d0d,	0x0e1b0c0c,	0x0e330c0c,	//R 105 ~ 112
	0x0e4b0d0d,	0x0e650d0d,	0x0e7f0d0d,	0x0e990d0d,	0x0eb30e0e,	0x0ecf0e0d,	0x0eea0e0e,	0x0f060e0e,	//R 113 ~ 120
	0x0f220e0e,	0x0f3e0e0e,	0x0f5a0e0e,	0x0f760e0f,	0x0f930f0f,	0x0fb10f0e,	0x0fce0d0d,	0x0fe80d0a,	//R 121 ~ 128	

	/* TBL G*/
	0x00001b1b,	0x00361b19,	0x006a1614,	0x00941313,	0x00ba1311,	0x00de1211,	0x01011110,	0x01220f10,	//G   1 ~   8
	0x0141100f,	0x01600f10,	0x017f0f10,	0x019e100f,	0x01bd1112,	0x01e01314,	0x02071415,	0x02301515,	//G   9 ~  16
	0x025a1515,	0x02841514,	0x02ad1514,	0x02d61414,	0x02fe1413,	0x03251413,	0x034c1213,	0x03711212,	//G  17 ~  24
	0x03951312,	0x03ba1312,	0x03df1212,	0x04031212,	0x04271212,	0x044b1211,	0x046e1112,	0x04911211,	//G  25 ~  32
	0x04b41211,	0x04d71112,	0x04fa1112,	0x051d1112,	0x05401111,	0x05621211,	0x05851112,	0x05a81112,	//G  33 ~  40
	0x05cb1112,	0x05ee1211,	0x06111211,	0x06341212,	0x06581212,	0x067c1112,	0x069f1212,	0x06c31212,	//G  41 ~  48
	0x06e71212,	0x070b1211,	0x072e1111,	0x07501110,	0x0771100f,	0x07900f0f,	0x07ae0f10,	0x07cd0f0f,	//G  49 ~  56
	0x07eb100f,	0x080a100f,	0x08291010,	0x08490f10,	0x08680f0e,	0x08850f0e,	0x08a20d0d,	0x08bc0d0d,	//G  57 ~  64
	0x08d60d0c,	0x08ef0d0c,	0x09080d0c,	0x09210c0d,	0x093a0d0c,	0x09530d0d,	0x096d0d0c,	0x09860e0c,	//G  65 ~  72
	0x09a00c0d,	0x09b90c0d,	0x09d20c0c,	0x09ea0d0e,	0x0a050d0d,	0x0a1f0d0e,	0x0a3a0e0e,	0x0a560f0f,	//G  73 ~  80
	0x0a74100e,	0x0a920f0f,	0x0ab00f0f,	0x0ace0e0e,	0x0aea0f0e,	0x0b070e0e,	0x0b230e0d,	0x0b3e0d0d,	//G  81 ~  88
	0x0b580e0e,	0x0b740d0d,	0x0b8e0e0e,	0x0baa0e0e,	0x0bc60f0f,	0x0be40f0f,	0x0c020f10,	0x0c210f0f,	//G  89 ~  96
	0x0c3f0e0d,	0x0c5a0e0e,	0x0c760e0f,	0x0c930f0f,	0x0cb10f10,	0x0cd00e0f,	0x0ced0f0f,	0x0d0b0e0e,	//G  97 ~ 104
	0x0d270f10,	0x0d460f0f,	0x0d64100f,	0x0d83100f,	0x0da2100f,	0x0dc1100f,	0x0de00e0e,	0x0dfc0e0d,	//G 105 ~ 112
	0x0e170c0d,	0x0e300c0d,	0x0e490d0d,	0x0e630d0e,	0x0e7e0e0e,	0x0e9a0e0e,	0x0eb60f0e,	0x0ed30f13,	//G 113 ~ 120
	0x0ef51411,	0x0f1a080e,	0x0f301213,	0x0f550d12,	0x0f741115,	0x0f9a1914,	0x0fc71210,	0x0fe90d09,	//G 121 ~ 128	

	/* TBL B*/
	0x00001011,	0x00211011,	0x00421010,	0x00621110,	0x00830f0d,	0x009f0e0d,	0x00ba0d0c,	0x00d30b0c,	//B   1 ~   8
	0x00ea0c0b,	0x01010b0b,	0x01170b0b,	0x012d0b0a,	0x01420b0a,	0x01570c0a,	0x016d0b0b,	0x01830b0b,	//B   9 ~  16
	0x01990b0b,	0x01af0b0c,	0x01c60c0b,	0x01dd0e0f,	0x01fa0f0f,	0x02180f10,	0x02371010,	0x02571010,	//B  17 ~  24
	0x0277100f,	0x0296100f,	0x02b50f0f,	0x02d30f0f,	0x02f10f0f,	0x030f0e10,	0x032d0e0f,	0x034a0e0f,	//B  25 ~  32
	0x03670e0f,	0x03840e0e,	0x03a00e0f,	0x03bd0e0e,	0x03d90e0e,	0x03f50e0e,	0x04110e0e,	0x042d0e0e,	//B  33 ~  40
	0x04490e0e,	0x04650e0e,	0x04810e0e,	0x049d0e0e,	0x04b90e0d,	0x04d40e0e,	0x04f00e0f,	0x050d0e0e,	//B  41 ~  48
	0x05290e0e,	0x05450e0e,	0x05610e0e,	0x057d0e0e,	0x05990e0f,	0x05b60e0e,	0x05d20e0e,	0x05ee0f0e,	//B  49 ~  56
	0x060b0e0f,	0x06280f0e,	0x06450f0f,	0x06630f0f,	0x06810f0e,	0x069e0f0f,	0x06bc0f0f,	0x06da0f0f,	//B  57 ~  64
	0x06f8100f,	0x07170f10,	0x07360e0e,	0x07520e0d,	0x076d0e0d,	0x07880c0e,	0x07a20d0c,	0x07bb0e0d,	//B  65 ~  72
	0x07d60d0d,	0x07f00d0e,	0x080b0e0d,	0x08260e0e,	0x08420e0d,	0x085d0e0d,	0x08780d0c,	0x08910d0c,	//B  73 ~  80
	0x08aa0b0c,	0x08c10c0b,	0x08d80c0b,	0x08ef0b0c,	0x09060b0b,	0x091c0c0b,	0x09330b0c,	0x094a0b0b,	//B  81 ~  88
	0x09600c0c,	0x09780b0c,	0x098f0c0c,	0x09a70c0c,	0x09bf0b0c,	0x09d60c0b,	0x09ed0c0d,	0x0a060c0d,	//B  89 ~  96
	0x0a1f0d0e,	0x0a3a0d0e,	0x0a550e0f,	0x0a720f0e,	0x0a8f0f0f,	0x0aad0f0f,	0x0acb0f0e,	0x0ae80f0f,	//B  97 ~ 104
	0x0b060e0f,	0x0b230e0e,	0x0b3f0e0e,	0x0b5b0e0e,	0x0b770e0e,	0x0b930e0e,	0x0baf1010,	0x0bcf1011,	//B 105 ~ 112
	0x0bf01212,	0x0c141211,	0x0c37100f,	0x0c561011,	0x0c771212,	0x0c9b1212,	0x0cbf1313,	0x0ce5131a,	//B 113 ~ 120
	0x0d121f1b,	0x0d4c1515,	0x0d761212,	0x0d9a1c1f,	0x0dd52129,	0x0e1f2b22,	0x0e6c1000,	0x0e7c0000,	//B 121 ~ 128	

	/* TBL W*/
	0x00001010,	0x00201010,	0x00401010,	0x00601010,	0x00801010,	0x00a01010,	0x00c01010,	0x00e01010,	//W   1 ~   8
	0x01001010,	0x01201010,	0x01401010,	0x01601010,	0x01801010,	0x01a01010,	0x01c01010,	0x01e01010,	//W   9 ~  16
	0x02001010,	0x02201010,	0x02401010,	0x02601010,	0x02801010,	0x02a01010,	0x02c01010,	0x02e01010,	//W  17 ~  24
	0x03001010,	0x03201010,	0x03401010,	0x03601010,	0x03801010,	0x03a01010,	0x03c01010,	0x03e01010,	//W  25 ~  32
	0x04001010,	0x04201010,	0x04401010,	0x04601010,	0x04801010,	0x04a01010,	0x04c01010,	0x04e01010,	//W  33 ~  40
	0x05001010,	0x05201010,	0x05401010,	0x05601010,	0x05801010,	0x05a01010,	0x05c01010,	0x05e01010,	//W  41 ~  48
	0x06001010,	0x06201010,	0x06401010,	0x06601010,	0x06801010,	0x06a01010,	0x06c01010,	0x06e01010,	//W  49 ~  56
	0x07001010,	0x07201010,	0x07401010,	0x07601010,	0x07801010,	0x07a01010,	0x07c01010,	0x07e01010,	//W  57 ~  64
	0x08001010,	0x08201010,	0x08401010,	0x08601010,	0x08801010,	0x08a01010,	0x08c01010,	0x08e01010,	//W  65 ~  72
	0x09001010,	0x09201010,	0x09401010,	0x09601010,	0x09801010,	0x09a01010,	0x09c01010,	0x09e01010,	//W  73 ~  80
	0x0a001010,	0x0a201010,	0x0a401010,	0x0a601010,	0x0a801010,	0x0aa01010,	0x0ac01010,	0x0ae01010,	//W  81 ~  88
	0x0b001010,	0x0b201010,	0x0b401010,	0x0b601010,	0x0b801010,	0x0ba01010,	0x0bc01010,	0x0be01010,	//W  89 ~  96
	0x0c001010,	0x0c201010,	0x0c401010,	0x0c601010,	0x0c801010,	0x0ca01010,	0x0cc01010,	0x0ce01010,	//W  97 ~ 104
	0x0d001010,	0x0d201010,	0x0d401010,	0x0d601010,	0x0d801010,	0x0da01010,	0x0dc01010,	0x0de01010,	//W 105 ~ 112
	0x0e001010,	0x0e201010,	0x0e401010,	0x0e601010,	0x0e801010,	0x0ea01010,	0x0ec01010,	0x0ee01010,	//W 113 ~ 120
	0x0f001010,	0x0f201010,	0x0f401010,	0x0f601010,	0x0f801010,	0x0fa01010,	0x0fc01010,	0x0fe01010,	//W 121 ~ 128	
};

int VALC_from_bin(char *src, int len)
{
	VALC_HEADER header = {0};
	unsigned char *pData = NULL;
	int i=0;

	if(src == NULL)
		return 0;

	pData = (unsigned char *)(&VALC_Table);

	if(len - sizeof(VALC_HEADER) != 6*289*sizeof(unsigned int))
		return 0;
	
	while(i<len)
	{
		if(i<sizeof(VALC_HEADER))
		{
			if((i+sizeof(VALC_HEADER))<=len)
			{
				header.Offset = *src<<24 | *(src+1)<<16 | *(src+2)<<8 | *(src+3);
				header.endCode = *(src+4)<<24 | *(src+5)<<16 | *(src+6)<<8 | *(src+7);

				if(header.Offset != sizeof(VALC_HEADER) || header.endCode != 0xffffffff)
					return 0;

				i+=sizeof(VALC_HEADER);
				src+=sizeof(VALC_HEADER);
			}
			else
				break;
		}
		else
		{
			if((i+sizeof(unsigned int)) <= len)
			{
				if(i%sizeof(unsigned int) == 0)
				{
					*pData = *(src+sizeof(unsigned int)-1);					
					pData++;
				}
				i+=sizeof(unsigned int);
				src+=sizeof(unsigned int);
			}
			else
				break;
		}
	}

	return 1;
}

int OutputGamma_from_bin(char *src, int len)
{
	OUTPUT_GAMMA_HEADER header = {0};
	unsigned int *pData = NULL;
	int i = 0;

	if(src == NULL)
		return 0;
	
	pData = (unsigned int*)(&OutputGamma_Table);

	if(len - sizeof(OUTPUT_GAMMA_HEADER) != 512*sizeof(unsigned int))
		return 0;
	
	while(i<len)
	{
		if(i<sizeof(OUTPUT_GAMMA_HEADER))
		{
			if((i+sizeof(OUTPUT_GAMMA_HEADER))<=len)
			{
				header.Offset = *src<<24 | *(src+1)<<16 | *(src+2)<<8 | *(src+3);
				header.endCode = *(src+4)<<24 | *(src+5)<<16 | *(src+6)<<8 | *(src+7);

				if(header.Offset != sizeof(OUTPUT_GAMMA_HEADER) || header.endCode != 0xffffffff)
					return 0;

				i+=sizeof(OUTPUT_GAMMA_HEADER);
				src+=sizeof(OUTPUT_GAMMA_HEADER);
			}
			else
				break;
		}
		else
		{
			if((i+sizeof(unsigned int)) <= len)
			{
				if(i%sizeof(unsigned int) == 0)
				{
					*pData = *src<<24 | *(src+1)<<16 | *(src+2)<<8 | *(src+3);
					pData++;
				}
				i+=sizeof(unsigned int);
				src+=sizeof(unsigned int);
			}
			else
				break;
		}
	}
	
	return 1;
}

static void drvif_color_pcid2_valuetable(unsigned int * ValueTBL, unsigned int TblSel, DRV_pcid2_channel_t Channel)
{
	//unsigned long flags;//for spin_lock_irqsave
	int i=0;
	ppoverlay_display_timing_ctrl2_RBUS display_timing_ctrl2_reg;
	pcid_pcid_lut_addr_RBUS pcid_lut_addr_reg;
	pcid_pcid_lut_data_RBUS pcid_pcid_lut_data_reg;

	if(!ValueTBL)
	{
		printf("[Error][%s][%s] Table Ptr is null!\n", __FILE__, __func__);
		return;
	}

	// ------------------------------------------------------------------------------------------------------
	// Enable module
	rtd_clearbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT10); // disable DTG double buffer
	display_timing_ctrl2_reg.regValue = rtd_inl(PPOVERLAY_Display_Timing_CTRL2_reg);
	display_timing_ctrl2_reg.pcid2_en = 1;
	rtd_outl(PPOVERLAY_Display_Timing_CTRL2_reg, display_timing_ctrl2_reg.regValue);
	rtd_setbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT10); // enable DTG double buffer
	// ------------------------------------------------------------------------------------------------------

	// Setup table access setting
	pcid_lut_addr_reg.regValue = rtd_inl(PCID_PCID_LUT_ADDR_reg);
	pcid_lut_addr_reg.pcid_lut_sel = Channel;
	pcid_lut_addr_reg.pcid_lut_adr_mode = 0;	/* Increase Current (column) Index first */
	pcid_lut_addr_reg.pcid_lut_ax_en = 1;		/* access en */
	pcid_lut_addr_reg.pcid_lut_row_addr = TblSel;
	pcid_lut_addr_reg.pcid_lut_column_addr = TblSel;

	rtd_outl(PCID_PCID_LUT_ADDR_reg, pcid_lut_addr_reg.regValue);

	// Write the value into table
	for(i=0; i<81; i++)
	{
		pcid_pcid_lut_data_reg.pcid_lut_data = ValueTBL[i];
		rtd_outl(PCID_PCID_LUT_DATA_reg, pcid_pcid_lut_data_reg.regValue);
	}

	// Disable the table access
	pcid_lut_addr_reg.regValue = rtd_inl(PCID_PCID_LUT_ADDR_reg);
	pcid_lut_addr_reg.pcid_lut_ax_en = 0;
	rtd_outl(PCID_PCID_LUT_ADDR_reg, pcid_lut_addr_reg.regValue);

	// Disable the PCID enable
	rtd_clearbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT10); // disable DTG double buffer
	display_timing_ctrl2_reg.regValue = rtd_inl(PPOVERLAY_Display_Timing_CTRL2_reg);
	display_timing_ctrl2_reg.pcid2_en = 0;
	rtd_outl(PPOVERLAY_Display_Timing_CTRL2_reg, display_timing_ctrl2_reg.regValue);
	rtd_setbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT10); // enable DTG double buffer
}


static void fwif_color_set_pcid2_valuetable_bycolor(unsigned char* InputLUT, DRV_pcid2_channel_t Channel)
{
    // Transfer the value to register value
    int Row = 0, Col = 0;
    unsigned int TblValue = 0;
    unsigned int POD_ValueTBL_1[81] = {0};
    unsigned int POD_ValueTBL_2[81] = {0};
    for(Row=0; Row<9; Row++)
    {
        for(Col=0; Col<9; Col++)
        {
            //-------------------
            // Value1 | Value3 |
            //-------------------     will be transform to {Value1, Value2, Value3, Value4} {MSB, LSB}
            // Value2 | Value4 |
            //-------------------
            unsigned int Value1 = 0, Value2 = 0, Value3 = 0, Value4 = 0;
            if( Row<8 && Col<8 )
            {
                Value1 = InputLUT[Row*34   +Col*2  ];
                Value2 = InputLUT[Row*34+17+Col*2  ];
                Value3 = InputLUT[Row*34   +Col*2+1];
                Value4 = InputLUT[Row*34+17+Col*2+1];
            }
            else if( Row < 8 )
            {
                Value1 = InputLUT[Row*34   +Col*2  ];
                Value2 = InputLUT[Row*34+17+Col*2  ];
                Value3 = Value1;
                Value4 = Value2;
            }
            else if( Col < 8 )
            {
                Value1 = InputLUT[Row*34   +Col*2  ];
                Value2 = Value1;
                Value3 = InputLUT[Row*34   +Col*2+1];
                Value4 = Value3;
            }
            else
            {
                Value1 = InputLUT[Row*34   +Col*2  ];
                Value2 = Value1;
                Value3 = Value1;
                Value4 = Value1;
            }
            TblValue = ( (Value1 << 24) + (Value2 << 16) + (Value3 << 8) + Value4 );
            POD_ValueTBL_1[Row*9+Col] = TblValue;
        }
    }

    for(Row=0; Row<9; Row++)
    {
        for(Col=0; Col<9; Col++)
        {
            //-------------------
            // Value1 | Value3 |
            //-------------------     will be transform to {Value1, Value2, Value3, Value4} {MSB, LSB}
            // Value2 | Value4 |
            //-------------------
            unsigned int Value1 = 0, Value2 = 0, Value3 = 0, Value4 = 0;
            if( Row<8 && Col<8 )
            {
                Value1 = InputLUT[289+Row*34   +Col*2  ];
                Value2 = InputLUT[289+Row*34+17+Col*2  ];
                Value3 = InputLUT[289+Row*34   +Col*2+1];
                Value4 = InputLUT[289+Row*34+17+Col*2+1];
            }
            else if( Row < 8 )
            {
                Value1 = InputLUT[289+Row*34   +Col*2  ];
                Value2 = InputLUT[289+Row*34+17+Col*2  ];
                Value3 = Value1;
                Value4 = Value2;
            }
            else if( Col < 8 )
            {
                Value1 = InputLUT[289+Row*34   +Col*2  ];
                Value2 = Value1;
                Value3 = InputLUT[289+Row*34   +Col*2+1];
                Value4 = Value3;
            }
            else
            {
                Value1 = InputLUT[289+Row*34   +Col*2  ];
                Value2 = Value1;
                Value3 = Value1;
                Value4 = Value1;
            }
            TblValue = ( (Value1 << 24) + (Value2 << 16) + (Value3 << 8) + Value4 );
            POD_ValueTBL_2[Row*9+Col] = TblValue;
        }
    }

    drvif_color_pcid2_valuetable(POD_ValueTBL_1, 0, Channel);
    drvif_color_pcid2_valuetable(POD_ValueTBL_2, 1, Channel);
}


static void drvif_color_output_gamma_control_front(unsigned char display)
{
	outputgamma_out_gamma_ctrl_1_RBUS out_gamma_ctrl_1_REG;

	out_gamma_ctrl_1_REG.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg);

	if (display == 0) {
//		out_gamma_port_REG.out_gamma_tab_index = 0;
		out_gamma_ctrl_1_REG.out_gamma_en = 0;
		out_gamma_ctrl_1_REG.out_gamma_ax_rw_sel = 1; /* writing table*/
	}

	rtd_outl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg, out_gamma_ctrl_1_REG.regValue);
}

static void drvif_color_output_gamma_control_back(unsigned char display)
{
	outputgamma_out_gamma_ctrl_1_RBUS out_gamma_ctrl_1_REG;

	out_gamma_ctrl_1_REG.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg);

	if (display == 0) {
		out_gamma_ctrl_1_REG.out_gamma_ax_rw_sel = 0; /* read table*/
		out_gamma_ctrl_1_REG.out_gamma_en = 1;
	}

	rtd_outl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg, out_gamma_ctrl_1_REG.regValue);
}

static void drvif_color_colorwrite_output_gamma(unsigned int *pArray, unsigned char RGB_chanel_idx)
{
	unsigned short ii, cnt = 10000;
	outputgamma_out_gamma_port_RBUS out_gamma_port_REG;
	outputgamma_out_gamma_ctrl_1_RBUS out_gamma_ctrl_1_REG;
	unsigned char status = 1;

	out_gamma_ctrl_1_REG.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg);
//	out_gamma_ctrl_1_REG.out_gamma_ax_rw_sel = 1; // enable write (0:disable, 1:Write, 2:Read)
	out_gamma_ctrl_1_REG.out_gamma_ax_ch_sel = RGB_chanel_idx; // 0:R, 1:G, 2:B, 3:RGB, 4:W
	rtd_outl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg, out_gamma_ctrl_1_REG.regValue);

	for (ii = 0; ii < (Bin_Num_Output_InvOutput_Gamma>>1); ii++) 
	{
		status = 1;
		cnt = 10000;

		out_gamma_port_REG.regValue = pArray[ii];
		rtd_outl(OUTPUTGAMMA_OUT_GAMMA_PORT_reg, out_gamma_port_REG.regValue);

		while ((cnt--) && (status) && ((ii % 16) == 15)) 
		{
			out_gamma_ctrl_1_REG.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_PORT_reg);
			status =  (out_gamma_ctrl_1_REG.out_gamma_fifo_empty == 1) ? 0 : 1;
		}
	}
}


static void drvif_color_set_output_gamma_format(unsigned char Enable, unsigned char fmt, unsigned char loc, unsigned char ch_order)
{
	// fmt = format,  0:RGB, 1:RGBW
	// loc = location,	0: After Demura (RGB),	1: After HCIC/FCIC (RGB or RGBW)
	// ch_order = channel order, only available when (fmt=1 && loc=1)
	outputgamma_out_gamma_ctrl_1_RBUS outputgamma_out_gamma_ctrl_1_reg;
	outputgamma_out_gamma_ctrl_2_RBUS out_gamma_ctrl_2_REG;

	unsigned char order_lst[2][8] = {
	// 0:even_R, 1:even_G, 2:even_B, 3:even_W, 4:odd_R, 5:odd_G, 6:odd_B, 7:odd_W,
	/*0*/{ 0, 1, 2, 3, 2, 3, 0, 1,}, // hw default
	/*1*/{ 2, 3, 0, 1, 0, 1, 2, 3,},
	};
	unsigned char order_sel[8];
	memcpy(order_sel, &order_lst[ch_order][0], sizeof(order_sel));

	outputgamma_out_gamma_ctrl_1_reg.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg);
	outputgamma_out_gamma_ctrl_1_reg.out_gamma_en = Enable;
	outputgamma_out_gamma_ctrl_1_reg.out_gamma_location = loc; // 0: After Demura (RGB),	1: After HCIC/FCIC (RGB or RGBW)
	rtd_outl(OUTPUTGAMMA_OUT_GAMMA_CTRL_1_reg, outputgamma_out_gamma_ctrl_1_reg.regValue);

	out_gamma_ctrl_2_REG.regValue = rtd_inl(OUTPUTGAMMA_OUT_GAMMA_CTRL_2_reg);
	out_gamma_ctrl_2_REG.out_gamma_format =  fmt; // 0:RGB, 1:RGBW
	out_gamma_ctrl_2_REG.even_ch_r =  order_sel[0];
	out_gamma_ctrl_2_REG.even_ch_g = order_sel[1] ;
	out_gamma_ctrl_2_REG.even_ch_b =  order_sel[2];
	out_gamma_ctrl_2_REG.even_ch_w =  order_sel[3];
	out_gamma_ctrl_2_REG.odd_ch_r =  order_sel[4];
	out_gamma_ctrl_2_REG.odd_ch_g =  order_sel[5];
	out_gamma_ctrl_2_REG.odd_ch_b =  order_sel[6];
	out_gamma_ctrl_2_REG.odd_ch_w =  order_sel[7];

	rtd_outl(OUTPUTGAMMA_OUT_GAMMA_CTRL_2_reg, out_gamma_ctrl_2_REG.regValue);
}

void drvif_color_valc_table(unsigned char* InputLUT)
{
	ppoverlay_display_timing_ctrl2_RBUS display_timing_ctrl2_reg;
	display_timing_ctrl2_reg.regValue = rtd_inl(PPOVERLAY_Display_Timing_CTRL2_reg);
	display_timing_ctrl2_reg.pcid2_en = 0;
	rtd_outl(PPOVERLAY_Display_Timing_CTRL2_reg, display_timing_ctrl2_reg.regValue);

	rtd_setbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT8); // Set double buffer apply bit

	// VALC setting, all subpixel refer to the upper line, current subpixel
	rtd_outl(0xb802ef40, 0x00001010); // basic setting
	rtd_outl(0xb802ef44, 0xc0000000); // basic setting
	rtd_outl(0xb802ef48, 0x00000000); // basic setting
	rtd_outl(0xb802ef50, 0x91509150); // line 1 pixel setting
	rtd_outl(0xb802ef54, 0x91509150); // line 2 pixel setting
	rtd_outl(0xb802ef58, 0x91509150); // line 3 pixel setting
	rtd_outl(0xb802ef5c, 0x91509150); // line 3 pixel setting
	rtd_outl(0xb802ef70, 0x0000aaaa); // R [ line2, line1 ]
	rtd_outl(0xb802ef74, 0xffff5555); // R [ line4, line3 ]
	rtd_outl(0xb802ef78, 0x0000aaaa); // R [ line6, line5 ]
	rtd_outl(0xb802ef7c, 0xffff5555); // R [ line8, line7 ]
	rtd_outl(0xb802ef80, 0x00005555); // G [ line2, line1 ]
	rtd_outl(0xb802ef84, 0xffffaaaa); // G [ line4, line3 ]
	rtd_outl(0xb802ef88, 0x00005555); // G [ line6, line5 ]
	rtd_outl(0xb802ef8c, 0xffffaaaa); // G [ line8, line7 ]
	rtd_outl(0xb802ef90, 0x0000aaaa); // B [ line2, line1 ]
	rtd_outl(0xb802ef94, 0xffff5555); // B [ line4, line3 ]
	rtd_outl(0xb802ef98, 0x0000aaaa); // B [ line6, line5 ]
	rtd_outl(0xb802ef9c, 0xffff5555); // B [ line8, line7 ]

	fwif_color_set_pcid2_valuetable_bycolor(InputLUT, _PCID2_COLOR_R);
	fwif_color_set_pcid2_valuetable_bycolor((InputLUT+289*2), _PCID2_COLOR_G);
	fwif_color_set_pcid2_valuetable_bycolor((InputLUT+289*4), _PCID2_COLOR_B);

	display_timing_ctrl2_reg.regValue = rtd_inl(PPOVERLAY_Display_Timing_CTRL2_reg);
	display_timing_ctrl2_reg.pcid_en = 1;
	rtd_outl(PPOVERLAY_Display_Timing_CTRL2_reg, display_timing_ctrl2_reg.regValue);
	rtd_setbits(PPOVERLAY_Double_Buffer_CTRL_reg, _BIT8); // Set double buffer apply bit
	
	rtd_outl(0xb802efa0, 0x00000007);
	rtd_outl(0xb802efa4, 0xfca87642);
	rtd_outl(0xb802efa8, 0x10000000);
	rtd_outl(0xb802efac, 0x00000000);
	rtd_outl(0xb802efb0, 0x00000000);
}


void drvif_color_outputgamma_table(unsigned int* pOutputGama)
{
	if(pOutputGama == NULL)
		return;

	drvif_color_output_gamma_control_front(0);
	drvif_color_colorwrite_output_gamma(pOutputGama, 0);	//R channel 
	drvif_color_colorwrite_output_gamma((pOutputGama+128), 1);	//G channel 
	drvif_color_colorwrite_output_gamma((pOutputGama+128*2), 2);	//B channel 
	drvif_color_colorwrite_output_gamma((pOutputGama+128*3), 4);	//W channel 
	drvif_color_output_gamma_control_back(0);

	drvif_color_set_output_gamma_format(1,0,1,0);
}
#endif

static int CusApi_panel_mLVDS_CSOT_GOA_Setting_from_bin(char *src, int len)
{
	TCON_SETTING_HEADER header = {0};
	TCON_SETTING_DATA Data = {0};
	int i = 0;
	
	while(i<len)
	{
		if(i<sizeof(TCON_SETTING_HEADER))
		{
			if((i+8)<=len)
			{
				header.Offset = *src<<24 | *(src+1)<<16 | *(src+2)<<8 | *(src+3);
				header.endCode = *(src+4)<<24 | *(src+5)<<16 | *(src+6)<<8 | *(src+7);

				if(header.Offset != sizeof(TCON_SETTING_HEADER) || header.endCode != 0xffffffff)
					return 0;

				i+=sizeof(TCON_SETTING_HEADER);
				src+=sizeof(TCON_SETTING_HEADER);;
			}
			else
				break;

		}
		else
		{
			if((i+sizeof(TCON_SETTING_DATA))<=len)
			{
				Data.type = *src<<24 | *(src+1)<<16 | *(src+2)<<8 | *(src+3);
				Data.addr = *(src+4)<<24 | *(src+5)<<16 | *(src+6)<<8 | *(src+7);
				Data.value1 = *(src+8)<<24 | *(src+9)<<16 | *(src+10)<<8 | *(src+11);
				Data.value2 = *(src+12)<<24 | *(src+13)<<16 | *(src+14)<<8 | *(src+15);
				//printf("\nData.type:%d Data.addr:0x%08x Data.value1:0x%08x Data.value2:0x%08x\n", Data.type,Data.addr,Data.value1,Data.value2);

				switch(Data.type)
				{
					case 0:
						//printf("\nrtd_outl(0x%08x,0x%08x)\n", Data.addr,Data.value1);
						rtd_outl(Data.addr,Data.value1);
						break;
					case 1:
						//printf("\nrtd_maskl(0x%08x,0x%08x,0x%08x)\n",Data.addr,Data.value1,Data.value2);
						rtd_maskl(Data.addr,Data.value1,Data.value2);
						break;
					case 2:
						break;
					case 3:
						mdelay(Data.addr);
						break;
					default:
						break;
				}

				i+=sizeof(TCON_SETTING_DATA);
				src+=sizeof(TCON_SETTING_DATA);
			}
			else
				break;	
		}
	}
	
	return 1;
}

static bool CusApi_panel_mLVDS_CSOT_GOA_tcon_setting(void)
{
	char *dst_addr = NULL;
	int dst_length;
    char path[256] = {0};

	ppoverlay_double_buffer_ctrl_RBUS double_buffer_ctrl_reg;	 
	unsigned int count = 0x3fffff;
	
	double_buffer_ctrl_reg.regValue = rtd_inl(PPOVERLAY_Double_Buffer_CTRL_reg);
	double_buffer_ctrl_reg.dreg_dbuf_en = 1;
	rtd_outl(PPOVERLAY_Double_Buffer_CTRL_reg, double_buffer_ctrl_reg.regValue);
	double_buffer_ctrl_reg.dreg_dbuf_set = 1;
	rtd_outl(PPOVERLAY_Double_Buffer_CTRL_reg, double_buffer_ctrl_reg.regValue);
	while((double_buffer_ctrl_reg.dreg_dbuf_set)&& --count){
		double_buffer_ctrl_reg.regValue = rtd_inl(PPOVERLAY_Double_Buffer_CTRL_reg);
	}
	if(count==0){
		printf("wait double buffer timeout\n");
	}

    sprintf(path,"%s",MLVDS_TCON_SETTING_FILENAME_IN_FACTORY);

    if (boot_power_on_mode != POWER_OFF)
		IO_Set(PIN_LVDS_ON_OFF, 1);	
	mdelay(50);
	if(factory_read((char *)path, &dst_addr, &dst_length)) 
	{	
		printf("\033[1;31m>>>>>>>>>>>>>>>> can't find , path:%s\033[m\n",path);
#if 1
		//---------------------------------------------------------------------------------------------------------------------------------
		//SFG: DH_ST_END = 0x01181018, DTG_HS_Width = 32, DTG_VS_Width = 12
		//---------------------------------------------------------------------------------------------------------------------------------
		rtd_outl(0xb802D900, 0x00100840);	// Common fifo use sfg_clk, 30-bit Mode, Dout Enable, Pre-R/B Swap disable,
							// Remove last too short dhs, any to any sub-pixel assign disable,
		rtd_outl(0xb802D904, 0x07FFFFFF);	// PIF_Clock = Dclk/8, Force to background enable,
		rtd_outl(0xb802D910, 0x00000003);	// CRC Enable,
		rtd_outl(0xb802D928, 0x00000000);	// PIF Forcre to BG disable,
		rtd_outl(0xb802D92C, 0x00000000);	// Line 8n+0/8n+1/8n+2/8n+3 no sub-pixel,
		rtd_outl(0xb802DBA8, 0x00000000);	// Line 8n+4/8n+5/8n+6/8n+7 no sub-pixel,
		rtd_outl(0xb802D930, 0x00000000);	// SEG0/SEG1 No front/back dummy pixel,
		rtd_outl(0xb802D934, 0x00000000);	// SEG2/SEG3 No front/back dummy pixel,
		//rtd_outl(0xb802D938, 0x00000000); // SEG4/SEG5 No front/back dummy pixel,
		//rtd_outl(0xb802D940, 0x00000000); // SEG6/SEG7 No front/back dummy pixel,
		//rtd_outl(0xb802D944, 0x00000000); // SEG8/SEG9 No front/back dummy pixel,
		//rtd_outl(0xb802D948, 0x00000000); // SEG10/SEG11 No front/back dummy pixel,
		rtd_outl(0xb802D964, 0x00000307);	// SFG Normal Mode, 4-SEG, 8-Port, H-Flip Disable,
		rtd_outl(0xb802D968, 0x000003C0);	// SEG0/SEG1 Start Address,
		rtd_outl(0xb802D96C, 0x07800B40);	// SEG2/SEG3 Start Address,
		//rtd_outl(0xb802D970, 0x0A000C80); // SEG4/SEG5 Start Address,
		//rtd_outl(0xb802D974, 0x00000000); // SEG6/SEG7 Start Address,
		//rtd_outl(0xb802D978, 0x00000000); // SEG8/SEG9 Start Address,
		//rtd_outl(0xb802D97C, 0x00000000); // SEG10/SEG11 Start Address,
		//rtd_outl(0xb802D980, 0x00000000); // Front dummy line(4N+0) data,
		//rtd_outl(0xb802D984, 0x00000000); // Front dummy line(4N+1) data,
		//rtd_outl(0xb802D94C, 0x00000000); // Front dummy line(4N+2) data,
		rtd_outl(0xb802D988, 0x40000000);	// Bypass SFG disable, Auto config enable, downsample disable,
		rtd_outl(0xb802D990, 0x08980300);	// Hs_distance, Vsync Start/End,
		rtd_outl(0xb802D998, 0x00000000);	// Segement-flip Disable,
		rtd_outl(0xb802D99C, 0x0F000000);	// Final line length,
		rtd_outl(0xb802D9A0, 0x00000000);	// Residue_pix_split_mode, Final_addr_split_mode,
		rtd_outl(0xb802D9A4, 0x01181128);	// hs_den_io_dist, hs_io_dist,
		rtd_outl(0xb802D9B8, 0x00000000);	// All PIF Clock disable, Data inverse disable,
		rtd_outl(0xb802D9E8, 0x00000000);	// SEG0/SEG1 No middle dummy sub-pixel,
		rtd_outl(0xb802D9EC, 0x00000000);	// SEG2/SEG3 No middle dummy sub-pixel,
		//rtd_outl(0xb802D9F0, 0x00000000); // SEG4/SEG5 No middle dummy sub-pixel,
		//rtd_outl(0xb802D9F4, 0x00000000); // SEG6/SEG7 No middle dummy sub-pixel,
		//rtd_outl(0xb802D9F8, 0x00000000); // SEG8/SEG9 No middle dummy sub-pixel,
		//rtd_outl(0xb802D9FC, 0x00000000); // SEG10/SEG11 No middle dummy sub-pixel,
		rtd_outl(0xb802DA08, 0x81000000);	// Active_pixel_RGB, ext_num use fw mode, unit is sub-pixel
		rtd_outl(0xb802DA0C, 0x00000000);		// G-dummy sub-pixel for line 8n+0, G-dummy data select from SFG input,
		rtd_outl(0xb802DA10, 0x00000000);		// G-dummy sub-pixel for line 8n+1,
		rtd_outl(0xb802DA14, 0x00000000);		// G-dummy sub-pixel for line 8n+2,
		rtd_outl(0xb802DA18, 0x00000000);		// G-dummy sub-pixel for line 8n+3,
		rtd_outl(0xb802DBAC, 0x00000000);		// G-dummy sub-pixel for line 8n+4,
		rtd_outl(0xb802DBB0, 0x00000000);		// G-dummy sub-pixel for line 8n+5,
		rtd_outl(0xb802DBB4, 0x00000000);		// G-dummy sub-pixel for line 8n+6,
		rtd_outl(0xb802DBB8, 0x00000000);		// G-dummy sub-pixel for line 8n+7,
		rtd_outl(0xb802DA1C, 0x00000000);	// dummy_pixel_data is black,
		rtd_outl(0xb802DA94, 0x00000000);	// Last_seg_start, last_seg_start_res for line 4n+0/1,
		rtd_outl(0xb802DA98, 0x00000000);	// Last_seg_start, last_seg_start_res for line 4n+2/3,
		rtd_outl(0xb802DBE0, 0x00000000);	// Last_seg_start, last_seg_start_res for line 4n+4/5,
		rtd_outl(0xb802DBE4, 0x00000000);	// Last_seg_start, last_seg_start_res for line 4n+6/7,
		//rtd_outl(0xb802DAA0, 0x01010101); // DRD Data Mapping,
		//rtd_outl(0xb802DAA4, 0x05050505); // DRD Data Mapping,
		//rtd_outl(0xb802DAA8, 0x03030303); // DRD Data Mapping,
		//rtd_outl(0xb802DAAC, 0x07070707); // DRD Data Mapping,
		//rtd_outl(0xb802DAB0, 0x0B0B0B0B); // DRD Data Mapping,
		//rtd_outl(0xb802DAB4, 0x09090909); // DRD Data Mapping,
		//rtd_outl(0xb802DAB8, 0x0D0D0D0D); // DRD Data Mapping,
		//rtd_outl(0xb802DABC, 0x11111111); // DRD Data Mapping,
		//rtd_outl(0xb802DAC0, 0x0F0F0F0F); // DRD Data Mapping,
		//rtd_outl(0xb802DAC4, 0x13131313); // DRD Data Mapping,
		//rtd_outl(0xb802DAC8, 0x17171717); // DRD Data Mapping,
		//rtd_outl(0xb802DACC, 0x15151515); // DRD Data Mapping,
		//rtd_outl(0xb802DAD0, 0x02020202); // DRD Data Mapping,
		//rtd_outl(0xb802DAD4, 0x00000000); // DRD Data Mapping,
		//rtd_outl(0xb802DAD8, 0x04040404); // DRD Data Mapping,
		//rtd_outl(0xb802DADC, 0x08080808); // DRD Data Mapping,
		//rtd_outl(0xb802DAE0, 0x06060606); // DRD Data Mapping,
		//rtd_outl(0xb802DAE4, 0x0A0A0A0A); // DRD Data Mapping,
		//rtd_outl(0xb802DAE8, 0x0E0E0E0E); // DRD Data Mapping,
		//rtd_outl(0xb802DAEC, 0x0C0C0C0C); // DRD Data Mapping,
		//rtd_outl(0xb802DAF0, 0x10101010); // DRD Data Mapping,
		//rtd_outl(0xb802DAF4, 0x14141414); // DRD Data Mapping,
		//rtd_outl(0xb802DAF8, 0x12121212); // DRD Data Mapping,
		//rtd_outl(0xb802DAFC, 0x16161616); // DRD Data Mapping,
		rtd_outl(0xb802DB68, 0x55550000);	// ext_num setting for case_a and case_b,
		//rtd_outl(0xb802DB78, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+0,
		//rtd_outl(0xb802DB7C, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+1,
		//rtd_outl(0xb802DB80, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+2,
		//rtd_outl(0xb802DB84, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+3,
		//rtd_outl(0xb802DB88, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+4,
		//rtd_outl(0xb802DB8C, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+5,
		//rtd_outl(0xb802DB90, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+6,
		//rtd_outl(0xb802DB94, 0x00000000); // Virtual data_0/1 for G-dummy line 8n+7,
		rtd_outl(0xb802DBF0, 0x1DF10E38);	// SFG Pattern Gen. Disable, RGB Mode, Width = 480, Hight = 270, pattern reset by hs,
		rtd_outl(0xb802DBF4, 0x00FFFFFF);	// SFG Pattern Gen. R/G/B/W Color for Bar0,
		rtd_outl(0xb802DBF8, 0x00FFFF00);	// SFG Pattern Gen. R/G/B/W Color for Bar1,
		rtd_outl(0xb802DBFC, 0x0000FFFF);	// SFG Pattern Gen. R/G/B/W Color for Bar2,
		rtd_outl(0xb802DC00, 0x0000FF00);	// SFG Pattern Gen. R/G/B/W Color for Bar3,
		rtd_outl(0xb802DC04, 0x00FF00FF);	// SFG Pattern Gen. R/G/B/W Color for Bar4,
		rtd_outl(0xb802DC08, 0x00FF0000);	// SFG Pattern Gen. R/G/B/W Color for Bar5,
		rtd_outl(0xb802DC0C, 0x000000FF);	// SFG Pattern Gen. R/G/B/W Color for Bar6,
		rtd_outl(0xb802DC10, 0x00000000);	// SFG Pattern Gen. R/G/B/W Color for Bar7,
		rtd_outl(0xb802DC1C, 0x00000000);	// Pattern Detect Function Disable,
		
		//---------------------------------------------------------------------------------------------------------------------------------
		//Mini-LVDS MAC & DISP_IF
		//---------------------------------------------------------------------------------------------------------------------------------
		rtd_outl(0xb802D014, 0x00000441);	// Mini-LVDS disable, 4-Port 6-Pair, P2S LSB first, 8-Bit Mode,
		rtd_outl(0xb802D018, 0x00FFFFFF);	// Mini-LVDS Reset Pulse Enable for all Lane,
		rtd_outl(0xb802D01C, 0x00000000);	// Mini-LVDS Reset Pulse Merge Tcon Disable,
		rtd_outl(0xb802D020, 0x00000000);	// Mini-LVDS Reset Pulse Only in Active Region, Mini-LVDS Clock inverse,
		rtd_outl(0xb802D024, 0xFFFFFF1F);	// Mini-LVDS Reset Pulse F/W Setting,
		rtd_outl(0xb802D028, 0x00000000);	// Mini-LVDS Reset Pulse F/W Setting,
		rtd_outl(0xb802D02C, 0x80000000);	// Async FIFO use LVDS/mLVDS Mode, P/N Swap enable,
		rtd_outl(0xb802D038, 0x00000000);	// Async FIFO Disable,
		rtd_outl(0xb802D03C, 0x00010010);	// Async FIFO reset enable, every vsync, apply after 16-clock,
		//rtd_outl(0xb802D060, 0x06050403); // Lane 0~3 pair assign,
		//rtd_outl(0xb802D064, 0x0201000d); // Lane 4~7 pair assign,
		//rtd_outl(0xb802D068, 0x0c0b0A09); // Lane 8~11 pair assign,
		//rtd_outl(0xb802D06C, 0x08071413); // Lane 12~15 pair assign,
		//rtd_outl(0xb802D070, 0x1211100f); // Lane 16~19 pair assign,
		//rtd_outl(0xb802D074, 0x0e1b1a19); // Lane 20~23 pair assign,
		//rtd_outl(0xb802D078, 0x18171615); // Lane 24~27 pair assign,
		rtd_outl(0xb802D060, 0x191a1b18);	// Lane 0~3 pair assign,
		rtd_outl(0xb802D064, 0x15161712);	// Lane 4~7 pair assign,
		rtd_outl(0xb802D068, 0x1314110e);	// Lane 8~11 pair assign,
		rtd_outl(0xb802D06C, 0x0f100b0c);	// Lane 12~15 pair assign,
		rtd_outl(0xb802D070, 0x0d0a0708);	// Lane 16~19 pair assign,
		rtd_outl(0xb802D074, 0x09040506);	// Lane 20~23 pair assign,
		rtd_outl(0xb802D078, 0x03000102);	// Lane 24~27 pair assign,
		
		//---------------------------------------------------------------------------------------------------------------------------------
		//VBy1 APHY
		//---------------------------------------------------------------------------------------------------------------------------------
		rtd_outl(0xb8000C00, 0x30018020);	// PI_A1(Phase16), PI_A2(Phase16), VBG,
		rtd_outl(0xb8000C04, 0x30018020);	// PI_B1(Phase16), PI_B2(Phase16),
		rtd_outl(0xb8000C08, 0x03080100);	// PI_AB(Phase0), LDO_1V = 0.988V, TXPLL use VBy1 Path, DIVB(1),
		rtd_outl(0xb8000C0C, 0x0005E380);	// Small KVCO, Through PI, CKREF_INV(0), PLL_ICP<2:0>, PLL_SC1, PLL_SC2, PLL_SR,
		rtd_outl(0xb8000C10, 0x0C000000);	// CK1X_INV(0), CK20X_INV(0), MACPLL_CKO_INV(0), PRESCALERDIV_LS(1), EPI_DIVM(8),
		rtd_outl(0xb8000C14, 0x00000000);	// Normal VCO, PLL Test PAD select, PLL_ICP<3>,
		rtd_outl(0xb8000C18, 0x00000002);	// TXPLL use LVDS/mLVDS feedback path,
		rtd_outl(0xb8000C20, 0x00000200);	// TXPLL_EN(1), TXPLL_RSTN(0),
		rtd_outl(0xb8000C50, 0x01F4C000);	// IBN Current, VCM Level, LDO1V_EN(1), LDO2V_EN(1), VCM_SHIFT_DOWN(0),
		rtd_outl(0xb8000C74, 0x80000000);	// GLOBAL_POW_EN(1),
		rtd_outl(0xb8000C78, 0x00000600);	// CKIN_DIVN(8),
		//
		rtd_outl(0xb8000D00, 0x33002152);	// Lane0 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D10, 0x33002152);	// Lane1 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D20, 0x33002152);	// Lane2 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D30, 0x33000152);	// Lane3 : LS Mode, PI_SEL(PI_B1), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D40, 0x33002152);	// Lane4 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D50, 0x33002152);	// Lane5 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D60, 0x33002152);	// Lane6 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D70, 0x33002152);	// Lane7 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D80, 0x33002152);	// Lane8 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000D90, 0x33002152);	// Lane9 : LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000DA0, 0x33002152);	// Lane10: LS Mode, PI_SEL(PI_B2), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000DB0, 0x33002152);	// Lane11: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000DC0, 0x33002152);	// Lane12: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000DD0, 0x33002152);	// Lane13: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E00, 0x33002152);	// Lane14: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E10, 0x33002152);	// Lane15: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E20, 0x33002152);	// Lane16: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E30, 0x33000152);	// Lane17: LS Mode, PI_SEL(PI_A1), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E40, 0x33002152);	// Lane18: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E50, 0x33002152);	// Lane19: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E60, 0x33002152);	// Lane20: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E70, 0x33002152);	// Lane21: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E80, 0x33002152);	// Lane22: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000E90, 0x33002152);	// Lane23: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000EA0, 0x33002152);	// Lane24: LS Mode, PI_SEL(PI_A2), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000EB0, 0x33002152);	// Lane25: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000EC0, 0x33002152);	// Lane26: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		rtd_outl(0xb8000ED0, 0x33002152);	// Lane27: LS Mode, PI_SEL(PI_DM), SR_SEL(0), TERM_SEL, IDRV_SEL, IEM_SEL,
		//
		rtd_outl(0xb8000D04, 0x44000008);	// Lane0 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D14, 0x44000008);	// Lane1 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D24, 0x44000008);	// Lane2 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D34, 0x44000000);	// Lane3 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_B1),
		rtd_outl(0xb8000D44, 0x44000008);	// Lane4 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D54, 0x44000008);	// Lane5 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D64, 0x44000008);	// Lane6 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D74, 0x44000008);	// Lane7 : CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000D84, 0x44000000);	// Lane8 : CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000D94, 0x44000000);	// Lane9 : CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000DA4, 0x44000000);	// Lane10: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000DB4, 0x44000000);	// Lane11: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000DC4, 0x44000000);	// Lane12: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000DD4, 0x44000000);	// Lane13: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E04, 0x44000000);	// Lane14: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E14, 0x44000000);	// Lane15: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E24, 0x44000000);	// Lane16: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E34, 0x44000000);	// Lane17: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E44, 0x44000000);	// Lane18: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E54, 0x44000000);	// Lane19: CKMX_INV(0), NDRV(0), IEM_EN(1), RT_EN(1), RP_EN(0), LANE_POW(0),
		rtd_outl(0xb8000E64, 0x44000008);	// Lane20: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000E74, 0x44000008);	// Lane21: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000E84, 0x44000008);	// Lane22: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000E94, 0x44000008);	// Lane23: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000EA4, 0x44000000);	// Lane24: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_A2),
		rtd_outl(0xb8000EB4, 0x44000008);	// Lane25: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000EC4, 0x44000008);	// Lane26: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		rtd_outl(0xb8000ED4, 0x44000008);	// Lane27: CKMX_INV(0), IEM_EN(1), RT_EN(1), LANE_POW(0), PI_SEL(PI_DM),
		//
		rtd_outl(0xb8000D88, 0x0000000A);	// Lane8 : PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000D98, 0x0000000A);	// Lane9 : PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000DA8, 0x00000002);	// Lane10: PI_SEL(PI_B2), CMFB_EN(1),
		rtd_outl(0xb8000DB8, 0x0000000A);	// Lane11: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000DC8, 0x0000000A);	// Lane12: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000DD8, 0x0000000A);	// Lane13: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000E08, 0x0000000A);	// Lane14: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000E18, 0x0000000A);	// Lane15: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000E28, 0x0000000A);	// Lane16: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000E38, 0x00000002);	// Lane17: PI_SEL(PI_A1), CMFB_EN(1),
		rtd_outl(0xb8000E48, 0x0000000A);	// Lane18: PI_SEL(PI_DM), CMFB_EN(1),
		rtd_outl(0xb8000E58, 0x0000000A);	// Lane19: PI_SEL(PI_DM), CMFB_EN(1),
		
		
		rtd_outl(0xb802D208, 0x00000000);		// Global Tcon disable,
		////////////////////////////////////////////////////////////////////////////

		mdelay(80);
		//---------------------------------------------------------------------------------------------------------------------------------
		//Initial Flow
		//--------------------------------------------------------------------------------------------------------------------------------
		// (1) DPLL Power Enable  (As above Script)
		//
		// (2) Wait DPLL stable at least 150us (SW Control)
		//
		// (3) Release DCLK Gating (SW Control)
		//
		// (4) TXPLL power up TXPLL during reset state (As above Script)
		//
		// (5) TXPLL reset release (Below Setting)
		rtd_part_outl(0xb8000C20, 10, 10, 0x1);
		//
		// (6) Wait TXPLL stable at least 150us (SW Control)
		//
		// (7) LVDS/mLVDS Clock Enable (Below Setting)
		rtd_outl(0xb802D9B8, 0x80000000);
		//
		// (8) mLVDS MAC Enable (Below Setting)
		rtd_part_outl(0xb802D014, 7, 7, 0x1);
		//
		// (9) Enable Async FIFO,
		rtd_outl(0xb802D038, 0x00000001);
		// (10) APHY Lane Power Enable (Below Setting)
		rtd_part_outl(0xb8000D04, 27, 27, 0x1); // LANE0
		rtd_part_outl(0xb8000D14, 27, 27, 0x1); // LANE1
		rtd_part_outl(0xb8000D24, 27, 27, 0x1); // LANE2
		rtd_part_outl(0xb8000D34, 27, 27, 0x1); // LANE3
		rtd_part_outl(0xb8000D44, 27, 27, 0x1); // LANE4
		rtd_part_outl(0xb8000D54, 27, 27, 0x1); // LANE5
		rtd_part_outl(0xb8000D64, 27, 27, 0x1); // LANE6
		rtd_part_outl(0xb8000D74, 27, 27, 0x1); // LANE7
		rtd_part_outl(0xb8000D84, 27, 27, 0x1); // LANE8
		rtd_part_outl(0xb8000D94, 27, 27, 0x1); // LANE9
		rtd_part_outl(0xb8000DA4, 27, 27, 0x1); // LANE10
		rtd_part_outl(0xb8000DB4, 27, 27, 0x1); // LANE11
		rtd_part_outl(0xb8000DC4, 27, 27, 0x1); // LANE12
		rtd_part_outl(0xb8000DD4, 27, 27, 0x1); // LANE13
		rtd_part_outl(0xb8000E04, 27, 27, 0x1); // LANE14
		rtd_part_outl(0xb8000E14, 27, 27, 0x1); // LANE15
		rtd_part_outl(0xb8000E24, 27, 27, 0x1); // LANE16
		rtd_part_outl(0xb8000E34, 27, 27, 0x1); // LANE17
		rtd_part_outl(0xb8000E44, 27, 27, 0x1); // LANE18
		rtd_part_outl(0xb8000E54, 27, 27, 0x1); // LANE19
		rtd_part_outl(0xb8000E64, 27, 27, 0x1); // LANE20
		rtd_part_outl(0xb8000E74, 27, 27, 0x1); // LANE21
		rtd_part_outl(0xb8000E84, 27, 27, 0x1); // LANE22
		rtd_part_outl(0xb8000E94, 27, 27, 0x1); // LANE23
		rtd_part_outl(0xb8000EA4, 27, 27, 0x1); // LANE24
		rtd_part_outl(0xb8000EB4, 27, 27, 0x1); // LANE25
		rtd_part_outl(0xb8000EC4, 27, 27, 0x1); // LANE26
		rtd_part_outl(0xb8000ED4, 27, 27, 0x1); // LANE27
		//
		// (11) Panel Power Enable (SW Control),
		
		mdelay(80);
		////////////////////////////////////////////////////////////////////////////		
		rtd_part_outl(0xb802D208, 1, 1, 0x1);		// Across frame and Across line 3 enable,
		rtd_part_outl(0xb802D208, 0, 0, 0x1);		// Global Tcon enable,
		////////////////////////////////////////////////////////////////////////////
		// Tcon23 (TP)
		rtd_outl(0xb802D4f0, 0x10580164);			   // H_start/H_end,
		rtd_outl(0xb802D4f4, 0x000008C9);			   // V_start/V_end,
		rtd_outl(0xb802D4f8, 0x00000080);			   // Control,
		////////////////////////////////////////////////////////////////////////////
		// Tcon7 (POL2)
		rtd_outl(0xB802D2f0, 0x08970897);			   // H_start/H_end,
		rtd_outl(0xB802D2f4, 0x00240024);			   // V_start/V_end,
		rtd_outl(0xb802D2f8, 0x00000480);			   // Control,
		rtd_outl(0xb802D300, 0x00000000);			   // Across Frame CTRL,
		// Tcon8 (POL1)
		rtd_outl(0xb802D310, 0x08970897);			   // H_start/H_end,
		rtd_outl(0xb802D314, 0x00001192);			   // V_start/V_end,
		rtd_outl(0xb802D318, 0x00000383);			   // Control,(00000383-->2 line inv,  -->1 frame inv)
		rtd_outl(0xb802D31c, 0x00040002);			   // Across Frame CTRL,
		////////////////////////////////////////////////////////////////////////////
		mdelay(25);
		// Tcon25 (CLK1)
		rtd_outl(0xb802d530, 0x0c0005f0);				// H_start/H_end,
		rtd_outl(0xb802d534, 0x002A08a6);			   // V_start/V_end,
		rtd_outl(0xb802d538, 0x00000380);					   // Control,
		rtd_outl(0xb802d53c, 0x00020001);					   // RP,HP
		////////////////////////////////////////////////////////////////////////////
		// Tcon2 (CLK2)
		rtd_outl(0xb802d250, 0x10FF0B00);			   // H_start/H_end,
		rtd_outl(0xb802d254, 0x002D08a6);			   // V_start/V_end,
		rtd_outl(0xb802d258, 0x00000380);			   // Control,
		rtd_outl(0xb802d25c, 0x00020001);					   // RP,HP
		////////////////////////////////////////////////////////////////////////////
		// Tcon19 (CLK3)
		rtd_outl(0xb802d470, 0x0C0005F0);			   // H_start/H_end,
		rtd_outl(0xb802d474, 0x002B08a6);			   // V_start/V_end,
		rtd_outl(0xb802d478, 0x00000380);
		rtd_outl(0xb802d47c, 0x00020001);					   // RP,HP
		////////////////////////////////////////////////////////////////////////////
		// Tcon4 (CLK4)
		rtd_outl(0xb802D290, 0x10FF0B00);			   // H_start/H_end,(0DC00260)
		rtd_outl(0xb802D294, 0x002E08a6);			   // V_start/V_end,
		rtd_outl(0xb802D298, 0x00000380);			   // Control,
		rtd_outl(0xb802D29C, 0x00020001);					  // RP,HP
		////////////////////////////////////////////////////////////////////////////
		mdelay(5);
		// Tcon1 (STV)
		rtd_outl(0xb802D230, 0x027E027E);			   // H_start/H_end,
		rtd_outl(0xb802D234, 0x0029002d);			   // V_start/V_end,
		rtd_outl(0xb802D238, 0x00000180);			   // Control,
		////////////////////////////////////////////////////////////////////////////
		// Tcon21 (LC)
		rtd_outl(0xb802d4b0, 0x04CC04CC);			  // H_start/H_end,
		rtd_outl(0xb802d4b4, 0x08C708C7);			   // V_start/V_end,
		rtd_outl(0xb802d4b8, 0x00000480);			   // Control,
		rtd_outl(0xb802d4c0, 0x00000064);			   // cross frame control, 
		////////////////////////////////////////////////////////////////////////////
#else
		return false;
#endif
	}
	else
	{
		printf(">>>>>>>>>>>>>>>> path:%s dst_length:%d\n",path, dst_length);
		CusApi_panel_mLVDS_CSOT_GOA_Setting_from_bin(dst_addr, dst_length); 
	}

	return true;
}


static bool CusApi_panel_mLVDS_tx(void)
{
	if(default_panel_parameter.iCONFIG_PANEL_CUSTOM_INDEX==P_MINI_LVDS_CSOT_PANEL)
		return CusApi_panel_mLVDS_CSOT_GOA_tcon_setting();
	
	return false;
}



bool CusApi_panel_interface_common(void)
{
	if(default_panel_parameter.iCONFIG_PANEL_TYPE == P_MINI_LVDS)
	{
		return CusApi_panel_mLVDS_tx();
	}

	return false;
}

void CusApi_panel_i2c_init(void)
{
	if(default_panel_parameter.iCONFIG_PANEL_TYPE == P_MINI_LVDS && default_panel_parameter.iCONFIG_PANEL_CUSTOM_INDEX == P_MINI_LVDS_CSOT_PANEL)
	{
		G1632_i2c_cmd();
		G2510S_i2c_cmd();
	}
}

void CusApi_panel_customized_setting(void)
{
	char *dst_addr = NULL;
	int dst_length;
    char path[256] = {0};

	if(default_panel_parameter.iCONFIG_PANEL_TYPE == P_MINI_LVDS && default_panel_parameter.iCONFIG_PANEL_CUSTOM_INDEX == P_MINI_LVDS_CSOT_PANEL)
	{
		memset(path,0,256);
		sprintf(path,"%s",VALC_FILENAME_IN_FACTORY);
		if(factory_read((char *)path, &dst_addr, &dst_length)) {
			printf("\033[1;31m>>>>>>>>>>>>>>>> can't find , path:%s\033[m\n",path);
		}
		else{
			printf(">>>>>>>>>>>>>>>> path:%s dst_length:%d\n",path, dst_length);
			VALC_from_bin(dst_addr, dst_length); 
		}
		
		drvif_color_valc_table((unsigned char *)VALC_Table);
		
		
		memset(path,0,256);
		sprintf(path,"%s",OUTPUT_GAMMA_FILENAME_IN_FACTORY);
		if(factory_read((char *)path, &dst_addr, &dst_length)) {
			printf("\033[1;31m>>>>>>>>>>>>>>>> can't find , path:%s\033[m\n",path);
		}
		else{
			printf(">>>>>>>>>>>>>>>> path:%s dst_length:%d\n",path, dst_length);
			OutputGamma_from_bin(dst_addr, dst_length); 
		}
		
		drvif_color_outputgamma_table(OutputGamma_Table);
	}
}
