/* Register */
#define REG_HW_SEMAPHORE		0xb801a000
#define REG_UART0_BASE			0xb8062300
#define REG_UART1_BASE			0xb801b100
#define DEFAULT_UART			REG_UART0_BASE

/* ACPU QC */
#define ACPU_QC_ENTRY_CODE_ADDR		(0xa1700000)
#define ACPU_QC_RESULT_SADDR		(0xa1780000)
#define ACPU_QC_RESULT_DEFAULT		(0x00000000)
#define ACPU_QC_RESULT_SUCCESS		(0x00cafe00)
#define ACPU_QC_RESULT_FAILED		(0xcafecafe)
#define ACPU_QC_PATTERN_SADDR		(0xa1800000)
#define ACPU_QC_PATTERN_BYTES		(0x00080000)
#define ACPU_QC_PATTERN_MAGIC		(0xa5a5aa55)

/* MIPS reg */
#define zero		$0
#define AT			$1
#define v0			$2
#define v1			$3
#define a0			$4
#define a1			$5
#define a2			$6
#define a3			$7
#define t0			$8
#define t1			$9
#define t2			$10
#define t3			$11
#define t4			$12
#define t5			$13
#define t6			$14
#define t7			$15
#define s0			$16
#define s1			$17
#define s2			$18
#define s3			$19
#define s4			$20
#define s5			$21
#define s6			$22
#define s7			$23
#define t8			$24
#define t9			$25
#define k0			$26
#define k1			$27
#define gp			$28
#define sp			$29
#define fp			$30
#define ra			$31

	.section ".text", "ax"
	.globl	a_entry
	.ent	a_entry
a_entry:
	.set 	noreorder

	/* set 5280 CP0 status register and vector interrupt address */
#if 1
	.word 0x40106000,0x3c118000,0x36310001,0x02118025,\
		 0x3c11ffbf,0x3631ffff,0x02118024,0x40906000,\
		 0x3c108000,0x36101900,0x40f01000,0x00000000,\
		 0x3c1000ff,0x40f00000,0x00000000
#else

	mfc0	s0, $12
	lui 	s1,0x8000
	ori 	s1,s1,0x1
	or		s0,s0,s1
	lui 	s1,0xffbf
	ori 	s1,s1,0xffff
	and 	s0,s0,s1
	mtc0	s0,$12
	lui 	s0,0x8000
	ori 	s0,s0,0x1900
	mtlxc0	s0,INTVEC
	nop
	lui 	s0,0xff
	mtlxc0	s0,ESTATUS
	nop

#endif

	/* Print Ago */
	li		t0, 'A'
	sw		t0, DEFAULT_UART

	li		t0, 'g'
	sw		t0, DEFAULT_UART

	li		t0, 'o'
	sw		t0, DEFAULT_UART

	li		t0, 0x0D
	sw		t0, DEFAULT_UART

	li		t0, 0x0A
	sw		t0, DEFAULT_UART

	li		t0, REG_HW_SEMAPHORE

get_HW_sem:
	# delay a while (reduce bus traffic)
	li		t2, 100
wait_HW_sem:
	subu	t2, 0x1
	bnez	t2, wait_HW_sem
	nop

	# check if HW sem is available
	lw		t1, 0(t0)
	nop		/* 5181 need nop after lw */
	beq		t1, zero, get_HW_sem
	nop

	# release HW sem
	li		s2, REG_HW_SEMAPHORE
	sw		zero, (s2)

start_qc:
	li		t0, ACPU_QC_PATTERN_BYTES	/* pattern total size */
	li		t1, ACPU_QC_PATTERN_SADDR	/* pattern address */
	li		t2, ACPU_QC_PATTERN_MAGIC	/* magic number for success pattern */
	li		t3, 0x0						/* counter is also the sample */

compare_loop:
	lw		t4, 0(t1)				/* load pattern */
	nop
	bne		t3, t4, qc_failed		/* diff pattern and sample */
	nop
	sw		t2, (t1)				/* store magic number to success pattern for scpu check */

	addi	t1, t1, 4				/* next pattern address */
	addi	t3, t3, 1				/* counter increment */
	bne		t3, t0, compare_loop	/* check end */
	nop

qc_success:
	li		t4, ACPU_QC_RESULT_SUCCESS
	li		t5, ACPU_QC_RESULT_SADDR
	sw		t4, 0(t5)
	b		a_loop
	nop

qc_failed:
	li		t4, ACPU_QC_RESULT_FAILED
	li		t5, ACPU_QC_RESULT_SADDR
	sw		t4, 0(t5)

a_loop:
	b		a_loop
	nop

	.set	reorder
	.end	a_entry
