;;----------------------------------------------------------------------------
;;
;;The confidential and proprietary information contained in this file may
;;only be used by a person authorised under and to the extent permitted
;;by a subsisting licensing agreement from ARM Limited.
;;
;;        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
;;             ALL RIGHTS RESERVED
;;             
;;This entire notice must be reproduced on all copies of this file
;;and copies of this file may only be made by a person if such person is
;;permitted to do so under the terms of a subsisting license agreement
;;from ARM Limited.
;;
;;    SVN Information
;;
;;    Checked In          : $Date$
;;
;;    Revision            : $Revision$
;;
;;    Release Information : r0p0-00rel0
;;
;;-----------------------------------------------------------------------------
;; 
;;-----------------------------------------------------------------------------
;; Abstract :Vectors initialisation
;;-----------------------------------------------------------------------------
;; Overview
;;----------
;; 
;;
;; 
;; 
;;----------------------------------------------------------------------------

        AREA    vectors, CODE

        ; -------
        ; Vectors
        ; -------

        ENTRY

        IMPORT  entrypoint
        IMPORT  uart_puts
        IMPORT  uart_putc
        EXPORT  __vector_entry

        EXPORT  vectors_irq_vector
        EXPORT  vectors_fiq_vector
        EXPORT  vectors_swi_vector
        EXPORT  wrapped_irq_vector

__vector_entry
reset           b       entrypoint
undef           b       undef_handler        
swi             LDR     pc,vectors_swi_vector
prefetch        b       prefetch_handler
abort           b       abort_handler
reserved        b       reserved_handler
irq             LDR     pc,vectors_irq_vector
fiq             LDR     pc,vectors_fiq_vector

	
;
; Indirect the vectors.
; - This enables other modules to override vector.
;                                                
vectors_swi_vector DCD swi_handler
vectors_irq_vector DCD irq_handler
vectors_fiq_vector DCD fiq_handler

; This is the pointer to the actual irq handler
; This is the irq_vector wrapped with wrapper to handle
; exception return
wrapped_irq_vector DCD default_irq_handler                                    

        ;; Undef handler
undef_handler
        LDR     r0,=undef_string
        B       error_exit

        ;; SWI handler
        ;; - armcc -> error
swi_handler
        LDR     r0, swi_string
        B       error_exit
        
        ;; Prefetch abort handler
prefetch_handler
        LDR     r0,=prefetch_string
        B       error_exit

        ;; Data abort handler
abort_handler
        LDR     r0,=abort_string
        B       error_exit

        ;; No handler
reserved_handler
        LDR     r0,=reserved_string
        B       error_exit

        ;; IRQ handler
irq_handler
        PUSH    {r0,r14}
        LDR     r0,wrapped_irq_vector
        BLX     r0
        POP     {r0,r14}
        SUBS    pc,r14,#4

default_irq_handler        
        LDR     r0,=irq_string
        B       error_exit

        ;; FIQ handler
fiq_handler
        LDR     r0,=fiq_string
        B       error_exit

        ;; Unsupported handler: print message and stop simulation
error_exit
        MOV     r4, lr
        BL      uart_puts               ; Report passed error string

        LDR     r0,=cpu_num_string
        BL      uart_puts

        [ ENABLE_SMP
        MRC     p15, 0, r0, c0, c0, 5
        AND     r0,r0,#0x3
        |
        MOV     r0,#0
        ]
        
        LDR     r1, =hextab
        LDRB    r0, [r1,r0]             ; convert to ASCII hex character
        BL      uart_putc

        LDR     r0,=newline
        BL      uart_puts        
        
        LDR     r0,=at_string           ; Start to output exceptional address
        BL      uart_puts

        MOV     r5, #28                 ; Start with bits 31:28
lp
        MOV     r2, r4, lsr r5          ; 
        AND     r2, r2, #0xF            ; get nybble 0x0 - 0xf
        LDR     r1, =hextab
        LDRB    r0, [r1,r2]             ; convert to ASCII hex character
        BL      uart_putc               ; squirt it out
        SUBS    r5, r5, #4              ; move to next nybble
        BCS     lp

        
        LDR     r0,=newline
        BL      uart_puts        
        MRC     p15,0,r4,c5,c0,0 ; read DFSR        
        
        MOV     r5, #28                 ; Start with bits 31:28
lp2
        MOV     r2, r4, lsr r5          ; 
        AND     r2, r2, #0xF            ; get nybble 0x0 - 0xf
        LDR     r1, =hextab
        LDRB    r0, [r1,r2]             ; convert to ASCII hex character
        BL      uart_putc               ; squirt it out
        SUBS    r5, r5, #4              ; move to next nybble
        BCS     lp2


        LDR     r0,=newline
        BL      uart_puts        
        MRC     p15,0,r4,c6,c0,0 ; read DFAR        
        
        MOV     r5, #28                 ; Start with bits 31:28
lp3
        MOV     r2, r4, lsr r5          ; 
        AND     r2, r2, #0xF            ; get nybble 0x0 - 0xf
        LDR     r1, =hextab
        LDRB    r0, [r1,r2]             ; convert to ASCII hex character
        BL      uart_putc               ; squirt it out
        SUBS    r5, r5, #4              ; move to next nybble
        BCS     lp3
		
        LDR     r0,=eof_string          ; output LF / <EOF> string
        BL      uart_puts               ; NOTE We print hex(4) to tube to exit.

        
err     B       err

undef_string    DCB     "FATAL: Unexpected undefined instruction.\n\0"
swi_string      DCB     "FATAL: Unexpected SWI/SVC instruction.\n\0"
prefetch_string DCB     "FATAL: Unexpected prefetch abort.\n\0"
abort_string    DCB     "FATAL: Unexpected data abort.\n\0"
reserved_string DCB     "FATAL: Unexpected reserved exception.\n\0"
irq_string      DCB     "FATAL: Unexpected interrupt request.\n\0"
fiq_string      DCB     "FATAL: Unexpected fast interrupt request.\n\0"
at_string       DCB     "At address 0x\0"
hextab          DCB     "0123456789abcdef"
eof_string      DCB     "\n<EOF>\n\004\000"
newline         DCB     "\n\0"
cpu_num_string  DCB     "CPU_#\0"

        ALIGN

        END
