/*-----------------------------------------------------------------------------
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from ARM Limited or its affiliates.
 *
 *            (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
 *                ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from ARM Limited or its affiliates.
 *
 *      Release Information : r0p0-00eac0
 *
 *-----------------------------------------------------------------------------
 *
 *-----------------------------------------------------------------------------
 * Abstract :Implementaion of all functions that are used to access MALI
 *-----------------------------------------------------------------------------
 * Overview
 *----------
 *
 **************************************************************/

//#include <stdlib.h>
//#include <stdio.h>
#include <qc_verify_common.h>
#include <common.h>

#include "MaliDefines_DVALIN.h"
#include "MaliFns_DVALIN.h"

static int gpu_errmask = 0;
static int mmu_errmask = 0;
static int job_errmask = 0;
static int event_errmask = 0;

void rt_suspend_gpu(void)
{
	//bit 0 is write data
	//CLKEN = 0
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x0); 
}

void Mali_Reset(void) 
{
#ifndef CONFIG_REALTEK_FPGA
	//bit 0 is write data
	//CLKEN = 0
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x0); 
	udelay(1000);

	//RSTN = 0
	rtd_outl(SYS_SRST1_reg,  0x1<<3 | 0x0); 
	udelay(1000);

	//RSTN = 1
	rtd_outl(SYS_SRST1_reg,  0x1<<3 | 0x1); 
	udelay(1000);

	//CLKEN = 1
	rtd_outl(SYS_CLKEN1_reg, 0x1<<3 | 0x1); 

#endif

	//gw_endian_swap
	rtd_outl(REGS_GPU_WRAPPER_ENDIAN,0x01);//endian

	//enable GPU interrupt
	rtd_outl(REGS_GPU_WRAPPER_INT_EN,0x0f);//gpu, mmu, job, event irq

#ifndef CONFIG_REALTEK_FPGA
	//Rounting to SCPU
	rtd_outl(SYS_INT_CTRL_SCPU,0x1<<20 | 0x1);
#endif
	// Release GPU core reset	
	rtd_outl(REGS_GPU_WRAPPER_RESET,0x3);

	udelay(1000);
};

void Mali_SetInterruptErrMask(int type, int e_mask)
{
  switch (type) {
	  case 1 : gpu_errmask = e_mask; break;
	  case 2 : mmu_errmask = e_mask; break;
	  case 3 : job_errmask = e_mask; break;
	  case 4 : event_errmask = e_mask; break;
	  default : 
		  printf("Programming Error\n"); 	
  }
}

void Mali_Identify(void)
{
	int gpuid = Mali_RdReg(GPU_ID);

	int l2features, tilerfeatures,memfeatures;
	int mmufeatures, aspresent, jspresent;
	int i, core_bitmap, num_cores;

	printf("GPU ID=0x%08x ProductID=0x%04x, Version=r%1dp%1d Version_status=%1d\n",
            (gpuid) & 0XFFFFFFFF,
            (gpuid >> PRODUCT_ID) & 0xFFFF,
            (gpuid >> VERSION_MAJOR) & 0xF,
            (gpuid >> VERSION_MINOR) & 0xFF,
            (gpuid >> VERSION_STATUS) & 0xF);

	l2features = Mali_RdReg(GPU_L2_FEATURES);
	printf("L2. LineSize=%2d Associativity=%2d CacheSize=%dkB BusWidth=%3d\n",
		   1 << ((l2features >> LINESIZE)       & 0xFF),
		   1 << ((l2features >> ASSOCIATIVITY)  & 0xFF),
		  (1 << ((l2features >> CACHE_SIZE) & 0xFF)) >> 10,
		   1 << ((l2features >> BUS_WIDTH) & 0xFF)
		);

	tilerfeatures =  Mali_RdReg(GPU_TILER_FEATURES);
	printf("Tiler. BinSize=%4d MaxActiveLevels=%4d\n",
		   1 << ((tilerfeatures >> BIN_SIZE)        & 0x3F),
		   1 << ((tilerfeatures >> MAX_LEVELS)   & 0xF));
		   
	memfeatures =  Mali_RdReg(GPU_MEM_FEATURES);
	printf("MemFeatures. CoherentCoreGroup=%d CoherentSuperGroup=%d L2 Slices=%d\n",
		   (memfeatures >> COHERENT_CORE_GROUP) & 0x1,
		   (memfeatures >> COHERENT_SUPER_GROUP) & 0x1,
         (((memfeatures >> L2_SLICES) & 0xf)+1));

	mmufeatures =  Mali_RdReg(GPU_MMU_FEATURES);
	printf("MMUFeatures. VA_BITS=%d PA_BITS=%d IPA_BITS=%d\n",
		   (mmufeatures >> VA_BITS) & 0xFF,
		   (mmufeatures >> PA_BITS) & 0xFF,
		   (mmufeatures >> IPA_BITS) & 0xFF);

	aspresent =  Mali_RdReg(GPU_AS_PRESENT);
	jspresent =  Mali_RdReg(GPU_JS_PRESENT);
	printf("ASPresent=%08x JSPresent=%08x\n",
		   aspresent,
		   jspresent);

  /* work out the number of cores and print out the result */
  core_bitmap = Mali_RdReg (GPU_SHADER_PRESENT);
  i = core_bitmap;
  num_cores = 0;
  
  while (i) {
    num_cores++;
    i &= i-1;
  }
  printf("Number of shader cores: %d\n", num_cores);
  
}


void *Mali_LdMem(void *srcptr,int size,int ttb_base) {
  /* Setup MMU to point to data structures for this image */
  Mali_WrReg(AS0_MEMATTR_HI,0);
  Mali_WrReg(AS0_MEMATTR_LO,0x88888888);
  Mali_WrReg(AS0_TRANSTAB_HI,0);
  Mali_WrReg(AS0_TRANSTAB_LO,ttb_base|0x3);
  printf (" - - - - - - - - > ttb_base = %x\n", ttb_base);
  Mali_WrReg(AS0_COMMAND,1); // Broadcast updates
  /* Clear interrupts and enable them */
  Mali_WrReg(MMU_IRQ_CLEAR,0xFFFFFFFF);
  Mali_WrReg(MMU_IRQ_MASK,0xFFFFFFFF);

  return srcptr;
};

static void Mali_GPUError(int stat,int mask,int faultstat,int faultvalo,int faultvahi)
{
	printf("Unexpected GPU Fault:\n");
	printf("Int Status=%08x",stat);
	printf(" Error Mask=%08x\n",mask);
	printf("FaultStats=%08x",faultstat);
	printf(" VA=%08x%08x\n",faultvahi,faultvalo);
	printf("TEST FAILED\n\n");
	exit(1);
}
static void Mali_MMUError(int stat,int mask,int faultstat,int faultvalo,int faultvahi)
{
	printf("Unexpected MMU Fault:\n");
	printf("Int Status=%08x",stat);
	printf(" Error Mask=%08x\n",mask);
	printf("FaultStats=%08x",faultstat);
	printf(" VA=%08x%08x\n",faultvahi,faultvalo);
	printf ("TEST FAILED\n\n");
	exit(1);
}
static void Mali_JobError(int stat,int mask)
{
	printf("Unexpected Job Interrupt:\n");
	printf("Int Status=%08x",stat);
	printf(" Error Mask=%08x\n",mask);
	printf ("TEST FAILED\n\n");
	exit(1);
}
static void Mali_EventError(int stat,int mask)
{
	printf("Unexpected Event Interrupt:\n");
	printf("Int Status=%08x",stat);
	printf(" Error Mask=%08x\n",mask);
	printf ("TEST FAILED\n\n");
	exit(1);
}

/* This function is backwards compatible with Mali400 API
 * For DVALIN it checks for job interrupts if mask and value given
 * We wait for any interrupt if mask and value is all set
 */
int Mali_InterruptCheck(int i_mask, int i_value)
{
	if ((i_mask == i_value) && i_mask == 0xFFFFFFFF) {
		return Mali_AnyInterruptCheck(0,i_mask,i_value);
	} else {
		return Mali_AnyInterruptCheck(3,i_mask,i_value);
	}
}

/* This function checks if we have the required interrupts.
 * It polls the Mali interrupt registers for this.
 * If we do not have the required interrupts, then it remembers which ones we 
 * have seen, and then masks them.
 */
int Mali_AnyInterruptCheck(int type, int i_mask, int i_value)
{
  int done;

  /* Read interrupt status */
  int gpu_stat = Mali_RdReg(GPU_IRQ_STATUS);
  int job_stat = Mali_RdReg(JOB_IRQ_STATUS);
  int mmu_stat = Mali_RdReg(MMU_IRQ_STATUS);
  int event_stat = Mali_RdReg(EVENT_IRQ_STATUS);

  int gpu_rawstat = Mali_RdReg(GPU_IRQ_RAWSTAT);
  int job_rawstat = Mali_RdReg(JOB_IRQ_RAWSTAT);
  int mmu_rawstat = Mali_RdReg(MMU_IRQ_RAWSTAT);
  int event_rawstat = Mali_RdReg(EVENT_IRQ_RAWSTAT);

  int v;

  /* Compare against failure conditions */
  if ((gpu_stat  & gpu_errmask) != 0) {
	  Mali_GPUError(gpu_stat,gpu_errmask,
					Mali_RdReg(GPU_FAULTSTATUS),
					Mali_RdReg(GPU_FAULTADDRESS_LO),
					Mali_RdReg(GPU_FAULTADDRESS_HI));
  }

  if ((mmu_stat & mmu_errmask) != 0) {
	  Mali_MMUError(mmu_stat,mmu_errmask,
					Mali_RdReg(AS0_FAULTSTATUS),
					Mali_RdReg(AS0_FAULTADDRESS_LO),
					Mali_RdReg(AS0_FAULTADDRESS_HI));
  }

  if ((job_stat & job_errmask) != 0) {
	  Mali_JobError(job_stat,job_errmask);
  }

  if ((event_stat & event_errmask) != 0) {
	  Mali_EventError(event_stat,event_errmask);
  }

  /* If no failure then test against masks as required */
  done = 0;
  switch (type) {
	  // This is wait for any interrupt, so check against real interrupt outputs
	  case 0 : done = ((gpu_stat | job_stat | mmu_stat | event_stat) != 0); break;

	  // These are masked checks, so check if required interrupts are triggered.
      // We do against rawstat, since if not all required interrupts have occured
      // then we need to mask the first ones to happen, whilst waiting for the others.
	  case 1 : done = ((gpu_rawstat & i_mask) == i_value); break;
	  case 2 : done = ((job_rawstat & i_mask) == i_value); break;
	  case 3 : done = ((mmu_rawstat & i_mask) == i_value); break;
	  case 4 : done = ((event_rawstat & i_mask) == i_value); break;
	  default : 
		  printf("Programming Error\n"); 	
		  exit(1);
  }





  /* If we do not satisfy final conditions then
   * mask all seen interrupts
   */
  if (!done) {
	  // If not done then mark the interrupts we have seen
	  Mali_WrReg(GPU_IRQ_MASK,Mali_RdReg(GPU_IRQ_MASK) &  ~gpu_stat);
	  Mali_WrReg(JOB_IRQ_MASK,Mali_RdReg(JOB_IRQ_MASK) &  ~job_stat);
	  Mali_WrReg(MMU_IRQ_MASK,Mali_RdReg(MMU_IRQ_MASK) &  ~mmu_stat);
	  Mali_WrReg(EVENT_IRQ_MASK,Mali_RdReg(EVENT_IRQ_MASK) &  ~event_stat);
	  printf ("Interrupt - conditions not met - WFI\n");
  }

  if (done) {
	  // When done - re-enable all the interrupts we were looking for
	  // This is only an issue for specific interrupt testing.
	  switch (type) {
		  case 0 :  break;
		  case 1 :  Mali_WrReg(GPU_IRQ_MASK, Mali_RdReg(GPU_IRQ_MASK) | i_value); break;
		  case 2 :  Mali_WrReg(JOB_IRQ_MASK, Mali_RdReg(JOB_IRQ_MASK) | i_value); break;
		  case 3 :  Mali_WrReg(MMU_IRQ_MASK, Mali_RdReg(MMU_IRQ_MASK) | i_value); break;
		  case 4 :  Mali_WrReg(EVENT_IRQ_MASK, Mali_RdReg(EVENT_IRQ_MASK) | i_value); break;
		  default :
			  printf("Programming error\n");
			  exit(1);
	  }

	  printf ("Interrupt - conditions met - continue test\n");
  }

  printf("gpu_raw_stat = %x\n",gpu_rawstat);
  printf("job_raw_stat = %x\n",job_rawstat);
  printf("mmu_raw_stat = %x\n",mmu_rawstat);
  printf("event_raw_stat = %x\n",event_rawstat);
  v =  Mali_RdReg(JS0_STATUS);
  printf("JS0Status = %x\n",v);
  v =  Mali_RdReg(JS1_STATUS);
  printf("JS1Status = %x\n",v);
  v =  Mali_RdReg(JS2_STATUS);
  printf("JS2Status = %x\n",v);

  return done;
};


void Mali_MaskAllInterrupts (void) {
	Mali_WrReg(GPU_IRQ_CLEAR,0xFFFFFFFF);
	Mali_WrReg(JOB_IRQ_CLEAR,0xFFFFFFFF);
	Mali_WrReg(MMU_IRQ_CLEAR,0xFFFFFFFF);
	Mali_WrReg(EVENT_IRQ_CLEAR,0xFFFFFFFF);
};


int Mali_RdReg(int regnum)
{
  int *reg_ptr = (int *)MALI_BASE + (regnum/4);
  #if 0
  return *reg_ptr;
  #else
  return rtd_inl(reg_ptr);
  #endif
};


void Mali_WrReg(int regnum,int value)
{
  
  int *reg_ptr = (int *)MALI_BASE + (regnum/4);
  #if 0
  *reg_ptr = value;
  #else
  rtd_outl(reg_ptr,value);
  #endif
};

unsigned int Mali_PollReg (unsigned int regnum, unsigned int bit_location, unsigned int val, char * name_str)
{
  int regnum_value =  Mali_RdReg(regnum);


  printf ("Polling (for %d) of register %s 0x%x bit %d started\n", val, name_str, regnum, bit_location);
  
  if (val)
  {
    while ((regnum_value & (1<<bit_location)) != (1<<bit_location))
    {
      regnum_value = Mali_RdReg(regnum);
    }  
  }
  else
  {
    while ((regnum_value & (1<<bit_location)) == (1<<bit_location))
    {
      regnum_value = Mali_RdReg(regnum);
    }  
  }
  printf ("Polling (for %d) of register %s 0x%x bit %d ended   current value=0x%x \n", val, name_str, regnum, bit_location, regnum_value);
  
  return regnum_value;
};

void Mali_clear_irqs_and_set_all_masks (void) {
  Mali_clear_and_set_masks_for_gpu_irq ();
  Mali_clear_and_set_masks_for_mmu_irq ();
  Mali_clear_and_set_masks_for_job_irq ();
  Mali_clear_and_set_masks_for_event_irq ();
}

void Mali_clear_and_set_masks_for_gpu_irq (void) {
  printf ("Clear the GPU IRQ flags\n");
  Mali_WrReg(GPU_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the GPU IRQ Mask to all 1s\n");
  Mali_WrReg(GPU_IRQ_MASK        , 0xFFFFFFFF);
}
void Mali_clear_and_set_masks_for_mmu_irq (void) {
  printf ("Clear the MMU IRQ flags\n");
  Mali_WrReg(MMU_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the MMU IRQ Mask to all 1s\n");
  Mali_WrReg(MMU_IRQ_MASK        , 0xFFFFFFFF);
}
void Mali_clear_and_set_masks_for_job_irq (void) {
  printf ("Clear the JOB IRQ flags\n");
  Mali_WrReg(JOB_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the JOB IRQ Mask to all 1s\n");
  Mali_WrReg(JOB_IRQ_MASK        , 0xFFFFFFFF);
}
void Mali_clear_and_set_masks_for_event_irq (void) {
  printf ("Clear the EVENT IRQ flags\n");
  Mali_WrReg(EVENT_IRQ_CLEAR       , 0xFFFFFFFF);
  printf ("Set the EVENT IRQ Mask to all 1s\n");
  Mali_WrReg(EVENT_IRQ_MASK        , 0xFFFFFFFF);
}

void Mali_write_ASn_register (int ASn_reg, int n, int value_to_write) {
  int ASn_reg_final = MEM_MANAGEMENT_BASE | ASn_BASE | ASn_reg | (n * 64);
#ifdef DEBUG_ON
  printf ("Write: ASn_reg_final=0x%x n=%d value=0x%x\n", ASn_reg_final, n, value_to_write);
#endif
  Mali_WrReg (ASn_reg_final, value_to_write);
}

void Mali_write_JSn_register (int JSn_reg, int n, int value_to_write) {
  int JSn_reg_final = JOB_CONTROL_BASE | JSn_BASE | JSn_reg | (n * 128);
#ifdef DEBUG_ON
  printf ("Write: JSn_reg_final=0x%x n=%d value=0x%x\n", JSn_reg_final, n, value_to_write);
#endif
  Mali_WrReg (JSn_reg_final, value_to_write);
}

int Mali_read_JSn_register (int JSn_reg, int n) {
  int JSn_reg_final = JOB_CONTROL_BASE | JSn_BASE | JSn_reg | (n * 128);
  int value_read = Mali_RdReg (JSn_reg_final);
#ifdef DEBUG_ON
  printf ("Read: JSn_reg_final=0x%x n=%d value=0x%x\n", JSn_reg_final, n, value_read);
#endif
  return value_read;
}
