/**
* @file rbusSmartfitReg.h
* RBus systemc program.
*
* @author RS_MM2_SD
* @email jason9.ccc@realtek.com
 * @date 2016/6/3
* @ingroup model_rbus
 * @version { 1.0 }
 **
 */

#ifndef _RBUS_SMARTFIT_REG_H_
#define _RBUS_SMARTFIT_REG_H_

#include "rbusTypes.h"



//  SMARTFIT Register Address



#define  SMARTFIT_AUTO_FIELD_VADDR                          	(0xb8022000)
#define  SMARTFIT_AUTO_H_BOUNDARY_VADDR                     	(0xb8022004)
#define  SMARTFIT_AUTO_V_BOUNDARY_VADDR                     	(0xb8022008)
#define  SMARTFIT_AUTO_R_G_B_MARGIN_VADDR                   	(0xb802200c)
#define  SMARTFIT_AUTO_ADJ_VADDR                            	(0xb8022010)
#define  SMARTFIT_AUTO_RESULT_VSTA_END_VADDR                	(0xb8022014)
#define  SMARTFIT_AUTO_RESULT_HSTA_END_VADDR                	(0xb8022018)
#define  SMARTFIT_AUTO_RESULT_PHASE_M_VADDR                 	(0xb802201c)
#define  SMARTFIT_AUTO_RESULT_PHASE_L_VADDR                 	(0xb8022020)
#define  SMARTFIT_AUTO_PHASE_CTRL0_VADDR                    	(0xb8022024)
#define  SMARTFIT_PLLPHASE_CTRL_VADDR                       	(0xb8022028)



#ifndef LITTLE_ENDIAN //apply BIG_ENDIAN

//======SMARTFIT register structure define==========

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  smartfit_src_sel:1;
        RBus_UInt32  auto_balance_rgb:1;
        RBus_UInt32  smfit_en_n:1;
        RBus_UInt32  res1:16;
        RBus_UInt32  vgip_db_disable:1;
        RBus_UInt32  adj_source:2;
        RBus_UInt32  rl_sel_en:1;
        RBus_UInt32  rl_sel:1;
        RBus_UInt32  odd_sel_en:1;
        RBus_UInt32  odd_sel:1;
        RBus_UInt32  res2:1;
        RBus_UInt32  rgb_in_one_en:1;
        RBus_UInt32  square_en:1;
        RBus_UInt32  sum_pixel_num:1;
        RBus_UInt32  filter_sel:2;
    };
}smartfit_auto_field_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:2;
        RBus_UInt32  hb_sta:14;
        RBus_UInt32  res2:2;
        RBus_UInt32  hb_end:14;
    };
}smartfit_auto_h_boundary_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:3;
        RBus_UInt32  vb_sta:13;
        RBus_UInt32  res2:3;
        RBus_UInt32  vb_end:13;
    };
}smartfit_auto_v_boundary_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  red_nm:6;
        RBus_UInt32  adj_even_odd:1;
        RBus_UInt32  adj_even_odd_en:1;
        RBus_UInt32  grn_nm:6;
        RBus_UInt32  vb_th:2;
        RBus_UInt32  blu_nm:6;
        RBus_UInt32  color_sel:2;
        RBus_UInt32  diff_th:8;
    };
}smartfit_auto_r_g_b_margin_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:24;
        RBus_UInt32  m_vgen_en:1;
        RBus_UInt32  diff_pixel_sel:1;
        RBus_UInt32  force_flip_:1;
        RBus_UInt32  sum_max:1;
        RBus_UInt32  pulse_det_en:1;
        RBus_UInt32  autophase_sel_tri:1;
        RBus_UInt32  diff_en:1;
        RBus_UInt32  now_af:1;
    };
}smartfit_auto_adj_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:3;
        RBus_UInt32  vx_sta:13;
        RBus_UInt32  res2:3;
        RBus_UInt32  vx_end:13;
    };
}smartfit_auto_result_vsta_end_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:2;
        RBus_UInt32  hx_sta:14;
        RBus_UInt32  res2:2;
        RBus_UInt32  hx_end:14;
    };
}smartfit_auto_result_hsta_end_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:22;
        RBus_UInt32  acc_41_32:10;
    };
}smartfit_auto_result_phase_m_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  acc_31_0:32;
    };
}smartfit_auto_result_phase_l_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  autophase_en:1;
        RBus_UInt32  res1:6;
        RBus_UInt32  autophase_step:3;
        RBus_UInt32  init_phase:6;
        RBus_UInt32  res2:1;
        RBus_UInt32  autophase_max_step:7;
        RBus_UInt32  autophase_flg:1;
        RBus_UInt32  autosod_step_num:7;
    };
}smartfit_auto_phase_ctrl0_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  res1:9;
        RBus_UInt32  pll2_ph_lut:1;
        RBus_UInt32  res2:1;
        RBus_UInt32  pll2d2:1;
        RBus_UInt32  pll2_d2x:1;
        RBus_UInt32  pll2_d2y:1;
        RBus_UInt32  pll2x:1;
        RBus_UInt32  pll2y:1;
        RBus_UInt32  res3:3;
        RBus_UInt32  pll2_psel:5;
        RBus_UInt32  res4:1;
        RBus_UInt32  pll2_ph_lut_index_msb:1;
        RBus_UInt32  pll2_ph_lut_index:6;
    };
}smartfit_pllphase_ctrl_RBUS;

#else //apply LITTLE_ENDIAN

//======SMARTFIT register structure define==========

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  filter_sel:2;
        RBus_UInt32  sum_pixel_num:1;
        RBus_UInt32  square_en:1;
        RBus_UInt32  rgb_in_one_en:1;
        RBus_UInt32  res1:1;
        RBus_UInt32  odd_sel:1;
        RBus_UInt32  odd_sel_en:1;
        RBus_UInt32  rl_sel:1;
        RBus_UInt32  rl_sel_en:1;
        RBus_UInt32  adj_source:2;
        RBus_UInt32  vgip_db_disable:1;
        RBus_UInt32  res2:16;
        RBus_UInt32  smfit_en_n:1;
        RBus_UInt32  auto_balance_rgb:1;
        RBus_UInt32  smartfit_src_sel:1;
    };
}smartfit_auto_field_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  hb_end:14;
        RBus_UInt32  res1:2;
        RBus_UInt32  hb_sta:14;
        RBus_UInt32  res2:2;
    };
}smartfit_auto_h_boundary_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  vb_end:13;
        RBus_UInt32  res1:3;
        RBus_UInt32  vb_sta:13;
        RBus_UInt32  res2:3;
    };
}smartfit_auto_v_boundary_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  diff_th:8;
        RBus_UInt32  color_sel:2;
        RBus_UInt32  blu_nm:6;
        RBus_UInt32  vb_th:2;
        RBus_UInt32  grn_nm:6;
        RBus_UInt32  adj_even_odd_en:1;
        RBus_UInt32  adj_even_odd:1;
        RBus_UInt32  red_nm:6;
    };
}smartfit_auto_r_g_b_margin_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  now_af:1;
        RBus_UInt32  diff_en:1;
        RBus_UInt32  autophase_sel_tri:1;
        RBus_UInt32  pulse_det_en:1;
        RBus_UInt32  sum_max:1;
        RBus_UInt32  force_flip_:1;
        RBus_UInt32  diff_pixel_sel:1;
        RBus_UInt32  m_vgen_en:1;
        RBus_UInt32  res1:24;
    };
}smartfit_auto_adj_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  vx_end:13;
        RBus_UInt32  res1:3;
        RBus_UInt32  vx_sta:13;
        RBus_UInt32  res2:3;
    };
}smartfit_auto_result_vsta_end_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  hx_end:14;
        RBus_UInt32  res1:2;
        RBus_UInt32  hx_sta:14;
        RBus_UInt32  res2:2;
    };
}smartfit_auto_result_hsta_end_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  acc_41_32:10;
        RBus_UInt32  res1:22;
    };
}smartfit_auto_result_phase_m_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  acc_31_0:32;
    };
}smartfit_auto_result_phase_l_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  autosod_step_num:7;
        RBus_UInt32  autophase_flg:1;
        RBus_UInt32  autophase_max_step:7;
        RBus_UInt32  res1:1;
        RBus_UInt32  init_phase:6;
        RBus_UInt32  autophase_step:3;
        RBus_UInt32  res2:6;
        RBus_UInt32  autophase_en:1;
    };
}smartfit_auto_phase_ctrl0_RBUS;

typedef union
{
    RBus_UInt32 regValue;
    struct{
        RBus_UInt32  pll2_ph_lut_index:6;
        RBus_UInt32  pll2_ph_lut_index_msb:1;
        RBus_UInt32  res1:1;
        RBus_UInt32  pll2_psel:5;
        RBus_UInt32  res2:3;
        RBus_UInt32  pll2y:1;
        RBus_UInt32  pll2x:1;
        RBus_UInt32  pll2_d2y:1;
        RBus_UInt32  pll2_d2x:1;
        RBus_UInt32  pll2d2:1;
        RBus_UInt32  res3:1;
        RBus_UInt32  pll2_ph_lut:1;
        RBus_UInt32  res4:9;
    };
}smartfit_pllphase_ctrl_RBUS;




#endif 


#endif 
