//------------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited or its affiliates.
//
//            (C) COPYRIGHT 2013-2018 ARM Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited or its affiliates.
//
//            Release Information : CORINTH-MP090-r4p0-00rel0
//
//------------------------------------------------------------------------------

#ifndef BENCHMARK_H
#define BENCHMARK_H

//#include <stdio.h>

// Trace benchmarking
#ifdef ETMBM
  // defines
  #define APBETM_ADDRESS ((volatile unsigned*) 0x14040000u)
  #define MAINEND benchmark_finish();
  #define BENCHSTART printf ("**  BENCHSTART  **\n"); program_etm_tpiu(); enable_etm();
  #define BENCHFINISHED disable_etm();

  // functions
  // programs ETM and TPIU
  static void program_etm_tpiu()
  {
    // TRCLAR: unlock software lock
    *(APBETM_ADDRESS+0x3ec) = 0xc5acce55;
    // TRCOSLAR: unlock os lock
    *(APBETM_ADDRESS+0xc0) = 0x0;
    // TRCCONFIGR: disable trace
    *(APBETM_ADDRESS+0x1)  = 0x0;
    // TRCEVENTCTL0R: all events disabled
    *(APBETM_ADDRESS+0x8)  = 0x0;
    // TRCEVENTCTL1R: all events disabled
    *(APBETM_ADDRESS+0x9)  = 0x0;
    // TRCSTALLCTLR: no stalling
    *(APBETM_ADDRESS+0xb)  = 0x0;
    // TRCSYNCPR: sync every 4096 bytes
    *(APBETM_ADDRESS+0xd)  = 0xc;
    // TRCTRACEID: ID is 0x1
    *(APBETM_ADDRESS+0x10) = 0x1;
    // TRCVICTLR: event set to Always, S/S to Start
    *(APBETM_ADDRESS+0x20) = 0x201;
    // TRCVIIECTLR: include all
    *(APBETM_ADDRESS+0x21) = 0x0;
    // TRCVISSCTLR: start stop not active
    *(APBETM_ADDRESS+0x22) = 0x0;
    // ETMBM = 1: P0 branch only, return stack enabled
    // ETMBM = 2: P0 branch only, return stack enabled, cycle count enabled, timestamp enabled
    if (ETMBM == 1) {
      // TRCCONFIGR
      *(APBETM_ADDRESS+0x4)  = 0x1001;
    }
    else if (ETMBM == 2) {
      // TRCCCCTLR: cycle Counting at 20 cycle threshold
      *(APBETM_ADDRESS+0xe)  = 0x14;
      // TSCTLR: Timestamping Enabled - select resource selector 2
      *(APBETM_ADDRESS+0xc)  = 0x2;
      // Write TRCRSCTLR2 - resource selector 2, select counter 2
      *(APBETM_ADDRESS+0x82) = 0x00020001;
      // TRCCNTVR0: set initial counter value (1000)
      *(APBETM_ADDRESS+0x58) = 0x3e8;
      // Write TRCCNTRLDVR0: set counter reload value (1000)
      *(APBETM_ADDRESS+0x50) = 0x3e8;
      // Write TRCCNTCTLR0: enable counter with self reload
      *(APBETM_ADDRESS+0x54) = 0x10001;
      // TRCCONFIGR
      *(APBETM_ADDRESS+0x4)  = 0x1811;
    }
    // Use reserved register offset 0 to initiate TPIU programming sequence in testbench
    // TPIUBM value = port size width
    *(APBETM_ADDRESS) = 1<<(TPIUBM-1);
    printf ("** ETM %0d and TPIU %0d programmed **\n", ETMBM, TPIUBM);
  }
  
  // enable ETM
  static void enable_etm()
  {
    printf ("** Turning on ETM **\n");
    *(APBETM_ADDRESS+0x1) = 1;
  }
  
  // disable ETM
  static void disable_etm()
  {
    *(APBETM_ADDRESS+0x1) = 0;
    printf ("** ETM disabled **\n");
  }
#else
  #define MAINEND
  #define BENCHSTART
  #define BENCHFINISHED benchmark_finish();
  #define ITERATIONS 40
#endif

#define TUBE_ADDRESS ((volatile char *) 0x13000000u)

#define MAINSTART

#define LOOPSTART
#define LOOPEND

// This function writes characters directly rather the use printf so that
// it can work when printf is not available.
static void benchmark_finish(void)
{
  char  p[] = "** TEST PASSED OK **\n";
  char* c   = p;

  while (*c)
  {
    *TUBE_ADDRESS = *c;
    c++;
  }

  *TUBE_ADDRESS = 0x4;
}


#endif
