/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Misc functions
 */
#include <common.h>
#include <command.h>
#include <asm-generic/gpio.h>
#include <drivers/gpio/rtk_gpio.h>
#include <common/rtk_menu.h>
#include <platform_lib/board/pcb_mgr.h>
#include <system.h>
DECLARE_GLOBAL_DATA_PTR;

#ifndef name_to_gpio
#define name_to_gpio(name) simple_strtoul(name, NULL, 10)
#endif

typedef enum gpio_cmd {
    GPIO_MISC_GET,
    GPIO_MISC_SET,
    GPIO_ISO_GET,
    GPIO_ISO_SET,
    GPI_MISC_GET,
    GPI_ISO_GET,
    GPO_MISC_SET,
    GPO_ISO_SET,
    GPO_MISC_GET,
    GPO_ISO_GET,
}GPIO_CMD;

extern unsigned long strtoul(const char *str, char **endptr, int base);

typedef struct
{
    unsigned long long  pcbenumValue;
    char                *namePtr;
    PCB_PIN_TYPE_T      pinType;
    PCB_GPIO_TYPE_T     gpioType;
    short int           gpioPinNum;
    char                invertFlag;
    char                initValue;
}RTK_GPIO_PIN_INFO;

typedef struct
{
    int                 gpioPinNum;
    RTK_GPIO_PIN_INFO   gpioPinInfoList[64];
}RTK_GPIO_DUMP_INFO;

typedef enum gpio_sort{
    INPUT  = 0,
    OUTPUT,
    ALL,
}GPIO_SORT;

int _gGpioInfoInitFlag = 0;
static RTK_GPIO_DUMP_INFO _gGpioDumpInfo;
char gpio_cmd_buf[256];

rtk_menu_state gpio_state;

/*check enter gpio num*/
//para:
//      gpioNum:  enter gpio num
//      gpioType: get gpio type max num
//return:
//      0:  pass
//      -1: failed

static int check_enter_gpioNum(int gpioNum, int gpioType_flag)
{
    int ret = 0;
    switch(gpioType_flag){
        case 0:
            if(gpioNum < 0 || gpioNum > MISC_GPIO_MAX_NUM){
                ret = -1;
                printf("  MAX MISC GPIO NUM : %d\n",MISC_GPIO_MAX_NUM);
            }
            break;
        case 1:
            if(gpioNum < 0 || gpioNum > ISO_GPIO_MAX_NUM){
                ret = -1;
                printf("  MAX ISO GPIO NUM: %d\n",ISO_GPIO_MAX_NUM);
            }
            break;
#ifdef MISC_GPI_SUPPORT
        case 2:
            if(gpioNum < 0 || gpioNum > MISC_GPI_MAX_NUM){
                ret = -1;
                printf("  MAX MISC GPI NUM: %d\n",MISC_GPI_MAX_NUM);
            }
            break;
#endif
#ifdef ISO_GPI_SUPPORT
        case 3:
            if(gpioNum < 0 || gpioNum > ISO_GPI_MAX_NUM){
                ret = -1;
                printf("  MAX ISO GPI NUM: %d\n",ISO_GPI_MAX_NUM);
            }
            break;
#endif
#ifdef MISC_GPO_SUPPORT
        case 4:
            if(gpioNum < 0 || gpioNum > MISC_GPO_MAX_NUM){
                ret = -1;
                printf("  MAX MISC GPO NUM: %d\n",MISC_GPO_MAX_NUM);
            }
            break;
#endif
#ifdef ISO_GPO_SUPPORT
        case 5:
            if(gpioNum < 0 || gpioNum > ISO_GPO_MAX_NUM){
                ret = -1;
                printf("  MAX ISO GPO NUM: %d\n",ISO_GPO_MAX_NUM);
            }
            break;
#endif
        default:
            ret = -1;
            break;
    }

    return ret;

}

/*get gpio cmd from user enter*/
//para:
//      gpioNum:  enter gpio num
//      gpioType: get gpio type max num
//return:
//      GPIO_CMD
//

static GPIO_CMD get_gpio_cmd(int isGet,int gpioType_flag)
{
    GPIO_CMD gpioCmd = GPIO_MISC_GET;

    switch(gpioType_flag){
        case 0:
            if(isGet == 1)
                gpioCmd = GPIO_MISC_GET;
            else
                gpioCmd = GPIO_MISC_SET;
            break;
        case 1:
            if(isGet == 1)
                gpioCmd = GPIO_ISO_GET;
            else
                gpioCmd = GPIO_ISO_SET;
            break;
        case 2:
            if(isGet == 1)
                gpioCmd = GPI_MISC_GET;
            break;
        case 3:
            if(isGet == 1)
                gpioCmd = GPI_ISO_GET;
            break;
        case 4:
            if(isGet == 1)
                gpioCmd = GPO_MISC_GET;
            else
                gpioCmd = GPO_MISC_SET;
            break;
        case 5:
            if(isGet == 1)
                gpioCmd = GPO_ISO_GET;
            else
                gpioCmd = GPO_ISO_SET;
            break;
        default:
            break;
    }

    return gpioCmd;

}

int do_gpio_get_info(void)
{
    PCB_PIN_TYPE_T     pin_type;
    int                i = 0;
    int                iPcbCounter = 0;
    unsigned long long ullPcbPinValue = 0;
    if(_gGpioInfoInitFlag == 0)
        _gGpioInfoInitFlag = 1;
    else
        return 0;
#if 0 //[CID-314118]: Logically dead code
    if(NULL == pcb_enum){
        printf("ppcb_enum is NULL\n");
        return -1;
    }
#endif
    do {
        if((pcb_enum[i].name == NULL) || (0 == pcb_enum[i].name[0])){
            break;
        }
        if(memcmp(pcb_enum[i].name, "PIN", 3) == 0){
            pin_type = GET_PIN_TYPE(pcb_enum[i].info);
            //printf("pin type :%d, pin name:%s\n", pin_type, pcb_enum[i].name);
            if((PCB_PIN_TYPE_GPIO == pin_type ) || (PCB_PIN_TYPE_ISO_GPIO == pin_type)
#ifdef MISC_GPI_SUPPORT
               || (PCB_PIN_TYPE_GPI == pin_type)
#endif
#ifdef MISC_GPO_SUPPORT
               || (PCB_PIN_TYPE_GPO == pin_type)
#endif
#ifdef ISO_GPI_SUPPORT
               ||(PCB_PIN_TYPE_ISO_GPI == pin_type)
#endif
#ifdef ISO_GPO_SUPPORT
               || (PCB_PIN_TYPE_ISO_GPO == pin_type)
#endif
               ) {
                ullPcbPinValue = pcb_enum[i].info;
                RTK_GPIO_PIN_INFO *tmpGpioInfoPtr = &(_gGpioDumpInfo.gpioPinInfoList[iPcbCounter]);
                tmpGpioInfoPtr->pcbenumValue = ullPcbPinValue;
                tmpGpioInfoPtr->namePtr      = pcb_enum[i].name;
                tmpGpioInfoPtr->pinType      = pin_type;
                tmpGpioInfoPtr->gpioPinNum   = GET_PIN_INDEX(ullPcbPinValue);
                tmpGpioInfoPtr->gpioType     = GET_PIN_PARAM1(ullPcbPinValue);
                tmpGpioInfoPtr->invertFlag   = GET_PIN_PARAM2(ullPcbPinValue);
                tmpGpioInfoPtr->initValue    = GET_PIN_PARAM3(ullPcbPinValue);
                iPcbCounter++;
                //printf("pin type :%d, iPcbCounter:%d, ullPcbPinValue:0x%llx\n", pin_type, iPcbCounter, ullPcbPinValue);
            }
        }
        i ++;
    } while(1);

    _gGpioDumpInfo.gpioPinNum = iPcbCounter;

    return 0;
}

int do_gpio_dump_info(void)
{
    int i;
    char *str;
    char *pinTypeStr;

    printf("        =====  Total GPIO pin : %d =====\n", _gGpioDumpInfo.gpioPinNum);

    if(0 == _gGpioDumpInfo.gpioPinNum)
        return -1;

    for(i = 0; i < _gGpioDumpInfo.gpioPinNum; i++){
        RTK_GPIO_PIN_INFO *tmpGpioInfoPtr = &(_gGpioDumpInfo.gpioPinInfoList[i]);
        switch(tmpGpioInfoPtr->gpioType)
        {
            case PCB_GPIO_TYPE_INPUT:
                str = " Input";
                break;
            case PCB_GPIO_TYPE_OUPUT:
                str = "Output";
                break;
            case PCB_GPIO_TYPE_TRI_IO:
                str = "Tri_IO";
                break;
            default:
                str = " Undef";
                break;
        }
        switch(tmpGpioInfoPtr->pinType)
        {
            case PCB_PIN_TYPE_GPIO:
                pinTypeStr = "MISC_GPIO";
                break;
            case PCB_PIN_TYPE_ISO_GPIO:
                pinTypeStr = " ISO_GPIO";
                break;
#ifdef MISC_GPI_SUPPORT
           case PCB_PIN_TYPE_GPI:
               pinTypeStr = "MISC_GPI";
               break;
#endif
#ifdef MISC_GPO_SUPPORT
           case PCB_PIN_TYPE_GPO:
               pinTypeStr = "MISC_GPO";
               break;
#endif
#ifdef ISO_GPI_SUPPORT
           case PCB_PIN_TYPE_ISO_GPI:
               pinTypeStr = "MISC_GPO";
               break;
#endif
#ifdef ISO_GPO_SUPPORT
           case PCB_PIN_TYPE_ISO_GPO:
               pinTypeStr = " IOS_GPO";
               break;
#endif
            default:
                pinTypeStr = " Undef";
                break;
        }
        printf("---------------------------------------------------------------------- \n");
        printf("GPIO name:%s, value:0x%016llx \n", tmpGpioInfoPtr->namePtr, tmpGpioInfoPtr->pcbenumValue);
        printf("GPIO type:%s, num:%3d, direction:%s, invert:%d, initValue:%d\n",
               pinTypeStr, tmpGpioInfoPtr->gpioPinNum, str, tmpGpioInfoPtr->invertFlag, tmpGpioInfoPtr->initValue);
    }
    return 0;
}

int do_gpio_dump_list(GPIO_SORT type)
{
    int i = 0;
    if(type == ALL){
        for(i=0;i<_gGpioDumpInfo.gpioPinNum;++i){
            if(strlen(_gGpioDumpInfo.gpioPinInfoList[i].namePtr) == 0)
                continue;
            printf("[%-2d]:%-20s  [%-6s]\n",i,_gGpioDumpInfo.gpioPinInfoList[i].namePtr,
                _gGpioDumpInfo.gpioPinInfoList[i].gpioType == PCB_GPIO_TYPE_INPUT ? "INPUT" : "OUTPUT");
        }
    }else if(type == OUTPUT){
        for(i=0;i<_gGpioDumpInfo.gpioPinNum;++i){
            if(strlen(_gGpioDumpInfo.gpioPinInfoList[i].namePtr) == 0 ||
                _gGpioDumpInfo.gpioPinInfoList[i].gpioType == PCB_GPIO_TYPE_INPUT)
                continue;
            printf("[%-2d]:%-20s  [OUTPUT]\n",i,_gGpioDumpInfo.gpioPinInfoList[i].namePtr);
        }
    }else if(type == INPUT){
        for(i=0;i<_gGpioDumpInfo.gpioPinNum;++i){
            if(strlen(_gGpioDumpInfo.gpioPinInfoList[i].namePtr) == 0 ||
                _gGpioDumpInfo.gpioPinInfoList[i].gpioType == PCB_GPIO_TYPE_OUPUT)
                continue;
            printf("[%-2d]:%-20s  [INPUT]\n",i,_gGpioDumpInfo.gpioPinInfoList[i].namePtr);
        }
    }
    return 0;
}

int do_gpio_set_cmd(int select,int setValue)
{
    int gpio;
    int ret = -1;
    if(select > _gGpioDumpInfo.gpioPinNum)
        return ret;

    gpio = _gGpioDumpInfo.gpioPinInfoList[select].gpioPinNum;
    if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_ISO_GPIO)
        ret = rt_gpio_set(gpio,setValue, ISO_GPIO);
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_GPIO)
        ret = rt_gpio_set(gpio,setValue, MISC_GPIO);
#ifdef MISC_GPO_SUPPORT
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_GPO)
        ret = rt_gpo_set(gpio,setValue, MISC_GPO);
#endif
#ifdef ISO_GPO_SUPPORT
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_ISO_GPO)
        ret = rt_gpo_set(gpio,setValue, ISO_GPO);
#endif
    return ret;
};

int do_gpio_get_cmd(int select,int *getValue)
{
    int gpio;
    if(select > _gGpioDumpInfo.gpioPinNum)
        return -1;

    gpio = _gGpioDumpInfo.gpioPinInfoList[select].gpioPinNum;
    if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_ISO_GPIO)
        *getValue = rt_gpio_get(gpio, ISO_GPIO);
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_GPIO)
        *getValue = rt_gpio_get(gpio, MISC_GPIO);
#ifdef MISC_GPO_SUPPORT
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_GPO)
        *getValue = rt_gpo_get(gpio, MISC_GPO);
#endif
#ifdef ISO_GPO_SUPPORT
    else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_ISO_GPO)
        *getValue = rt_gpo_get(gpio, ISO_GPO);
#endif
#ifdef MISC_GPI_SUPPORT
        else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_GPI)
        *getValue = rt_gpo_get(gpio, MISC_GPO);
#endif
#ifdef ISO_GPI_SUPPORT
        else if(_gGpioDumpInfo.gpioPinInfoList[select].pinType == PCB_PIN_TYPE_ISO_GPI)
        *getValue = rt_gpi_get(gpio, ISO_GPI);
#endif

    if(*getValue == -1)
        return -1;
    else
        return 0;
};

int do_gpio_senior_debug_cmd(int gpioNum,GPIO_CMD subCmd,int* value)
{
    int ret = 0;
    switch (subCmd) {
        case GPIO_MISC_GET:
            *value = rt_gpio_get(gpioNum, MISC_GPIO);
            if(*value == -1)
                ret = -1;
        break;
        case GPIO_MISC_SET:
            ret = rt_gpio_set(gpioNum,*value, MISC_GPIO);
            break;
        case GPIO_ISO_GET:
            *value = rt_gpio_get(gpioNum, ISO_GPIO);
            if(*value == -1)
                ret = -1;
        break;
        case GPIO_ISO_SET:
            ret = rt_gpio_set(gpioNum,*value, ISO_GPIO);
            break;
#ifdef MISC_GPO_SUPPORT
        case GPO_MISC_GET:
            *value = rt_gpo_get(gpioNum, MISC_GPO);
            if(*value == -1)
                ret = -1;
            break;
        case GPO_MISC_SET:
            ret = rt_gpo_set(gpioNum,*value, MISC_GPO);
            break;
#endif
#ifdef ISO_GPO_SUPPORT
        case GPO_ISO_GET:
            *value = rt_gpo_get(gpioNum, ISO_GPO);
            if(*value == -1)
                ret = -1;
            break;
        case GPO_ISO_SET:
            ret = rt_gpo_set(gpioNum,*value, ISO_GPO);
            break;
#endif
#ifdef MISC_GPI_SUPPORT
        case GPI_MISC_GET:
            *value = rt_gpi_get(gpioNum, MISC_GPI);
            if(*value == -1)
                ret = -1;
            break;
#endif
#ifdef ISO_GPI_SUPPORT
        case GPI_ISO_GET:
            *value = rt_gpi_get(gpioNum, ISO_GPI);
            if(*value == -1)
                ret = -1;
            break;
#endif
        default:
            break;
    }
    return ret;
}

int go_gpio_list(void)
{
    do_gpio_dump_list(ALL);
    return 0;
}

int go_gpio_get_value_top(void)
{
    int select = 0;
    int getValue = 0;
    int ret;
    char* buf = gpio_cmd_buf;
    do_gpio_dump_list(ALL);
    while(1){
        printf("\nP:PREVIOUS\nQ:QUIT\nNUM:SELECT PIN\n");
        printf("input choice:");
        fgets(buf,sizeof(gpio_cmd_buf)-1);
        printf("\n");
        if(buf[0] == 'Q'||buf[0] == 'q')
        {
            printf("\n");
            rtk_go_up_menu(&gpio_state);
            break;
        }if(buf[0] == 'P'||buf[0] == 'p')
        {
            return 0;
        }
        select = simple_strtoul(buf, NULL, 10);
        ret = do_gpio_get_cmd(select,&getValue);
        if(ret != 0){
            printf("Get pin value error,please check select num!");
            return 0;
        }
        if(ret == 0){
            printf("Get PIN[%d]%-16s value = %d Success\n",
                   select,_gGpioDumpInfo.gpioPinInfoList[select].namePtr,getValue);
        }
    }
    return 0;
}

int go_gpio_set_value_top(void)
{
    int select = 0;
    int getValue = 0;
    int setValue = 0;
    int ret = 0;
    char* buf = gpio_cmd_buf;
    do_gpio_dump_list(OUTPUT);
    while(1){
        printf("\nP:PREVIOUS\nQ:QUIT\nNUM:SELECT PIN\n");
        printf("input choice:");
        fgets(buf,sizeof(gpio_cmd_buf)-1);
        printf("\n");
        if(buf[0] == 'Q'||buf[0] == 'q')
        {
            printf("\n");
            rtk_go_up_menu(&gpio_state);
            break;
        }if(buf[0] == 'P'||buf[0] == 'p')
        {
            return 0;
        }
        select = simple_strtoul(buf, NULL, 10);
        ret = do_gpio_get_cmd(select,&getValue);
        if(ret != 0){
            printf("Get pin value error,please check select num!");
            return 0;
        }
        if(ret == 0){
            printf("Current PIN[%d] value = %d\n",select,getValue);
        }
        printf("(1: high  0:low)\n");
        printf("set value:");
        fgets(buf,sizeof(gpio_cmd_buf)-1);
        printf("\n");
        setValue = simple_strtoul(buf, NULL, 10);
        ret = do_gpio_set_cmd(select,setValue);
        if(ret != 0){
            printf("Get pin value error,please check select num!");
        }
        if(ret == 0){
            printf("Set PIN[%d]%-16s value = %d Success\n",select,
                _gGpioDumpInfo.gpioPinInfoList[select].namePtr,
                setValue);
        }
    }
    return 0;
}

int go_gpio_senior_debug_top(void)
{
    int cmd_flag;  //0:set ; 1:get
    int gpioType_flag; //0:MISC; 1:ISO
    int gpioNum;
    char* gpioStr;
    int value;
    int tmpValue;
    int ret;
    GPIO_CMD sub_cmd;
    char* buf = gpio_cmd_buf;

    printf("Senior Set USage:\n");
    printf("  get mis/iso <gpio num> - Get MISC/ISO GPIO Value\n");
    printf("  Example: get mis/iso 80\n");
    printf("  set mis/iso <gpio num> <set value> - Set MISC/ISO GPIO Value\n");
    printf("  Example: set mis/iso s 80 1\n");

    while(1){
        cmd_flag = -1;
        gpioType_flag = -1;
        gpioNum = -1;
        value = -1;
        tmpValue = -1;
        ret = -1;
        gpioStr = NULL;
        printf("\n****Choose Set/Get Cmd****\n\n");
        printf("  [0]:SET--set gpio value\n");
        printf("  [1]:GET--get gpio value\n");
        printf("\nP:PREVIOUS\nQ:QUIT\n");
        printf("input choice:");
        gets(buf);
        printf("\n");
        if(buf[0] == 'Q'||buf[0] == 'q')
        {
            printf("\n");
            rtk_go_up_menu(&gpio_state);
            break;
        }if(buf[0] == 'P'||buf[0] == 'p')
        {
            return 0;
        }
        cmd_flag = simple_strtoul(buf, NULL, 10);
        if(cmd_flag == 0 || cmd_flag == 1)
            printf("\nTo %s gpio\n",cmd_flag == 0 ? "set" : "get");
        else {
            printf("\nunkown choice,Please restart the choice again\n ");
            continue;
        }

        printf("\n****Choose MISC/ISO GPIO TYPE****\n\n");
        printf("  [0]:MISC-GPIO  --for MISC gpio\n");
        printf("  [1]:ISO-GPIO   --for iso gpio\n");
#ifdef MISC_GPI_SUPPORT
        printf("  [2]:MISC-GPI   --for MISC gpi\n");
#else
        printf("  [2]:MISC-GPI   --MISC gpi not support on this platform\n");
#endif
#ifdef ISO_GPI_SUPPORT
        printf("  [3]:ISO-GPI    --for MISC gpi\n");
#else
        printf("  [3]:ISO-GPI    --ISO-GPI not support on this platform\n");
#endif
#ifdef MISC_GPO_SUPPORT
        printf("  [4]:MISC-GPO   --for MISC gpi\n");
#else
        printf("  [4]:MISC-GPO   --MISC-GPO not support on this platform\n");
#endif
#ifdef ISO_GPO_SUPPORT
        printf("  [5]:ISO-GPO    --for MISC gpi\n");
#else
        printf("  [5]:ISO-GPO    --ISO-GPO not support on this platform\n");
#endif

        printf("\nP:PREVIOUS\nQ:QUIT\n");
        printf("input choice:");
        gets(buf);
        printf("\n");
        if(buf[0] == 'Q'||buf[0] == 'q')
        {
            printf("\n");
            rtk_go_up_menu(&gpio_state);
            break;
        }if(buf[0] == 'P'||buf[0] == 'p')
        {
            return 0;
        }
        gpioType_flag = simple_strtoul(buf, NULL, 10);

        switch(gpioType_flag){
            case 0:
                gpioStr = "misc-gpio";
                break;
            case 1:
                gpioStr = "iso-gpio";
                break;
#ifdef MISC_GPI_SUPPORT
            case 2:
                gpioStr = "misc-gpi";
                break;
#endif
#ifdef ISO_GPI_SUPPORT
            case 3:
                gpioStr = "iso-gpi";
                break;
#endif
#ifdef MISC_GPO_SUPPORT
            case 4:
                gpioStr = "misc-gpo";
                break;
#endif
#ifdef ISO_GPO_SUPPORT
            case 5:
                gpioStr = "iso-gpo";
                break;
#endif
            default:
                break;
        }
        if(gpioStr == NULL){
            printf("\n Not sopport choosed GPIO type,Please restart the choice again\n ");
            continue;
        }
        if(strcmp(gpioStr,"misc-gpi") == 0 || strcmp(gpioStr,"iso-gpi") == 0){
            if(cmd_flag == 0){
                printf("\n GPI can not set ,Please restart the choice again\n ");
                continue;
            }
        }
        printf("\nTo %s %s \n",cmd_flag == 0 ? "set" : "get",gpioStr);


        printf("\n****ENTER GPIO NUM****\n\n");
        switch(gpioType_flag){
            case 0:
                printf("  MAX MISC GPIO NUM : %d\n",MISC_GPIO_MAX_NUM);
                break;
            case 1:
                printf("  MAX ISO GPIO NUM: %d\n",ISO_GPIO_MAX_NUM);
                break;
#ifdef MISC_GPI_SUPPORT
            case 2:
                printf("  MAX MISC GPI NUM: %d\n",MISC_GPI_MAX_NUM);
                break;
#endif
#ifdef ISO_GPI_SUPPORT
            case 3:
                printf("  MAX ISO GPI NUM: %d\n",ISO_GPI_MAX_NUM);
                break;
#endif
#ifdef MISC_GPO_SUPPORT
            case 4:
                printf("  MAX MISC GPO NUM: %d\n",MISC_GPO_MAX_NUM);
                break;
#endif
#ifdef ISO_GPO_SUPPORT
            case 5:
                printf("  MAX ISO GPO NUM: %d\n",ISO_GPO_MAX_NUM);
                break;
#endif
            default:
                break;
        }
        printf("\nP:PREVIOUS\nQ:QUIT\n");
        printf("input choice:");
        gets(buf);
        printf("\n");
        if(buf[0] == 'Q'||buf[0] == 'q')
        {
            printf("\n");
            rtk_go_up_menu(&gpio_state);
            break;
        }if(buf[0] == 'P'||buf[0] == 'p')
        {
            return 0;
        }
        gpioNum = simple_strtoul(buf, NULL, 10);

        if(check_enter_gpioNum(gpioNum,gpioType_flag) == 0){
            printf("\nTo %s %s %d \n",cmd_flag == 0 ? "set" : "get",
                gpioStr,
                gpioNum);
        }else{
            printf("\nerror gpio num,Please restart the choice again\n ");
            continue;
        }

        if(cmd_flag == 0){
            printf("\n****ENTER GPIO SET VALUE****\n\n");
            printf("\nP:PREVIOUS\nQ:QUIT\n");
            printf("(1: high  0:low)\n");
            printf("input choice:");
            fgets(buf,sizeof(gpio_cmd_buf)-1);
            printf("\n");
            if(buf[0] == 'Q'||buf[0] == 'q')
            {
                printf("\n");
                rtk_go_up_menu(&gpio_state);
                break;
            }if(buf[0] == 'P'||buf[0] == 'p')
            {
                return 0;
            }
            value = simple_strtoul(buf, NULL, 10);
            printf("\nTo %s %s %d %d \n","set",
                    gpioStr,
                    gpioNum, value);
            sub_cmd = get_gpio_cmd(1,gpioType_flag);
            ret = do_gpio_senior_debug_cmd(gpioNum,sub_cmd,&tmpValue);
            printf("Current %s %d value = %d\n",
                    gpioStr,
                    gpioNum,tmpValue);
        }

        sub_cmd = get_gpio_cmd(cmd_flag,gpioType_flag);
        ret = do_gpio_senior_debug_cmd(gpioNum,sub_cmd,&value);
        if(ret != -1){
            printf("%s %s %d value = %d success\n",cmd_flag == 0 ? "set" : "get",
                gpioStr,
                gpioNum,value);
        }else{
            printf("%s %s %d value = %d failed\n",cmd_flag == 0 ? "set" : "get",
                gpioStr,
                gpioNum,value);
        }
    }
    return 0;
}


int go_gpio_show_pcb_info_top(void)
{
    do_gpio_dump_info();
    return 0;
}




rtk_setmenu gpio_top_menu[] =
{
    {"ROOT", "********GPIO Main menu ********", NULL, NULL}, //title
    {"LIST-DUMP", "SHOW GPIO LIST",go_gpio_list, NULL},
    {"GET-PORT-VALUE", "Get GPIO Value TEST", go_gpio_get_value_top, NULL},
    {"SET-OUTPUT-VALUE", "Set Output GPIO  TEST", go_gpio_set_value_top, NULL},
    {"PCB_INFO", "SHOW GPIO PCB INFO", go_gpio_show_pcb_info_top, NULL},
    {"SENIOR-DEBUG","FOR ADVANCED DEBUG",go_gpio_senior_debug_top,NULL},
    {NULL, NULL, NULL, NULL},
};

int go_gpio_main_menu(void)
{
    gpio_state.menu = gpio_top_menu;
    rtk_prepare_go_ahead(&gpio_state);
    do_gpio_get_info();
    while(gpio_state.menu != NULL)
    {
        rtk_menu_wait(&gpio_state);
    }
    rtk_go_menu_exit();

    return 0;
}


int do_rtk_gpiotest(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    go_gpio_main_menu();
    return 0;
}

U_BOOT_CMD(
        rtkgpio ,    5,    0,     do_rtk_gpiotest,
        "GPIO CMD Menu",
        "\n"

);

