#include <common.h>
#include <asm/arch/system.h>
#include "flashdev_s.h"
#include <asm/arch/sys_flash_layout.h>
#include "../../../src/include/flash/flash_spi_list.h"
#include <linux/compiler.h>
#include <malloc.h>
#include <rbus/sb2_reg.h>
#include <rbus/md_reg.h>

#ifdef CONFIG_MTD_SFC_DEBUG
#define SFC_MSG_DEBUG(fmt, args...) printf(fmt, ##args)
#else
#define SFC_MSG_DEBUG(fmt, args...)     /* empty macro */
#endif

void  mask_register(const unsigned int reg_addr, const unsigned int AND_filter, const unsigned int OR_filter)
{
	rtd_outl(reg_addr, (rtd_inl(reg_addr) & AND_filter) | OR_filter);
}


#define CONFIG_MTD_SFC_PRINTK_INFO
#ifdef CONFIG_MTD_SFC_PRINTK_INFO
#define SFC_MSG_INFO(fmt, args...) printf("[SFC_LOG][INFO] " fmt, ## args)
#else
#define SFC_MSG_INFO(fmt, args...)     /* empty macro */
#endif

#define SFC_MSG_ERR(fmt, args...) printf("[SFC_LOG][ERROR] " fmt, ## args)

#define MD_READ_WRITE
#ifdef MD_READ_WRITE
static int md_rw_flag = 1;
#else
static int md_rw_flag = 0;
#endif

/* must keep sync with dvrmain/uboot/kernel */
#define CONFIG_RTK_SPI_WP_SOFT
#ifdef CONFIG_RTK_SPI_WP_SOFT
#endif

/************************************************************************
 *  Defines SFC registers
 ************************************************************************/
#define SPI_BASE_ADDR   0xB8200000      /* NOR flash start addess */
#define MD_PP_DATA_SIZE_SHIFT   8               /* 256 bytes */
#define MD_PP_DATA_SIZE         (1 << MD_PP_DATA_SIZE_SHIFT)

/* REG_SFC_OPCODE */
#define DUAL_QUAL_MODE_OFFSET 9
#define DUAL_QUAL_MODE_MASE 0xF
#define FAT_MODE_OFFSET (8)
#define WRITE_BYTES_MASK  (0xf)
#define CMD_MASK  0xff
#define NORMAL_MODE    ((0b0000&DUAL_QUAL_MODE_MASE)<<DUAL_QUAL_MODE_OFFSET)
#define DUAL_READ_MODE ((0b0001&DUAL_QUAL_MODE_MASE)<<DUAL_QUAL_MODE_OFFSET)
#define QUAD_READ_MODE ((0b0101&DUAL_QUAL_MODE_MASE)<<DUAL_QUAL_MODE_OFFSET)

#define CMD_READ_ID                 0x9f
#define CMD_READ_STATUS1            0x05
#define CMD_READ_STATUS2            0x35
#define CMD_READ_STATUS3            0x15
#define CMD_WRITE_ENABLE            0x06
#define CMD_WRITE_STATUS1           0X01
#define CMD_WRITE_STATUS2           0X31
#define CMD_WRITE_STATUS3           0X11
#define CMD_PAGE_PROGRAM            0x02
#define CMD_QUAD_PAGE_PROGRAM       0x32
#define CMD_READ_NORMAL             0x03
#define CMD_READ_NORMAL_FAST        0x0b
#define CMD_READ_DUAL_FAST          0x3b
#define CMD_READ_QUAD_FAST          0x6b
#define CMD_ERASE_BLOCK_64K         0xd8
#define CMD_ERASE_BLOCK_32K         0x52
#define CMD_ERASE_SECTOR_4K         0x20

#define CMD_ENTER_4BYTES_ADDR_MODE	0xB7
#define CMD_EXIT_4BYTES_ADDR_MODE	0xE9

#define CMD_SOFTWARE_DIE_SELECT		0xC2


/* REG_SFC_WP */
#define HW_WP_N_MASK       0x01

/* SPI STATUS REGISTER */
#define STATUS_WIP                      0x01
/************************************************************************
 *  Defines Move Data Engine registers
 ************************************************************************/
/* REG_MD_FDMA_CTRL2 */
#define SET_DMA_LEN_512           2
#define FLASH_2_DDR               0
#define DDR_2_FLASH               1

/************************************************************************
 *  Define misc
 ************************************************************************/
#define SFC_DELAY 100
#define MD_PP_DATA_SIZE_SHIFT   8       /* 256 bytes */
#define MD_PP_DATA_SIZE         (1 << MD_PP_DATA_SIZE_SHIFT)
#define ERASE_DATA_TIMEOUT 25000 // 2500ms
#define WRITE_DATA_TIMEOUT 100   // 5ms
#define SET_FEATURE_TIMEOUT 1000   // 50ms

#define SFC_WP_MASK_BIT2_TO_BIT7 0xfc
#define SFC_WP_MASK_BIT6         0x40
#define SFC_WP_MASK_BIT2_TO_BIT5 0x3c


#define SPI_MD_READ_UNIT  (384)

/************************************************************************
 ************************************************************************
 ************************************************************************/
static spinor_info_t *spi_dev = NULL;
uint spi_flash_check_flag = 0;

unsigned int swap_endian(unsigned int input)
{
        unsigned int output;

        output = (input & 0xff000000) >> 24 |
                 (input & 0x00ff0000) >> 8 |
                 (input & 0x0000ff00) << 8 |
                 (input & 0x000000ff) << 24;

        return output;
}

static void sfc_enable_write(void)
{
        __maybe_unused unsigned char tmp;
        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_WRITE_ENABLE));
        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(0) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0));  // data_en = 0, addr_en=0 , dmy_en=0
        asm("DSB");
        udelay(SFC_DELAY);
        tmp = rtd_inb(SPI_BASE_ADDR);
	tmp = 0;
        return;
}

static void sfc_get_feature(unsigned char rdsr, unsigned char *data)
{
        asm("DSB");
        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(rdsr));
        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0)); 
        asm("DSB");

        udelay(SFC_DELAY);
        *data = rtd_inb(SPI_BASE_ADDR);
        asm("DSB");
        return;
}

static int sfc_set_feature(unsigned char wrsr, unsigned char data)
{
        unsigned char tmp;
        int ret = 0, retry = 0;

        sfc_enable_write();
        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(wrsr));
        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0)); 
        asm("DSB");
        udelay(SFC_DELAY);
        rtd_outb(SPI_BASE_ADDR, data);

        do {
                udelay(SFC_DELAY);
                sfc_get_feature(CMD_READ_STATUS1, &tmp);
        } while ((tmp & STATUS_WIP) && (++retry < SET_FEATURE_TIMEOUT));

        if (retry == SET_FEATURE_TIMEOUT) {
                SFC_MSG_ERR("%s wait timet out  0x%x\n", __FUNCTION__, tmp);
                ret = -1;
        }
        return ret;
}

static int sfc_set_feature_16bit(unsigned char wrsr, unsigned short data)
{
        unsigned char tmp;
        int ret = 0, retry = 0;

        sfc_enable_write();
        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(wrsr));
        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0)); 
        asm("DSB");
        udelay(SFC_DELAY);
        rtd_outw(SPI_BASE_ADDR, data);

        do {
                udelay(SFC_DELAY);
                sfc_get_feature(CMD_READ_STATUS1, &tmp);
        } while ((tmp & STATUS_WIP) && (++retry < SET_FEATURE_TIMEOUT));

        if (retry == SET_FEATURE_TIMEOUT) {
                SFC_MSG_ERR("%s wait timet out  0x%x\n", __FUNCTION__, tmp);
                ret = -1;
        }
        return ret;
}

static int sfc_wait_erase_done(void)
{
        int ret = 0, retry = 0;
        unsigned char data = 0;

        do {
                udelay(SFC_DELAY);
                sfc_get_feature(CMD_READ_STATUS1, &data);
        } while ((data & STATUS_WIP) && (++retry < ERASE_DATA_TIMEOUT));

        if (retry == ERASE_DATA_TIMEOUT) {
                SFC_MSG_ERR("%s wait timet out  0x%x\n", __FUNCTION__, data);
                ret = -1;
        }
        return ret;
}

static int sfc_wait_write_done(void)
{
        int ret = 0, retry = 0;
        unsigned char data = 0;

        do {
                udelay(SFC_DELAY);
                sfc_get_feature(CMD_READ_STATUS1, &data);
        } while ((data & STATUS_WIP) && (++retry < WRITE_DATA_TIMEOUT));

        if (retry == WRITE_DATA_TIMEOUT) {
                SFC_MSG_ERR("%s wait timet out  0x%x\n", __FUNCTION__, data);
                ret = -1;
        }
        return ret;
}

static __maybe_unused int sfc_wait_set_feature_done(void)
{
        int ret = 0, retry = 0;
        unsigned char data = 0;

        do {
                udelay(SFC_DELAY);
                sfc_get_feature(CMD_READ_STATUS1, &data);
        } while ((data & STATUS_WIP) && (++retry < SET_FEATURE_TIMEOUT));

        if (retry == SET_FEATURE_TIMEOUT) {
                SFC_MSG_ERR("%s wait timet out  0x%x\n", __FUNCTION__, data);
                ret = -1;
        }
        printf("***11retry %d\n", retry);
        return ret;
}

// return 1 : quad enable  return 0: quad disable
static int sfc_check_quad(spinor_info_t *spinor_info)
{
        unsigned char data = 0;

        sfc_get_feature(spinor_info->cmd_quad_rdsr, &data);
        if(data & spinor_info->cmd_quad_bit)
                return 1;
        else
                return 0;
}
static int sfc_enable_quad(spinor_info_t *spinor_info)
{
        unsigned char data = 0;
        unsigned char tmp = 0;
        unsigned short data_16bit = 0xffff;
        int ret = 0;

        if(spinor_info->id == WINBOND_W32Q64_64Mbit) {
                sfc_get_feature(CMD_READ_STATUS1, &data);
                data_16bit = data;
                sfc_get_feature(spinor_info->cmd_quad_rdsr, &data);
                data = data | spinor_info->cmd_quad_bit;
                data_16bit = ((data << 8) | data_16bit);
                ret = sfc_set_feature_16bit(spinor_info->cmd_quad_wrsr, data_16bit);
                if(ret < 0) {
                        return ret;
                }
                sfc_get_feature(spinor_info->cmd_quad_rdsr, &tmp);
                SFC_MSG_DEBUG("%s tmp:0x%x data:0x%x\n", __FUNCTION__, tmp, data);
                if(tmp != data) {
                        SFC_MSG_DEBUG("%s 0x%x != 0x%x\n", __FUNCTION__, tmp, data);
                        return -1;
                }
        } else {
                sfc_get_feature(spinor_info->cmd_quad_rdsr, &data);
                data = data | spinor_info->cmd_quad_bit;
                ret = sfc_set_feature(spinor_info->cmd_quad_wrsr, data);
                if(ret < 0) {
                        return ret;
                }
                sfc_get_feature(spinor_info->cmd_quad_rdsr, &tmp);
                SFC_MSG_DEBUG("%s tmp:0x%x data:0x%x\n", __FUNCTION__, tmp, data);

                if(tmp != data) {
                        SFC_MSG_DEBUG("%s 0x%x != 0x%x\n", __FUNCTION__, tmp, data);
                        return -1;
                }
        }
        return ret;
}

static __maybe_unused void sfc_disable_quad(spinor_info_t *spinor_info)
{
        unsigned char data = 0;
	  __maybe_unused int ret = 0;
	  
        sfc_get_feature(spinor_info->cmd_quad_rdsr, &data);
        data = data & (~spinor_info->cmd_quad_bit);
        ret = sfc_set_feature(spinor_info->cmd_quad_wrsr, data);
        return;
}

static void switch_read_mode(spinor_info_t *spinor_info)
{
        switch(spinor_info->read_mode) {
                case eMODE_READ_NORMAL:
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_NORMAL) | SB2_SFC_OPCODE_dual_quad_mode(0));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(0));
                        break;
                case eMODE_READ_NORMAL_FAST:
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_NORMAL_FAST) | SB2_SFC_OPCODE_dual_quad_mode(0));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(1));
                        break;
                case eMODE_READ_DUAL_FAST:
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_DUAL_FAST) | SB2_SFC_OPCODE_dual_quad_mode(1));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(1));
                        break;
                case eMODE_READ_QUAD_FAST:
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_QUAD_FAST) | SB2_SFC_OPCODE_dual_quad_mode(0x5));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(1));
                        break;
                default:
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_NORMAL) | SB2_SFC_OPCODE_dual_quad_mode(0));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(0));
                        break;
        }

        asm("DSB");
}

static void switch_write_mode(spinor_info_t *spinor_info)
{
        SFC_MSG_DEBUG("%s_%d\n", __FUNCTION__, __LINE__);

        if(spinor_info->write_mode == eMODE_WRITE_NORMAL) {
                rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_PAGE_PROGRAM));
                rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(0));
                asm("DSB");
        } else if(spinor_info->write_mode == eMODE_WRITE_QUAD_FAST) {
                rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_QUAD_PAGE_PROGRAM) | SB2_SFC_OPCODE_dual_quad_mode(0x7)); 
                rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(1));		
                asm("DSB");
        }
}

static void switch_erase_mode(spinor_info_t *spinor_info)
{
        SFC_MSG_DEBUG("%s_%d\n", __FUNCTION__, __LINE__);
        asm("DSB");

        return;
}

static void sfc_enable_4bytes_addr_mode(void)
{
	__maybe_unused unsigned char tmp = 0;

	/*enable spi device 4bytes address mode*/
	rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_ENTER_4BYTES_ADDR_MODE));
	rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(0) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0));
	asm("DSB");
	udelay(SFC_DELAY);
	/*trigger cmd*/
	tmp = rtd_inb(SPI_BASE_ADDR);

	/*enable spi controler 4bytes address mode*/
	rtd_outl(SB2_SFC_OPCODE2_reg, SB2_SFC_OPCODE2_adr4byte_en(1));
}

static void sfc_disable_4bytes_addr_mode(void)
{
	__maybe_unused unsigned char tmp = 0;

	/*disable spi device 4bytes address mode*/
	rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_EXIT_4BYTES_ADDR_MODE));
	rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(0) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0));
	asm("DSB");
	udelay(SFC_DELAY);
	/*trigger cmd*/
	tmp = rtd_inb(SPI_BASE_ADDR);

	/*disable spi controler 4bytes address mode*/
	rtd_outl(SB2_SFC_OPCODE2_reg, SB2_SFC_OPCODE2_adr4byte_en(0));
}

static void sfc_die_select(unsigned char die_id)
{
	/*select die*/
	rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_SOFTWARE_DIE_SELECT));
	rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0));
	asm("DSB");
	udelay(SFC_DELAY);

	rtd_outb(SPI_BASE_ADDR, die_id);
}

static void sfc_read_id(unsigned int* id)
{
        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(CMD_READ_ID));
        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(1) | SB2_SFC_CTL_adren(0) | SB2_SFC_CTL_dmycnt(0)); /* dataen = 1, adren = 0, dmycnt = 0 */
        asm("DSB");
        udelay(SFC_DELAY);
        *id = rtd_inl(SPI_BASE_ADDR);
        sync();
        *id = swap_endian(*id);
        (*id) = (*id) >> 8;
        return;
}

static int sfc_match_chip(spinor_info_t **dev, unsigned int chip_id)
{
        int i = 0;

        for (i = 0; i < DEV_SIZE_S; i++) {
                if (chip_id != s_device[i].id) // find the match flash brand
                        continue;

                memcpy(*dev, &s_device[i], sizeof(spinor_info_t));
                break;
        }

        if (i == DEV_SIZE_S) {
                SFC_MSG_ERR("[SPINOR-INFO]Cannot find any match ID!!!\n");
                *dev =  NULL;
                return -1;
        }
        return 0;
}

static int sfc_lock(spinor_info_t *spinor_info)
{
        unsigned char status1 = 0xFF, status2 = 0xFF, status3 = 0xFF;
        unsigned short status_16bit = 0xffff;
        unsigned int i = 0, max = 0;

        switch(spinor_info->id) {
                case GD25Q128C_128Mbit:
                case WINBOND_W25Q128FV_128Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("old wp status for lock: 0x%x 0x%x 0x%x\n", status1, status2, status3);

                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x0c) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7)) | 0x0c;
                                status2 = status2 | SFC_WP_MASK_BIT6;
                                if((-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1)) \
                                    || (-1 == sfc_set_feature(CMD_WRITE_STATUS2, status2))) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

                case GD25Q64C_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);

                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x10) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7)) | 0x10;
                                status2 = status2 | SFC_WP_MASK_BIT6;
                                if((-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1)) \
                                    || (-1 == sfc_set_feature(CMD_WRITE_STATUS2, status2))) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

                case WINBOND_W32Q64_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x\n", status1, status2);
                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x10) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7)) | 0x10;
                                status2 = status2 | SFC_WP_MASK_BIT6;
                                status_16bit = ((status2 << 8) | status1);
                                if(-1 == sfc_set_feature_16bit(CMD_WRITE_STATUS1, status_16bit)) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

               case WINBOND_W25M512JV_512Mbit:
                        max = spinor_info->size/spinor_info->per_die_size;
                        for(i = 0; i < max; i++){
	                        sfc_die_select(i);

	                        sfc_get_feature(CMD_READ_STATUS1, &status1);
	                        sfc_get_feature(CMD_READ_STATUS2, &status2);
	                        sfc_get_feature(CMD_READ_STATUS3, &status3);
	                        SFC_MSG_INFO("old wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);

				    /*check status*/
				    if((!(status1 & SFC_WP_MASK_BIT2_TO_BIT5)) && (status2 & SFC_WP_MASK_BIT6)){
				            SFC_MSG_INFO("spi flash already locked! ,wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);
				            continue;
				    }

				    /*lock spi flash*/
	                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT5) != 0x0) \
	                                || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
	                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT5)) | 0x0;
	                                status2 = status2 | SFC_WP_MASK_BIT6;
	                                if((-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1)) \
	                                        || (-1 == sfc_set_feature(CMD_WRITE_STATUS2, status2))) {
	                                        SFC_MSG_ERR("sfc_lock fail\n");
	                                        return -1;
	                                }
	                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
	                        }

#if 0
 	                        sfc_get_feature(CMD_READ_STATUS2, &status2);
	                        SFC_MSG_INFO("new wp status2: 0x%x\n", status2);
 #endif
                        }
			    sfc_die_select(0);
                        break;
						
                default:
                        break;
        }

        return 0;
}

static int sfc_unlock(spinor_info_t *spinor_info)
{
        unsigned char status1 = 0xFF, status2 = 0xFF, status3 = 0xFF;
        unsigned short status_16bit = 0xffff;
        unsigned int i = 0, max = 0;
        switch(spinor_info->id) {
                case GD25Q128C_128Mbit:
                case WINBOND_W25Q128FV_128Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);
                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7)) \
                            || (status2 & SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7));
                                status2 = status2 & (~SFC_WP_MASK_BIT6);
                                if((-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1)) \
                                    || (-1 == sfc_set_feature(CMD_WRITE_STATUS2, status2))) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

                case GD25Q64C_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);
                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7)) \
                            || (status2 & SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7));
                                status2 = status2 & (~SFC_WP_MASK_BIT6);
                                if((-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1)) \
                                    || (-1 == sfc_set_feature(CMD_WRITE_STATUS2, status2))) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

                case WINBOND_W32Q64_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x \n", status1, status2);
                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7)) \
                            || (status2 & SFC_WP_MASK_BIT6)) {
                                status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT7));
                                status2 = status2 & (~SFC_WP_MASK_BIT6);
                                status_16bit = ((status2 << 8) | status1);
                                if(-1 == sfc_set_feature_16bit(CMD_WRITE_STATUS1, status_16bit)) {
                                        SFC_MSG_ERR("sfc_lock fail\n");
                                        return -1;
                                }
                                SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                        }
                        break;

	        case WINBOND_W25M512JV_512Mbit:
                        max = spinor_info->size/spinor_info->per_die_size;
                        for(i = 0; i < max; i++){
	                        sfc_die_select(i);

	                        sfc_get_feature(CMD_READ_STATUS1, &status1);
	                        sfc_get_feature(CMD_READ_STATUS2, &status2);
	                        sfc_get_feature(CMD_READ_STATUS3, &status3);
	                        SFC_MSG_INFO("old wp status for unlock: 0x%x 0x%x 0x%x\n", status1, status2, status3);
							
                               /*check status*/
                               if((!(status1 & SFC_WP_MASK_BIT2_TO_BIT5)) && (!(status2 & SFC_WP_MASK_BIT6))){
                                   SFC_MSG_INFO("spi flash already unlocked! ,wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);
                                   continue;
                               }

                               /*unlock spi flash*/
                               if(((status1 & SFC_WP_MASK_BIT2_TO_BIT5)) \
                                   || (status2 & SFC_WP_MASK_BIT6)){
                                   status1 = (status1 & (~SFC_WP_MASK_BIT2_TO_BIT5));
                                   status2 = status2 & (~SFC_WP_MASK_BIT6);
                                   if(-1 == sfc_set_feature(CMD_WRITE_STATUS1, status1) ||
                                       sfc_set_feature(CMD_WRITE_STATUS2, status2) == -1){
                                       SFC_MSG_ERR("sfc_unlock fail\n");
                                       return -1;
                                   }
                                   SFC_MSG_INFO("new wp status: 0x%x 0x%x\n", status1, status2);
                               }

#if 0
 	                        sfc_get_feature(CMD_READ_STATUS2, &status2);
	                        SFC_MSG_INFO("new wp status2: 0x%x\n", status2);
 #endif
			    }
			    sfc_die_select(0);
			    break;
				
                default:
                        break;

        }

        return 0;
}

static int sfc_lock_check(spinor_info_t *spinor_info)
{
        unsigned char status1 = 0xFF, status2 = 0xFF, status3 = 0xFF;
        unsigned int i = 0, max = 0;

        switch(spinor_info->id) {
                case GD25Q128C_128Mbit:
                case WINBOND_W25Q128FV_128Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("wp status for lock: 0x%x 0x%x 0x%x\n", status1, status2, status3);

                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x0c) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                SFC_MSG_INFO("error!spi flash is not locked!\n");
                                spi_flash_check_flag = 1;
                        }else{
                                SFC_MSG_INFO("right!spi flash is locked!\n");
                        }
                        break;

                case GD25Q64C_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        sfc_get_feature(CMD_READ_STATUS3, &status3);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);

                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x10) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                SFC_MSG_INFO("error!spi flash is not locked!\n");
                                spi_flash_check_flag = 1;
                        }else{
                                SFC_MSG_INFO("right!spi flash is locked!\n");
                        }
                        break;

                case WINBOND_W32Q64_64Mbit:
                        sfc_get_feature(CMD_READ_STATUS1, &status1);
                        sfc_get_feature(CMD_READ_STATUS2, &status2);
                        SFC_MSG_INFO("old wp status: 0x%x 0x%x\n", status1, status2);
                        if(((status1 & SFC_WP_MASK_BIT2_TO_BIT7) != 0x10) \
                            || ((status2 & SFC_WP_MASK_BIT6) != SFC_WP_MASK_BIT6)) {
                                SFC_MSG_INFO("error!spi flash is not locked!\n");
                                spi_flash_check_flag = 1;
                        }else{
                                SFC_MSG_INFO("right!spi flash is locked!\n");
                        }
                        break;

               case WINBOND_W25M512JV_512Mbit:
                        max = spinor_info->size/spinor_info->per_die_size;
                        for(i = 0; i < max; i++){
	                        sfc_die_select(i);

	                        sfc_get_feature(CMD_READ_STATUS1, &status1);
	                        sfc_get_feature(CMD_READ_STATUS2, &status2);
	                        sfc_get_feature(CMD_READ_STATUS3, &status3);
	                        SFC_MSG_INFO("wp status: 0x%x 0x%x 0x%x\n", status1, status2, status3);

				    /*check status*/
				    if(!((!(status1 & SFC_WP_MASK_BIT2_TO_BIT5)) 
	                                && (status2 & SFC_WP_MASK_BIT6))){
				            SFC_MSG_INFO("error!spi flash die %d is not locked!\n", i);
				            spi_flash_check_flag = 1;
				    }else{
				            SFC_MSG_INFO("right!spi flash die %d is locked!\n", i);
				    }
                        }
			    sfc_die_select(0);
                        break;
						
                default:
                        break;
        }

        return 0;
}

static int sfc_get_status(void)
{
	unsigned char status3 = 0xFF;
	
	sfc_get_feature(CMD_READ_STATUS3, &status3);
	SFC_MSG_INFO("status3 value is: 0x%x\n", status3);

	return 0;
}

static void sfc_init_early(void)
{
        rtd_outl(SB2_SFC_SCK_reg, SB2_SFC_SCK_fdiv(0xB) | SB2_SFC_SCK_clk_duty50_en(0));

        /*HW SD provide the value :   Phcnt=0x60 / Plcnt=0xF(default) / tdt=0x39 */
        rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_phcnt_mask, SB2_SFC_CE_phcnt(0x60)); 
        rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_plcnt_mask, SB2_SFC_CE_plcnt(0xF));
        rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_tdt_mask, SB2_SFC_CE_tdt(0x39));
	
        rtd_outl(SB2_SFC_POS_LATCH_reg, SB2_SFC_POS_LATCH_delay_sel(0x5) | SB2_SFC_POS_LATCH_pos_latch(0));

        // auto
        rtd_outl(SB2_SFC_EN_WR_reg, SB2_SFC_EN_WR_wt_prog_en(1) | SB2_SFC_EN_WR_wr_en_op(CMD_WRITE_ENABLE));
        rtd_outl(SB2_SFC_WAIT_WR_reg, SB2_SFC_WAIT_WR_wt_prog_done(1) | SB2_SFC_WAIT_WR_rdsr_op(CMD_READ_STATUS1));
	 
        return;
}

static int sfc_init(spinor_info_t *spinor_info)
{
        int ret = 0;

        if((spinor_info->read_mode == eMODE_READ_QUAD_FAST || spinor_info->write_mode == eMODE_WRITE_QUAD_FAST)) {
                if((spinor_info->cmd_quad_rdsr == EMPTY_VALUE) || (spinor_info->cmd_quad_wrsr == EMPTY_VALUE) || (spinor_info->cmd_quad_bit == EMPTY_VALUE)) {
                        SFC_MSG_ERR("quad mode set error, please check s_device[]\n");
                        return -1;
                }

                if(sfc_check_quad(spinor_info) == 0) {
                        SFC_MSG_INFO("enabel quad mode\n");
                        ret = sfc_enable_quad(spinor_info);
                        if(ret == -1) {
                                SFC_MSG_ERR("enabel quad mode fail\n");
                                return ret;
                        }
                }
        }

#ifdef CONFIG_RTK_SPI_WP_HW
        if(sfc_check_quad(spinor_info) == 1) {
                SFC_MSG_ERR("the WP_HW is conflict with quad mode, only chose the one\n");
                return -1;
        }
#endif

#ifdef CONFIG_RTK_SPI_WP_SOFT
        sfc_lock_check(spi_dev);
#endif
		
        return ret;
}

static int sfc_identify(spinor_info_t **dev)
{
        unsigned int id = 0;
        unsigned int retry = 2;

#if defined(Config_Target_Board_MERLIN2) || defined(Config_Target_Board_RTD284X)
        rtd_outl(SB2_CTRL_REG, rtd_inl(SB2_CTRL_REG) & 0xfffffffe); // release spi flash
#endif

        *dev = malloc(sizeof(spinor_info_t));

        sfc_init_early();
        
RETRY:
        sfc_read_id(&id);
        //sfc_debug();

        SFC_MSG_INFO("spi nor id: 0x%x\n", id);
        if(sfc_match_chip(dev, id) == -1) {
                if(retry) {
                        retry--;
                        SFC_MSG_INFO("[SPINOR-INFO] do_identify_s retry %d!!!\n", retry);
                        goto RETRY;
                } else {
                        SFC_MSG_ERR("[SPINOR-INFO] do_identify_s fail!!!\n");
                        free(*dev);
                        *dev = NULL;
                        return -1;
                }
        }

        SFC_MSG_INFO("do_identify_s success!!!\n");

        return 0;
}

inline size_t sfc_memcpy(unsigned int dst, unsigned int src, size_t len)
{
        unsigned int i;
        for (i = 0; i < len; i++) {
                rtd_outb((dst + i), rtd_inb(src + i));
        }

        return len;
}

static void sfc_read_bytes(spinor_info_t *spinor_info, unsigned int from, unsigned int len, unsigned int *retlen, unsigned char *buf)
{
        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();

        switch_read_mode(spinor_info);

        *retlen = sfc_memcpy((unsigned int)buf, (SPI_BASE_ADDR + (unsigned int)from), len);

        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();

        return;
}

static int sfc_write_bytes(spinor_info_t *spinor_info, unsigned int to, unsigned int len, unsigned int *retlen, unsigned char *buf)
{
        int ret = 0;

        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();
		
        switch_write_mode(spinor_info);
        *retlen = sfc_memcpy((SPI_BASE_ADDR + (unsigned int)to), (unsigned int)buf, len);

        ret = sfc_wait_write_done();

        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();
        return ret;
}

static void sfc_read_md(spinor_info_t *spinor_info, unsigned int from, unsigned int len, unsigned int *retlen, unsigned char *buf)
{
        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();
        switch_read_mode(spinor_info);

        SFC_MSG_DEBUG("%s_0x%x_from:0x%x  buf_0x%x len_0x%x\n", __FUNCTION__, __LINE__, from, (unsigned int)buf, (unsigned int)len);
        SFC_MSG_DEBUG("(unsigned long)buf %x, (unsigned longt)len %x\n", (unsigned long)buf, (unsigned long)len);
        rtd_outl(MD_MD_FDMA_DDR_SADDR_reg, (unsigned int)virt_to_phys(buf));
        rtd_outl(MD_MD_FDMA_FL_SADDR_reg, MD_MD_FDMA_FL_SADDR_fdma_fl_saddr((unsigned int)from));  
        rtd_outl(MD_MD_FDMA_CTRL2_reg, MD_MD_FDMA_CTRL2_fdma_length(len) | MD_MD_FDMA_CTRL2_fdma_dir(FLASH_2_DDR) | \
                MD_MD_FDMA_CTRL2_fdma_max_xfer(SET_DMA_LEN_512) | MD_MD_FDMA_CTRL2_fdma_swap(0) | \
                MD_MD_FDMA_CTRL2_fl_mapping_mode(1)); 
        flush_dcache_range((unsigned long)buf, (unsigned long)buf + len);
        rtd_outl(MD_MD_FDMA_CTRL1_reg, MD_MD_FDMA_CTRL1_write_en1(1) | MD_MD_FDMA_CTRL1_fdma_go(1));
        asm("DSB");
        while(rtd_inl(MD_MD_FDMA_CTRL1_reg) & MD_MD_FDMA_CTRL1_fdma_go_mask);
        rtd_outl(MD_MD_SMQ_INT_STATUS_reg, MD_MD_SMQ_INT_STATUS_fdma_done(1));
        *retlen = len;
        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();
        return;
}

static int sfc_write_md(spinor_info_t* spinor_info, unsigned int to, unsigned int len, unsigned int *retlen, unsigned char *buf)
{
        int ret = 0;

        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();
        switch_write_mode(spinor_info);

        rtd_outl(MD_MD_FDMA_DDR_SADDR_reg, (unsigned int)virt_to_phys(buf));
        rtd_outl(MD_MD_FDMA_FL_SADDR_reg, MD_MD_FDMA_FL_SADDR_fdma_fl_saddr((unsigned int)to));
        rtd_outl(MD_MD_FDMA_CTRL2_reg, MD_MD_FDMA_CTRL2_fdma_length(len) | MD_MD_FDMA_CTRL2_fdma_dir(DDR_2_FLASH) | \
                MD_MD_FDMA_CTRL2_fdma_max_xfer(SET_DMA_LEN_512) | MD_MD_FDMA_CTRL2_fdma_swap(0) | \
                MD_MD_FDMA_CTRL2_fl_mapping_mode(1));
	 
        flush_dcache_range((unsigned long)buf, (unsigned long)(buf + len));
        asm("DSB");

        rtd_outl(MD_MD_FDMA_CTRL1_reg, MD_MD_FDMA_CTRL1_write_en1(1) | MD_MD_FDMA_CTRL1_fdma_go(1));
        while(rtd_inl(MD_MD_FDMA_CTRL1_reg) & MD_MD_FDMA_CTRL1_fdma_go_mask);
        rtd_outl(MD_MD_SMQ_INT_STATUS_reg, MD_MD_SMQ_INT_STATUS_fdma_done(1));

        *retlen = len;
        ret = sfc_wait_write_done();
        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();
        return ret;
}

static __maybe_unused int sfc_verify_after_write(spinor_info_t *spinor_info, unsigned char * to_verify, unsigned int len_verify, unsigned char *buf_verify)
{
        size_t i = 0;
        to_verify += SPI_BASE_ADDR;
		
        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();
		
        switch_read_mode(spinor_info);
        mdelay(10);
        do {
                if (*to_verify != *buf_verify) {
                        SFC_MSG_ERR("0x%08x -> 0x%02x,  0x%08x -> 0x%02x\n", (unsigned int)to_verify, *to_verify, (unsigned int)buf_verify, *buf_verify);
                        goto fail;
                }
                to_verify++;
                buf_verify++;
                asm("DSB");
        } while (++i < len_verify);

        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();

        return 0;

fail:
        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();

        return -1;
}


// the from and len must align with rtk_spi_nor_get_align
static int sfc_erase_flash(spinor_info_t *spinor_info, unsigned int from, unsigned int len)
{
        __maybe_unused unsigned char tmp;
        unsigned int size;
        unsigned int erase_addr = SPI_BASE_ADDR + from;
        unsigned int erase_size = len;
        unsigned int cmd_erase = 0;
        int ret = 0;

        if(spinor_info->min_erase_size == 0) {
                SFC_MSG_ERR("the spi nor min_erase_size is not init\n");
                return -1;
        }

        if(len % spinor_info->min_erase_size != 0) {
                SFC_MSG_ERR("the erase size is not align\n");
                return -1;
        }

        if((erase_addr & (spinor_info->min_erase_size - 1)) != 0) {
                SFC_MSG_ERR("the erase addr is not align\n");
                return -1;
        }

        if(spinor_info->address_4bytes_enable)
                sfc_enable_4bytes_addr_mode();
        switch_erase_mode(spinor_info);

        while(erase_size > 0) {
                //sfc_enable_write();

                if ((erase_size >= ERASE_SZIE_64K) && (((unsigned int)erase_addr & (ERASE_SZIE_64K - 1)) == 0) && (spinor_info->erase_capacity & ERASE_64K)) {
                        //printf("(64k)");
                        cmd_erase = CMD_ERASE_BLOCK_64K;
                        size = ERASE_SZIE_64K;
                } else if ((erase_size >= ERASE_SZIE_32K) && (((unsigned int)erase_addr & (ERASE_SZIE_32K - 1)) == 0) && (spinor_info->erase_capacity & ERASE_32K)) {
                        //printf("(32k)");
                        cmd_erase = CMD_ERASE_BLOCK_32K;
                        size = ERASE_SZIE_32K;
                } else if ((erase_size >= ERASE_SZIE_4K) && (((unsigned int)erase_addr & (ERASE_SZIE_4K - 1)) == 0) && (spinor_info->erase_capacity & ERASE_4K)) {
                        //printf("(4k)");
                        cmd_erase = CMD_ERASE_SECTOR_4K;
                        size = ERASE_SZIE_4K;
                }

                if(cmd_erase) {
                        rtd_outl(SB2_SFC_OPCODE_reg, SB2_SFC_OPCODE_opc(cmd_erase));
                        rtd_outl(SB2_SFC_CTL_reg, SB2_SFC_CTL_dataen(0) | SB2_SFC_CTL_adren(1) | SB2_SFC_CTL_dmycnt(0));
                        asm("DSB");
                        tmp = rtd_inb(erase_addr);
                        udelay(SFC_DELAY);
                        cmd_erase = 0;
                } else {
                        SFC_MSG_ERR("the erase size is not aligned,please use rtk_spi_nor_get_align()\n");
                        goto fail;
                }

                ret = sfc_wait_erase_done();
                if(ret < 0) {
                        goto fail;
                }

                size = (erase_size > size) ? size : erase_size ;
                erase_size -= size;
                erase_addr += size;
        }

        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();
        return 0;
fail:
        if(spinor_info->address_4bytes_enable)
                sfc_disable_4bytes_addr_mode();
        return -1;
}

static int sfc_check_pin_mux(void)
{
	volatile unsigned int regval1;
	volatile unsigned int regval2;
	
	regval1 = (rtd_inl(0xb8000850) & 0x0f0f0f0f) | 0x20202020;
	rtd_outl(0xb8000850, regval1);
	regval2 = (rtd_inl(0xb8000854) & 0x0f0fffff) | 0x20200000;
	rtd_outl(0xb8000854, regval2);

	/*set clock drivering strength to 8mA*/
	rtd_maskl(0xb8000854,0xFFFDFFFF,0x00020000);//8mA
        
	regval1 = rtd_inl(0xb8000850);
	regval2 = rtd_inl(0xb8000854);

	printf("SPI Flash pinmux setting check, reg-0x%x: 0x%x , reg-0x%x: 0x%x!\r\n", \
		0xb8000850, regval1, 0xb8000854, regval2);

	return 0;
}

#ifdef CONFIG_RTK_SPI_WP_HW
// wp high: sfc_hw_wp_set(HW_WP_N_DISABLE);
// wp low:  sfc_hw_wp_set(HW_WP_N_ENABLE);
void sfc_hw_wp_set(unsigned char status)
{
        unsigned char tmp;
        rtd_outl(SB2_SFC_WP_reg,  rtd_inl(SB2_SFC_WP_reg) & (~HW_WP_N_MASK) | (SB2_SFC_WP_wp_n(status)) | SB2_SFC_WP_write_en1(1));
        sfc_get_feature(CMD_READ_STATUS1, &tmp);  // for HW_WP_ENABLE is accept when next cmd

        return;
}
#endif

static int rtk_spi_nor_read_operation(unsigned int from, unsigned int len, unsigned char* buf)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        unsigned int rlen = 0;
        unsigned int len_to_read = 0;
        unsigned int len_done_read = 0;
        unsigned int len_verify = len;

        if(spinor_info == NULL) {
                SFC_MSG_ERR("the spi is not init\n");
                return -1;
        }

        if(buf == NULL)
                return -1;

        if(md_rw_flag == 0) {
                len_to_read = len;
                sfc_read_bytes(spinor_info, from, len_to_read, &rlen, buf);
                len_done_read += rlen;
        } else {
                len_to_read = len;
                if(len_to_read < SPI_MD_READ_UNIT) { // if data len smaller than 16, not need md
                        sfc_read_bytes(spinor_info, from, len_to_read, &rlen, buf);
                        len_done_read = rlen;
                } else {
                        unsigned int remainder_len = len_to_read % SPI_MD_READ_UNIT;
                        unsigned int align_len = (len_to_read / SPI_MD_READ_UNIT) * SPI_MD_READ_UNIT;
			
                        sfc_read_md(spinor_info, from, align_len, &rlen, buf);
                        len_done_read += rlen;
                        sfc_read_bytes(spinor_info, from+ align_len, remainder_len, &rlen, buf+ align_len);
                        len_done_read += rlen;
                }
                
        }


        if(len_done_read != len_verify) {
                SFC_MSG_ERR("Error, len_done_read:0x%x != len_verify:0x%x!\n", len_done_read, len_verify);
                return -1;
        }
        return 0;
}

/************************************************************************
 *                          rtk_spi_nor_read
 *  Description :
 *  implement the flash write
 *  Parameters :
 *  Return values :
 ************************************************************************/
int rtk_spi_nor_read(unsigned int from, unsigned int len, unsigned char* buf)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        int ret = 0;

        if(spinor_info == NULL) {
                SFC_MSG_ERR("the spi is not init\n");
                return -1;
        }

        if(spinor_info->multi_die_support){
                 if(from >= spinor_info->per_die_size){
                        sfc_die_select(0x1);

                        from -= spinor_info->per_die_size;
                }else{
                        sfc_die_select(0x0);

                        if((from + len) > spinor_info->per_die_size){
                                unsigned int read_len = spinor_info->per_die_size - from;
                                
                                ret = rtk_spi_nor_read_operation(from,read_len,buf);
                                if(ret < 0){
                                        SFC_MSG_ERR("%s_%d: read fail\n",__func__, __LINE__);
                                        return ret;
                                }

                                from = 0x0;
                                len -= read_len;
                                buf += read_len;
                                sfc_die_select(0x1);
                        }
                }
        }

        ret = rtk_spi_nor_read_operation(from,len,buf);
        if(spinor_info->multi_die_support)
        	sfc_die_select(0x0);
		
        return ret;
}

int rtk_spi_nor_write_md(unsigned int to, unsigned int len, unsigned char *buf)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        unsigned int wlen = 0;
        unsigned int len_to_write = 0;
        unsigned int len_done_write = 0;
        __maybe_unused unsigned char *to_verify = (unsigned char *)to;
        __maybe_unused unsigned int len_verify = len;
        __maybe_unused unsigned char *buf_verify = buf;
        int ret = 0;

        if(spinor_info == NULL) {
                SFC_MSG_ERR("the spi is not init\n");
                return -1;
        }
        if(buf == NULL)
                return -1;

        // 1st pass: write data which smaller than a single page (256 bytes) to make destination address within 256-bytes alignment
        if((len > 0) && (((to + MD_PP_DATA_SIZE - 1) & (~(MD_PP_DATA_SIZE - 1))) - to) > 0) {
                len_to_write = ((to + MD_PP_DATA_SIZE - 1) & (~(MD_PP_DATA_SIZE - 1))) - to;
                if(len_to_write > len)
                        len_to_write = len;

                if(len_to_write < WRITE_BYTES_MASK) { // if data len smaller than 16, not need md
                        SFC_MSG_DEBUG("write 1st pass use cpu\n");
                        ret = sfc_write_bytes(spinor_info, to, len_to_write, &wlen, buf);
                        if(ret < 0) {
                                goto to_error;
                        }

                } else {
                        SFC_MSG_DEBUG("write 1st pass use md\n");
                        ret = sfc_write_md(spinor_info, to, len_to_write, &wlen, buf);
                        if(ret < 0) {
                                goto to_error;
                        }
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        // 2rd pass: write data in page unit (256 bytes)
        while(len >= MD_PP_DATA_SIZE) {
                SFC_MSG_DEBUG("write 2rd pass use md\n");
                len_to_write = MD_PP_DATA_SIZE;
                ret = sfc_write_md(spinor_info, to, len_to_write, &wlen, buf);
                if(ret < 0) {
                        goto to_error;
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        // 3th pass: write data which smaller than a single page (256 bytes)
        if(len > 0) {
                len_to_write = len;
                if(len_to_write < WRITE_BYTES_MASK) { // if data len smaller than 16, not need md
                        SFC_MSG_DEBUG("write 3st pass use cpu\n");
                        ret = sfc_write_bytes(spinor_info, to, len_to_write, &wlen, buf);
                        if(ret < 0) {
                                goto to_error;
                        }
                } else {
                        SFC_MSG_DEBUG("write 3th pass use md\n");
                        ret = sfc_write_md(spinor_info, to, len_to_write, &wlen, buf);
                        if(ret < 0) {
                                goto to_error;
                        }
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        if(len_done_write != len_verify) {
                SFC_MSG_ERR("Error, len_done_write:0x%x != len_verify:0x%x!\n", len_done_write, len_verify);
                return -1;
        }

to_error:
        return ret;
}

int rtk_spi_nor_write_cpu(unsigned int to, unsigned int len, unsigned char *buf)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        unsigned int wlen = 0;
        unsigned int len_to_write = 0;
        unsigned int len_done_write = 0;
        __maybe_unused unsigned char *to_verify = (unsigned char *)to;
        __maybe_unused unsigned int len_verify = len;
        __maybe_unused unsigned char *buf_verify = buf;
        int ret = 0;
        if(spinor_info == NULL) {
                SFC_MSG_ERR("the spi is not init\n");
                return -1;
        }
        if(buf == NULL)
                return -1;

        // 1st pass: write data which smaller than a single page (256 bytes) to make destination address within 256-bytes alignment
        if((len > 0) && (((to + MD_PP_DATA_SIZE - 1) & (~(MD_PP_DATA_SIZE - 1))) - to) > 0) {
                len_to_write = ((to + MD_PP_DATA_SIZE - 1) & (~(MD_PP_DATA_SIZE - 1))) - to;
                if(len_to_write > len)
                        len_to_write = len;

                SFC_MSG_DEBUG("write 1st pass use cpu\n");
                ret = sfc_write_bytes(spinor_info, to, len_to_write, &wlen, buf);
                if(ret < 0) {
                        goto to_error;
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        // 2rd pass: write data in page unit (256 bytes)
        while(len >= MD_PP_DATA_SIZE) {
                SFC_MSG_DEBUG("write 2rd pass use md\n");
                len_to_write = MD_PP_DATA_SIZE;
                ret = sfc_write_bytes(spinor_info, to, len_to_write, &wlen, buf);
                if(ret < 0) {
                        goto to_error;
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        // 3th pass: write data which smaller than a single page (256 bytes)
        if(len > 0) {
                len_to_write = len;
                SFC_MSG_DEBUG("write 3st pass use cpu\n");
                ret = sfc_write_bytes(spinor_info, to, len_to_write, &wlen, buf);
                if(ret < 0) {
                        goto to_error;
                }
                to += wlen;
                buf += wlen;
                len -= wlen;
                len_done_write += wlen;
        }

        if(len_done_write != len_verify) {
                SFC_MSG_ERR("Error, len_done_write:0x%x != len_verify:0x%x!\n", len_done_write, len_verify);
                return -1;
        }

to_error:
        return 0;
}

static int rtk_spi_nor_write_operation(unsigned int to, unsigned int len, unsigned char *buf)
{        
        int ret = 0;
        
        if(md_rw_flag == 0)
                ret = rtk_spi_nor_write_cpu(to, len, buf);
        else
                ret = rtk_spi_nor_write_md(to, len, buf);
        
        return ret;
}

int rtk_spi_nor_write(unsigned int to, unsigned int len, unsigned char *buf)
{
        spinor_info_t *spinor_info =  (spinor_info_t *)spi_dev;
        int ret = 0;

        if(spinor_info == NULL) {
                SFC_MSG_ERR("the spi is not init\n");
                return -1;
        }
        
        if(spinor_info->multi_die_support){
                if(to >= spinor_info->per_die_size){
                        sfc_die_select(0x1);

                        to -= spinor_info->per_die_size;
                }else{
                        sfc_die_select(0x0);
                        
                        if((to + len) > spinor_info->per_die_size){
                                unsigned int write_len = spinor_info->per_die_size - to;
                                
                                ret = rtk_spi_nor_write_operation(to, write_len, buf);                             
                                if(ret < 0){
                                        SFC_MSG_ERR("%s_%d: write fail\n",__func__, __LINE__);
                                        return ret;
                                }

                                to = 0x0;
                                len -= write_len;
                                buf += write_len;
                                sfc_die_select(0x1);
                        }
                }
        }          

        ret =  rtk_spi_nor_write_operation(to, len, buf);
        if(spinor_info->multi_die_support)
        	sfc_die_select(0x0);

        return ret;
}

int rtk_spi_nor_ioctl(char type)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        if(spinor_info == NULL) {
                SFC_MSG_ERR("spi_nor is not init\n");
                return -1;
        }

        switch(type) {
                case SFC_LOCK:
                        sfc_lock(spinor_info);
                        break;
                case SFC_UNLOCK:
                        sfc_unlock(spinor_info);
                        break;
                default:
                        break;

        }
        return 0;
}

static int rtk_spi_nor_erase_operation(unsigned int from, unsigned int len)
{
        int ret = 0;
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        if(spinor_info == NULL) {
                SFC_MSG_ERR("spi_nor is not init\n");
                return -1;
        }
        ret = sfc_erase_flash(spinor_info, from, len);
        return ret;
}

// the from and len must align with rtk_spi_nor_get_align
int rtk_spi_nor_erase(unsigned int from, unsigned int len)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        int ret = 0;
		
        if(spinor_info == NULL) {
                SFC_MSG_ERR("spi_nor is not init\n");
                return -1;
        }
        
        if(spinor_info->multi_die_support){
                 if(from >= spinor_info->per_die_size){
                        sfc_die_select(0x1);

                        from -= spinor_info->per_die_size;
                }else{
                        sfc_die_select(0x0);

                        if((from + len) > spinor_info->per_die_size){
                                unsigned int erase_len = spinor_info->per_die_size - from;
                                
                                ret = rtk_spi_nor_erase_operation(from,erase_len);
                                if(ret < 0){
                                        SFC_MSG_ERR("%s_%d: erase fail\n",__func__, __LINE__);
                                        return ret;
                                }

                                from = 0x0;
                                len -= erase_len;
                                sfc_die_select(0x1);
                        }
                }

        }
        
        ret = rtk_spi_nor_erase_operation(from,len);
        if(spinor_info->multi_die_support)
        	sfc_die_select(0x0);

        return ret;
}

int rtk_spi_nor_get_align(void)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        if(spinor_info == NULL) {
                SFC_MSG_ERR("spi_nor is not init\n");
                return -1;
        }
        return spinor_info->min_erase_size;
}

int rtk_spi_nor_get_size(void)
{
        spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;
        if(spinor_info == NULL) {
                SFC_MSG_ERR("spi_nor is not init\n");
                return -1;
        }

        return spinor_info->size;
}

int rtk_spi_nor_init(void)
{
        int ret;

        if(sfc_check_pin_mux() < 0)
                return -1;

        ret = sfc_identify(&spi_dev);
        if(ret < 0 || spi_dev == NULL) {
                return -1;
        }
        SFC_MSG_INFO("SPI FLASH   =  %s   \n", spi_dev->string);
        ret = sfc_init(spi_dev);
        if(ret < 0) {
                return -1;
        }
        
        if(spi_dev->read_mode == eMODE_READ_NORMAL) {
                SFC_MSG_INFO("support eMODE_READ_NORMAL\n");
        } else if(spi_dev->read_mode == eMODE_READ_NORMAL_FAST) {
                SFC_MSG_INFO("support eMODE_READ_NORMAL_FAST\n");
        } else if(spi_dev->read_mode == eMODE_READ_DUAL_FAST) {
                SFC_MSG_INFO("support eMODE_READ_DUAL_FAST\n");
        } else if(spi_dev->read_mode == eMODE_READ_QUAD_FAST) {
                SFC_MSG_INFO("support eMODE_READ_QUAD_FAST\n");
        }

	 if(spi_dev->address_4bytes_enable){
                SFC_MSG_INFO("support 4Bytes address feature!\r\n");
	 }
	 
        if(spi_dev->multi_die_support){
                SFC_MSG_INFO("support multi die feature, total_size = 0x%x die_size = 0x%x total_die_counts = 0x%x!\r\n", \
                        spi_dev->size, spi_dev->per_die_size, spi_dev->size/spi_dev->per_die_size);
        }

        return 0;
}

static void fill_buffer(unsigned char * buffer, unsigned int size)
{
	int i = 0;

	for(i = 0; i < size; i = i + 4) {
		*(buffer++) = 0x1a;
		*(buffer++) = 0x2b;
		*(buffer++) = 0x3c;
		*(buffer++) = 0x4d;
	}
}

static void fill_buffer_erase(unsigned char * buffer, unsigned int size)
{
	int i = 0;

	for(i = 0; i < size; i++) {
		*(buffer++) = 0xff;
	}
}

int rtk_spi_test(unsigned int addr_test, unsigned int size_test)
{
	spinor_info_t *spinor_info = (spinor_info_t *)spi_dev;

	unsigned int addr_erase = addr_test ;
	unsigned int size_erase = size_test;

	unsigned int addr = addr_test;
	unsigned int size = size_test;
	
	unsigned char * buffer = NULL;
	unsigned char * buffer_bak = NULL;
	unsigned char * buffer_erase = NULL;

	unsigned int test_count = 0;
	unsigned long time_start = 0;
	unsigned long time_end = 0;

	if((addr_test & (spinor_info->min_erase_size - 1)) || (size_test & (spinor_info->min_erase_size - 1))) {
		printf("%s_%d: addr_test(0x%x) and size_test(0x%x) must align %x \n",\
			__func__, __LINE__, addr_test, size_test, spinor_info->min_erase_size);
		return -1;
	}

	if(size_test > 0x200000) {
		printf("%s_%d: size_test(0x%x) is bigger than 2MBytes \n", __func__, __LINE__,  size_test);
		return -1;
	}
		
	buffer_erase = (unsigned char *)TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
	buffer = (unsigned char *)TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x200000;
	buffer_bak =  (unsigned char *)TEMP_BUFFER_FOR_FLASH_DATA_ADDR + 0x400000;

	printf("%s_%d: buffer_erase: 0x%p buffer: 0x%p buffer_bak: 0x%p\n", __func__, __LINE__, buffer_erase, buffer, buffer_bak);
	printf("%s_%d: addr_test:0x%x size_test: %x\n",__func__, __LINE__, addr_test, size_test);
		
	fill_buffer(buffer, size);
	fill_buffer_erase(buffer_erase, size_erase);
	printf("\n\ntest start:\n");
	
	while(1) {
		time_start = get_boot_time_ms();
		if(rtk_spi_nor_erase(addr_erase, size_erase) != 0) {
			printf("rtk_spi_nor_erase error\n");
			break;
		}

		time_end = get_boot_time_ms();
		printf("erase time: %ldms\n", time_end - time_start);
		
		if(rtk_spi_nor_read(addr_erase, size_erase, buffer_bak) != 0) {
			printf("rtk_spi_nor_read  error\n");
			break;
		} 

		if(memcmp(buffer_erase, buffer_bak, size_erase) != 0) {
			printf("erase memcmp error: test count %d\n", test_count);
			break;
		}

		/*write*/
		time_start = get_boot_time_ms();
		if(rtk_spi_nor_write(addr, size, buffer) != 0) {
			printf("rtk_spi_nor_write error\n");
			break;
		} 

		time_end = get_boot_time_ms();
		printf("write time: %ldms\n", time_end - time_start);

		/*read */
		time_start = get_boot_time_ms();
		if(rtk_spi_nor_read(addr, size, buffer_bak) != 0) {
			printf("rtk_spi_nor_read error\n");
			break;
		} 
		time_end = get_boot_time_ms();
		printf("read time: %ldms\n", time_end - time_start);
		
		if(memcmp(buffer, buffer_bak, size) == 0) {
			printf("test_count %d OK\n", test_count);
			test_count++;
		} else {
			printf("write memcmp error: test count: %d\n", test_count);
			break;
		}
	}

	printf("test_count %d FAIL\n", test_count);
	return 0;
}

static void sfc_set_clk(unsigned int clk)
{
	unsigned int clk_div = 0;

	clk_div  = 500 / clk - 1;
	
	rtd_outl(SB2_SFC_SCK_reg, SB2_SFC_SCK_fdiv(clk_div) | SB2_SFC_SCK_clk_duty50_en(0));
	/*HW SD provide the value :   Phcnt=0x60 / Plcnt=0xF(default) / tdt=0x39 */
	rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_phcnt_mask, SB2_SFC_CE_phcnt(0x60)); 
	rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_plcnt_mask, SB2_SFC_CE_plcnt(0xF));
	rtd_maskl(SB2_SFC_CE_reg, ~SB2_SFC_CE_tdt_mask, SB2_SFC_CE_tdt(0x39));
	rtd_outl(SB2_SFC_POS_LATCH_reg, SB2_SFC_POS_LATCH_delay_sel(0x5) | SB2_SFC_POS_LATCH_pos_latch(0));

	// auto
	rtd_outl(SB2_SFC_EN_WR_reg, SB2_SFC_EN_WR_wt_prog_en(1) | SB2_SFC_EN_WR_wr_en_op(CMD_WRITE_ENABLE));
	rtd_outl(SB2_SFC_WAIT_WR_reg, SB2_SFC_WAIT_WR_wt_prog_done(1) | SB2_SFC_WAIT_WR_rdsr_op(CMD_READ_STATUS1));

	printf("current clk %dMhz, clk_div %d !\r\n", clk, clk_div);

	return;
}

int do_rtk_spi(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = CMD_RET_USAGE;
	unsigned long mem_addr;
	unsigned long spi_addr;
	unsigned long byte_size;

	if( argc == 5 ) {
		mem_addr   = simple_strtoul(argv[2], NULL, 16);
		spi_addr   = simple_strtoul(argv[3], NULL, 16);
		byte_size  = simple_strtoul(argv[4], NULL, 16);
		if( strncmp( argv[1], "read", 4) == 0 ) {
			ret = rtk_spi_nor_read(spi_addr, byte_size, (unsigned char*)mem_addr);
			return (ret ? 0 : 1);
		} else if( strncmp( argv[1], "write", 5 ) == 0 ) {
			ret = rtk_spi_nor_write(spi_addr, byte_size, (unsigned char*)mem_addr);
			return (ret ? 0 : 1);
		}
	} else if(argc == 4) {
		spi_addr   = simple_strtoul(argv[2], NULL, 16);
		byte_size  = simple_strtoul(argv[3], NULL, 16);
		if( strncmp( argv[1], "erase", 5 ) == 0 ) {
			ret = rtk_spi_nor_erase(spi_addr, byte_size);
			return (ret ? 0 : 1);
		}
		if( strncmp( argv[1], "test_cpu", 8 ) == 0 ) {
			md_rw_flag = 0;
			ret = rtk_spi_test(spi_addr, byte_size);
			return (ret ? 0 : 1);
		}
		if( strncmp( argv[1], "test_md", 8 ) == 0 ) {
			md_rw_flag = 1;
			ret = rtk_spi_test(spi_addr, byte_size);
			return (ret ? 0 : 1);
		}
	} else if( argc == 3 ){
		if( strncmp( argv[1], "select", 6 ) == 0 ) {
			unsigned long die_id = simple_strtoul(argv[2], NULL, 16);
			sfc_die_select((unsigned char)die_id);
			SFC_MSG_INFO("Select die%d done!\r\n",  (unsigned char)die_id);
			return 0;
		}
		if( strncmp( argv[1], "setclk", 6 ) == 0 ) {
			unsigned long clock = simple_strtoul(argv[2], NULL, 10);
			sfc_set_clk((unsigned char)clock);
			return 0;
		}
	} else if( argc == 2 ) {
		if( strncmp( argv[1], "init", 4 ) == 0 ) {
			ret = rtk_spi_nor_init();
			return (ret ? 0 : 1);
		} else if( strncmp( argv[1], "unlock", 4 ) == 0 ) {
			ret = rtk_spi_nor_ioctl(SFC_UNLOCK);
			return (ret ? 0 : 1);
		} else if( strncmp( argv[1], "lock", 4 ) == 0 ) {
			ret = rtk_spi_nor_ioctl(SFC_LOCK);
			return (ret ? 0 : 1);
		} else if( strncmp( argv[1], "status", 6 ) == 0 ) {
			ret = sfc_get_status();
			return (ret ? 0 : 1);
		}
	}

	return  ret;
}

U_BOOT_CMD(
	rtkspi, 5, 0, do_rtk_spi,
	"RTK SPI NOR direct function",
	": all argument in HEX type.\n"
	"rtkspi read  DDR_addr SPI_addr size\n"
	"rtkspi write DDR_addr SPI_addr size\n"
	"rtkspi erase SPI_addr size\n"
	"rtkspi init : execute re-initial flow\n"
	"rtkspi test_cpu : test driver with cpu\n"
	"rtkspi test_md : test driver with md\n"
	"rtkspi unlock : cancle write protect\n"
	"rtkspi lock : enable write protect\n"
	"rtkspi select die_id : select die\n"
	"rtkspi status : get status3 value\n"
	"rtkspi setclk clk_value: set spi clock (MHZ)\n"
	"\n"
);
