/************************************************************************
 *
 *  rtk_mcp_core.h
 *
 *  Public header file for CP module
 *
 ************************************************************************/

#ifndef __RTK_MCP_CORE_H__
#define __RTK_MCP_CORE_H__

#include "rtk_mcp_hw.h"

#define MAX_CP_EN_DE_SEGMENT_SIZE   0x00FFFFF0  /*16Mbytes - 16bytes*/
#define MAX_CP_SHA_SEGMENT_SIZE       0x00FFFFC0  /*16Mbytes - 64bytes*/
#define MCP_MAX_CW_ENTRY     128
#define CW_KEY_ENTRY(key)    ((unsigned int)(key) + 1)
#define CP_RESERVE_CW          50

typedef enum _MEM_ALLOC_TYPE
{
    MEM_ALLOC_CACHE = 0,
    MEM_ALLOC_UNCACHE,
}MEM_ALLOC_TYPE;

#define DEFUALT_MEM_ALLOC_TYPE MEM_ALLOC_UNCACHE
#define CP_RESERVE_CW          50

void * MCP_MALLOC(unsigned int len, MEM_ALLOC_TYPE cache_type);
void MCP_FREE(void *data, unsigned int size, MEM_ALLOC_TYPE cache_type);
unsigned int alloc_mcp_descriptor_buffer(MEM_ALLOC_TYPE cache_type);
void free_mcp_descriptor_buffer(unsigned int alloc_mem, MEM_ALLOC_TYPE cache_type);
t_mcp_descriptor *alloc_mcp_descriptor(void);



/************************************************************************
 *  Public functions
 ************************************************************************/
// DES
int DES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int DES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int DES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int DES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
// TDES
int TDES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int TDES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int TDES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int TDES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);

int MCP_AES_Pattern_Decryption(
        unsigned int           BCM,
        unsigned int           Key[4],
        unsigned int           IV[4],
        unsigned long           DataIn,
        unsigned long           DataOut,
        unsigned long           Len,
        unsigned char           crypt_byte_block,
        unsigned char           skip_byte_block);

int MCP_AES_Pattern_Decryption_DADR(
        unsigned int            BCM,
        unsigned int            Key[4],
        unsigned int            IV[4],
        unsigned long            DataIn,
        unsigned long            DataOut[2],
        unsigned long            Len[2],
        unsigned char            crypt_byte_block,
        unsigned char            skip_byte_block);

// AES_128
int AES_CBC_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry, unsigned int ini_key[4]);
int AES_ECB_decrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry);
int AES_ECB_encrypt_key_in_CW(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key_entry);
int AES_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// AES_192
int AES_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// AES_256
int AES_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int AES_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int AES_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// CAM_128
int CAM_ECB_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int CAM_ECB_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4]);
int CAM_CBC_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CBC_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CTR_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
int CAM_CTR_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int key[4], unsigned int ini_key[4]);
// CAM_192
int CAM_ECB_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_ECB_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_CBC_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CBC_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_192_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_192_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
// CAM_256
int CAM_ECB_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_ECB_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key);
int CAM_CBC_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CBC_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_256_decrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);
int CAM_CTR_256_encrypt(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int * key, unsigned int ini_key[4]);

// Digest
int AES_hash(unsigned char * src_addr, int length, unsigned char * dst_addr, unsigned int block_size);

int SHA512_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[16]);
int SHA512_padding(char *p_buff, const unsigned int length);
int SHA384_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[16]);
int SHA384_padding(char *p_buff, const unsigned int length);

int SHA1_hash(unsigned char * src_addr, unsigned int length, unsigned char * dst_addr, unsigned int iv[5]);
int SHA256_hash(unsigned char * src_addr, unsigned int length, unsigned char *dst_addr, unsigned int iv[8]);
int SHA256_padding(char *p_buff, const unsigned int length);
int SHA1_padding(char* p_buff, unsigned long buff_len, const unsigned int data_len);
int MCP_HMAC_SHA256(unsigned char* keyIn, unsigned int keyIn_len, unsigned char* msg, unsigned int msg_len, unsigned char* p_out);
void reverse_signature( unsigned char * pSignature );

int CP_SHA256(unsigned char *p_img, unsigned int img_size,unsigned char *sha256_value);

int HKDF_SHA256(
        unsigned char * ikm, unsigned int ikm_len,
        unsigned char * salt, unsigned int salt_len,
        unsigned char * info, unsigned int info_len,
        unsigned char * okm, unsigned int okm_len);

int CP_SET_CW_KEY(unsigned int cw_key_id, unsigned char key[], unsigned int key_len);
int CP_GET_CW_KEY(unsigned int cw_key_id,unsigned char key[], unsigned int key_len);

#define ALGO_ID_AES_ECB         0
#define ALGO_ID_AES_CBC         1
int CP_AES_Engine_from_CW(unsigned char algo_id,
            unsigned int enc_dec,
            unsigned int key_entry, unsigned int iv_entry,
            void *src_addr, void *dst_addr,
            unsigned long length,unsigned int update_iv);
int RT_AES_Decrypt_from_CW(unsigned char algo_id,
            unsigned int key_entry, unsigned int iv_entry,
            void *src_addr, void *dst_addr,
            unsigned long length,unsigned int update_iv);

#endif
