/************************************************************************
 *
 *  mcp.h
 *
 *  Public header file for CP module
 *
 ************************************************************************/

#ifndef __RTK_MCP_HW_H__
#define __RTK_MCP_HW_H__
#include "rtk_mcp_reg.h"
#include "rtk_mcp_dbg.h"
#include <common.h>
#define BIT(x)                                          (1 <<x)
#define MCP_B4TL(b1, b2, b3, b4)              (((b1) << 24) + ((b2) << 16) + ((b3) << 8) + (b4))


/* 128 bit of AES_H initial vector(h0) */
#define AES_H_IV_0                                  0x2dc2df39
#define AES_H_IV_1                                  0x420321d0
#define AES_H_IV_2                                  0xcef1fe23
#define AES_H_IV_3                                  0x74029d95

/* 128 bit of AES initial vector */
#define AES_IV_0                                      0x2C0AD630
#define AES_IV_1                                      0xF616F1D7
#define AES_IV_2                                      0xF8569AEB
#define AES_IV_3                                      0x5E749771

/* 160 bit SHA1 initial vector */
#ifndef SHA1_IV_0
#define SHA1_IV_0                                   0x67452301
#define SHA1_IV_1                                   0xEFCDAB89
#define SHA1_IV_2                                   0x98BADCFE
#define SHA1_IV_3                                   0x10325476
#define SHA1_IV_4                                   0xC3D2E1F0
#endif

#define SHA1_SIZE                                   20

/* 256 bit SHA256 initial vector */
#define SHA256_H0                                   0x6A09E667
#define SHA256_H1                                   0xBB67AE85
#define SHA256_H2                                   0x3C6EF372
#define SHA256_H3                                   0xA54FF53A
#define SHA256_H4                                   0x510E527F
#define SHA256_H5                                   0x9B05688C
#define SHA256_H6                                   0x1F83D9AB
#define SHA256_H7                                   0x5BE0CD19


#define SHA256_SIZE                                 32
#define MCP_SHA256_BLOCK_SIZE             64
#define MCP_HMAC_SHA256_SIZE               SHA256_SIZE

#define SECURE_KH_KEY_STR                   "abcdef1213572468a1b2c3d49090babe"
#define SECURE_KH_KEY0                          0xabcdef12
#define SECURE_KH_KEY1                          0x13572468
#define SECURE_KH_KEY2                          0xa1b2c3d4
#define SECURE_KH_KEY3                          0x9090babe

#define RSA_SIGNATURE_LENGTH             256

#define DCACHE_FLUSH_INVALIDATE()       
#define DCACHE_FLUSH()                          flush_dcache_all() 
#define SYNC()                                          asm("DSB")
#define PHYS(addr)                                   ((uint)(addr))

typedef struct mcp_descriptor {
    unsigned int mode;
    unsigned int key[6];
    unsigned int ini_key[4];
    unsigned int src_addr;
    unsigned int dst_addr;
    unsigned int length;
    unsigned int dst_addr2;
    unsigned int dst_len;
} __attribute__((aligned(16))) t_mcp_descriptor ;

extern int GenerateMCPDescriptor(
    unsigned int mcp_mode, unsigned int mcp_bcd,
    unsigned int mcp_ende, unsigned int mcp_key_sel,
    unsigned int mcp_iv_sel, unsigned char *src_addr,
    unsigned int length, unsigned char *dst_addr,
    unsigned int *key, unsigned int *ini_key);

extern int reload_otp(void);
extern void update_iv(t_mcp_descriptor *dscpt, unsigned char * vector, bool is_big_endian);
extern void update_key(t_mcp_descriptor *dscpt, unsigned char * vector);
extern int rtk_mcp_do_hw_crypt(t_mcp_descriptor *dscpt);

#endif // __MCP_H__
